@@ -1,5 +1,5 @@
 /*
- * $Id: common.h 12293 2006-11-09 22:55:17Z cbiere $
+ * $Id: common.h 14064 2007-07-07 04:13:57Z cbiere $
  *
  * Copyright (c) 2001-2003, Richard Eckart
  *
@@ -27,7 +27,7 @@
  * @ingroup core
  * @file
  *
- * Common header for Gtk-Gnutella.
+ * Common header for gtk-gnutella.
  *
  * @author Richard Eckart
  * @date 2001-2003
@@ -44,10 +44,10 @@
 
 #define GTA_VERSION 0				/**< major version */
 #define GTA_SUBVERSION 96			/**< minor version */
-#define GTA_PATCHLEVEL 3			/**< patch level or teeny version */
+#define GTA_PATCHLEVEL 4			/**< patch level or teeny version */
 #define GTA_REVISION "stable"		/**< unstable, beta, stable */
 #define GTA_REVCHAR ""				/**< (u)nstable, (b)eta, none -> stable */
-#define GTA_RELEASE "2006-11-09"	/**< ISO 8601 format YYYY-MM-DD */
+#define GTA_RELEASE "2007-07-07"	/**< ISO 8601 format YYYY-MM-DD */
 #define GTA_WEBSITE "http://gtk-gnutella.sourceforge.net/"
 
 #if defined(USE_GTK1)
@@ -61,7 +61,7 @@
 #endif
 
 #ifndef HAS_LIBXML2
-#error "You need libxml2 (http://www.xmlsoft.org/) to compile Gtk-Gnutella"
+#error "You need libxml2 (http://www.xmlsoft.org/) to compile gtk-gnutella"
 #endif
 
 /*
@@ -166,6 +166,7 @@ struct passwd
 #else /* !MINGW32 */
 
 #include <sys/uio.h>		/* For writev(), readv(), struct iovec */
+#include <sys/wait.h>
 
 #ifdef I_NETINET_IN
 #include <netinet/in.h>
@@ -181,13 +182,12 @@ struct passwd
 
 #include <ctype.h>
 
-#ifdef I_FCNTL
-#include <fcntl.h>
-#endif
 #ifdef I_SYS_FILE
 #include <sys/file.h>
 #endif
 
+#include <fcntl.h>
+
 #if !defined(I_FCNTL) && !defined(I_SYS_FILE)
 #include <sys/fcntl.h>		/* Fallback */
 #endif
@@ -211,17 +211,15 @@ struct passwd
 #ifdef I_SYS_PARAM
 #include <sys/param.h>
 #endif
-#ifdef I_SYS_SYSCTL
-#include <sys/sysctl.h>
-#endif
-#ifdef I_INVENT
-#include <invent.h>
-#endif
 
 #ifdef I_INTTYPES
 #include <inttypes.h>
 #endif /* I_INTTYPES */
 
+#ifdef I_SYS_UTSNAME
+#include <sys/utsname.h>		/* For uname() */
+#endif
+
 #ifdef I_SYS_MMAN
 #include <sys/mman.h>
 #endif
@@ -293,11 +291,11 @@ struct passwd
 #include <glib.h>
 
 #if defined(USE_GLIB1) && !defined(GLIB_MAJOR_VERSION)
-#error "Install GLib 1.2 to compile Gtk-Gnutella against GLib 1.2."
+#error "Install GLib 1.2 to compile gtk-gnutella against GLib 1.2."
 #endif
 
 #if defined(USE_GLIB2) && !defined(GLIB_MAJOR_VERSION)
-#error "Install GLib 2.x to compile Gtk-Gnutella against GLib 2.x."
+#error "Install GLib 2.x to compile gtk-gnutella against GLib 2.x."
 #endif
 
 #ifdef USE_LINT
@@ -318,8 +316,6 @@ typedef guint64 filesize_t; /**< Use filesize_t to hold filesizes */
 #include <regex.h>
 #endif
 
-#include <zlib.h>
-
 #ifdef USE_GLIB1
 typedef void (*GCallback) (void);
 #define G_STRLOC __FILE__ ":" STRINGIFY(__LINE__)
@@ -368,12 +364,29 @@ typedef void (*GCallback) (void);
 #define STDERR_FILENO 2
 #endif /* STDERR_FILENO */
 
-/* Determines the maximum value of the given integer type "t". This
- * works for signed as well as unsigned types. However, it's assumed
- * the type consists of exactly sizeof (type) * CHAR_BIT bits. */
+/*
+ * S_ISLNK() is in POSIX.1-1997 but not in older revisions of the standard.
+ */
+#ifndef S_ISLNK
+#define S_ISLNK(mode) (((mode) & S_IFMT) == S_IFLNK)
+#endif	/* S_ISLNK */
+
+/*
+ * These macros determine the maximum/minimum value of the given integer type
+ * "t". This works for signed as well as unsigned types. This code does
+ * carefully avoid integer overflows and undefined behaviour.
+ * However, it's assumed the type consists of exactly sizeof(type) * CHAR_BIT
+ * bits.
+ */
+
+#define MAX_INT_VAL_STEP(t) \
+	((t) 1 << (CHAR_BIT * sizeof(t) - 1 - ((t) -1 < 1)))
+	 
 #define MAX_INT_VAL(t) \
-	(((t) 1 << (CHAR_BIT * sizeof(t) - 1 - ((t) -1 < 0))) \
-   	- 1 + ((t) 1 << (CHAR_BIT * sizeof(t) - 1 - ((t) -1 < 0))))
+	((MAX_INT_VAL_STEP(t) - 1) + MAX_INT_VAL_STEP(t))
+
+#define MIN_INT_VAL(t) \
+	((t) -MAX_INT_VAL(t) - 1)
 
 #ifndef TIME_T_MAX
 /* This assumes time_t is an integer, not a float */
@@ -426,6 +439,15 @@ G_STMT_START {			\
 } G_STMT_END
 
 #if defined(__GNUC__) && defined(__GNUC_MINOR__)
+
+/** HAVE_GCC allows conditionalization depending on the version of gcc
+ *  being used to compile the source.
+ *
+ *  Check each version at "http://gcc.gnu.org/onlinedocs/" for
+ *  support.  Specific functionality may also be broken in some
+ *  compiler revisions, so it is useful to conditionalize on the
+ *  version.
+ */
 #define HAVE_GCC(major, minor) \
 	((__GNUC__ > (major)) || \
 	 (__GNUC__ == (major) && __GNUC_MINOR__ >= (minor)))
@@ -442,7 +464,7 @@ G_STMT_START {			\
  * void my_memcpy(void *dst, const void *src, size_t n) NON_NULL_PARAM((1, 2));
  */
 #if HAVE_GCC(3, 3)
-#define NON_NULL_PARAM(x) __attribute__((nonnull x))
+#define NON_NULL_PARAM(x) __attribute__((__nonnull__ x))
 #else /* GCC < 3.3 */
 #define NON_NULL_PARAM(x)
 #endif
@@ -452,7 +474,7 @@ G_STMT_START {			\
  * of GCC do not allow function attributes for function pointers.
  */
 #if HAVE_GCC(3, 0)
-#define PRINTF_FUNC_PTR(x, y) __attribute__((format(printf, (x), (y))))
+#define PRINTF_FUNC_PTR(x, y) __attribute__((format(__printf__, (x), (y))))
 #else /* GCC < 3.0 */
 #define PRINTF_FUNC_PTR(x, y)
 #endif
@@ -460,7 +482,7 @@ G_STMT_START {			\
 /* Functions using this attribute cause a warning if the returned
  * value is not used. */
 #if HAVE_GCC(3, 4)
-#define WARN_UNUSED_RESULT __attribute__((warn_unused_result))
+#define WARN_UNUSED_RESULT __attribute__((__warn_unused_result__))
 #else /* GCC < 3.4 */
 #define WARN_UNUSED_RESULT
 #endif
@@ -468,7 +490,7 @@ G_STMT_START {			\
 /* Instructs the compiler to emit code for this function even if it is
  * or seems to be unused. */
 #if HAVE_GCC(3, 1)
-#define KEEP_FUNCTION __attribute__((used))
+#define KEEP_FUNCTION __attribute__((__used__))
 #else /* GCC < 3.1 || !GCC */
 #define KEEP_FUNCTION
 #endif
@@ -478,7 +500,8 @@ G_STMT_START {			\
  * ignored in contrast to MUST NOT. Unfortunately, a simple "(void)"
  * does not suppress this warning.
  */
-#define IGNORE_RESULT(x) G_STMT_START { (void) (0 != (x)); }  G_STMT_END
+#define IGNORE_RESULT(x) \
+	G_STMT_START { switch (0 != (x)) { default: ; } }  G_STMT_END
 
 /* Functions using this attribute cause a warning if the variable
  * argument list does not contain a NULL pointer. */
@@ -511,17 +534,17 @@ G_STMT_START {			\
 #endif	/* GCC >= 3.0 */
 #endif	/* G_GNUC_MALLOC */
 
-#if HAVE_GCC(3, 0)
-#define REGPARM(n)	__attribute__((__regparm__((n))))
-#else
-#define REGPARM(n)
-#endif	/* GCC >= 3.0 */
-
-#if HAVE_GCC(3, 0)
+#if HAVE_GCC(3, 1)
 #define ALWAYS_INLINE __attribute__((__always_inline__))
 #else
 #define ALWAYS_INLINE
-#endif	/* GCC >= 3.0 */
+#endif	/* GCC >= 3.1 */
+
+#ifdef HAS_REGPARM
+#define REGPARM(n)	__attribute__((__regparm__((n))))
+#else
+#define REGPARM(n)
+#endif	/* HAS_REGPARM */
 
 /**
  * CMP() returns the sign of a-b, that means -1, 0, or 1.
@@ -562,9 +585,10 @@ G_STMT_START {			\
 
 #define GTA_PORT			6346	/**< Default "standard" port */
 #define MAX_HOSTLEN			256		/**< Max length for FQDN host */
-#define GTA_BUILD					"$Revision: 12293 $"
+#define GTA_BUILD					"$Revision: 14064 $"
 
 /* The next two defines came from huge.h --- Emile */
+#define SHA1_BASE16_SIZE 	40		/**< 160 bits in base16 representation */
 #define SHA1_BASE32_SIZE 	32		/**< 160 bits in base32 representation */
 #define SHA1_RAW_SIZE		20		/**< 160 bits in binary representation */
 
@@ -575,6 +599,9 @@ G_STMT_START {			\
 									 **  includes dot between sha1 and tiger */
 #define BITPRINT_RAW_SIZE		44	/**< 352 bits in binary representation */
 
+/** Maximum bytes in filename i.e., including NUL */
+#define	FILENAME_MAXBYTES 256
+
 /*
  * Forbidden glib calls.
  */
