@@ -1,5 +1,5 @@
 /*
- * $Id: ban.c 12075 2006-09-28 19:56:37Z cbiere $
+ * $Id: ban.c 14029 2007-07-03 15:44:47Z cbiere $
  *
  * Copyright (c) 2002-2003, Raphael Manfredi
  *
@@ -35,13 +35,13 @@
 
 #include "common.h"
 
-RCSID("$Id: ban.c 12075 2006-09-28 19:56:37Z cbiere $")
+RCSID("$Id: ban.c 14029 2007-07-03 15:44:47Z cbiere $")
 
 #include "ban.h"
-#include "sockets.h"
-#include "version.h"		/* For version_is_too_old() */
-#include "token.h"
 #include "sockets.h"		/* For socket_register_fd_reclaimer() */
+#include "token.h"
+#include "version.h"		/* For version_is_too_old() */
+#include "whitelist.h"
 
 #include "lib/atoms.h"
 #include "lib/file.h"		/* For file_register_fd_reclaimer() */
@@ -70,6 +70,8 @@ RCSID("$Id: ban.c 12075 2006-09-28 19:56:37Z cbiere $")
 #define MAX_BAN			10800	/**< 3 hours */
 #define BAN_REMIND		5		/**< Every so many attemps, tell them about it */
 
+#define ban_reason(p)	((p)->ban_msg ? (p)->ban_msg : "N/A")
+
 static GHashTable *info;		/**< Info by IP address */
 static zone_t *ipf_zone;		/**< Zone for addr_info allocation */
 
@@ -87,10 +89,10 @@ struct addr_info {
 	gfloat counter;				/**< Counts connection, decayed linearily */
 	host_addr_t addr;			/**< IP address */
 	time_t ctime;				/**< When did last connection occur? */
-	gpointer cq_ev;				/**< Scheduled callout event */
+	cevent_t *cq_ev;			/**< Scheduled callout event */
 	gint ban_delay;				/**< Banning delay, in seconds */
 	gint ban_count;				/**< Amount of time we banned this source */
-	gchar *ban_msg;				/**< Banning message (atom) */
+	const gchar *ban_msg;		/**< Banning message (atom) */
 	gboolean banned;			/**< Is this IP currently banned? */
 };
 
@@ -140,12 +142,8 @@ ipf_free(struct addr_info *ipf)
 {
 	g_assert(ipf);
 
-	if (ipf->cq_ev)
-		cq_cancel(callout_queue, ipf->cq_ev);
-
-	if (ipf->ban_msg)
-		atom_str_free(ipf->ban_msg);
-
+	cq_cancel(callout_queue, &ipf->cq_ev);
+	atom_str_free_null(&ipf->ban_msg);
 	zfree(ipf_zone, ipf);
 }
 
@@ -162,8 +160,9 @@ ipf_destroy(cqueue_t *unused_cq, gpointer obj)
 	g_assert(!ipf->banned);
 	g_assert(ipf == g_hash_table_lookup(info, &ipf->addr));
 
-	if (ban_debug > 8)
-		g_message("disposing of BAN %s", host_addr_to_string(ipf->addr));
+	if (GNET_PROPERTY(ban_debug) > 8)
+		g_message("disposing of BAN %s: %s",
+			host_addr_to_string(ipf->addr), ban_reason(ipf));
 
 	g_hash_table_remove(info, &ipf->addr);
 	ipf->cq_ev = NULL;
@@ -193,9 +192,9 @@ ipf_unban(cqueue_t *unused_cq, gpointer obj)
 	ipf->counter -= delta_time(now, ipf->ctime) * decay_coeff;
 	ipf->ctime = now;
 
-	if (ban_debug > 4)
-		g_message("removing BAN for %s, counter = %.3f",
-			host_addr_to_string(ipf->addr), ipf->counter);
+	if (GNET_PROPERTY(ban_debug) > 2)
+		g_message("lifting BAN for %s (%s), counter = %.3f",
+			host_addr_to_string(ipf->addr), ban_reason(ipf), ipf->counter);
 
 	/**
 	 * Compute new scheduling delay.
@@ -210,8 +209,9 @@ ipf_unban(cqueue_t *unused_cq, gpointer obj)
 	 */
 
 	if (delay <= 0) {
-		if (ban_debug > 8)
-			g_message("disposing of BAN %s", host_addr_to_string(ipf->addr));
+		if (GNET_PROPERTY(ban_debug) > 8)
+			g_message("disposing of BAN %s: %s",
+				host_addr_to_string(ipf->addr), ban_reason(ipf));
 
 		g_hash_table_remove(info, &ipf->addr);
 		ipf->cq_ev = NULL;
@@ -220,6 +220,7 @@ ipf_unban(cqueue_t *unused_cq, gpointer obj)
 	}
 
 	ipf->banned = FALSE;
+	atom_str_free_null(&ipf->ban_msg);
 	ipf->cq_ev = cq_insert(callout_queue, delay, ipf_destroy, ipf);
 }
 
@@ -247,6 +248,9 @@ ban_allow(const host_addr_t addr)
 		return BAN_OK;
 	}
 
+	if (whitelist_check(addr))
+		return BAN_OK;
+
 	ipf = g_hash_table_lookup(info, &addr);
 
 	/*
@@ -279,7 +283,7 @@ ban_allow(const host_addr_t addr)
 	ipf->counter += 1.0;
 	ipf->ctime = now;
 
-	if (ban_debug > 4)
+	if (GNET_PROPERTY(ban_debug) > 4)
 		g_message("BAN %s, counter = %.3f (%s)",
 			host_addr_to_string(ipf->addr), ipf->counter,
 			ipf->banned ? "already banned" :
@@ -317,9 +321,10 @@ ban_allow(const host_addr_t addr)
 	 */
 
 	if (ipf->counter > (gfloat) MAX_REQUEST) {
-		cq_cancel(callout_queue, ipf->cq_ev);	/* Cancel ipf_destroy */
+		cq_cancel(callout_queue, &ipf->cq_ev);	/* Cancel ipf_destroy */
 
 		ipf->banned = TRUE;
+		atom_str_change(&ipf->ban_msg, "Too frequent connections");
 
 		if (ipf->ban_delay)
 			ipf->ban_delay *= 2;
@@ -368,19 +373,20 @@ ban_record(const host_addr_t addr, const gchar *msg)
 		g_hash_table_insert(info, &ipf->addr, ipf);
 	}
 
-	if (ipf->ban_msg != NULL)
-		atom_str_free(ipf->ban_msg);
-
-	ipf->ban_msg = atom_str_get(msg);
+	atom_str_change(&ipf->ban_msg, msg);
 	ipf->ban_delay = MAX_BAN;
 
+	if (GNET_PROPERTY(ban_debug))
+		g_message("BAN %s record %s: %s",
+			ipf->banned ? "updating" : "new",
+			host_addr_to_string(ipf->addr), ban_reason(ipf));
+
 	if (ipf->banned)
 		cq_resched(callout_queue, ipf->cq_ev, MAX_BAN * 1000);
 	else {
-		cq_cancel(callout_queue, ipf->cq_ev);	/* Cancel ipf_destroy */
+		cq_cancel(callout_queue, &ipf->cq_ev);	/* Cancel ipf_destroy */
 		ipf->banned = TRUE;
-		ipf->cq_ev =
-			cq_insert(callout_queue, MAX_BAN * 1000, ipf_unban, ipf);
+		ipf->cq_ev = cq_insert(callout_queue, MAX_BAN * 1000, ipf_unban, ipf);
 	}
 }
 
@@ -410,16 +416,16 @@ reclaim_fd(void)
 
 	if (banned_tail == NULL) {
 		g_assert(banned_head == NULL);
-		g_assert(banned_count == 0);
+		g_assert(GNET_PROPERTY(banned_count) == 0);
 		return FALSE;					/* Empty list */
 	}
 
 	g_assert(banned_head != NULL);
-	g_assert(banned_count > 0);
+	g_assert(GNET_PROPERTY(banned_count) > 0);
 
 	(void) close(GPOINTER_TO_INT(banned_tail->data));	/* Reclaim fd */
 
-	if (ban_debug > 9)
+	if (GNET_PROPERTY(ban_debug) > 9)
 		g_message("closed BAN fd #%d", GPOINTER_TO_INT(banned_tail->data));
 
 	prev = g_list_previous(banned_tail);
@@ -427,7 +433,7 @@ reclaim_fd(void)
 	g_list_free_1(banned_tail);
 	banned_tail = prev;
 
-	gnet_prop_set_guint32_val(PROP_BANNED_COUNT, banned_count - 1);
+	gnet_prop_decr_guint32(PROP_BANNED_COUNT);
 
 	return TRUE;
 }
@@ -472,15 +478,23 @@ ban_reclaim_fd(void)
 void
 ban_force(struct gnutella_socket *s)
 {
-	gint fd = s->file_desc;
+	gint fd;
 
-	if (banned_count >= max_banned_fd) {
+	socket_check(s);
+	fd = s->file_desc;
+	g_return_if_fail(fd >= 0);
+
+	if (GNET_PROPERTY(banned_count) >= GNET_PROPERTY(max_banned_fd)) {
 		g_assert(banned_tail);
-		g_assert(max_banned_fd <= 1 || (banned_tail != banned_head));
+		g_assert(GNET_PROPERTY(max_banned_fd) <= 1 ||
+			banned_tail != banned_head);
 
 		reclaim_fd();
 	}
 
+	/* Ensure we're not listening to I/O events anymore. */
+	socket_evt_clear(s);
+
 	/*
 	 * Shrink socket buffers.
 	 */
@@ -504,7 +518,7 @@ ban_force(struct gnutella_socket *s)
 	if (banned_tail == NULL)
 		banned_tail = banned_head;
 
-	gnet_prop_set_guint32_val(PROP_BANNED_COUNT, banned_count + 1);
+	gnet_prop_incr_guint32(PROP_BANNED_COUNT);
 }
 
 /**
@@ -537,7 +551,7 @@ ban_delay(const host_addr_t addr)
 /**
  * @return banning message for banned IP.
  */
-gchar *
+const gchar *
 ban_message(const host_addr_t addr)
 {
 	struct addr_info *ipf;
@@ -570,10 +584,11 @@ ban_max_recompute(void)
 {
 	guint32 max;
 
-	max = MIN(ban_max_fds, sys_nofile * ban_ratio_fds / 100);
+	max = (GNET_PROPERTY(sys_nofile) * GNET_PROPERTY(ban_ratio_fds)) / 100;
+	max = MIN(GNET_PROPERTY(ban_max_fds), max);
 	max = MAX(1, max);
 
-	if (ban_debug)
+	if (GNET_PROPERTY(ban_debug))
 		g_message("will use at most %d file descriptor%s for banning",
 			max, max == 1 ? "" : "s");
 
