@@ -1,5 +1,5 @@
 /*
- * $Id: bh_upload.c 12058 2006-09-27 19:33:55Z cbiere $
+ * $Id: bh_upload.c 13916 2007-06-21 20:39:04Z cbiere $
  *
  * Copyright (c) 2005, Christian Biere & Raphael Manfredi
  *
@@ -36,7 +36,7 @@
 
 #include "common.h"
 
-RCSID("$Id: bh_upload.c 12058 2006-09-27 19:33:55Z cbiere $")
+RCSID("$Id: bh_upload.c 13916 2007-06-21 20:39:04Z cbiere $")
 
 #include "bh_upload.h"
 #include "share.h"
@@ -50,13 +50,16 @@ RCSID("$Id: bh_upload.c 12058 2006-09-27 19:33:55Z cbiere $")
 #include "guid.h"
 #include "version.h"
 
+#include "if/core/hosts.h"
 #include "if/gnet_property.h"
 #include "if/gnet_property_priv.h"
 
+#include "lib/array.h"
 #include "lib/header.h"
 #include "lib/misc.h"
 #include "lib/url.h"
 #include "lib/walloc.h"
+
 #include "lib/override.h"	/* Must be the last header included */
 
 /**
@@ -88,8 +91,8 @@ enum bh_type {
 	BH_TYPE_QHIT,			/* Send back Gnutella query hits */
 };
 
-struct browse_host_ctx {
-	struct special_ctx special;	/**< vtable, MUST be first field */
+struct browse_host_upload {
+	struct special_upload special;	/**< vtable, MUST be first field */
 	enum bh_type type;		/**< Type of data to send back */
 	gint flags;				/**< Opening flags */
 	txdrv_t *tx;			/**< The transmission stack */
@@ -101,10 +104,16 @@ struct browse_host_ctx {
 	guint file_index;		/**< Current file index (iterator) */
 	enum bh_state state;	/**< Current state of the state machine */
 	GSList *hits;			/**< Pending query hits to send back */
-	bh_closed_t cb;			/**< Callback to invoke when TX fully flushed */
+	special_upload_closed_t cb;	/**< Callback to invoke when TX fully flushed */
 	gpointer cb_arg;		/**< Callback argument */
 };
 
+struct browse_host_upload *
+cast_to_browse_host_upload(struct special_upload *p)
+{
+	return (void *) p;
+}
+
 /**
  * Copies up to ``*size'' bytes from current data block
  * (bh->b_data + bh->b_offset) to the buffer ``dest''.
@@ -117,7 +126,7 @@ struct browse_host_ctx {
  * @return The amount of bytes copied. Use this to advance ``dest''.
  */
 static inline size_t
-browse_host_read_data(struct browse_host_ctx *bh, gchar *dest, size_t *size)
+browse_host_read_data(struct browse_host_upload *bh, gchar *dest, size_t *size)
 {
 	size_t len;
 
@@ -140,7 +149,7 @@ browse_host_read_data(struct browse_host_ctx *bh, gchar *dest, size_t *size)
  * data block variables.
  */
 static inline void
-browse_host_next_state(struct browse_host_ctx *bh, enum bh_state state)
+browse_host_next_state(struct browse_host_upload *bh, enum bh_state state)
 {
 	g_assert(NULL != bh);
 	g_assert((gint) state >= 0 && state < NUM_BH_STATES);
@@ -168,7 +177,8 @@ browse_host_next_state(struct browse_host_ctx *bh, enum bh_state state)
  *         is returned.
  */
 static ssize_t
-browse_host_read_html(gpointer ctx, gpointer const dest, size_t size)
+browse_host_read_html(struct special_upload *ctx,
+	gpointer const dest, size_t size)
 {
 	static const gchar header[] =
 		"<!DOCTYPE html PUBLIC \"-//W3C//DTD HTML 4.01//EN\">\r\n"
@@ -178,7 +188,7 @@ browse_host_read_html(gpointer ctx, gpointer const dest, size_t size)
 		"</head>\r\n"
 		"<body>\r\n";
 	static const gchar trailer[] = "</ul>\r\n</body>\r\n</html>\r\n";
-	struct browse_host_ctx *bh = ctx;
+	struct browse_host_upload *bh = cast_to_browse_host_upload(ctx);
 	gchar *p = dest;
 
 	g_assert(NULL != bh);
@@ -204,14 +214,15 @@ browse_host_read_html(gpointer ctx, gpointer const dest, size_t size)
 		case BH_STATE_LIBRARY_INFO:
 			if (!bh->b_data) {
 				bh->w_buf_size = w_concat_strings(&bh->w_buf,
-					"<h1>Gtk-Gnutella</h1>\r\n"
+					"<h1>gtk-gnutella</h1>\r\n"
 					"<h3>", version_get_string(),
 				   	" sharing ",
 					uint64_to_string(shared_files_scanned()),
 					" file",
 					shared_files_scanned() == 1 ? "" : "s",
 					" ",
-					short_kb_size(shared_kbytes_scanned(), FALSE),
+					short_kb_size(shared_kbytes_scanned(),
+						GNET_PROPERTY(display_metric_units)),
 					" total</h3>\r\n"
 					"<ul>\r\n", (void *) 0);
 				bh->b_data = bh->w_buf;
@@ -259,18 +270,34 @@ browse_host_read_html(gpointer ctx, gpointer const dest, size_t size)
 					size_t html_size;
 					gchar *html_name;
 
-					html_size = 1 + html_escape(name_nfc, NULL, 0);
-					html_name = walloc(html_size);
-					html_escape(name_nfc, html_name, html_size);
+					{
+						const gchar *dir;
+						gchar *name;
+						
+						dir = shared_file_relative_path(sf);
+						if (dir) {
+							name = g_strconcat(dir, "/", name_nfc, (void *) 0);
+						} else {
+							name = deconstify_gchar(name_nfc);
+						}
+
+						html_size = 1 + html_escape(name, NULL, 0);
+						html_name = walloc(html_size);
+						html_escape(name, html_name, html_size);
+						if (name != name_nfc) {
+							G_FREE_NULL(name);
+						}
+					}
 
 					if (sha1_hash_available(sf)) {
-						const gchar * const sha1 = shared_file_sha1(sf);
+						const struct sha1 *sha1 = shared_file_sha1(sf);
 
 						bh->w_buf_size = w_concat_strings(&bh->w_buf,
 							"<li><a href=\"/uri-res/N2R?urn:sha1:",
 							sha1_base32(sha1),
 							"\">", html_name, "</a>&nbsp;[",
-							short_html_size(file_size, display_metric_units),
+							short_html_size(file_size,
+								GNET_PROPERTY(display_metric_units)),
 							"]</li>\r\n",
 							(void *) 0);
 					} else {
@@ -279,10 +306,11 @@ browse_host_read_html(gpointer ctx, gpointer const dest, size_t size)
 						escaped = url_escape(name_nfc);
 						bh->w_buf_size = w_concat_strings(&bh->w_buf,
 							"<li><a href=\"/get/",
-							uint64_to_string(shared_file_index(sf)),
+							uint32_to_string(shared_file_index(sf)),
 							"/", escaped, "\">", html_name, "</a>"
 							"&nbsp;[",
-							short_html_size(file_size, display_metric_units),
+							short_html_size(file_size,
+								GNET_PROPERTY(display_metric_units)),
 							"]</li>\r\n", (void *) 0);
 
 						if (escaped != name_nfc) {
@@ -338,7 +366,7 @@ browse_host_read_html(gpointer ctx, gpointer const dest, size_t size)
 static void
 browse_host_record_hit(gpointer data, size_t len, gpointer udata)
 {
-	struct browse_host_ctx *bh = udata;
+	struct browse_host_upload *bh = udata;
 
 	bh->hits = g_slist_prepend(bh->hits, gmsg_to_pmsg(data, len));
 }
@@ -359,9 +387,10 @@ browse_host_record_hit(gpointer data, size_t len, gpointer udata)
  *         is returned.
  */
 static ssize_t
-browse_host_read_qhits(gpointer ctx, gpointer const dest, size_t size)
+browse_host_read_qhits(struct special_upload *ctx,
+	gpointer const dest, size_t size)
 {
-	struct browse_host_ctx *bh = ctx;
+	struct browse_host_upload *bh = cast_to_browse_host_upload(ctx);
 	size_t remain = size;
 	gchar *p = dest;
 
@@ -376,18 +405,16 @@ browse_host_read_qhits(gpointer ctx, gpointer const dest, size_t size)
 		for (i = 0; i < BH_SCAN_AHEAD; i++) {
 			const shared_file_t *sf;
 
-		skip:
-			bh->file_index++;
-			sf = shared_file(bh->file_index);
+			do {
+				/* Skip holes in indices */
+				bh->file_index++;
+				sf = shared_file(bh->file_index);
+			} while (NULL == sf && bh->file_index <= shared_files_scanned());
 
-			if (NULL == sf) {
-				if (bh->file_index > shared_files_scanned())
-					break;
-				goto skip;		/* Skip holes in indices */
-			} else if (SHARE_REBUILDING == sf)
+			if (SHARE_REBUILDING == sf || NULL == sf)
 				break;
-			else
-				files = g_slist_prepend(files, deconstify_gpointer(sf));
+			
+			files = g_slist_prepend(files, deconstify_gpointer(sf));
 		}
 
 		if (NULL == files)		/* Did not find any more file to include */
@@ -400,13 +427,14 @@ browse_host_read_qhits(gpointer ctx, gpointer const dest, size_t size)
 		files = g_slist_reverse(files);			/* Preserve order */
 
 		qhit_build_results(files, i, BH_MAX_QHIT_SIZE,
-			browse_host_record_hit, bh,
-			blank_guid, TRUE);
+			browse_host_record_hit, bh, blank_guid, FALSE, &zero_array);
 
 		g_assert(bh->hits != NULL);		/* At least 1 hit enqueued */
 
 		bh->hits = g_slist_reverse(bh->hits);	/* Preserve order */
- 	}
+ 		g_slist_free(files);
+		files = NULL;
+	}
 
 	/*
 	 * Read each query hit in turn.
@@ -421,8 +449,8 @@ browse_host_read_qhits(gpointer ctx, gpointer const dest, size_t size)
 		remain -= r;
 
 		if (r == 0 || 0 == pmsg_size(mb)) {
-			pmsg_free(mb);
 			bh->hits = g_slist_remove(bh->hits, mb);
+			pmsg_free(mb);
 		}
 	}
 
@@ -433,9 +461,9 @@ browse_host_read_qhits(gpointer ctx, gpointer const dest, size_t size)
  * Write data to the TX stack.
  */
 ssize_t
-browse_host_write(gpointer ctx, gconstpointer data, size_t size)
+browse_host_write(struct special_upload *ctx, gconstpointer data, size_t size)
 {
-	struct browse_host_ctx *bh = ctx;
+	struct browse_host_upload *bh = cast_to_browse_host_upload(ctx);
 
 	g_assert(bh->tx);
 
@@ -448,7 +476,7 @@ browse_host_write(gpointer ctx, gconstpointer data, size_t size)
 static void
 browse_tx_flushed(txdrv_t *unused_tx, gpointer arg)
 {
-	struct browse_host_ctx *bh = arg;
+	struct browse_host_upload *bh = arg;
 
 	(void) unused_tx;
 
@@ -463,9 +491,10 @@ browse_tx_flushed(txdrv_t *unused_tx, gpointer arg)
  * Flush the TX stack, invoking callback when it's done.
  */
 static void
-browse_host_flush(gpointer ctx, bh_closed_t cb, gpointer arg)
+browse_host_flush(struct special_upload *ctx,
+	special_upload_closed_t cb, gpointer arg)
 {
-	struct browse_host_ctx *bh = ctx;
+	struct browse_host_upload *bh = cast_to_browse_host_upload(ctx);
 
 	g_assert(bh->tx);
 
@@ -486,9 +515,9 @@ browse_host_flush(gpointer ctx, bh_closed_t cb, gpointer arg)
  * @return An initialized browse host context.
  */
 void
-browse_host_close(gpointer ctx, gboolean fully_served)
+browse_host_close(struct special_upload *ctx, gboolean fully_served)
 {
-	struct browse_host_ctx *bh = ctx;
+	struct browse_host_upload *bh = cast_to_browse_host_upload(ctx);
 	GSList *sl;
 
 	g_assert(bh);
@@ -510,12 +539,10 @@ browse_host_close(gpointer ctx, gboolean fully_served)
 	 */
 
 	if (fully_served) {
-		if (bh->flags & BH_HTML) {
-			gnet_prop_set_guint32_val(PROP_HTML_BROWSE_SERVED,
-				html_browse_served + 1);
-		} else if (bh->flags & BH_QHITS) {
-			gnet_prop_set_guint32_val(PROP_QHITS_BROWSE_SERVED,
-				qhits_browse_served + 1);
+		if (bh->flags & BH_F_HTML) {
+			gnet_prop_incr_guint32(PROP_HTML_BROWSE_SERVED);
+		} else if (bh->flags & BH_F_QHITS) {
+			gnet_prop_incr_guint32(PROP_QHITS_BROWSE_SERVED);
 		}
 	}
 
@@ -536,25 +563,26 @@ browse_host_close(gpointer ctx, gboolean fully_served)
  *
  * @return An initialized browse host context.
  */
-struct special_ctx *
+struct special_upload *
 browse_host_open(
 	gpointer owner,
-	gnet_host_t *host,
-	bh_writable_t writable,
-	struct tx_deflate_cb *deflate_cb,
-	struct tx_link_cb *link_cb,
-	wrap_io_t *wio,
+	struct gnutella_host *host,
+	special_upload_writable_t writable,
+	const struct tx_deflate_cb *deflate_cb,
+	const struct tx_link_cb *link_cb,
+	struct wrap_io *wio,
 	gint flags)
 {
-	struct browse_host_ctx *bh;
+	struct browse_host_upload *bh;
 
 	/* BH_HTML xor BH_QHITS set */
-	g_assert(flags & (BH_HTML|BH_QHITS));
-	g_assert((flags & (BH_HTML|BH_QHITS)) != (BH_HTML|BH_QHITS));
+	g_assert(flags & (BH_F_HTML|BH_F_QHITS));
+	g_assert((flags & (BH_F_HTML|BH_F_QHITS)) != (BH_F_HTML|BH_F_QHITS));
 
 	bh = walloc(sizeof *bh);
-	bh->special.read =
-		(flags & BH_HTML) ? browse_host_read_html : browse_host_read_qhits;
+	bh->special.read = (flags & BH_F_HTML)
+						? browse_host_read_html
+						: browse_host_read_qhits;
 	bh->special.write = browse_host_write;
 	bh->special.flush = browse_host_flush;
 	bh->special.close = browse_host_close;
@@ -573,34 +601,34 @@ browse_host_open(
 
 		args.cb = link_cb;
 		args.wio = wio;
-		args.bs = bws.out;
+		args.bws = BSCHED_BWS_OUT;
 
 		bh->tx = tx_make(owner, host, tx_link_get_ops(), &args);
 	}
 
-	if (flags & BH_CHUNKED)
+	if (flags & BH_F_CHUNKED) {
 		bh->tx = tx_make_above(bh->tx, tx_chunk_get_ops(), 0);
-
-	if (flags & (BH_DEFLATE | BH_GZIP)) {
+	}
+	if (flags & (BH_F_DEFLATE | BH_F_GZIP)) {
 		struct tx_deflate_args args;
-		txdrv_t *ctx;
+		txdrv_t *tx;
 
 		args.cq = callout_queue;
 		args.cb = deflate_cb;
 		args.nagle = FALSE;
-		args.gzip = 0 != (flags & BH_GZIP);
+		args.gzip = 0 != (flags & BH_F_GZIP);
 		args.buffer_flush = INT_MAX;		/* Flush only at the end */
 		args.buffer_size = BH_BUFSIZ;
 
-		ctx = tx_make_above(bh->tx, tx_deflate_get_ops(), &args);
-		if (ctx == NULL) {
+		tx = tx_make_above(bh->tx, tx_deflate_get_ops(), &args);
+		if (tx == NULL) {
 			tx_free(bh->tx);
 			link_cb->eof_remove(owner, "Cannot setup compressing TX stack");
 			wfree(bh, sizeof *bh);
 			return NULL;
 		}
 
-		bh->tx = ctx;
+		bh->tx = tx;
 	}
 
 	/*
@@ -615,13 +643,10 @@ browse_host_open(
 	 * Update statistics.
 	 */
 
-	if (flags & BH_HTML) {
-		gnet_prop_set_guint32_val(PROP_HTML_BROWSE_COUNT,
-			html_browse_count + 1);
-	}
-	if (flags & BH_QHITS) {
-		gnet_prop_set_guint32_val(PROP_QHITS_BROWSE_COUNT,
-			qhits_browse_count + 1);
+	if (flags & BH_F_HTML) {
+		gnet_prop_incr_guint32(PROP_HTML_BROWSE_COUNT);
+	} else if (flags & BH_F_QHITS) {
+		gnet_prop_incr_guint32(PROP_QHITS_BROWSE_COUNT);
 	}
 	return &bh->special;
 }
