@@ -1,5 +1,5 @@
 /*
- * $Id: bitzi.c 11701 2006-08-23 17:39:48Z cbiere $
+ * $Id: bitzi.c 13970 2007-06-24 20:17:48Z cbiere $
  *
  * Copyright (c) 2004, Alex Bennee <alex@bennee.com>
  *
@@ -56,6 +56,7 @@
 #include "lib/getdate.h"	/* date2time() */
 #include "lib/glib-missing.h"
 #include "lib/tm.h"
+#include "lib/urn.h"
 #include "lib/walloc.h"
 #include "lib/override.h"	/* This file MUST be the last one included */
 
@@ -70,7 +71,8 @@
 static const gchar bitzi_url_fmt[] = "http://ticket.bitzi.com/rdf/urn:sha1:%s";
 
 typedef struct {
-	gchar *sha1;			/**< binary SHA-1, atom */
+	const struct sha1 *sha1;			/**< binary SHA-1, atom */
+	filesize_t filesize;
 	gchar bitzi_url[SHA1_BASE32_SIZE + sizeof bitzi_url_fmt]; /**< request URL */
 
 	/*
@@ -82,9 +84,9 @@ typedef struct {
 /*
  * The request queue, the searches to the Bitzi data service are queued
  */
-static GSList *bitzi_rq = NULL;
+static GSList *bitzi_rq;
 
-static bitzi_request_t	*current_bitzi_request = NULL;
+static bitzi_request_t	*current_bitzi_request;
 static gpointer	 current_bitzi_request_handle;
 static guint bitzi_heartbeat_id;
 
@@ -123,6 +125,19 @@ xml_get_string(xmlNode *node, const gchar *id)
 	return (gchar *) xmlGetProp(node, (const xmlChar *) id);
 }
 
+/**
+ * Use this to free strings returned by xml_get_string().
+ */
+static inline void
+xml_free_null(gchar **ptr)
+{
+	g_assert(ptr);
+	if (*ptr) {
+		xmlFree(*ptr);
+		*ptr = NULL;
+	}
+}
+
 static inline const xmlChar *
 string_to_xmlChar(const gchar *s)
 {
@@ -159,15 +174,17 @@ bitzi_create(void)
 static void
 bitzi_destroy(bitzi_data_t *data)
 {
-	if (bitzi_debug) {
-		g_message("bitzi_destroy: %p", cast_to_gconstpointer(data));
+	g_assert(data);
+
+	if (GNET_PROPERTY(bitzi_debug)) {
+		g_message("bitzi_clear: %p", cast_to_gconstpointer(data));
 	}
 
 	atom_sha1_free_null(&data->sha1);
-	G_FREE_NULL(data->mime_type);
-	G_FREE_NULL(data->mime_desc);
+	atom_str_free_null(&data->mime_type);
+	atom_str_free_null(&data->mime_desc);
 
-	if (bitzi_debug) {
+	if (GNET_PROPERTY(bitzi_debug)) {
 		g_message("bitzi_destroy: freeing data");
 	}
 	wfree(data, sizeof *data);
@@ -183,7 +200,7 @@ bitzi_destroy(bitzi_data_t *data)
  * the parsing of the document tree and processes the ticket.
  */
 static void
-bitzi_host_data_ind(gpointer unused_handle, gchar *data, gint len)
+bitzi_host_data_ind(struct http_async *unused_handle, gchar *data, gint len)
 {
 	gint result;
 
@@ -215,7 +232,7 @@ bitzi_host_data_ind(gpointer unused_handle, gchar *data, gint len)
  * HTTP request is being stopped.
  */
 static void
-bitzi_host_error_ind(gpointer handle,
+bitzi_host_error_ind(struct http_async *handle,
 	http_errtype_t unused_type, gpointer unused_v)
 {
 	(void) unused_type;
@@ -262,6 +279,8 @@ struct efj_t {
 
 static const struct efj_t enum_fj_table[] = {
 	{ "Unknown",				BITZI_FJ_UNKNOWN },
+	{ "Bitzi lookup failure",	BITZI_FJ_FAILURE },
+	{ "Filesize mismatch",		BITZI_FJ_WRONG_FILESIZE },
 	{ "Dangerous/Misleading",	BITZI_FJ_DANGEROUS_MISLEADING },
 	{ "Incomplete/Damaged",		BITZI_FJ_INCOMPLETE_DAMAGED },
 	{ "Substandard",			BITZI_FJ_SUBSTANDARD },
@@ -280,39 +299,27 @@ static const struct efj_t enum_fj_table[] = {
 static void
 process_rdf_description(xmlNode *node, bitzi_data_t *data)
 {
-	gchar *s;
+	gchar *value;
 
 	/*
 	 * We extract the urn:sha1 from the ticket as we may be processing
 	 * cached tickets not associated with any actual request. The
 	 * bitprint itself will be at offset 9 into the string.
 	 */
-	s = xml_get_string(node, "about");
-	if (s) {
-		static const gchar urn_prefix[] = "urn:sha1:";
-		const gchar *sha1;
+	value = STRTRACK(xml_get_string(node, "about"));
+	if (value) {
+		struct sha1 sha1;
 
-		sha1 = is_strprefix(s, urn_prefix);
-		if (sha1) {
-			size_t len;
-
-			/* Skip the "urn:sha1:" prefix and check whether an SHA1
-			 * follows. We need to ensure that buf contains at least
-			 * SHA1_BASE32_SIZE bytes because that's what base32_sha1()
-			 * assumes. We allow trailing characters. */
-
-			len = strlen(sha1);
-			sha1 = len < SHA1_BASE32_SIZE ? NULL : base32_sha1(sha1);
-		}
-
-		if (!sha1) {
-			g_warning("process_rdf_description: bad 'about' string: \"%s\"", s);
+		if (urn_get_sha1(value, &sha1)) {
+			data->sha1 = atom_sha1_get(&sha1);
 		} else {
-			data->sha1 = atom_sha1_get(sha1);
+			g_warning("process_rdf_description: bad 'about' string: \"%s\"",
+				value);
 		}
 	} else {
 		g_warning("process_rdf_description: No SHA-1!");
 	}
+	xml_free_null(&value);
 
 
 	/*
@@ -321,39 +328,40 @@ process_rdf_description(xmlNode *node, bitzi_data_t *data)
 	 *
 	 * CHECK: date parse deals with timezone? can it fail?
 	 */
-	s = xml_get_string(node, "ticketExpires");
-	if (s) {
-		data->expiry = date2time(s, tm_time());
+	value = STRTRACK(xml_get_string(node, "ticketExpires"));
+	if (value) {
+		data->expiry = date2time(value, tm_time());
 		if ((time_t) -1 == data->expiry)
-			g_warning("process_rdf_description: Bad expiration date \"%s\"", s);
+			g_warning("process_rdf_description: Bad expiration date \"%s\"",
+				value);
 	} else {
 		g_warning("process_rdf_description: No ticketExpires!");
 	}
+	xml_free_null(&value);
 
 	/*
 	 * fileGoodness amd fileJudgement are the two most imeadiatly
 	 * useful values.
 	 */
-	s = xml_get_string(node, "fileGoodness");
-	if (s) {
-		data->goodness = g_strtod(s, NULL);
-		if (bitzi_debug)
-			g_message("fileGoodness is %s/%f", s, data->goodness);
+	value = STRTRACK(xml_get_string(node, "fileGoodness"));
+	if (value) {
+		data->goodness = g_strtod(value, NULL);
+		if (GNET_PROPERTY(bitzi_debug))
+			g_message("fileGoodness is %s/%f", value, data->goodness);
 	} else {
 		data->goodness = 0;
 	}
+	xml_free_null(&value);
 
-	data->judgement = BITZI_FJ_UNKNOWN;
-
-	s = xml_get_string(node, "fileJudgement");
-	if (s) {
+	value = STRTRACK(xml_get_string(node, "fileJudgement"));
+	if (value) {
 		size_t i;
 
 		STATIC_ASSERT(NUM_BITZI_FJ == G_N_ELEMENTS(enum_fj_table));
 
 		for (i = 0; i < G_N_ELEMENTS(enum_fj_table); i++) {
 			if (
-				xmlStrEqual(string_to_xmlChar(s),
+				xmlStrEqual(string_to_xmlChar(value),
 					string_to_xmlChar(enum_fj_table[i].string))
 			) {
 				data->judgement = enum_fj_table[i].judgement;
@@ -361,17 +369,18 @@ process_rdf_description(xmlNode *node, bitzi_data_t *data)
 			}
 		}
 	}
-
+	xml_free_null(&value);
 
 	/*
 	 * fileLength, useful for comparing to result
 	 */
 
-	s = xml_get_string(node, "fileLength");
-	if (s) {
+	value = STRTRACK(xml_get_string(node, "fileLength"));
+	if (value) {
 		gint error;
-		data->size = parse_uint64(s, NULL, 10, &error);
+		data->size = parse_uint64(value, NULL, 10, &error);
 	}
+	xml_free_null(&value);
 
 	/*
 	 * The multimedia type, bitrate etc is all built into one
@@ -380,58 +389,93 @@ process_rdf_description(xmlNode *node, bitzi_data_t *data)
 	 * Currently we handle video and audio
 	 */
 
-	s = xml_get_string(node, "format");
-	if (s) {
-		if (xmlStrstr(string_to_xmlChar(s), string_to_xmlChar("video"))) {
-			gchar *xml_width = xml_get_string(node, "videoWidth");
-			gchar *xml_height = xml_get_string(node, "videoHeight");
-			gchar *xml_bitrate = xml_get_string(node, "videoBitrate");
-			gchar *xml_fps = xml_get_string(node, "videoFPS");
-
-			/*
-			 * copy the mime type
-			 */
-			data->mime_type = g_strdup(s);
+	value = STRTRACK(xml_get_string(node, "format"));
+	if (value) {
+		/*
+		 * copy the mime type
+		 */
+		atom_str_change(&data->mime_type, value);
 
+		if (is_strcaseprefix(value, "video")) {
+			gchar *bitrate = STRTRACK(xml_get_string(node, "videoBitrate"));
+			gchar *fps = STRTRACK(xml_get_string(node, "videoFPS"));
+			gchar *height = STRTRACK(xml_get_string(node, "videoHeight"));
+			gchar *width = STRTRACK(xml_get_string(node, "videoWidth"));
+			gboolean has_res = width && height;
+			gchar desc[256];
+				
 			/*
 			 * format the mime details
 			 */
-			{
-				gboolean has_res = xml_width && xml_height;
-				/**
-	 			 * TRANSLATORS: This describes video parameters;
-				 * The first part is used as <width>x<height> (resolution).
-				 * fps stands for "frames per second".
-				 * kbps stands for "kilobit per second" (metric kilo).
-	 			 */
-				data->mime_desc =
-					g_strdup_printf(_("%s%s%s%s%s fps, %s kbps"),
-						has_res ? xml_width : "",
-						has_res ? Q_("times|x") : "",
-						has_res ? xml_height : "",
-						has_res ? ", " : "",
-						(xml_fps != NULL) ? xml_fps : "?",
-						(xml_bitrate != NULL) ? xml_bitrate : "?");
+
+			/**
+			 * TRANSLATORS: This describes video parameters;
+			 * The first part is used as <width>x<height> (resolution).
+			 * fps stands for "frames per second".
+			 * kbps stands for "kilobit per second" (metric kilo).
+			 */
+			gm_snprintf(desc, sizeof desc, _("%s%s%s%s%s fps, %s kbps"),
+					has_res ? width : "",
+					has_res ? Q_("times|x") : "",
+					has_res ? height : "",
+					has_res ? ", " : "",
+					(fps != NULL) ? fps : "?",
+					(bitrate != NULL) ? bitrate : "?");
+
+			atom_str_change(&data->mime_desc, desc);
+
+			xml_free_null(&bitrate);
+			xml_free_null(&fps);
+			xml_free_null(&height);
+			xml_free_null(&width);
+		} else if (is_strcaseprefix(value, "audio")) {
+			gchar *channels = STRTRACK(xml_get_string(node, "audioChannels"));
+			gchar *duration = STRTRACK(xml_get_string(node, "duration"));
+			gchar *kbps = STRTRACK(xml_get_string(node, "audioBitrate"));
+			gchar *srate = STRTRACK(xml_get_string(node, "audioSamplerate"));
+			guint32 seconds;
+			gchar desc[256];
+
+			if (duration) {
+				gint error;
+				seconds = parse_uint32(duration, NULL, 10, &error) / 1000;
+			} else {
+				seconds = 0;
 			}
-		} else if (
-			xmlStrstr(string_to_xmlChar(s), string_to_xmlChar("audio"))
-		) {
-			data->mime_type = g_strdup(s);
+
+			gm_snprintf(desc, sizeof desc, "%s%s%s%s%s%s%s",
+				kbps ? kbps : "", kbps ? "kbps " : "",
+				srate ? srate : "", srate ? "Hz " : "",
+				channels ? channels : "", channels ? "ch " : "",
+				seconds ? short_time(seconds) : "");
+
+			atom_str_change(&data->mime_desc, desc);
+
+			xml_free_null(&channels);
+			xml_free_null(&duration);
+			xml_free_null(&kbps);
+			xml_free_null(&srate);
 		}
 	}
+	xml_free_null(&value);
 
 	/*
 	 ** For debugging/development - dump all the attributes
 	 */
 
-	if (bitzi_debug) {
+	if (GNET_PROPERTY(bitzi_debug)) {
 		xmlAttr *cur_attr;
 
 		for (cur_attr = node->properties; cur_attr; cur_attr = cur_attr->next) {
-			const gchar *name = xmlChar_to_gchar(cur_attr->name);
+			const gchar *name;
+		   
+			name = xmlChar_to_gchar(cur_attr->name);
+			value = STRTRACK(xml_get_string(node, name));
 
 			g_message("bitzi rdf attrib: %s, type %d = %s",
-				name, cur_attr->type, xml_get_string(node, name));
+				name, cur_attr->type, value);
+
+			xml_free_null(&value);
 		}
 	}
 }
@@ -450,7 +494,7 @@ process_bitzi_ticket(xmlNode *a_node, bitzi_data_t *data)
 
 	for (cur_node = a_node; cur_node; cur_node = cur_node->next) {
 		if (cur_node->type == XML_ELEMENT_NODE) {
-			if (bitzi_debug)
+			if (GNET_PROPERTY(bitzi_debug))
 				g_message("node type: Element, name: %s, children %p",
 					cur_node->name, cast_to_gconstpointer(cur_node->children));
 
@@ -465,17 +509,44 @@ process_bitzi_ticket(xmlNode *a_node, bitzi_data_t *data)
 	}
 }
 
+static void
+bitzi_failure(const struct sha1 *sha1, filesize_t filesize, bitzi_fj_t error)
+{
+	if (sha1) {
+		bitzi_data_t *dummy = bitzi_create();
+
+		dummy->sha1 = atom_sha1_get(sha1);
+		dummy->size = filesize;
+		dummy->judgement = error;
+		gcu_bitzi_result(dummy);
+		bitzi_destroy(dummy);
+	}
+}
+
+static void
+bitzi_request_free(bitzi_request_t **ptr)
+{
+	if (*ptr) {
+		bitzi_request_t *req = *ptr;
+
+		atom_sha1_free_null(&req->sha1);
+		wfree(req, sizeof *req);
+		*ptr = NULL;
+	}
+}
+
 /**
  * Walk the parsed document tree and free up the data.
  */
 static void
 process_meta_data(bitzi_request_t *request)
 {
+	bitzi_data_t *data;
 	xmlDoc	*doc; 	/* the resulting document tree */
 	xmlNode	*root;
-	gint result;
+	gboolean wellformed;
 
-	if (bitzi_debug)
+	if (GNET_PROPERTY(bitzi_debug))
 		g_message("process_meta_data: %p", cast_to_gconstpointer(request));
 
 	g_assert(request != NULL);
@@ -485,63 +556,87 @@ process_meta_data(bitzi_request_t *request)
 	 */
 
 	doc = request->ctxt->myDoc;
-	result = request->ctxt->wellFormed;
+	wellformed = 0 != request->ctxt->wellFormed;
 	xmlFreeParserCtxt(request->ctxt);
 
-	if (bitzi_debug)
-		g_message("process_meta_data: doc = %p, result = %d",
-			cast_to_gconstpointer(doc), result);
+	if (GNET_PROPERTY(bitzi_debug))
+		g_message("process_meta_data: doc = %p, well-formed = %s",
+			cast_to_gconstpointer(doc), wellformed ? "yes" : "no");
+
+	if (!wellformed) {
+		bitzi_failure(request->sha1, request->filesize, BITZI_FJ_FAILURE);
+		goto finish;
+	}
 
 	/*
 	 * Now we can have a look at the data
 	 */
 
-	if (result) {
-		bitzi_data_t *data;
+   	data = bitzi_create();
 
-	   	data = bitzi_create();
+	/*
+	 * This just dumps the data
+	 */
 
-		/*
-		 * This just dumps the data
-		 */
+	root = xmlDocGetRootElement(doc);
+	process_bitzi_ticket(root, data);
 
-		root = xmlDocGetRootElement(doc);
-		process_bitzi_ticket(root, data);
-		if (!data->sha1) {
+	if (NULL == data->sha1) {
+		if (GNET_PROPERTY(bitzi_debug))  {
 			g_warning("process_meta_data: missing SHA-1");
-		} else {
-
-			/*
-			 * If the data has a valid date then we can cache the result
-			 * and re-echo the XML ticket to the file based cache.
-			 */
-
-			if (
-				(time_t) -1 == data->expiry ||
-				delta_time(data->expiry, tm_time()) <= 0
-			) {
-				g_warning("process_meta_data: stale bitzi data");
-			} else if (bitzi_cache_add(data)) {
-				if (bitzi_cache_file) {
-					xmlDocDump(bitzi_cache_file, doc);
-					fputs("\n", bitzi_cache_file);
-				}
-
-				gcu_bitzi_result(data);
-			}
 		}
+		bitzi_failure(request->sha1, request->filesize, BITZI_FJ_FAILURE);
+		bitzi_destroy(data);
+		goto finish;
+	}
 
-		/* we are now finished with this XML doc */
-		xmlFreeDoc(doc);
+	if (request->sha1 && !sha1_eq(data->sha1, request->sha1)) {
+		if (GNET_PROPERTY(bitzi_debug))  {
+			g_warning("process_meta_data: SHA-1 mismatch");
+		}
+		bitzi_failure(request->sha1, request->filesize, BITZI_FJ_FAILURE);
+		bitzi_destroy(data);
+		goto finish;
 	}
 
 	/*
-	 * free used memory by the request
+	 * If the data has a valid date then we can cache the result
+	 * and re-echo the XML ticket to the file based cache.
 	 */
 
-	atom_sha1_free_null(&request->sha1);
+	if (
+		(time_t) -1 == data->expiry ||
+		delta_time(data->expiry, tm_time()) <= 0
+	) {
+		if (GNET_PROPERTY(bitzi_debug))  {
+			g_message("process_meta_data: stale bitzi data");
+		}
+		bitzi_failure(request->sha1, request->filesize, BITZI_FJ_FAILURE);
+		goto finish;
+	}
 
-	wfree(request, sizeof *request);
+	if (bitzi_cache_add(data) && bitzi_cache_file) {
+		xmlDocDump(bitzi_cache_file, doc);
+		fputs("\n", bitzi_cache_file);
+	}
+
+	if (request->filesize && request->filesize != data->size) {
+		if (GNET_PROPERTY(bitzi_debug))  {
+			g_message("process_meta_data: filesize mismatch");
+		}
+		/* We keep the ticket anyway because there's only one per SHA-1 */
+		bitzi_failure(request->sha1, request->filesize,
+			data->size ? BITZI_FJ_WRONG_FILESIZE : BITZI_FJ_UNKNOWN);
+		goto finish;
+	}
+
+	gcu_bitzi_result(data);
+
+finish:
+
+	/* we are now finished with this XML doc */
+	xmlFreeDoc(doc);
+	bitzi_request_free(&request);
 }
 
 /**
@@ -552,7 +647,7 @@ process_meta_data(bitzi_request_t *request)
 static gboolean
 do_metadata_query(bitzi_request_t *req)
 {
-	if (bitzi_debug)
+	if (GNET_PROPERTY(bitzi_debug))
 		g_message("do_metadata_query: %p", cast_to_gconstpointer(req));
 
 	/*
@@ -563,7 +658,7 @@ do_metadata_query(bitzi_request_t *req)
 	/*
 	 * check we haven't already got a response from a previous query
 	 */
-	if (NULL != bitzi_query_cache_by_sha1(req->sha1))
+	if (bitzi_has_cached_ticket(req->sha1))
 		return FALSE;
 
 	current_bitzi_request = req;
@@ -593,7 +688,7 @@ do_metadata_query(bitzi_request_t *req)
 					current_bitzi_request->bitzi_url,
 					http_async_strerror(http_async_errno));
 		} else {
-			if (bitzi_debug)
+			if (GNET_PROPERTY(bitzi_debug))
 				g_message("do_metadata_query: request %s launched",
 					current_bitzi_request->bitzi_url);
 			return TRUE;
@@ -635,10 +730,10 @@ bitzi_cache_add(bitzi_data_t *data)
 		return FALSE;
 	}
 
-	g_hash_table_insert(bitzi_cache_ht, data->sha1, data);
+	gm_hash_table_insert_const(bitzi_cache_ht, data->sha1, data);
 	bitzi_cache = g_list_insert_sorted(bitzi_cache, data, bitzi_date_compare);
 
-	if (bitzi_debug)
+	if (GNET_PROPERTY(bitzi_debug))
 		g_message("bitzi_cache_add: data %p, now %u entries",
 			cast_to_gconstpointer(data), g_hash_table_size(bitzi_cache_ht));
 
@@ -646,9 +741,9 @@ bitzi_cache_add(bitzi_data_t *data)
 }
 
 static void
-bitzi_cache_remove(bitzi_data_t * data)
+bitzi_cache_remove(bitzi_data_t *data)
 {
-	if (bitzi_debug)
+	if (GNET_PROPERTY(bitzi_debug))
 		g_message("bitzi_cache_remove: %p", cast_to_gconstpointer(data));
 
 	g_assert(data);
@@ -721,20 +816,24 @@ bitzi_heartbeat(gpointer unused_data)
 	return TRUE;		/* Always requeue */
 }
 
+static bitzi_data_t *
+bitzi_query_cache_by_sha1(const struct sha1 *sha1)
+{
+	g_return_val_if_fail(NULL != sha1, FALSE);
+	return g_hash_table_lookup(bitzi_cache_ht, sha1);
+}
 
 /**************************************************************
  ** Bitzi API
  *************************************************************/
 
 /**
- * Query the bitzi cache for this given SHA-1, return NULL if
- * nothing otherwise we return the
+ * Query the bitzi cache for this given SHA-1.
  */
-bitzi_data_t *
-bitzi_query_cache_by_sha1(const gchar *sha1)
+gboolean
+bitzi_has_cached_ticket(const struct sha1 *sha1)
 {
-	g_return_val_if_fail(NULL != sha1, NULL);
-	return g_hash_table_lookup(bitzi_cache_ht, sha1);
+	return NULL != bitzi_query_cache_by_sha1(sha1);
 }
 
 /**
@@ -745,8 +844,8 @@ bitzi_query_cache_by_sha1(const gchar *sha1)
  * If no query succeds then the call back is never made, however we
  * should always get some sort of data back from the service.
  */
-gpointer
-bitzi_query_by_sha1(const gchar *sha1)
+bitzi_data_t *
+bitzi_query_by_sha1(const struct sha1 *sha1, filesize_t filesize)
 {
 	bitzi_data_t *data = NULL;
 	bitzi_request_t	*request;
@@ -754,7 +853,19 @@ bitzi_query_by_sha1(const gchar *sha1)
 	g_return_val_if_fail(NULL != sha1, NULL);
 
 	data = bitzi_query_cache_by_sha1(sha1);
-	if (data == NULL) {
+	if (data) {
+		if (GNET_PROPERTY(bitzi_debug)) {
+			g_message("bitzi_query_by_sha1: result already in cache");
+		}
+
+		if (filesize && data->size != filesize) {
+			bitzi_failure(sha1, filesize,
+				data->size ? BITZI_FJ_WRONG_FILESIZE : BITZI_FJ_UNKNOWN);
+			data = NULL;
+		} else {
+			gcu_bitzi_result(data);
+		}
+	} else {
 		size_t len;
 
 		request = walloc(sizeof *request);
@@ -763,12 +874,13 @@ bitzi_query_by_sha1(const gchar *sha1)
 		 * build the bitzi url
 		 */
 		request->sha1 = atom_sha1_get(sha1);
+		request->filesize = filesize;
 		len = gm_snprintf(request->bitzi_url, sizeof request->bitzi_url,
 				bitzi_url_fmt, sha1_base32(sha1));
 		g_assert(len < sizeof request->bitzi_url);
 
 		bitzi_rq = g_slist_append(bitzi_rq, request);
-		if (bitzi_debug) {
+		if (GNET_PROPERTY(bitzi_debug)) {
 			g_message("bitzi_query_by_sha1: queued query, %d in queue",
 				g_slist_position(bitzi_rq, g_slist_last(bitzi_rq)) + 1);
 		}
@@ -776,10 +888,6 @@ bitzi_query_by_sha1(const gchar *sha1)
 		/*
 		 * the heartbeat will pick up the request
 		 */
-	} else {
-		if (bitzi_debug)
-			g_message("bitzi_query_by_sha1: result already in cache");
-				gcu_bitzi_result(data);
 	}
 
 	return data;
@@ -863,6 +971,7 @@ bitzi_load_cache(void)
 				/* new pseudo request */
 				request = walloc(sizeof *request);
 				request->sha1 = NULL;
+				request->filesize = 0;
 
 				request->ctxt = xmlCreatePushParserCtxt(
 					NULL, NULL, NULL, 0, NULL);
@@ -894,7 +1003,7 @@ bitzi_load_cache(void)
 		fclose(old_data);
 	} /* if (old_data) */
 
-	if (bitzi_debug)
+	if (GNET_PROPERTY(bitzi_debug))
 		g_message("Loaded %d bitzi ticket(s) from \"%s\"",
 			ticket_count, oldpath);
 
@@ -909,7 +1018,7 @@ bitzi_load_cache(void)
 void
 bitzi_init(void)
 {
-	bitzi_cache_ht = g_hash_table_new(NULL, NULL);
+	bitzi_cache_ht = g_hash_table_new(pointer_hash_func, NULL);
 
 	bitzi_load_cache();
 
@@ -937,6 +1046,17 @@ bitzi_close(void)
 		g_list_free(bitzi_cache);
 		bitzi_cache = NULL;
 	}
+
+	{
+		GSList *iter;
+		
+		for (iter = bitzi_rq; NULL != iter; iter = g_slist_next(iter)) {
+			bitzi_request_t *req = iter->data;
+			bitzi_request_free(&req);
+		}
+		g_slist_free(bitzi_rq);
+		bitzi_rq = NULL;
+	}
 	
 	g_return_if_fail(bitzi_heartbeat_id);
 	g_source_remove(bitzi_heartbeat_id);
