@@ -1,5 +1,5 @@
 /*
- * $Id: downloads.h,v 1.14 2005/12/18 06:09:10 cbiere Exp $
+ * $Id: downloads.h 13919 2007-06-22 00:51:50Z cbiere $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi
  *
@@ -36,18 +36,15 @@
 #ifndef _core_downloads_h_
 #define _core_downloads_h_
 
+#include "common.h"
+
 #include "lib/header.h"
+
 #include "fileinfo.h"
 
 #include "if/core/downloads.h"
 #include "if/core/search.h"			/* For gnet_host_vec_t */
 
-/**
- * Global Data.
- */
-
-extern GSList *sl_unqueued;
-
 /*
  * Global Functions.
  */
@@ -57,13 +54,13 @@ void download_restore_state(void);
 void download_store_if_dirty(void);
 void download_timer(time_t now);
 void download_info_change_all(fileinfo_t *old_fi, fileinfo_t *new_fi);
-void download_orphan_new(
-	gchar *file, filesize_t size, gchar *sha1, fileinfo_t *fi);
+void download_orphan_new(const gchar *file, filesize_t size,
+		const struct sha1 *sha1, fileinfo_t *fi);
 void download_queue(struct download *d,
 	const gchar *fmt, ...) G_GNUC_PRINTF(2, 3);
-void download_stop(struct download *, guint32,
+void download_stop(struct download *, download_status_t,
 	const gchar *, ...) G_GNUC_PRINTF(3, 4);
-void download_stop_v(struct download *d, guint32 new_status,
+void download_stop_v(struct download *d, download_status_t new_status,
     const gchar * reason, va_list ap);
 void download_push_ack(struct gnutella_socket *);
 void download_fallback_to_push(struct download *, gboolean, gboolean);
@@ -72,23 +69,19 @@ void download_forget(struct download *, gboolean unavailable);
 gboolean download_start_prepare(struct download *d);
 gboolean download_start_prepare_running(struct download *d);
 void download_send_request(struct download *);
-void download_retry(struct download *);
+void download_connected(struct download *);
 void download_close(void);
-gboolean download_server_nopush(gchar *guid,
+gboolean download_server_nopush(const gchar *guid,
 			const host_addr_t addr, guint16 port);
 void download_free_removed(void);
 void download_redirect_to_server(struct download *d,
 		const host_addr_t addr, guint16 port);
 void download_actively_queued(struct download *d, gboolean queued);
 
-void download_verify_start(struct download *d);
-void download_verify_progress(struct download *d, guint32 hashed);
-void download_verify_done(struct download *d, gchar *digest, guint elapsed);
-void download_verify_error(struct download *d);
-
 void download_move_start(struct download *d);
 void download_move_progress(struct download *d, filesize_t copied);
-void download_move_done(struct download *d, guint elapsed);
+void download_move_done(struct download *d, const gchar *pathname,
+		guint elapsed);
 void download_move_error(struct download *d);
 
 guint extract_retry_after(struct download *d, const header_t *header);
@@ -102,16 +95,26 @@ void download_proxy_newstate(struct download *d);
 void download_proxy_sent(struct download *d);
 void download_proxy_failed(struct download *d);
 
-struct download * download_browse_start(const gchar *name,
+struct download * download_browse_start(
 	const gchar *hostname, host_addr_t addr, guint16 port,
-	const gchar *guid, gboolean push, const gnet_host_vec_t *proxies,
-	gnet_search_t search);
+	const gchar *guid, const gnet_host_vec_t *proxies,
+	gnet_search_t search, guint32 flags);
 
-void download_abort_browse_host(gpointer download, gnet_search_t sh);
+struct download * download_thex_start(const gchar *uri,
+	const struct sha1 *sha1, const struct tth *tth, filesize_t filesize,
+	const gchar *hostname, host_addr_t addr, guint16 port,
+	const gchar *guid, const gnet_host_vec_t *proxies,
+	guint32 flags);
+
+void download_abort_browse_host(struct download *d, gnet_search_t sh);
 void download_got_eof(struct download *d);
 void download_rx_done(struct download *d);
-void download_browse_received(struct download *d, ssize_t received);
-void download_browse_maybe_finished(struct download *d);
+
+void download_data_received(struct download *d, ssize_t received);
+void download_maybe_finished(struct download *d);
+
+gboolean download_handle_http(const gchar *url);
+gboolean download_is_stalled(struct download *d);
 
 #endif /* _core_downloads_h_ */
 
