@@ -1,5 +1,5 @@
 /*
- * $Id: extensions.c 12020 2006-09-21 17:17:35Z cbiere $
+ * $Id: extensions.c 13831 2007-06-16 01:22:39Z cbiere $
  *
  * Copyright (c) 2002-2003, Raphael Manfredi
  *
@@ -35,7 +35,9 @@
 
 #include "common.h"
 
-RCSID("$Id: extensions.c 12020 2006-09-21 17:17:35Z cbiere $")
+RCSID("$Id: extensions.c 13831 2007-06-16 01:22:39Z cbiere $")
+
+#include <zlib.h>
 
 #include "extensions.h"
 #include "ggep.h"
@@ -47,7 +49,7 @@ RCSID("$Id: extensions.c 12020 2006-09-21 17:17:35Z cbiere $")
 
 #include "if/gnet_property_priv.h"
 
-#define HUGE_FS		'\x1c'		/**< Field separator (HUGE) */
+#define HUGE_FS		0x1CU		/**< Field separator (HUGE) */
 
 #define GGEP_MAXLEN	65535		/**< Maximum decompressed length */
 #define GGEP_GROW	512			/**< Minimum chunk growth when resizing */
@@ -92,8 +94,8 @@ RCSID("$Id: extensions.c 12020 2006-09-21 17:17:35Z cbiere $")
  * each time a new extension is found.
  */
 typedef struct extdesc {
-	gchar *ext_phys_payload;	/**< Start of payload buffer */
-	gchar *ext_payload;			/**< "virtual" payload */
+	const gchar *ext_phys_payload;	/**< Start of payload buffer */
+	const gchar *ext_payload;		/**< "virtual" payload */
 	guint16 ext_phys_len;		/**< Extension length (header + payload) */
 	guint16 ext_phys_paylen;	/**< Extension payload length */
 	guint16 ext_paylen;			/**< "virtual" payload length */
@@ -150,14 +152,18 @@ static const struct rwtable urntable[] =
 /** GGEP extension table (sorted) */
 static const struct rwtable ggeptable[] =
 {
-#define GGEP_ID(x) { STRINGIFY(x), CAT2(EXT_T_GGEP_,x) }
-#define GGEP_GTKG_ID(x) { "GTKG." STRINGIFY(x), CAT2(EXT_T_GGEP_GTKG_,x) }
+#define GGEP_ID(x) { #x, EXT_T_GGEP_ ## x }
+#define GGEP_GTKG_ID(x) { "GTKG." #x, EXT_T_GGEP_GTKG_ ## x }
 
 	{ "<", EXT_T_GGEP_LIME_XML }, /**< '<' is less that 'A' */
+	GGEP_ID(A),			/**< Same as GGEP ALT but used in HEAD Pongs */
 	GGEP_ID(ALT),		/**< Alt-locs in qhits */
+	GGEP_ID(ALT_TLS),	/**< TLS-capability bitmap for GGEP ALT */
 	GGEP_ID(BH),		/**< Browseable host indication */
+	GGEP_ID(C),			/**< Result Code in HEAD Pongs */
 	GGEP_ID(CT),		/**< Resource creation time */
 	GGEP_ID(DU),		/**< Average servent uptime */
+	GGEP_ID(F),			/**< Flags in HEAD Pongs */
 	GGEP_ID(FW),		/**< Firewalled-to-Firewalled protocol version */
 	GGEP_ID(GGEP),		/**< GGEP extension names known, NUL-separated */
 	GGEP_GTKG_ID(IPV6),	/**< GTKG IPv6 address */
@@ -168,16 +174,24 @@ static const struct rwtable ggeptable[] =
 	GGEP_ID(HNAME),		/**< Hostname */
 	GGEP_ID(IP),		/**< Ip:Port in ping and pongs (F2F) */
 	GGEP_ID(IPP),		/**< IP:Port in pongs (UHC) */
+	GGEP_ID(IPP_TLS),	/**< TLS-capability bitmap for GGEP IPP */
 	GGEP_ID(LF),		/**< Large file size in qhits */
 	GGEP_ID(LOC),		/**< Locale preferences, for clustering  */
+	GGEP_ID(NP),		/**< do Not Proxy (queries; OOB) */
+	GGEP_ID(P),			/**< Push alt-locs in HEAD Pongs */
 	GGEP_ID(PATH),		/**< Shared file path, in query hits */
 	GGEP_ID(PHC),		/**< Packed host caches (UHC) in pongs */
 	GGEP_ID(PUSH),		/**< Push proxy info, in qhits */
+	GGEP_ID(PUSH_TLS),	/**< TLS-capability bitmap for GGEP PUSH */
+	GGEP_ID(Q),			/**< Queue status in HEAD Pongs */
 	GGEP_ID(SCP),		/**< Supports cached pongs, in pings (UHC) */
-	GGEP_ID(T),			/**< Textual information in qhits */
+	GGEP_ID(SO),		/**< Secure OOB */
+	GGEP_ID(T),			/**< Same as ALT_TLS but for HEAD Pongs */
+	GGEP_ID(TLS),		/**< TLS support indication */
 	GGEP_ID(UA),		/**< User-Agent string */
 	GGEP_ID(UDPHC),		/**< Is an UDP hostcache (UHC) , in pongs */
 	GGEP_ID(UP),		/**< Ultrapeer information about free slots */
+	GGEP_ID(V),			/**< Vendor code, in HEAD Pongs */
 	GGEP_ID(VC),		/**< Vendor code, in pongs */
 	GGEP_ID(VMSG),		/**< Array of supported vendor message codes */
 	GGEP_ID(u),			/**< HUGE URN in ASCII */
@@ -274,7 +288,7 @@ rw_ggep_screen(gchar *word, const gchar **retkw)
  * If keyword was found, its static shared string is returned in `retkw'.
  */
 static ext_token_t
-rw_urn_screen(gchar *word, const gchar **retkw)
+rw_urn_screen(const gchar *word, const gchar **retkw)
 {
 	return rw_screen(FALSE, urntable, G_N_ELEMENTS(urntable), word, retkw);
 }
@@ -359,11 +373,11 @@ ext_names_kv_free(gpointer key, gpointer value, gpointer unused_udata)
  * Parses a GGEP block (can hold several extensions).
  */
 static gint
-ext_ggep_parse(gchar **retp, gint len, extvec_t *exv, gint exvcnt)
+ext_ggep_parse(const gchar **retp, gint len, extvec_t *exv, gint exvcnt)
 {
-	gchar *p = *retp;
-	gchar *end = p + len;
-	gchar *lastp = p;				/* Last parsed point */
+	const gchar *p = *retp;
+	const gchar *end = &p[len];
+	const gchar *lastp = p;				/* Last parsed point */
 	gint count;
 
 	for (count = 0; count < exvcnt && p < end; /* empty */) {
@@ -564,19 +578,58 @@ abort:
 	return 0;		/* Cannot be a GGEP block: leave parsing pointer intact */
 }
 
+static gint
+ext_urn_bad_parse(const gchar **retp, gint len, extvec_t *exv, gint exvcnt)
+{
+	const gchar *p = *retp;
+	const gchar *lastp = p;				/* Last parsed point */
+	extdesc_t *d;
+
+	g_assert(exvcnt > 0);
+	g_assert(exv->opaque == NULL);
+
+	if (len != 3)
+		return 0;
+	p = is_strcaseprefix(p, "urn");
+	if (!p)
+		return 0;
+
+	/*
+	 * Encapsulate as one big opaque chunk.
+	 */
+
+	d = walloc(sizeof(*d));
+
+	d->ext_phys_payload = lastp;
+	d->ext_phys_len = d->ext_phys_paylen = p - lastp;
+	d->ext_payload = d->ext_phys_payload;
+	d->ext_paylen = d->ext_phys_paylen;
+
+	exv->opaque = d;
+	exv->ext_type = EXT_NONE;
+	exv->ext_name = NULL;
+	exv->ext_token = EXT_T_URN_BAD;
+
+	g_assert(p - lastp == d->ext_phys_len);
+
+	*retp = p;			/* Points to first byte after what we parsed */
+
+	return 1;
+}
+
+
 /**
  * Parses a URN block (one URN only).
  */
 static gint
-ext_huge_parse(gchar **retp, gint len, extvec_t *exv, gint exvcnt)
+ext_huge_parse(const gchar **retp, gint len, extvec_t *exv, gint exvcnt)
 {
-	gchar *p = *retp;
-	gchar *end = p + len;
-	gchar *lastp = p;				/* Last parsed point */
-	gchar *name_start;
+	const gchar *p = *retp;
+	const gchar *end = &p[len];
+	const gchar *lastp = p;				/* Last parsed point */
 	ext_token_t token;
-	gchar *payload_start = NULL;
-	gint data_length = 0;
+	const gchar *payload_start = NULL;
+	gint data_length;
 	const gchar *name = NULL;
 	extdesc_t *d;
 
@@ -588,7 +641,7 @@ ext_huge_parse(gchar **retp, gint len, extvec_t *exv, gint exvcnt)
 	 */
 
 	if (len < 4)
-		return 0;
+		return ext_urn_bad_parse(retp, len, exv, exvcnt);
 
 	/*
 	 * Recognize "urn:".
@@ -605,7 +658,7 @@ ext_huge_parse(gchar **retp, gint len, extvec_t *exv, gint exvcnt)
 	if (p == end || *p == '\0' || *p == HUGE_FS) {
 		token = EXT_T_URN_EMPTY;
 		payload_start = p;
-		g_assert(data_length == 0);
+		data_length = 0;
 		goto found;
 	}
 
@@ -613,27 +666,30 @@ ext_huge_parse(gchar **retp, gint len, extvec_t *exv, gint exvcnt)
 	 * Look for the end of the name, identified by ':'.
 	 */
 
-	name_start = p;
+	{
+		const gchar *name_start, *name_end;
+		size_t name_len;
+		gchar name_buf[16];
 
-	while (p < end) {
-		if (*p == ':')
-			break;
-		p++;
+		name_start = p;
+		name_end = memchr(p, ':', end - name_start);
+		name_len = name_end ? name_end - name_start : 0;
+
+		/* Not found, empty name or too long */
+		if (0 == name_len || name_len >= sizeof name_buf)
+			return 0;
+
+		memcpy(name_buf, name_start, name_len);
+		name_buf[name_len] = '\0';
+
+		/*
+		 * Lookup the token.
+		 */
+
+		token = rw_urn_screen(name_buf, &name);
+		p = &name_end[1];
 	}
 
-	if (p == end || p == name_start)	/* Not found, or empty name */
-		return 0;
-
-	g_assert(*p == ':');
-
-	/*
-	 * Lookup the token.
-	 */
-
-	*p = '\0';
-	token = rw_urn_screen(name_start, &name);
-	*p++ = ':';
-
 	/*
 	 * Now extract the payload (must be made of alphanum chars),
 	 * until we reach a delimiter (NUL byte, GGEP header, GEM separator).
@@ -641,17 +697,13 @@ ext_huge_parse(gchar **retp, gint len, extvec_t *exv, gint exvcnt)
 	 */
 
 	payload_start = p;
-
-	while (p < end) {
-		guchar c = *p++;
-		if (!is_ascii_alnum(c) || c == (guchar) GGEP_MAGIC) {
-			p--;
+	for (/* NOTHING*/; p < end; p++) {
+		guchar c = *p;
+		if (!is_ascii_alnum(c) || c == GGEP_MAGIC) {
 			break;
 		}
-		data_length++;
 	}
-
-	g_assert(data_length == p - payload_start);
+	data_length = p - payload_start;
 
 found:
 	g_assert(payload_start);
@@ -681,20 +733,19 @@ found:
  * Parses a XML block (grabs the whole xml up to the first NUL or separator).
  */
 static gint
-ext_xml_parse(gchar **retp, gint len, extvec_t *exv, gint exvcnt)
+ext_xml_parse(const gchar **retp, gint len, extvec_t *exv, gint exvcnt)
 {
-	gchar *p = *retp;
-	gchar *end = p + len;
-	gchar *lastp = p;				/* Last parsed point */
+	const gchar *p = *retp;
+	const gchar *end = &p[len];
+	const gchar *lastp = p;				/* Last parsed point */
 	extdesc_t *d;
 
 	g_assert(exvcnt > 0);
 	g_assert(exv->opaque == NULL);
 
-	while (p < end) {
-		guchar c = *p++;
-		if (c == '\0' || c == (guchar) HUGE_FS) {
-			p--;
+	for (/* NOTHING */; p != end; p++) {
+		guchar c = *p;
+		if (c == '\0' || c == HUGE_FS) {
 			break;
 		}
 	}
@@ -729,11 +780,11 @@ ext_xml_parse(gchar **retp, gint len, extvec_t *exv, gint exvcnt)
  * If `skip' is TRUE, we don't resync on the first resync point.
  */
 static gint
-ext_unknown_parse(gchar **retp, gint len, extvec_t *exv,
+ext_unknown_parse(const gchar **retp, gint len, extvec_t *exv,
 	gint exvcnt, gboolean skip)
 {
-	gchar *p = *retp;
-	gchar *lastp = p;				/* Last parsed point */
+	const gchar *p = *retp;
+	const gchar *lastp = p;				/* Last parsed point */
 	extdesc_t *d;
 
 	g_assert(exvcnt > 0);
@@ -749,8 +800,8 @@ ext_unknown_parse(gchar **retp, gint len, extvec_t *exv,
 
 		switch ((guchar) *p) {
 		case '\0':
-		case (guchar) HUGE_FS:
-		case (guchar) GGEP_MAGIC:
+		case HUGE_FS:
+		case GGEP_MAGIC:
 			found = TRUE;
 			break;
 		case 'u':
@@ -805,22 +856,20 @@ ext_unknown_parse(gchar **retp, gint len, extvec_t *exv,
  * "none" extension.
  */
 static gint
-ext_none_parse(gchar **retp, gint len, extvec_t *exv, gint exvcnt)
+ext_none_parse(const gchar **retp, gint len, extvec_t *exv, gint exvcnt)
 {
-	gchar *p = *retp;
-	gchar *end = p + len;
-	gchar *lastp = p;				/* Last parsed point */
+	const gchar *p = *retp;
+	const gchar *end = &p[len];
+	const gchar *lastp = p;				/* Last parsed point */
 	extdesc_t *d;
 
 	g_assert(exvcnt > 0);
 	g_assert(exv->opaque == NULL);
 
-	while (p < end) {
-		guchar c = *p++;
-		if (c == '\0' || c == (guchar) HUGE_FS)
-			continue;
-		p--;						/* Point back to the non-NULL char */
-		break;
+	for (/* NOTHING */; p != end; p++) {
+		guchar c = *p;
+		if (c != '\0' && c != HUGE_FS)
+			break;
 	}
 
 	/*
@@ -861,9 +910,9 @@ ext_none_parse(gchar **retp, gint len, extvec_t *exv, gint exvcnt)
 static void
 ext_merge_adjacent(extvec_t *exv, extvec_t *next)
 {
-	gchar *end;
-	gchar *nend;
-	gchar *nbase;
+	const gchar *end;
+	const gchar *nend;
+	const gchar *nbase;
 	guint16 added;
 	extdesc_t *d = exv->opaque;
 	extdesc_t *nd = next->opaque;
@@ -920,10 +969,9 @@ ext_merge_adjacent(extvec_t *exv, extvec_t *next)
  * @return the number of filled entries.
  */
 gint
-ext_parse(gchar *buf, gint len, extvec_t *exv, gint exvcnt)
+ext_parse(const gchar *buf, gint len, extvec_t *exv, gint exvcnt)
 {
-	gchar *p = buf;
-	gchar *end = buf + len;
+	const gchar *p = buf, *end = &buf[len];
 	gint cnt = 0;
 
 	g_assert(buf);
@@ -933,8 +981,8 @@ ext_parse(gchar *buf, gint len, extvec_t *exv, gint exvcnt)
 	g_assert(exv->opaque == NULL);
 
 	while (p < end && exvcnt > 0) {
+		const gchar *old_p = p;
 		gint found = 0;
-		gchar *old_p = p;
 
 		g_assert(len > 0);
 
@@ -944,7 +992,7 @@ ext_parse(gchar *buf, gint len, extvec_t *exv, gint exvcnt)
 		 * that predate GGEP (HUGE and XML) and were not properly encapsulated.
 		 */
 
-		switch (*p) {
+		switch ((guchar) *p) {
 		case GGEP_MAGIC:
 			p++;
 			if (p == end)
@@ -982,7 +1030,7 @@ ext_parse(gchar *buf, gint len, extvec_t *exv, gint exvcnt)
 		g_assert(found == 0 || p != old_p);
 
 		if (found == 0) {
-			g_assert(*old_p == GGEP_MAGIC || p == old_p);
+			g_assert((guchar) *old_p == GGEP_MAGIC || p == old_p);
 
 			/*
 			 * If we were initially on a GGEP magic byte, and since we did
@@ -990,7 +1038,7 @@ ext_parse(gchar *buf, gint len, extvec_t *exv, gint exvcnt)
 			 * about to skip the first synchronization point...
 			 */
 
-			if (*old_p == GGEP_MAGIC) {
+			if ((guchar) *old_p == GGEP_MAGIC) {
 				p--;
 				g_assert(p == old_p);
 			}
@@ -1037,7 +1085,7 @@ out:
  * @returns NULL on error.
  */
 static gchar *
-ext_ggep_inflate(gchar *buf, gint len, guint16 *retlen, const gchar *name)
+ext_ggep_inflate(const gchar *buf, gint len, guint16 *retlen, const gchar *name)
 {
 	gchar *result;					/* Inflated buffer */
 	gint rsize;						/* Result's buffer size */
@@ -1164,7 +1212,7 @@ ext_ggep_inflate(gchar *buf, gint len, guint16 *retlen, const gchar *name)
 static void
 ext_ggep_decode(const extvec_t *e)
 {
-	gchar *pbase;					/* Current payload base */
+	const gchar *pbase;				/* Current payload base */
 	size_t plen;					/* Curernt payload length */
 	gchar *uncobs = NULL;			/* COBS-decoded buffer */
 	size_t uncobs_len = 0;			/* Length of walloc()'ed buffer */
@@ -1196,7 +1244,7 @@ ext_ggep_decode(const extvec_t *e)
 
 		if (!d->ext_ggep_deflate) {
 			if (!cobs_decode_into(pbase, plen, uncobs, plen, &result)) {
-				if (ggep_debug)
+				if (GNET_PROPERTY(ggep_debug))
 					g_warning("unable to decode COBS buffer");
 				goto out;
 			}
@@ -1210,7 +1258,7 @@ ext_ggep_decode(const extvec_t *e)
 			return;
 		} else {
 			if (!cobs_decode_into(pbase, plen, uncobs, plen, &result)) {
-				if (ggep_debug)
+				if (GNET_PROPERTY(ggep_debug))
 					g_warning("unable to decode COBS buffer");
 				goto out;
 			}
@@ -1252,7 +1300,7 @@ out:
 	 */
 
 	if (d->ext_payload == NULL) {
-		if (dbg || ggep_debug)
+		if (GNET_PROPERTY(dbg) || GNET_PROPERTY(ggep_debug))
 			g_warning("unable to get GGEP \"%s\" %d-byte payload (%s)",
 				d->ext_ggep_id, d->ext_phys_paylen,
 				(d->ext_ggep_deflate && d->ext_ggep_cobs) ? "COBS + deflated" :
@@ -1266,22 +1314,19 @@ out:
 /**
  * @returns a pointer to the extension's payload.
  */
-const gchar *
+gconstpointer
 ext_payload(const extvec_t *e)
 {
 	extdesc_t *d = e->opaque;
 
 	g_assert(e->opaque != NULL);
 
-	if (d->ext_payload != NULL)
-		return d->ext_payload;
-
-	/*
-	 * GGEP payload is COBS-ed and/or deflated.
-	 */
-
-	ext_ggep_decode(e);
-
+	if (NULL == d->ext_payload) {
+		/*
+		 * GGEP payload is COBS-ed and/or deflated.
+		 */
+		ext_ggep_decode(e);
+	}
 	return d->ext_payload;
 }
 
@@ -1295,15 +1340,12 @@ ext_paylen(const extvec_t *e)
 
 	g_assert(e->opaque != NULL);
 
-	if (d->ext_payload != NULL)
-		return d->ext_paylen;
-
-	/*
-	 * GGEP payload is COBS-ed and/or deflated.
-	 */
-
-	ext_ggep_decode(e);
-
+	if (NULL == d->ext_payload) {
+		/*
+		 * GGEP payload is COBS-ed and/or deflated.
+		 */
+		ext_ggep_decode(e);
+	}
 	return d->ext_paylen;
 }
 
@@ -1378,16 +1420,13 @@ ext_ggep_id_str(const extvec_t *e)
 gboolean
 ext_is_printable(const extvec_t *e)
 {
-	const gchar *p = ext_payload(e);
-	gint len = ext_paylen(e);
+	const guchar *p = ext_payload(e);
+	size_t len;
 
-	g_assert(len >= 0);
-	while (len--) {
-		guchar c = *p++;
-		if (!isprint(c))
+	for (len = ext_paylen(e); len > 0; len--, p++) {
+		if (!isprint(*p))
 			return FALSE;
 	}
-
 	return TRUE;
 }
 
@@ -1397,16 +1436,13 @@ ext_is_printable(const extvec_t *e)
 gboolean
 ext_is_ascii(const extvec_t *e)
 {
-	const gchar *p = ext_payload(e);
-	gint len = ext_paylen(e);
+	const guchar *p = ext_payload(e);
+	size_t len;
 
-	g_assert(len >= 0);
-	while (len--) {
-		guchar c = *p++;
-		if (!isascii(c))
+	for (len = ext_paylen(e); len > 0; len--, p++) {
+		if (!isascii(*p))
 			return FALSE;
 	}
-
 	return TRUE;
 }
 
@@ -1416,19 +1452,15 @@ ext_is_ascii(const extvec_t *e)
 gboolean
 ext_has_ascii_word(const extvec_t *e)
 {
-	const gchar *p = ext_payload(e);
-	gint len = ext_paylen(e);
+	const guchar *p = ext_payload(e);
+	size_t len;
 	gboolean has_alnum = FALSE;
 
-	g_assert(len >= 0);
-	while (len--) {
-		guchar c = *p++;
-		if (!isascii(c))
+	for (len = ext_paylen(e); len > 0; len--, p++) {
+		if (!isascii(*p))
 			return FALSE;
-		if (!has_alnum && is_ascii_alnum(c))
-			has_alnum = TRUE;
+		has_alnum |= is_ascii_alnum(*p);
 	}
-
 	return has_alnum;
 }
 
@@ -1498,8 +1530,10 @@ void
 ext_dump(FILE *fd, const extvec_t *exv, gint exvcnt,
 	const gchar *prefix, const gchar *postfix, gboolean payload)
 {
-	while (exvcnt--)
-		ext_dump_one(fd, exv++, prefix, postfix, payload);
+	gint i;
+
+	for (i = 0; i < exvcnt; i++)
+		ext_dump_one(fd, &exv[i], prefix, postfix, payload);
 }
 
 /**
@@ -1509,8 +1543,10 @@ ext_dump(FILE *fd, const extvec_t *exv, gint exvcnt,
 void
 ext_prepare(extvec_t *exv, gint exvcnt)
 {
-	while (exvcnt--)
-		(exv++)->opaque = NULL;
+	gint i;
+
+	for (i = 0; i < exvcnt; i++)
+		exv[i].opaque = NULL;
 }
 
 /**
@@ -1520,8 +1556,10 @@ ext_prepare(extvec_t *exv, gint exvcnt)
 void
 ext_reset(extvec_t *exv, gint exvcnt)
 {
-	while (exvcnt--) {
-		extvec_t *e = exv++;
+	gint i;
+	
+	for (i = 0; i < exvcnt; i++) {
+		extvec_t *e = &exv[i];
 		extdesc_t *d;
 
 		if (e->opaque == NULL)		/* No more allocated extensions */
@@ -1530,10 +1568,13 @@ ext_reset(extvec_t *exv, gint exvcnt)
 		d = e->opaque;
 
 		if (d->ext_payload != NULL && d->ext_payload != d->ext_phys_payload) {
-			if (d->ext_rpaylen == 0)
-				g_free(d->ext_payload);
-			else
-				wfree(d->ext_payload, d->ext_rpaylen);
+			gpointer p = deconstify_gpointer(d->ext_payload);
+			if (d->ext_rpaylen == 0) {
+				g_free(p);
+			} else {
+				wfree(p, d->ext_rpaylen);
+			}
+			d->ext_payload = NULL;
 		}
 
 		wfree(d, sizeof(*d));
