@@ -1,5 +1,5 @@
 /*
- * $Id: extensions.h 11936 2006-09-10 20:20:07Z rmanfredi $
+ * $Id: extensions.h 13831 2007-06-16 01:22:39Z cbiere $
  *
  * Copyright (c) 2002-2003, Raphael Manfredi
  *
@@ -36,7 +36,7 @@
 #ifndef _core_extensions_h_
 #define _core_extensions_h_
 
-#include <glib.h>
+#include "common.h"
 
 /**
  * Known extension types.
@@ -69,15 +69,20 @@ typedef enum ext_token {
 	EXT_T_URN_BITPRINT,		/**< urn:bitprint: */
 	EXT_T_URN_SHA1,			/**< urn:sha1: */
 	EXT_T_URN_EMPTY,		/**< urn: */
+	EXT_T_URN_BAD,			/**< urn */
 	EXT_T_XML,				/**< XML payload */
 	EXT_T_UNKNOWN_GGEP,		/**< Unknown GGEP extension */
 	EXT_T_OVERHEAD,			/**< Pure overhead */
 	EXT_T_GGEP_LIME_XML,	/**< LimeWire XML metadata, in query hits */
 	/* sort below */
+	EXT_T_GGEP_A,			/**< Same as GGEP ALT but used in HEAD Pongs */
 	EXT_T_GGEP_ALT,			/**< Alternate locations in query hits */
+	EXT_T_GGEP_ALT_TLS,		/**< TLS-capability bitmap for GGEP ALT */
 	EXT_T_GGEP_BH,			/**< Browseable host indication */
+	EXT_T_GGEP_C,			/**< Result Code in HEAD Pongs */
 	EXT_T_GGEP_CT,			/**< Resource creation time */
 	EXT_T_GGEP_DU,			/**< Daily Uptime */
+	EXT_T_GGEP_F,			/**< Flags in HEAD Pongs */
 	EXT_T_GGEP_FW,			/**< Firewalled-to-Firewalled protocol version */
 	EXT_T_GGEP_GGEP,		/**< Name of known GGEP extensions, NUL-separated */
 	EXT_T_GGEP_GTKG_IPV6,	/**< GTKG IPv6 address */
@@ -89,16 +94,24 @@ typedef enum ext_token {
 	EXT_T_GGEP_HNAME,		/**< Hostname info, in query hits */
 	EXT_T_GGEP_IP,			/**< IP:Port, in ping and pongs (F2F) */
 	EXT_T_GGEP_IPP,			/**< IP:Port, in pongs (UHC) */
+	EXT_T_GGEP_IPP_TLS,		/**< TLS-capability bitmap for GGEP IPP */
 	EXT_T_GGEP_LF,			/**< Large File, in query hits */
 	EXT_T_GGEP_LOC,			/**< Locale preferences */
+	EXT_T_GGEP_NP,			/**< do Not Proxy the query (OOB) */
+	EXT_T_GGEP_P,			/**< Push alt-locs in HEAD Pongs */
 	EXT_T_GGEP_PATH,		/**< Shared file path, in query hits */
 	EXT_T_GGEP_PHC,			/**< Packed HostCaches, in pongs (UHC) */
 	EXT_T_GGEP_PUSH,		/**< Push proxy info, in query hits */
+	EXT_T_GGEP_PUSH_TLS,	/**< TLS-capability bitmap for GGEP PUSH */
+	EXT_T_GGEP_Q,			/**< Queue status in HEAD Pongs */
 	EXT_T_GGEP_SCP,			/**< Support Cached Pongs, in pings (UHC) */
-	EXT_T_GGEP_T,			/**< Textual information in query hits */
+	EXT_T_GGEP_SO,			/**< Secure OOB */
+	EXT_T_GGEP_T,			/**< Same as ALT_TLS but for HEAD Pongs */
+	EXT_T_GGEP_TLS,			/**< Supports TLS */
 	EXT_T_GGEP_UA,			/**< User-Agent string */
 	EXT_T_GGEP_UDPHC,		/**< UDP HostCache, in pongs (UHC) */
 	EXT_T_GGEP_UP,			/**< UltraPeer information */
+	EXT_T_GGEP_V,			/**< Vendor Code in HEAD Pongs */
 	EXT_T_GGEP_VC,			/**< Vendor Code */
 	EXT_T_GGEP_VMSG,		/**< Array of vendor message codes supported */
 	EXT_T_GGEP_u,			/**< HUGE URN in ASCII */
@@ -106,8 +119,8 @@ typedef enum ext_token {
 	EXT_T_TOKEN_COUNT
 } ext_token_t;
 
-#define GGEP_NAME(x) ext_ggep_name(CAT2(EXT_T_GGEP_,x))
-#define GGEP_GTKG_NAME(x) ext_ggep_name(CAT2(EXT_T_GGEP_GTKG_,x))
+#define GGEP_NAME(x) ext_ggep_name(EXT_T_GGEP_ ## x)
+#define GGEP_GTKG_NAME(x) ext_ggep_name(EXT_T_GGEP_GTKG_ ## x)
 
 /**
  * A public extension descriptor.
@@ -148,7 +161,7 @@ void ext_init(void);
 void ext_close(void);
 
 void ext_prepare(extvec_t *exv, gint exvcnt);
-gint ext_parse(gchar *buf, gint len, extvec_t *exv, gint exvcnt);
+gint ext_parse(const gchar *buf, gint len, extvec_t *exv, gint exvcnt);
 void ext_reset(extvec_t *exv, gint exvcnt);
 
 gboolean ext_is_printable(const extvec_t *e);
@@ -158,7 +171,7 @@ gboolean ext_has_ascii_word(const extvec_t *e);
 void ext_dump(FILE *fd, const extvec_t *extvec, gint extcnt,
 	const gchar *prefix, const gchar *postfix, gboolean payload);
 
-const gchar *ext_payload(const extvec_t *e);
+gconstpointer ext_payload(const extvec_t *e);
 guint16 ext_paylen(const extvec_t *e);
 const gchar *ext_base(const extvec_t *e);
 guint16 ext_headlen(const extvec_t *e);
@@ -166,9 +179,6 @@ guint16 ext_len(const extvec_t *e);
 const gchar *ext_ggep_id_str(const extvec_t *e);
 const gchar *ext_ggep_name(ext_token_t id);
 
-void ext_prepare(extvec_t *exv, gint exvcnt);
-void ext_reset(extvec_t *exv, gint exvcnt);
-
 #endif	/* _core_extensions_h_ */
 
 /* vi: set ts=4 sw=4 cindent: */
