@@ -1,5 +1,5 @@
 /*
- * $Id: guid.c 12242 2006-11-06 20:41:55Z cbiere $
+ * $Id: guid.c 13777 2007-05-29 00:23:11Z cbiere $
  *
  * Copyright (c) 2002-2003, Raphael Manfredi
  *
@@ -39,7 +39,7 @@
 
 #include "common.h"
 
-RCSID("$Id: guid.c 12242 2006-11-06 20:41:55Z cbiere $")
+RCSID("$Id: guid.c 13777 2007-05-29 00:23:11Z cbiere $")
 
 #include "guid.h"
 #include "lib/misc.h"
@@ -157,7 +157,7 @@ guid_init(void)
 	gtkg_version_mark =
 		guid_gtkg_encode_version(GTA_VERSION, GTA_SUBVERSION, *rev == '\0');
 
-	if (dbg)
+	if (GNET_PROPERTY(dbg))
 		printf("GTKG version mark is 0x%x\n", gtkg_version_mark);
 }
 
@@ -430,7 +430,7 @@ void
 guid_oob_get_addr_port(const gchar *guid, host_addr_t *addr, guint16 *port)
 {
 	if (addr) {
-		*addr = host_addr_get_ipv4(peek_be32(&guid[0])); /* @todo TODO: IPv6 */
+		*addr = host_addr_peek_ipv4(&guid[0]); /* @todo TODO: IPv6 */
 	}
 	if (port) {
 		*port = peek_le16(&guid[13]);
