@@ -1,5 +1,5 @@
 /*
- * $Id: hcache.c,v 1.31 2005/11/09 19:16:53 cbiere Exp $
+ * $Id: hcache.c 13787 2007-05-29 21:09:50Z cbiere $
  *
  * Copyright (c) 2002-2003, Raphael Manfredi, Richard Eckart
  *
@@ -40,9 +40,6 @@
  *		-# hits/misses while adding,
  *		-# hits/misses while bad checking
  *		-# how many hosts were tried to connect to?
- *	- display stats about gwcache usage:
- *		-# how often
- *		-# how many hosts got
  *	- move unstable servant code from nodes.c to hcache.c
  *	- make sure hosts we are currently connected too are also saved
  *		to disk on exit!
@@ -52,7 +49,7 @@
 
 #include "common.h"
 
-RCSID("$Id: hcache.c,v 1.31 2005/11/09 19:16:53 cbiere Exp $");
+RCSID("$Id: hcache.c 13787 2007-05-29 21:09:50Z cbiere $")
 
 #include "hosts.h"
 #include "hcache.h"
@@ -62,7 +59,6 @@ RCSID("$Id: hcache.c,v 1.31 2005/11/09 19:16:53 cbiere Exp $");
 #include "bogons.h"
 #include "hostiles.h"
 
-#include "lib/bg.h"
 #include "lib/file.h"
 #include "lib/hashlist.h"
 #include "lib/tm.h"
@@ -83,12 +79,8 @@ RCSID("$Id: hcache.c,v 1.31 2005/11/09 19:16:53 cbiere Exp $");
  * the host.
  */
 typedef struct hostcache_entry {
-    hcache_type_t type;					/**< Hostcache which contains this host */
-    time_t        time_added;			/**< Time when entry was added */
-#if 0
-	guint32       avg_uptime;			/**< Reported average uptime (seconds) */
-	gchar *       vendor;				/**< Latest known vendor name (atom) */
-#endif
+    hcache_type_t type;				/**< Hostcache which contains this host */
+    time_t        time_added;		/**< Time when entry was added */
 } hostcache_entry_t;
 
 /** No metadata for host */
@@ -98,7 +90,7 @@ typedef struct hostcache_entry {
  * A hostcache table.
  */
 typedef struct hostcache {
-	gchar *         name;		        /**< Name of the cache */
+	const gchar		*name;		        /**< Name of the cache */
 	hcache_type_t   type;				/**< Cache type */
 
     gboolean        addr_only;          /**< Use IP only, port always 0 */
@@ -108,8 +100,6 @@ typedef struct hostcache {
     guint           hits;               /**< Hits to the cache */
     guint           misses;             /**< Misses to the cache */
 
-    guint           host_count;			/**< Amount of hosts in cache */
-	gnet_property_t reading;			/**< Property to signal reading */
     gnet_property_t hosts_in_catcher;   /**< Property to update host count */
     gint            mass_update;        /**< If a mass update is in progess */
 } hostcache_t;
@@ -142,8 +132,6 @@ static const gchar * const host_type_names[HOST_MAX] = {
 };
 
 
-static gpointer bg_reader[HCACHE_MAX];
-
 enum {
     HCACHE_ALREADY_CONNECTED,
     HCACHE_INVALID_HOST,
@@ -177,25 +165,22 @@ stop_mass_update(hostcache_t *hc)
     if (hc->mass_update == 0) {
         switch (hc->type) {
         case HCACHE_FRESH_ANY:
-        case HCACHE_VALID_ANY: {
-            gnet_prop_set_guint32_val(hc->hosts_in_catcher,
-                caches[HCACHE_FRESH_ANY]->host_count +
-                caches[HCACHE_VALID_ANY]->host_count);
+        case HCACHE_VALID_ANY:
+           	gnet_prop_set_guint32_val(hc->hosts_in_catcher,
+				hcache_size(HOST_ANY));
             break;
-        }
         case HCACHE_FRESH_ULTRA:
         case HCACHE_VALID_ULTRA:
             gnet_prop_set_guint32_val(hc->hosts_in_catcher,
-                caches[HCACHE_FRESH_ULTRA]->host_count +
-                caches[HCACHE_VALID_ULTRA]->host_count);
+				hcache_size(HOST_ULTRA));
             break;
         case HCACHE_TIMEOUT:
         case HCACHE_UNSTABLE:
         case HCACHE_BUSY:
             gnet_prop_set_guint32_val(hc->hosts_in_catcher,
-                caches[HCACHE_TIMEOUT]->host_count +
-                caches[HCACHE_UNSTABLE]->host_count +
-                caches[HCACHE_BUSY]->host_count);
+                hash_list_length(caches[HCACHE_TIMEOUT]->hostlist) +
+                hash_list_length(caches[HCACHE_UNSTABLE]->hostlist) +
+                hash_list_length(caches[HCACHE_BUSY]->hostlist));
             break;
         default:
             g_error("stop_mass_update: unknown cache type: %d", hc->type);
@@ -211,10 +196,8 @@ static GHashTable * ht_known_hosts = NULL;
 static void
 hcache_update_low_on_pongs(void)
 {
-    host_low_on_pongs = (guint) (
-            caches[HCACHE_FRESH_ANY]->host_count +
-            caches[HCACHE_VALID_ANY]->host_count
-        ) < (max_hosts_cached >> 3);
+    host_low_on_pongs = hcache_size(HOST_ANY) <
+							(GNET_PROPERTY(max_hosts_cached) / 8);
 }
 
 /***
@@ -224,14 +207,21 @@ hcache_update_low_on_pongs(void)
 static hostcache_entry_t *
 hce_alloc(void)
 {
-	return walloc0(sizeof(struct hostcache_entry));
+	static const hostcache_entry_t zero_hce;
+	hostcache_entry_t *hce;
+
+	hce = walloc(sizeof *hce);
+	*hce = zero_hce;
+	return hce;
 }
 
 static void
 hce_free(struct hostcache_entry *hce)
 {
-	g_assert(hce && hce != NO_METADATA);
-	wfree(hce, sizeof(*hce));
+	g_assert(hce);
+	g_assert(hce != NO_METADATA);
+
+	wfree(hce, sizeof *hce);
 }
 
 /**
@@ -240,10 +230,8 @@ hce_free(struct hostcache_entry *hce)
 static void
 hcache_dump_info(const struct hostcache *hc, const gchar *what)
 {
-    g_message("[%s|%s] %u (%d) hosts (%u hits, %u misses)",
-        hc->name, what, hc->host_count,
-        hash_list_length(hc->hostlist),
-        hc->hits, hc->misses);
+    g_message("[%s|%s] %u hosts (%u hits, %u misses)",
+        hc->name, what, hash_list_length(hc->hostlist), hc->hits, hc->misses);
 }
 
 /***
@@ -268,8 +256,7 @@ hcache_ht_get(const host_addr_t addr, guint16 port,
 	gpointer k, v;
 	gboolean found;
 
-	host.addr = addr;
-	host.port = port;
+	gnet_host_set(&host, addr, port);
 
 	found = g_hash_table_lookup_extended(ht_known_hosts, &host, &k, &v);
 	if (found) {
@@ -313,26 +300,19 @@ hcache_ht_add(hcache_type_t type, gnet_host_t *host)
 static void
 hcache_ht_remove(gnet_host_t *host)
 {
-	union {
-		hostcache_entry_t *hce;
-		gpointer ptr;
-	} entry;
-	gpointer key;
-	gboolean found;
+	hostcache_entry_t *hce;
+	gpointer key, value;
 
-	found = g_hash_table_lookup_extended(ht_known_hosts,
-		(gconstpointer) host, &key, &entry.ptr);
-
-	if (!found) {
+	if (!g_hash_table_lookup_extended(ht_known_hosts, host, &key, &value)) {
 		g_warning("hcache_ht_remove: attempt to remove unknown host: %s",
-				  host_addr_port_to_string(host->addr, host->port));
+			  gnet_host_to_string(host));
 		return;
 	}
-
+	hce = value;
 	g_hash_table_remove(ht_known_hosts, host);
 
-	if (entry.hce != NO_METADATA)
-		hce_free(entry.hce);
+	if (hce != NO_METADATA)
+		hce_free(hce);
 }
 
 /**
@@ -364,11 +344,10 @@ hcache_node_is_bad(const host_addr_t addr)
 	if (host_low_on_pongs)
 		return FALSE;
 
-    h.addr = addr;
-    h.port = 0;
+	gnet_host_set(&h, addr, 0);
     hce = hcache_get_metadata(&h);
 
-    if ((hce == NULL) || (hce == NO_METADATA))
+    if (hce == NULL || hce == NO_METADATA)
         return FALSE;
 
     caches[hce->type]->hits++;
@@ -395,13 +374,10 @@ hcache_move_entries(hostcache_t *to, hostcache_t *from)
 	gpointer item;
 
     g_assert(hash_list_length(to->hostlist) == 0);
-    g_assert(to->host_count == 0);
 
-	hash_list_free(to->hostlist);
+	hash_list_free(&to->hostlist);
     to->hostlist = from->hostlist;
-    to->host_count = from->host_count;
-    from->hostlist = hash_list_new();
-    from->host_count = 0;
+    from->hostlist = hash_list_new(NULL, NULL);
 
     /*
      * Make sure that after switching hce->list points to the new
@@ -410,16 +386,16 @@ hcache_move_entries(hostcache_t *to, hostcache_t *from)
 
 	iter = hash_list_iterator(to->hostlist);
 
-	while (NULL != (item = hash_list_next(iter))) {
+	while (NULL != (item = hash_list_iter_next(iter))) {
         hostcache_entry_t *hce;
 
-        hce = hcache_get_metadata((gnet_host_t *) item);
+        hce = hcache_get_metadata(item);
         if (hce == NULL || hce == NO_METADATA)
             continue;
         hce->type = to->type;
     }
 
-	hash_list_release(iter);
+	hash_list_iter_release(&iter);
 }
 
 /**
@@ -440,20 +416,18 @@ hcache_require_caught(hostcache_t *hc)
     switch (hc->type) {
     case HCACHE_FRESH_ANY:
     case HCACHE_VALID_ANY:
-        if (caches[HCACHE_FRESH_ANY]->host_count == 0) {
-            hcache_move_entries(
-                caches[HCACHE_FRESH_ANY], caches[HCACHE_VALID_ANY]);
+        if (hash_list_length(caches[hc->type]->hostlist) == 0) {
+            hcache_move_entries(caches[hc->type], caches[HCACHE_VALID_ANY]);
         }
-        return caches[HCACHE_FRESH_ANY]->host_count != 0;
+        return hash_list_length(caches[hc->type]->hostlist) != 0;
     case HCACHE_FRESH_ULTRA:
     case HCACHE_VALID_ULTRA:
-        if (caches[HCACHE_FRESH_ULTRA]->host_count == 0) {
-            hcache_move_entries(
-                caches[hc->type], caches[HCACHE_VALID_ULTRA]);
+        if (hash_list_length(caches[hc->type]->hostlist) == 0) {
+            hcache_move_entries(caches[hc->type], caches[HCACHE_VALID_ULTRA]);
         }
-        return caches[HCACHE_FRESH_ULTRA]->host_count != 0;
+        return hash_list_length(caches[hc->type]->hostlist) != 0;
     default:
-        return hc->host_count != 0;
+        return hash_list_length(hc->hostlist) != 0;
     }
 }
 
@@ -463,11 +437,13 @@ hcache_require_caught(hostcache_t *hc)
 static void
 hcache_unlink_host(hostcache_t *hc, gnet_host_t *host)
 {
-	g_assert(hc->host_count > 0 && hc->hostlist != NULL);
-	g_assert(hash_list_contains(hc->hostlist, host));
+	gconstpointer orig_key;
+	
+	g_assert(hc->hostlist != NULL);
+	g_assert(hash_list_length(hc->hostlist) > 0);
 
-	hash_list_remove(hc->hostlist, host);
-    hc->host_count--;
+	orig_key = hash_list_remove(hc->hostlist, host);
+	g_assert(orig_key);
 
     if (hc->mass_update == 0) {
         guint32 cur;
@@ -516,26 +492,23 @@ host_type_to_string(host_type_t type)
  * Several types share common pools. Adding a host of one type may
  * affect the number of slots left on other types.
  */
-static gint32
+static gint
 hcache_slots_left(hcache_type_t type)
 {
-    g_assert((guint) type < HCACHE_MAX);
+    g_assert(UNSIGNED(type) < HCACHE_MAX);
 
     switch (type) {
     case HCACHE_FRESH_ANY:
     case HCACHE_VALID_ANY:
-        return max_hosts_cached -
-            caches[HCACHE_FRESH_ANY]->host_count -
-            caches[HCACHE_VALID_ANY]->host_count;
+        return GNET_PROPERTY(max_hosts_cached) - hcache_size(HOST_ANY);
     case HCACHE_FRESH_ULTRA:
     case HCACHE_VALID_ULTRA:
-        return max_ultra_hosts_cached -
-            caches[HCACHE_FRESH_ULTRA]->host_count -
-            caches[HCACHE_VALID_ULTRA]->host_count;
+        return GNET_PROPERTY(max_ultra_hosts_cached) - hcache_size(HOST_ULTRA);
 	case HCACHE_NONE:
 		g_assert_not_reached();
     default:
-        return max_bad_hosts_cached - caches[type]->host_count;
+        return GNET_PROPERTY(max_bad_hosts_cached) -
+					hash_list_length(caches[type]->hostlist);
     }
 }
 
@@ -560,7 +533,11 @@ hcache_add(hcache_type_t type, const host_addr_t addr, guint16 port,
 	hostcache_t *hc;
 	hostcache_entry_t *hce;
 
-	g_assert((guint) type < HCACHE_MAX && type != HCACHE_NONE);
+	g_assert(UNSIGNED(type) < HCACHE_MAX);
+	g_assert(type != HCACHE_NONE);
+
+	if (GNET_PROPERTY(stop_host_get))
+		return FALSE;
 
 	/*
 	 * Don't add anything to the "unstable" cache if they don't want to
@@ -571,11 +548,11 @@ hcache_add(hcache_type_t type, const host_addr_t addr, guint16 port,
 	 */
 
     if (type == HCACHE_UNSTABLE) {
-    	if (!node_monitor_unstable_ip || host_low_on_pongs)
+    	if (!GNET_PROPERTY(node_monitor_unstable_ip) || host_low_on_pongs)
 			return FALSE;
 	}
 
-	if (host_addr_equal(addr, listen_addr()) && port == listen_port) {
+	if (is_my_address_and_port(addr, port)) {
         stats[HCACHE_LOCAL_INSTANCE]++;
 		return FALSE;
     }
@@ -607,6 +584,8 @@ hcache_add(hcache_type_t type, const host_addr_t addr, guint16 port,
 	 */
 
 	if (hcache_ht_get(addr, port, &host, &hce)) {
+		gconstpointer orig_key;
+
         g_assert(hce != NULL);
 
         hc->hits++;
@@ -653,12 +632,10 @@ hcache_add(hcache_type_t type, const host_addr_t addr, guint16 port,
 		 * OK, we can move it from the `hce->type' cache to the `type' one.
 		 */
 
-		g_assert(hash_list_contains(caches[hce->type]->hostlist, host));
+		orig_key = hash_list_remove(caches[hce->type]->hostlist, host);
+		g_assert(orig_key);
 
-		hash_list_remove(caches[hce->type]->hostlist, host);
-		caches[hce->type]->host_count--;
 		hash_list_prepend(hc->hostlist, host);
-		hc->host_count++;
 		caches[hce->type]->dirty = hc->dirty = TRUE;
 
 		hce->type = type;
@@ -670,8 +647,7 @@ hcache_add(hcache_type_t type, const host_addr_t addr, guint16 port,
 	/* Okay, we got a new host */
 	host = walloc(sizeof *host);
 
-	host->addr = addr;
-	host->port = port;
+	gnet_host_set(host, addr, port);
 
 	hcache_ht_add(type, host);
 
@@ -701,7 +677,6 @@ hcache_add(hcache_type_t type, const host_addr_t addr, guint16 port,
     }
 
     hc->misses++;
-	hc->host_count++;
 	hc->dirty = TRUE;
 
     if (hc->mass_update == 0) {
@@ -713,10 +688,9 @@ hcache_add(hcache_type_t type, const host_addr_t addr, guint16 port,
     hcache_prune(hc->type);
     hcache_update_low_on_pongs();
 
-    if (dbg > 8) {
-        printf("Added %s %s (%s)\n",
-			what, host_addr_port_to_string(addr, port),
-            ((type == HCACHE_FRESH_ANY) || (type == HCACHE_VALID_ANY)) ?
+    if (GNET_PROPERTY(dbg) > 8) {
+        printf("Added %s %s (%s)\n", what, gnet_host_to_string(host),
+            (type == HCACHE_FRESH_ANY || type == HCACHE_VALID_ANY) ?
                 (host_low_on_pongs ? "LOW" : "OK") : "");
     }
 
@@ -777,7 +751,7 @@ hcache_remove(gnet_host_t *h)
     hce = hcache_get_metadata(h);
     if (hce == NULL) {
 		g_warning("hcache_remove: attempt to remove unknown host: %s",
-				  host_addr_port_to_string(h->addr, h->port));
+			gnet_host_to_string(h));
         return; /* Host is not in hashtable */
     }
 
@@ -792,18 +766,7 @@ hcache_remove(gnet_host_t *h)
 gboolean
 hcache_is_low(host_type_t type)
 {
-    switch (type) {
-    case HOST_ANY:
-        return (caches[HCACHE_FRESH_ANY]->host_count +
-            caches[HCACHE_VALID_ANY]->host_count) < MIN_RESERVE_SIZE;
-    case HOST_ULTRA:
-        return (caches[HCACHE_FRESH_ULTRA]->host_count +
-            caches[HCACHE_VALID_ULTRA]->host_count) < MIN_RESERVE_SIZE;
-    case HOST_MAX:
-		g_assert_not_reached();
-    }
-    g_error("hcache_is_low: unknown host type: %d", type);
-    return FALSE; /* Only here to make -Wall happy */
+	return hcache_size(type) < MIN_RESERVE_SIZE;
 }
 
 /**
@@ -814,23 +777,20 @@ hcache_remove_all(hostcache_t *hc)
 {
 	gnet_host_t *h;
 
-    if (hc->host_count == 0) {
-		g_assert(hash_list_length(hc->hostlist) == 0);
+    if (hash_list_length(hc->hostlist) == 0)
         return;
-	}
 
     /* FIXME: may be possible to do this faster */
 
     start_mass_update(hc);
 
-    while (NULL != (h = hash_list_first(hc->hostlist)))
+    while (NULL != (h = hash_list_head(hc->hostlist)))
         hcache_remove(h);
 
     g_assert(hash_list_length(hc->hostlist) == 0);
-    g_assert(hc->host_count == 0);
 
     stop_mass_update(hc);
-    g_assert(hc->host_count == 0);
+    g_assert(hash_list_length(hc->hostlist) == 0);
 }
 
 /**
@@ -878,16 +838,16 @@ hcache_clear(hcache_type_t type)
 /**
  * @return the amount of hosts in the cache.
  */
-gint
+guint
 hcache_size(host_type_t type)
 {
     switch (type) {
     case HOST_ANY:
-        return (caches[HCACHE_FRESH_ANY]->host_count +
-            caches[HCACHE_VALID_ANY]->host_count);
+        return hash_list_length(caches[HCACHE_FRESH_ANY]->hostlist) +
+            hash_list_length(caches[HCACHE_VALID_ANY]->hostlist);
     case HOST_ULTRA:
-        return (caches[HCACHE_FRESH_ULTRA]->host_count +
-            caches[HCACHE_VALID_ULTRA]->host_count);
+        return hash_list_length(caches[HCACHE_FRESH_ULTRA]->hostlist) +
+            hash_list_length(caches[HCACHE_VALID_ULTRA]->hostlist);
     case HOST_MAX:
 		g_assert_not_reached();
     }
@@ -915,7 +875,7 @@ hcache_expire_cache(hostcache_t *hc, time_t now)
      * sorted by time_added
 	 */
 
-    while (NULL != (h = hash_list_last(hc->hostlist))) {
+    while (NULL != (h = hash_list_tail(hc->hostlist))) {
         hostcache_entry_t *hce = hcache_get_metadata(h);
 
         g_assert(hce != NULL);
@@ -970,10 +930,11 @@ hcache_prune(hcache_type_t type)
 
 	hc = caches[type];
 
-#define HALF_PRUNE(x) do {		\
-	if (hc->host_count < caches[x]->host_count) \
+#define HALF_PRUNE(x) G_STMT_START {		\
+	if (hash_list_length(hc->hostlist) < \
+			hash_list_length(caches[x]->hostlist)) \
 		hc = caches[x];			\
-} while (0)
+} G_STMT_END
 
     switch (type) {
     case HCACHE_VALID_ANY:
@@ -994,23 +955,21 @@ hcache_prune(hcache_type_t type)
 
 #undef HALF_PRUNE
 
-    extra = -hcache_slots_left(hc->type);
-
-    if (extra <= 0)
+    extra = hcache_slots_left(hc->type);
+    if (extra >= 0)
         return;
 
     start_mass_update(hc);
 
     hcache_require_caught(hc);
-	while (extra > 0) {
-		gnet_host_t *h = hash_list_first(hc->hostlist);
+	while (extra++ < 0) {
+		gnet_host_t *h = hash_list_head(hc->hostlist);
 		if (NULL == h) {
 			g_warning("BUG: asked to remove hosts, "
                 "but hostcache list is empty: %s", hc->name);
 			break;
 		}
 		hcache_remove(h);
-        extra--;
 	}
 
     stop_mass_update(hc);
@@ -1053,16 +1012,19 @@ hcache_fill_caught_array(host_type_t type, gnet_host_t *hosts, gint hcount)
 
 	for (i = 0; i < hcount; i++) {
 		gnet_host_t host;
+		host_addr_t addr;
+		guint16 port;
 
-		if (!pcache_get_recent(type, &host.addr, &host.port))
+		if (!pcache_get_recent(type, &addr, &port))
 			break;
 
+		gnet_host_set(&host, addr, port);
 		if (g_hash_table_lookup(seen_host, &host))
 			break;
 
 		hosts[i] = host;		/* struct copy */
 
-		g_hash_table_insert(seen_host, &hosts[i], (gpointer) 0x1);
+		g_hash_table_insert(seen_host, &hosts[i], GUINT_TO_POINTER(1));
 	}
 
 	if (i == hcount)
@@ -1072,22 +1034,22 @@ hcache_fill_caught_array(host_type_t type, gnet_host_t *hosts, gint hcount)
 	 * Not enough fresh pongs, get some from our reserve.
 	 */
 
-	iter = hash_list_iterator_last(hc->hostlist);
+	iter = hash_list_iterator_tail(hc->hostlist);
 
 	for (
-        h = hash_list_previous(iter);
+        h = hash_list_iter_previous(iter);
         i < hcount && h != NULL;
-        i++, h = hash_list_previous(iter)
+        i++, h = hash_list_iter_previous(iter)
     ) {
 		if (g_hash_table_lookup(seen_host, h))
 			continue;
 
 		hosts[i] = *h;			/* struct copy */
 
-		g_hash_table_insert(seen_host, &hosts[i], (gpointer) 0x1);
+		g_hash_table_insert(seen_host, &hosts[i], GUINT_TO_POINTER(1));
 	}
 
-	hash_list_release(iter);
+	hash_list_iter_release(&iter);
 
 done:
 	g_hash_table_destroy(seen_host);	/* Keys point directly into vector */
@@ -1110,8 +1072,6 @@ hcache_find_nearby(host_type_t type, host_addr_t *addr, guint16 *port)
 	guint16 first_port;
 	gboolean got_recent;
 	hostcache_t *hc = NULL;
-	gboolean reading;
-    gboolean result = FALSE;
 	hash_list_iter_t *iter;
 
     switch (type) {
@@ -1128,8 +1088,6 @@ hcache_find_nearby(host_type_t type, host_addr_t *addr, guint16 *port)
 	if (!hc)
         g_error("hcache_get_caught: unknown host type: %d", type);
 
-    gnet_prop_get_boolean_val(hc->reading, &reading);
-
 	if (alternate++ & 1) {
 		/* Iterate through all recent pongs */
 
@@ -1149,43 +1107,45 @@ hcache_find_nearby(host_type_t type, host_addr_t *addr, guint16 *port)
 
 	/* iterate through whole list */
 
-	iter = reading ? hash_list_iterator(hc->hostlist) :
-		hash_list_iterator_last(hc->hostlist);
-
-	for (h = hash_list_follower(iter); h; h = hash_list_follower(iter)) {
-
-		if (host_is_nearby(h->addr)) {
-            *addr = h->addr;
-            *port = h->port;
-
-            result = TRUE;
+	iter = hash_list_iterator(hc->hostlist);
+	while (NULL != (h = hash_list_iter_next(iter))) {
+		if (host_is_nearby(gnet_host_get_addr(h))) {
+            *addr = gnet_host_get_addr(h);
+            *port = gnet_host_get_port(h);
 			break;
 		}
 	}
+	hash_list_iter_release(&iter);
 
-	hash_list_release(iter);
-
-	if (result)
+	if (h) {
 		hcache_unlink_host(hc, h);
-
-	return result;
+		return TRUE;
+	}
+	return FALSE;
 }
 
 /**
  * Get host IP/port information from our caught host list, or from the
  * recent pong cache, in alternance.
+ *
+ * @return TRUE on sucess, FALSE on failure.
  */
-void
+gboolean
 hcache_get_caught(host_type_t type, host_addr_t *addr, guint16 *port)
 {
 	static guint alternate = 0;
 	hostcache_t *hc = NULL;
-	gboolean reading;
 	extern guint32 number_local_networks;
 	gnet_host_t *h;
 	gboolean available;
 
-    switch(type) {
+	g_assert(addr);
+	g_assert(port);
+
+	*addr = zero_host_addr;
+	*port = 0;
+	
+    switch (type) {
     case HOST_ANY:
         hc = caches[HCACHE_FRESH_ANY];
         break;
@@ -1199,41 +1159,40 @@ hcache_get_caught(host_type_t type, host_addr_t *addr, guint16 *port)
 	if (!hc)
         g_error("hcache_get_caught: unknown host type: %d", type);
 
-    gnet_prop_get_boolean_val(hc->reading, &reading);
     available = hcache_require_caught(hc);
 
-    g_assert(available);
-
     hcache_update_low_on_pongs();
 
+	if (!available)
+		return FALSE;
+
 	/*
 	 * First, try to find a local host
 	 */
 
 	if (
-		use_netmasks && number_local_networks &&
+		GNET_PROPERTY(use_netmasks) &&
+		number_local_networks &&
 		hcache_find_nearby(type, addr, port)
 	)
-		return;
+		return TRUE;
 
 	/*
 	 * Try the recent pong cache when `alternate' is odd.
 	 */
 
 	if (alternate++ & 0x1 && pcache_get_recent(type, addr, port))
-		return;
+		return TRUE;
 
-	/*
-	 * If we're done reading from the host file, get latest host, at the
-	 * tail of the list.  Otherwise, get the first host in that list.
-	 */
+	h = hash_list_head(hc->hostlist);
+	if (h) {
+		*addr = gnet_host_get_addr(h);
+		*port = gnet_host_get_port(h);
+		hcache_unlink_host(hc, h);
+		return TRUE;
+	}
 
-	h = reading ? hash_list_first(hc->hostlist) : hash_list_last(hc->hostlist);
-
-	*addr = h->addr;
-	*port = h->port;
-
-    hcache_unlink_host(hc, h);
+	return FALSE;
 }
 
 /***
@@ -1248,13 +1207,9 @@ hcache_get_caught(host_type_t type, host_addr_t *addr, guint16 *port)
  *
  * The `maxhosts' variable is the pointer to the variable giving the maximum
  * amount of hosts we can store.
- *
- * The `reading' variable is the property to update to signal whether we're
- * reading the persisted file.
  */
 static hostcache_t *
-hcache_alloc(hcache_type_t type, gnet_property_t reading,
-	gnet_property_t catcher, gchar *name)
+hcache_alloc(hcache_type_t type, gnet_property_t catcher, const gchar *name)
 {
 	struct hostcache *hc;
 
@@ -1262,10 +1217,9 @@ hcache_alloc(hcache_type_t type, gnet_property_t reading,
 
 	hc = g_malloc0(sizeof *hc);
 
-	hc->hostlist = hash_list_new();
+	hc->hostlist = hash_list_new(NULL, NULL);
 	hc->name = name;
 	hc->type = type;
-	hc->reading = reading;
     hc->hosts_in_catcher = catcher;
     hc->addr_only = FALSE;
 
@@ -1279,130 +1233,34 @@ static void
 hcache_free(hostcache_t *hc)
 {
     g_assert(hc != NULL);
-    g_assert(hc->host_count == 0);
     g_assert(hash_list_length(hc->hostlist) == 0);
 
-	hash_list_free(hc->hostlist);
+	hash_list_free(&hc->hostlist);
 	G_FREE_NULL(hc);
 }
 
-/***
- *** Hosts text files
- ***/
-
-/*
- * Host reading context.
- */
-
-#define READ_MAGIC		0x3d00003d
-#define HOST_READ_CNT	20			/**< Amount of hosts to read each tick */
-
-struct read_ctx {
-	gint magic;						/**< Magic number */
-	FILE *fd;						/**< File descriptor to read from */
-	hostcache_t *hc;				/**< Hostcache to fill */
-};
-
 /**
- * Dispose of the read context.
+ * Parse and load the hostcache file.
  */
 static void
-read_ctx_free(gpointer u)
+hcache_load_file(hostcache_t *hc, FILE *f)
 {
-	struct read_ctx *rctx = (struct read_ctx *) u;
+	gchar buffer[1024];
 
-	g_assert(rctx->magic == READ_MAGIC);
+	g_return_if_fail(hc);
+	g_return_if_fail(f);
 
-	if (rctx->fd != NULL)
-		fclose(rctx->fd);
+	while (fgets(buffer, sizeof buffer, f)) {
+		host_addr_t addr;
+		guint16 port;
 
-	wfree(rctx, sizeof(*rctx));
-}
-
-/**
- * Read is finished.
- */
-static void
-read_done(hostcache_t *hc)
-{
-    /*
-     * Order is important so the GUI can update properly. First we say
-     * that loading has finished, then we tell the GUI the number of
-     * hosts in the catcher.
-     *      -- Richard, 6/8/2002
-     */
-
-    gnet_prop_set_boolean_val(hc->reading, FALSE);
-}
-
-/**
- * One reading step.
- */
-static bgret_t
-read_step(gpointer unused_h, gpointer u, gint ticks)
-{
-	struct read_ctx *rctx = u;
-	hostcache_t *hc;
-	gint max_read;
-	gint count;
-	gint i;
-	static gchar h_tmp[1024];
-
-	(void) unused_h;
-	g_assert(READ_MAGIC == rctx->magic);
-	g_assert(rctx->fd);
-
-	hc = rctx->hc;
-
-	max_read = hcache_slots_left(hc->type);
-	count = ticks * HOST_READ_CNT;
-	count = MIN(max_read, count);
-
-	if (dbg > 9)
-		printf("read_step(%s): ticks=%d, count=%d\n", hc->name, ticks, count);
-
-	for (i = 0; i < count; i++) {
-		if (fgets(h_tmp, sizeof(h_tmp) - 1, rctx->fd)) { /* NUL appended */
-			host_addr_t addr;
-			guint16 port;
-
-			if (string_to_host_addr_port(h_tmp, NULL, &addr, &port))
-                hcache_add(hc->type, addr, port, "on-disk cache");
-		} else
-			goto done;
+		if (string_to_host_addr_port(buffer, NULL, &addr, &port)) {
+			hcache_add(hc->type, addr, port, "on-disk cache");
+			if (hcache_slots_left(hc->type) < 1) {
+				break;
+			}
+		}
 	}
-
-	if (count < max_read)
-		return BGR_MORE;		/* Host cache not full, need to read more */
-
-	/* Fall through */
-
-done:
-	fclose(rctx->fd);
-	rctx->fd = NULL;
-
-	read_done(hc);
-
-	return BGR_DONE;
-}
-
-/**
- * Invoked when the task is completed.
- */
-static void
-bg_reader_done(gpointer unused_h, gpointer ctx,
-		bgstatus_t unused_status, gpointer unused_arg)
-{
-	struct read_ctx *rctx = (struct read_ctx *) ctx;
-	hostcache_t *hc;
-
-	(void) unused_h;
-	(void) unused_status;
-	(void) unused_arg;
-	g_assert(rctx->magic == READ_MAGIC);
-
-	hc = rctx->hc;
-	bg_reader[hc->type] = NULL;
 }
 
 /**
@@ -1411,31 +1269,15 @@ bg_reader_done(gpointer unused_h, gpointer ctx,
 static void
 hcache_retrieve(hostcache_t *hc, const gchar *filename)
 {
-	struct read_ctx *rctx;
-	FILE *fd;
-	bgstep_cb_t step = read_step;
+	file_path_t fp[1];
+	FILE *f;
 
-    {
-		file_path_t fp;
-
-		file_path_set(&fp, settings_config_dir(), filename);
-		fd = file_config_open_read("hosts", &fp, 1);
+	file_path_set(fp, settings_config_dir(), filename);
+	f = file_config_open_read("hosts", fp, G_N_ELEMENTS(fp));
+	if (f) {
+		hcache_load_file(hc, f);
+		fclose(f);
 	}
-
-	if (!fd)
-		return;
-
-	rctx = walloc(sizeof(*rctx));
-	rctx->magic = READ_MAGIC;
-	rctx->fd = fd;
-	rctx->hc = hc;
-
-    gnet_prop_set_boolean_val(hc->reading, TRUE);
-
-	bg_reader[hc->type] = bg_task_create(
-		hc->type == HCACHE_FRESH_ANY ?
-			"Hostcache reading" : "Ultracache reading",
-		&step, 1, rctx, read_ctx_free, bg_reader_done, NULL);
 }
 
 /**
@@ -1448,11 +1290,10 @@ hcache_write(FILE *f, hostcache_t *hc)
 	gnet_host_t *h;
 
 	iter = hash_list_iterator(hc->hostlist);
-
-	for (h = hash_list_next(iter); h != NULL; h = hash_list_next(iter))
-		fprintf(f, "%s\n", host_addr_port_to_string(h->addr, h->port));
-
-	hash_list_release(iter);
+	while (NULL != (h = hash_list_iter_next(iter))) {
+		fprintf(f, "%s\n", gnet_host_to_string(h));
+	}
+	hash_list_iter_release(&iter);
 }
 
 /**
@@ -1497,7 +1338,7 @@ hcache_get_stats(hcache_stats_t *s)
     for (n = 0; n < HCACHE_MAX; n++) {
 		if (n == HCACHE_NONE)
 			continue;
-        s[n].host_count = caches[n]->host_count;
+        s[n].host_count = hash_list_length(caches[n]->hostlist);
         s[n].hits       = caches[n]->hits;
         s[n].misses     = caches[n]->misses;
         s[n].reading    = FALSE;
@@ -1512,7 +1353,7 @@ hcache_timer(time_t now)
 {
     hcache_expire_all(now);
 
-    if (dbg >= 15) {
+    if (GNET_PROPERTY(dbg) >= 15) {
         hcache_dump_info(caches[HCACHE_FRESH_ANY],   "timer");
         hcache_dump_info(caches[HCACHE_VALID_ANY],   "timer");
 
@@ -1535,45 +1376,42 @@ hcache_timer(time_t now)
 void
 hcache_init(void)
 {
-    memset(bg_reader, 0, sizeof(bg_reader));
-    memset(stats, 0, sizeof(stats));
-
 	ht_known_hosts = g_hash_table_new(host_hash, host_eq);
 
     caches[HCACHE_FRESH_ANY] = hcache_alloc(
-        HCACHE_FRESH_ANY, PROP_READING_HOSTFILE,
+        HCACHE_FRESH_ANY,
         PROP_HOSTS_IN_CATCHER,
         "hosts.fresh.any");
 
     caches[HCACHE_FRESH_ULTRA] = hcache_alloc(
-        HCACHE_FRESH_ULTRA, PROP_READING_ULTRAFILE,
+        HCACHE_FRESH_ULTRA,
         PROP_HOSTS_IN_ULTRA_CATCHER,
         "hosts.fresh.ultra");
 
     caches[HCACHE_VALID_ANY] = hcache_alloc(
-        HCACHE_VALID_ANY, PROP_READING_HOSTFILE,
+        HCACHE_VALID_ANY,
         PROP_HOSTS_IN_CATCHER,
         "hosts.valid.any");
 
     caches[HCACHE_VALID_ULTRA] = hcache_alloc(
-        HCACHE_VALID_ULTRA, PROP_READING_ULTRAFILE,
+        HCACHE_VALID_ULTRA,
         PROP_HOSTS_IN_ULTRA_CATCHER,
         "hosts.valid.ultra");
 
     caches[HCACHE_TIMEOUT] = hcache_alloc(
-        HCACHE_TIMEOUT, PROP_READING_HOSTFILE,
+        HCACHE_TIMEOUT,
         PROP_HOSTS_IN_BAD_CATCHER,
         "hosts.timeout");
     caches[HCACHE_TIMEOUT]->addr_only = TRUE;
 
     caches[HCACHE_BUSY] = hcache_alloc(
-        HCACHE_BUSY, PROP_READING_HOSTFILE,
+        HCACHE_BUSY,
         PROP_HOSTS_IN_BAD_CATCHER,
         "hosts.busy");
     caches[HCACHE_BUSY]->addr_only = TRUE;
 
     caches[HCACHE_UNSTABLE] = hcache_alloc(
-        HCACHE_UNSTABLE, PROP_READING_HOSTFILE,
+        HCACHE_UNSTABLE,
         PROP_HOSTS_IN_BAD_CATCHER,
         "hosts.unstable");
     caches[HCACHE_UNSTABLE]->addr_only = TRUE;
@@ -1596,9 +1434,8 @@ void
 hcache_store_if_dirty(host_type_t type)
 {
 	gnet_property_t prop;
-	gboolean reading;
 	hcache_type_t first, second;
-	gchar *file;
+	const gchar *file;
 
 	switch (type) {
     case HOST_ANY:
@@ -1618,11 +1455,6 @@ hcache_store_if_dirty(host_type_t type)
 		return;
 	}
 
-	gnet_prop_get_boolean_val(prop, &reading);
-
-	if (reading)
-		return;
-
 	if (!caches[first]->dirty && !caches[second]->dirty)
 		return;
 
@@ -1631,70 +1463,21 @@ hcache_store_if_dirty(host_type_t type)
 	caches[first]->dirty = caches[second]->dirty = FALSE;
 }
 
-/**
- * Check whether the host caches are completely read. This should be
- * used before contacting a GWebCache at start-up time.
- *
- * @returns TRUE if the host caches have been completely loaded and FALSE
- *			if the loading is still in process.
- */
-gboolean
-hcache_read_finished(void)
-{
-	gboolean reading;
-
-	gnet_prop_get_boolean_val(PROP_READING_HOSTFILE, &reading);
-	if (reading)
-		return FALSE;
-
-	gnet_prop_get_boolean_val(PROP_READING_ULTRAFILE, &reading);
-	if (reading)
-		return FALSE;
-
-	return TRUE;
-}
-
 /**
  * Shutdown host caches.
  */
 void
 hcache_shutdown(void)
 {
-	gboolean reading;
-
-	/*
-	 * Write "valid" hosts first.  Next time we are launched, we'll first
-	 * start reading from the head first.  And once the whole cache has
-	 * been read in memory, we'll begin using the tail of the list, i.e.
-	 * possibly older hosts, which will help ensure we don't always connect
-	 * to the same set of hosts.
-	 */
-
-	/* Save the caught hosts */
-
-	gnet_prop_get_boolean_val(PROP_READING_HOSTFILE, &reading);
-
-	if (reading)
-		g_warning("exit() while still reading the hosts file, "
-			"caught hosts not saved!");
-	else
-		hcache_store(HCACHE_VALID_ANY, "hosts", HCACHE_FRESH_ANY);
-
-	/* Save the caught ultra hosts */
-
-	gnet_prop_get_boolean_val(PROP_READING_ULTRAFILE, &reading);
-
-	if (reading)
-		g_warning("exit() while still reading the ultrahosts file, "
-			"caught hosts not saved !");
-	else
-		hcache_store(HCACHE_VALID_ULTRA, "ultras", HCACHE_FRESH_ULTRA);
+	hcache_store(HCACHE_VALID_ANY, "hosts", HCACHE_FRESH_ANY);
+	hcache_store(HCACHE_VALID_ULTRA, "ultras", HCACHE_FRESH_ULTRA);
 }
 
 /**
  * Destroy all host caches.
  */
-void hcache_close(void)
+void
+hcache_close(void)
 {
 	static const hcache_type_t types[] = {
         HCACHE_FRESH_ANY,
@@ -1720,18 +1503,15 @@ void hcache_close(void)
 		guint j;
 		hcache_type_t type = types[i];
 
-		if (bg_reader[type] != NULL)
-			bg_task_cancel(bg_reader[type]);
-
 		/* Make sure all previous caches have been cleared */
 		for (j = 0; j < type; j++)
-    		g_assert(caches[j]->host_count == 0);
+    		g_assert(hash_list_length(caches[j]->hostlist) == 0);
 
         hcache_remove_all(caches[type]);
 
 		/* Make sure no caches have been refilled */
 		for (j = 0; j <= type; j++)
-    		g_assert(caches[j]->host_count == 0);
+    		g_assert(hash_list_length(caches[j]->hostlist) == 0);
 	}
 
 	for (i = 0; i < G_N_ELEMENTS(types); i++) {
