@@ -1,5 +1,5 @@
 /*
- * $Id: hostiles.c,v 1.27 2005/11/14 13:04:22 cbiere Exp $
+ * $Id: hostiles.c 13777 2007-05-29 00:23:11Z cbiere $
  *
  * Copyright (c) 2004, Raphael Manfredi
  * Copyright (c) 2003, Markus Goetz
@@ -40,7 +40,7 @@
 
 #include "common.h"
 
-RCSID("$Id: hostiles.c,v 1.27 2005/11/14 13:04:22 cbiere Exp $");
+RCSID("$Id: hostiles.c 13777 2007-05-29 00:23:11Z cbiere $")
 
 #include "hostiles.h"
 #include "settings.h"
@@ -59,8 +59,6 @@ RCSID("$Id: hostiles.c,v 1.27 2005/11/14 13:04:22 cbiere Exp $");
 
 #include "lib/override.h"		/* Must be the last header included */
 
-#define THERE	GUINT_TO_POINTER(0x2)
-
 typedef enum {
 	HOSTILE_GLOBAL = 0,
 	HOSTILE_PRIVATE = 1,
@@ -68,10 +66,14 @@ typedef enum {
 	NUM_HOSTILES
 } hostiles_t;
 
+static const gchar hostile[] = "hostile";
 static const gchar hostiles_file[] = "hostiles.txt";
-static const gchar hostiles_what[] = "hostile IP addresses";
+static const gchar * const hostiles_what[NUM_HOSTILES] = {
+	"hostile IP addresses (global)",
+	"hostile IP addresses (private)"
+};
 
-static gpointer hostile_db[NUM_HOSTILES];		/**< The hostile database */
+static struct iprange_db *hostile_db[NUM_HOSTILES];	/**< The hostile database */
 
 /**
  * Frees all entries in the given hostiles.
@@ -79,12 +81,10 @@ static gpointer hostile_db[NUM_HOSTILES];		/**< The hostile database */
 static void
 hostiles_close_one(hostiles_t which)
 {
-	g_assert((gint) which >= 0 && which < NUM_HOSTILES);
-
-	if (hostile_db[which]) {
-		iprange_free_each(hostile_db[which], NULL);
-		hostile_db[which] = NULL;
-	}
+	guint i = which;
+	
+	g_assert(i < NUM_HOSTILES);
+	iprange_free(&hostile_db[i]);
 }
 
 /**
@@ -99,15 +99,13 @@ hostiles_load(FILE *f, hostiles_t which)
 	gchar *p;
 	guint32 ip, netmask;
 	int linenum = 0;
-	gint count = 0;
 	gint bits;
 	iprange_err_t error;
-	guint32 hosts = 0;
 
 	g_assert((gint) which >= 0 && which < NUM_HOSTILES);
 	g_assert(NULL == hostile_db[which]);
 
-	hostile_db[which] = iprange_make(NULL, NULL);
+	hostile_db[which] = iprange_new();
 
 	while (fgets(line, sizeof(line), f)) {
 		linenum++;
@@ -136,46 +134,31 @@ hostiles_load(FILE *f, hostiles_t which)
 		}
 
 		bits = netmask_to_cidr(netmask);
-		error = iprange_add_cidr(hostile_db[which], ip, bits, THERE);
+		error = iprange_add_cidr(hostile_db[which], ip, bits,
+					deconstify_gchar(hostile));
 
 		switch (error) {
 		case IPR_ERR_OK:
-			hosts += 1 << (32 - bits);
 			break;
-		case IPR_ERR_RANGE_OVERLAP:
-			error = iprange_add_cidr_force(hostile_db[which],
-						ip, bits, THERE, NULL);
-			if (dbg > 0 && error == IPR_ERR_OK) {
-				g_warning("%s: line %d: "
-					"entry \"%s\" (%s/%d) superseded earlier smaller range",
-					hostiles_file, linenum, line, ip_to_string(ip), bits);
-				break;
-			}
 			/* FALL THROUGH */
 		default:
-			if (dbg > 0 || error != IPR_ERR_RANGE_SUBNET) {
+			if (GNET_PROPERTY(dbg) > 0 || error != IPR_ERR_RANGE_SUBNET) {
 				g_warning("%s, line %d: rejected entry \"%s\" (%s/%d): %s",
 					hostiles_file, linenum, line, ip_to_string(ip), bits,
 					iprange_strerror(error));
 			}
 			continue;
 		}
-
-		count++;
 	}
 
-	if (dbg) {
-		iprange_stats_t stats;
+	iprange_sync(hostile_db[which]);
 
-		iprange_get_stats(hostile_db[which], &stats);
-
-		g_message("loaded %d hostile IP addresses/netmasks (%u hosts)",
-			count, hosts);
-		g_message("hostile stats: count=%d level2=%d heads=%d enlisted=%d",
-			stats.count, stats.level2, stats.heads, stats.enlisted);
+	if (GNET_PROPERTY(dbg)) {
+		g_message("loaded %u hostile IP addresses/netmasks (%u hosts)",
+			iprange_get_item_count(hostile_db[which]),
+			iprange_get_host_count(hostile_db[which]));
 	}
-
-	return count;
+	return iprange_get_item_count(hostile_db[which]);
 }
 
 /**
@@ -203,6 +186,8 @@ hostiles_changed(const gchar *filename, gpointer udata)
 
 	gm_snprintf(buf, sizeof(buf), "Reloaded %d hostile IP addresses.", count);
 	gcu_statusbar_message(buf);
+
+	node_kill_hostiles();
 }
 
 static void
@@ -249,7 +234,8 @@ hostiles_retrieve(hostiles_t which)
 
 			file_path_set(&fp_private[0], settings_config_dir(), hostiles_file);
 			f = file_config_open_read_norename_chosen(
-				hostiles_what, fp_private, G_N_ELEMENTS(fp_private), &idx);
+					hostiles_what[HOSTILE_PRIVATE],
+					fp_private, G_N_ELEMENTS(fp_private), &idx);
 
 			if (f) {
 				hostiles_retrieve_from_file(f, HOSTILE_PRIVATE,
@@ -272,7 +258,7 @@ hostiles_retrieve(hostiles_t which)
 
 
 			f = file_config_open_read_norename_chosen(
-				hostiles_what, fp, G_N_ELEMENTS(fp), &idx);
+					hostiles_what[HOSTILE_GLOBAL], fp, G_N_ELEMENTS(fp), &idx);
 			if (f) {
 				hostiles_retrieve_from_file(f,
 				HOSTILE_GLOBAL, fp[idx].dir, fp[idx].name);
@@ -298,7 +284,7 @@ use_global_hostiles_txt_changed(property_t unused_prop)
 {
 	(void) unused_prop;
 
-	if (use_global_hostiles_txt && !hostile_db[HOSTILE_GLOBAL]) {
+	if (GNET_PROPERTY(use_global_hostiles_txt) && !hostile_db[HOSTILE_GLOBAL]) {
 		hostiles_retrieve(HOSTILE_GLOBAL);
 	}
 
@@ -350,12 +336,12 @@ hostiles_check(const host_addr_t ha)
 		ip = host_addr_ipv4(to);
 
 		for (i = 0; i < NUM_HOSTILES; i++) {
-			if (i == HOSTILE_GLOBAL && !use_global_hostiles_txt)
+			if (i == HOSTILE_GLOBAL && !GNET_PROPERTY(use_global_hostiles_txt))
 				continue;
 
 			if (
 				NULL != hostile_db[i] &&
-				THERE == iprange_get(hostile_db[i], ip)
+				NULL != iprange_get(hostile_db[i], ip)
 			)
 				return TRUE;
 		}
