@@ -1,5 +1,5 @@
 /*
- * $Id: hosts.c 11846 2006-08-31 22:00:15Z cbiere $
+ * $Id: hosts.c 13777 2007-05-29 00:23:11Z cbiere $
  *
  * Copyright (c) 2001-2004, Raphael Manfredi
  *
@@ -35,7 +35,7 @@
 
 #include "common.h"
 
-RCSID("$Id: hosts.c 11846 2006-08-31 22:00:15Z cbiere $")
+RCSID("$Id: hosts.c 13777 2007-05-29 00:23:11Z cbiere $")
 
 #include "sockets.h"
 #include "hosts.h"
@@ -45,7 +45,6 @@ RCSID("$Id: hosts.c 11846 2006-08-31 22:00:15Z cbiere $")
 #include "gmsg.h"
 #include "pcache.h"
 #include "whitelist.h"
-#include "gwcache.h"
 #include "settings.h"
 #include "bogons.h"
 #include "uhc.h"
@@ -55,6 +54,8 @@ RCSID("$Id: hosts.c 11846 2006-08-31 22:00:15Z cbiere $")
 #include "lib/endian.h"
 #include "lib/glib-missing.h"
 #include "lib/misc.h"
+#include "lib/walloc.h"
+
 #include "lib/override.h"	/* Must be the last header included */
 
 gboolean host_low_on_pongs = FALSE;			/**< True when less than 12% full */
@@ -72,8 +73,12 @@ guint
 host_hash(gconstpointer key)
 {
 	const gnet_host_t *host = key;
+	host_addr_t addr;
+	guint16 port;
 
-	return host_addr_hash(host->addr) ^ ((host->port << 16) | host->port);
+	addr = gnet_host_get_addr(host);
+	port = gnet_host_get_port(host);
+	return host_addr_hash(addr) ^ ((port << 16) | port);
 }
 
 /**
@@ -86,7 +91,8 @@ host_eq(gconstpointer v1, gconstpointer v2)
 {
 	const gnet_host_t *h1 = v1, *h2 = v2;
 
-	return host_addr_equal(h1->addr, h2->addr) && h1->port == h2->port;
+	return gnet_host_get_port(h1) == gnet_host_get_port(h2) &&
+		host_addr_equal(gnet_host_get_addr(h1), gnet_host_get_addr(h2));
 }
 
 /**
@@ -100,6 +106,151 @@ host_cmp(gconstpointer v1, gconstpointer v2)
 	return host_eq(v1, v2) ? 0 : 1;
 }
 
+void
+gnet_host_vec_free(gnet_host_vec_t **vec_ptr)
+{
+	g_assert(vec_ptr != NULL);
+
+	if (*vec_ptr) {
+		gnet_host_vec_t *vec;
+	
+		vec = *vec_ptr;
+		WFREE_NULL(vec->hvec_v4, vec->n_ipv4 * sizeof vec->hvec_v4[0]);
+		WFREE_NULL(vec->hvec_v6, vec->n_ipv6 * sizeof vec->hvec_v6[0]);
+		wfree(vec, sizeof *vec);
+		*vec_ptr = NULL;
+	}
+}
+
+gnet_host_vec_t *
+gnet_host_vec_alloc(void)
+{
+	static const gnet_host_vec_t zero_vec;
+	return wcopy(&zero_vec, sizeof zero_vec);
+}
+
+gnet_host_vec_t *
+gnet_host_vec_copy(const gnet_host_vec_t *vec)
+{
+	gnet_host_vec_t *vec_copy;
+
+	g_return_val_if_fail(vec, NULL);
+	g_return_val_if_fail(vec->n_ipv4 + vec->n_ipv6 > 0, NULL);
+
+	vec_copy = wcopy(vec, sizeof *vec);
+	if (vec->n_ipv4 > 0) {
+		vec_copy->hvec_v4 = wcopy(vec->hvec_v4,
+								vec->n_ipv4 * sizeof *vec->hvec_v4);
+	}
+	if (vec->n_ipv6 > 0) {
+		vec_copy->hvec_v6 = wcopy(vec->hvec_v6,
+								vec->n_ipv6 * sizeof *vec->hvec_v6);
+	}
+	return vec_copy;
+}
+
+void
+gnet_host_vec_add(gnet_host_vec_t *vec, host_addr_t addr, guint16 port)
+{
+	g_return_if_fail(vec);
+
+	switch (host_addr_net(addr)) {
+	case NET_TYPE_IPV4:
+		if (vec->n_ipv4 < 255) {
+			gchar *dest;
+			size_t size, old_size;
+			
+			old_size = vec->n_ipv4 * sizeof *vec->hvec_v4;
+			vec->n_ipv4++;	
+			size = vec->n_ipv4 * sizeof *vec->hvec_v4;
+			vec->hvec_v4 = vec->hvec_v4
+								? wrealloc(vec->hvec_v4, old_size, size)
+								: walloc(size);
+			dest = cast_to_gpointer(&vec->hvec_v4[vec->n_ipv4 - 1]);
+			poke_be32(&dest[0], host_addr_ipv4(addr));
+			poke_le16(&dest[4], port);
+		}
+		break;
+	case NET_TYPE_IPV6:
+		if (vec->n_ipv6 < 255) {
+			gchar *dest;
+			size_t size, old_size;
+			
+			old_size = vec->n_ipv6 * sizeof *vec->hvec_v6;
+			vec->n_ipv6++;	
+			size = vec->n_ipv6 * sizeof *vec->hvec_v6;
+			vec->hvec_v6 = vec->hvec_v6
+								? wrealloc(vec->hvec_v6, old_size, size)
+								: walloc(size);
+			dest = cast_to_gpointer(&vec->hvec_v6[vec->n_ipv6 - 1]);
+			memcpy(dest, host_addr_ipv6(&addr), 16);
+			poke_le16(&dest[16], port);
+		}
+		break;
+	case NET_TYPE_LOCAL:
+	case NET_TYPE_NONE:
+		break;
+	}
+}
+
+gnet_host_vec_t *
+gnet_host_vec_create(gnet_host_t *hvec, gint hcnt)
+{
+	gnet_host_vec_t *vec;
+	guint n_ipv6 = 0, n_ipv4 = 0;
+	gint i;
+
+	g_assert(hcnt >= 0);
+
+	for (i = 0; i < hcnt; i++) {
+		switch (gnet_host_get_net(&hvec[i])) {
+		case NET_TYPE_IPV4: n_ipv4++; break;
+		case NET_TYPE_IPV6: n_ipv6++; break;
+		case NET_TYPE_LOCAL:
+		case NET_TYPE_NONE:
+			break;
+		}
+	}
+
+	vec = gnet_host_vec_alloc();
+	vec->n_ipv4 = MIN(n_ipv4, 255);
+	vec->n_ipv6 = MIN(n_ipv6, 255);
+
+	if (vec->n_ipv4 > 0) {
+		vec->hvec_v4 = walloc(vec->n_ipv4 * sizeof *vec->hvec_v4);
+	}
+	if (vec->n_ipv6 > 0) {
+		vec->hvec_v6 = walloc(vec->n_ipv6 * sizeof *vec->hvec_v6);
+	}
+
+	n_ipv4 = 0;
+	n_ipv6 = 0;
+
+	for (i = 0; i < hcnt; i++) {
+		host_addr_t addr = gnet_host_get_addr(&hvec[i]);
+		guint16 port = gnet_host_get_port(&hvec[i]);
+		
+		switch (gnet_host_get_net(&hvec[i])) {
+		case NET_TYPE_IPV4:
+			if (n_ipv4 < vec->n_ipv4) {
+				gchar *dest = cast_to_gpointer(&vec->hvec_v4[n_ipv4++]);
+				poke_be32(&dest[0], host_addr_ipv4(addr));
+				poke_le16(&dest[4], port);
+			}
+		case NET_TYPE_IPV6:
+			if (n_ipv6 < vec->n_ipv6) {
+				gchar *dest = cast_to_gpointer(&vec->hvec_v6[n_ipv6++]);
+				memcpy(dest, host_addr_ipv6(&addr), 16);
+				poke_le16(&dest[16], port);
+			}
+			break;
+		case NET_TYPE_LOCAL:
+		case NET_TYPE_NONE:
+			break;
+		}
+	}
+	return vec;
+}
 
 /***
  *** Host periodic timer.
@@ -119,11 +270,11 @@ host_timer(void)
 	host_type_t htype;
 	guint max_nodes;
 
-	if (in_shutdown || !online_mode)
+	if (in_shutdown || !GNET_PROPERTY(online_mode))
 		return;
 
-	max_nodes = (current_peermode == NODE_P_LEAF) ?
-		max_ultrapeers : max_connections;
+	max_nodes = (GNET_PROPERTY(current_peermode) == NODE_P_LEAF) ?
+		GNET_PROPERTY(max_ultrapeers) : GNET_PROPERTY(max_connections);
 	count = node_count();
 	missing = node_keep_missing();
 
@@ -133,7 +284,7 @@ host_timer(void)
 	 * Also, we don't connect each time we are called.
 	 */
 
-	if (!is_inet_connected && missing) {
+	if (!GNET_PROPERTY(is_inet_connected) && missing) {
 		if (0 == (called++ & 0xf))		/* Once every 16 attempts */
 			missing = 1;
 		else
@@ -146,17 +297,18 @@ host_timer(void)
 	 * than quick_connect_pool_size   This is the "greedy mode".
 	 */
 
-	if (count >= quick_connect_pool_size)
-		if (dbg > 10) {
+	if (count >= GNET_PROPERTY(quick_connect_pool_size)) {
+		if (GNET_PROPERTY(dbg) > 10) {
 			g_message("host_timer - count %d >= pool size %d",
-				count, quick_connect_pool_size);
+				count, GNET_PROPERTY(quick_connect_pool_size));
+		}
 		return;
 	}
 
 	if (count < max_nodes)
 		missing -= whitelist_connect();
 
-	if (dbg > 10 && missing > 0)
+	if (GNET_PROPERTY(dbg) > 10 && missing > 0)
 		g_message("host_timer - missing %d host%s",
 			missing, missing == 1 ? "" : "s");
 
@@ -165,13 +317,14 @@ host_timer(void)
 	 * to the connection list
 	 */
 
-	htype = (current_peermode == NODE_P_NORMAL) ?
+	htype = (GNET_PROPERTY(current_peermode) == NODE_P_NORMAL) ?
         HOST_ANY : HOST_ULTRA;
 
 	if (
-        current_peermode == NODE_P_ULTRA &&
-        node_normal_count < normal_connections &&
-        node_ultra_count >= (up_connections - normal_connections)
+        GNET_PROPERTY(current_peermode) == NODE_P_ULTRA &&
+        GNET_PROPERTY(node_normal_count) < GNET_PROPERTY(normal_connections) &&
+        GNET_PROPERTY(node_ultra_count) >=
+			(GNET_PROPERTY(up_connections) - GNET_PROPERTY(normal_connections))
 	) {
 		htype = HOST_ANY;
     }
@@ -179,14 +332,13 @@ host_timer(void)
 	if (hcache_size(htype) == 0)
 		htype = HOST_ANY;
 
-    if (!stop_host_get) {
+    if (!GNET_PROPERTY(stop_host_get)) {
         if (missing > 0) {
-            guint fan;
-            guint max_pool = MAX(quick_connect_pool_size, max_nodes);
-            guint to_add;
+            guint fan, max_pool, to_add;
 
-            fan = (missing * quick_connect_pool_size) / max_nodes;
-            to_add = is_inet_connected ? fan : (guint) missing;
+            max_pool = MAX(GNET_PROPERTY(quick_connect_pool_size), max_nodes);
+            fan = (missing * GNET_PROPERTY(quick_connect_pool_size))/ max_nodes;
+            to_add = GNET_PROPERTY(is_inet_connected) ? fan : (guint) missing;
 
             /*
              * Make sure that we never use more connections then the
@@ -195,35 +347,31 @@ host_timer(void)
             if (to_add + count > max_pool)
                 to_add = max_pool - count;
 
-            if (dbg > 10) {
+            if (GNET_PROPERTY(dbg) > 10) {
                 g_message("host_timer - connecting - add: %d fan:%d  miss:%d "
                      "max_hosts:%d   count:%d   extra:%d",
 					 to_add, fan, missing, max_nodes, count,
-					 quick_connect_pool_size);
+					 GNET_PROPERTY(quick_connect_pool_size));
             }
 
             missing = to_add;
 
 			while (hcache_size(htype) && missing-- > 0) {
-				hcache_get_caught(htype, &addr, &port);
-				node_add(addr, port, 0);
+				if (hcache_get_caught(htype, &addr, &port)) {
+					node_add(addr, port, 0);
+				}
 			}
 
-			if (missing > 0 && hcache_read_finished()) {
-				static gint rotate = 0;
-
-				if (!uhc_is_waiting() && !gwc_is_waiting()) {
-					if (!udp_active() || (0x3 == (rotate++ & 0x3)))
-						gwc_get_hosts(); 	/* Get from web host cache */
-					else
-						uhc_get_hosts();	/* Get from UDP pong caches */
-				} else if (bootstrap_debug > 2)
+			if (missing > 0) {
+				if (!uhc_is_waiting()) {
+					uhc_get_hosts();	/* Get from UDP pong caches */
+				} else if (GNET_PROPERTY(bootstrap_debug) > 2)
 					g_message("BOOT host_timer - waiting for reply from %s",
 						uhc_is_waiting() ? "UDP host cache" : "web cache");
 			}
 		}
 
-	} else if (use_netmasks) {
+	} else if (GNET_PROPERTY(use_netmasks)) {
 		/* Try to find better hosts */
 		if (hcache_find_nearby(htype, &addr, &port)) {
 			if (node_remove_worst(TRUE))
@@ -248,15 +396,14 @@ host_init(void)
  * @return the address:port of a host
  */
 const gchar *
-host_to_string(const gnet_host_t *h)
+gnet_host_to_string(const struct gnutella_host *h)
 {
-	static gchar buf[64];
-	gchar host[48];
+	static gchar buf[HOST_ADDR_PORT_BUFLEN];
+	host_addr_t addr;
+	guint16 port;
 
-	g_assert(h != NULL);
-
-	host_addr_to_string_buf(h->addr, host, sizeof host);
-	gm_snprintf(buf, sizeof buf, "%s:%u", host, h->port);
+	packed_host_unpack(h->data, &addr, &port);
+	host_addr_port_to_string_buf(addr, port, buf, sizeof buf);
 	return buf;
 }
 
@@ -289,7 +436,7 @@ host_is_valid(const host_addr_t addr, guint16 port)
 void
 host_add(const host_addr_t addr, guint16 port, gboolean do_connect)
 {
-	if (!hcache_add_caught(HOST_ANY, addr, port, "pong"))
+	if (!do_connect || !hcache_add_caught(HOST_ANY, addr, port, "pong"))
 		return;
 
 	/*
@@ -305,17 +452,17 @@ host_add(const host_addr_t addr, guint16 port, gboolean do_connect)
 	 */
 
 
-	if (do_connect) {
-		if (node_keep_missing() > 0)
-				node_add(addr, port, 0);
-		else {
-			/* If we are above the max connections, delete a non-nearby
-			 * connection before adding this better one
-			 */
-			if (use_netmasks && host_is_nearby(addr) && node_remove_worst(TRUE))
-				node_add(addr, port, 0);
-		}
-
+	if (node_keep_missing() > 0)
+		node_add(addr, port, 0);
+	else if (
+		GNET_PROPERTY(use_netmasks) &&
+		host_is_nearby(addr) &&
+		node_remove_worst(TRUE)
+	) {
+		/* If we are above the max connections, delete a non-nearby
+		 * connection before adding this better one
+		 */
+		node_add(addr, port, 0);
 	}
 }
 
