@@ -1,5 +1,5 @@
 /*
- * $Id: http.c 12242 2006-11-06 20:41:55Z cbiere $
+ * $Id: http.c 13949 2007-06-24 00:46:05Z cbiere $
  *
  * Copyright (c) 2002-2003, Raphael Manfredi
  *
@@ -38,7 +38,7 @@
 
 #include "common.h"
 
-RCSID("$Id: http.c 12242 2006-11-06 20:41:55Z cbiere $")
+RCSID("$Id: http.c 13949 2007-06-24 00:46:05Z cbiere $")
 
 #include "nodes.h"
 #include "http.h"
@@ -91,10 +91,9 @@ http_send_status(
 {
 	gchar header[2560];			/* 2.5 K max */
 	gchar status_msg[512];
-	gchar xlive[512];
-	gint rw;
-	gint mrw;
-	gint sent;
+	size_t rw, minimal_rw;
+	size_t header_size = sizeof(header);
+	ssize_t sent;
 	gint i;
 	va_list args;
 	const gchar *conn_close = keep_alive ? "" : "Connection: close\r\n";
@@ -103,8 +102,7 @@ http_send_status(
 	const gchar *date;
 	const gchar *token;
 	const gchar *body = NULL;
-	gint header_size = sizeof(header);
-	gboolean saturated = bsched_saturated(bws.out);
+	gboolean saturated = bsched_saturated(BSCHED_BWS_OUT);
 	gint cb_flags = 0;
 
 	va_start(args, reason);
@@ -143,20 +141,17 @@ http_send_status(
 	}
 
 	/*
-	 * If bandwidth is short, drop X-Live-Since, and reduce the header
-	 * size noticeably, so that only the most important stuff gets out.
+	 * If bandwidth is short, reduce the header size noticeably, so that only
+	 * the most important stuff gets out.
 	 *		--RAM, 12/10/2003
 	 */
 
 	if (saturated && code >= 300) {
-		xlive[0] = '\0';
 		version = version_short_string;
 		token = socket_omit_token(s) ? NULL : tok_short_version();
 		header_size = 512;
 		cb_flags |= HTTP_CBF_SMALL_REPLY;
 	} else {
-		gm_snprintf(xlive, sizeof(xlive)-1,
-			"X-Live-Since: %s\r\n", start_rfc822_date);
 		version = version_string;
 		token = socket_omit_token(s) ? NULL : tok_version();
 	}
@@ -174,7 +169,7 @@ http_send_status(
 	if (code < 300 || !keep_alive || body)
 		no_content = "";
 
-	g_assert((size_t) header_size <= sizeof header);
+	g_assert(header_size <= sizeof header);
 
 	date = timestamp_rfc1123_to_string(clock_loc2gmt(tm_time()));
 	rw = gm_snprintf(header, header_size,
@@ -183,85 +178,90 @@ http_send_status(
 		"Date: %s\r\n"
 		"%s"			/* Connection */
 		"%s%s%s"		/* X-Token (optional) */
-		"%s"			/* X-Live-Since */
 		"%s",			/* Content length */
 		code, status_msg, version, date, conn_close,
 		token ? "X-Token: " : "",
 		token ? token : "",
 		token ? "\r\n" : "",
-		xlive, no_content);
+		no_content);
 
-	mrw = rw;		/* Minimal header length */
+	minimal_rw = rw;		/* Minimal header length */
 
 	/*
 	 * Append extra information to the minimal header created above.
 	 */
 
-	/*
-	 * The +3 is there to leave room for "\r\n\0"
-	 *		-- JA, 09/02/2004
-	 */
 	for (i = 0; i < hevcnt && rw + 3 < header_size; i++) {
 		http_extra_desc_t *he = &hev[i];
 		http_extra_type_t type = he->he_type;
+		size_t size;
+
+		g_assert(header_size >= rw);
+	   	size = header_size - rw;
+
+		if (size <= sizeof("\r\n"))
+			break;
+		size -= sizeof("\r\n");
 
 		switch (type) {
 		case HTTP_EXTRA_BODY:
 			/* Already handled above */
 			break;
 		case HTTP_EXTRA_LINE:
-			rw += gm_snprintf(&header[rw], header_size - rw, "%s", he->he_msg);
+			if (size > strlen(he->he_msg)) {
+				/* Don't emit truncated lines */
+				rw += gm_snprintf(&header[rw], size, "%s", he->he_msg);
+			}
 			break;
 		case HTTP_EXTRA_CALLBACK:
 			{
-				gint len = header_size - rw;
-
-				g_assert(header_size > rw);
-				g_assert(len > 0);
-
-				(*he->he_cb)(&header[rw], &len, he->he_arg, cb_flags);
-
-				g_assert(len >= 0);
-				g_assert(len + rw <= header_size);
+				size_t len;
 
+				len = (*he->he_cb)(&header[rw], size, he->he_arg, cb_flags);
+				g_assert(len < size);
 				rw += len;
 			}
 			break;
 		}
 	}
 
-	if (body)
+	if (body) {
 		rw += gm_snprintf(&header[rw], header_size - rw,
 						"Content-Length: %lu\r\n", (gulong) strlen(body));
-
-	if (rw < header_size)
+	}
+	if (rw < header_size) {
 		rw += gm_snprintf(&header[rw], header_size - rw, "\r\n");
-
-	if (body)
+	}
+	if (body) {
 		rw += gm_snprintf(&header[rw], header_size - rw, "%s", body);
-
+	}
 	if (rw >= header_size && hev) {
 		g_warning("HTTP status %d (%s) too big, ignoring extra information",
 			code, status_msg);
 
-		rw = mrw + gm_snprintf(&header[mrw], header_size - mrw, "\r\n");
+		rw = minimal_rw;
+		rw += gm_snprintf(&header[rw], header_size - rw, "\r\n");
 		g_assert(rw < header_size);
 	}
 
-	if (-1 == (sent = bws_write(bws.out, &s->wio, header, rw))) {
+	sent = bws_write(BSCHED_BWS_OUT, &s->wio, header, rw);
+	if ((ssize_t) -1 == sent) {
 		socket_eof(s);
-		if (http_debug > 1)
+		if (GNET_PROPERTY(http_debug) > 1)
 			g_warning("unable to send back HTTP status %d (%s) to %s: %s",
 			code, status_msg, host_addr_to_string(s->addr), g_strerror(errno));
 		return FALSE;
-	} else if (sent < rw) {
-		if (http_debug) g_warning(
-			"only sent %d out of %d bytes of status %d (%s) to %s",
-			sent, rw, code, status_msg, host_addr_to_string(s->addr));
+	} else if ((size_t) sent < rw) {
+		if (GNET_PROPERTY(http_debug)) g_warning(
+			"only sent %lu out of %lu bytes of status %d (%s) to %s",
+			(gulong) sent, (gulong) rw, code, status_msg,
+			host_addr_to_string(s->addr));
 		return FALSE;
-	} else if (http_debug > 2) {
-		g_message("----Sent HTTP Status to %s (%d bytes):\n%.*s----\n",
-			host_addr_to_string(s->addr), rw, rw, header);
+	} else {
+		if (GNET_PROPERTY(http_debug) > 2)
+			g_message("----Sent HTTP Status to %s (%lu bytes):\n%.*s\n----",
+				host_addr_to_string(s->addr), (gulong) rw,
+				(int) MIN(rw, INT_MAX), header);
 	}
 
 	return TRUE;
@@ -272,22 +272,40 @@ http_send_status(
  *
  * Add an X-Hostname line bearing the fully qualified hostname.
  */
-void
-http_hostname_add(gchar *buf, gint *retval, gpointer unused_arg, guint32 flags)
+size_t
+http_hostname_add(gchar *buf, size_t size, gpointer unused_arg, guint32 flags)
 {
-	size_t length = *retval;
-	size_t rw;
+	size_t len;
 
 	(void) unused_arg;
-	g_assert(length <= INT_MAX);
 
 	if (flags & HTTP_CBF_SMALL_REPLY)
-		rw = 0;
-	else
-		rw = gm_snprintf(buf, length, "X-Hostname: %s\r\n", server_hostname);
+		return 0;
+	if (GNET_PROPERTY(server_hostname)[0])
+		return 0;
 
-	if (rw != length - 1)
-		*retval = rw;
+	len = concat_strings(buf, size,
+			"X-Hostname: ", GNET_PROPERTY(server_hostname), "\r\n",
+			(void *) 0);
+	return len < size ? len : 0;
+}
+
+/**
+ * HTTP status callback.
+ *
+ * Add a Retry-After header.
+ */
+size_t
+http_retry_after_add(gchar *buf, size_t size,
+	gpointer arg, guint32 unused_flags)
+{
+	size_t len;
+
+	(void) unused_flags;
+	len = concat_strings(buf, size,
+			"Retry-After: ", uint32_to_string(GPOINTER_TO_UINT(arg)), "\r\n",
+			(void *) 0);
+	return len < size ? len : 0;
 }
 
 /***
@@ -473,8 +491,8 @@ http_extract_version(
 
 	limit = sizeof("X / HTTP/1.0") - 1;
 
-	if (http_debug > 4)
-		printf("HTTP req (%lu bytes): %s\n", (gulong) len, request);
+	if (GNET_PROPERTY(http_debug) > 4)
+		g_message("HTTP req (%lu bytes): %s", (gulong) len, request);
 
 	if (len < limit)
 		return FALSE;
@@ -489,7 +507,7 @@ http_extract_version(
 			break;
 	}
 
-	if (http_debug > 4)
+	if (GNET_PROPERTY(http_debug) > 4)
 		g_message("HTTP i = %lu, limit = %lu", (gulong) i, (gulong) limit);
 
 	if (i == limit)
@@ -506,14 +524,14 @@ http_extract_version(
 		NULL == (p = is_strprefix(p, "HTTP/")) ||
 		0 != parse_major_minor(p, NULL, major, minor)
 	) {
-		if (http_debug > 1)
-			printf("HTTP req (%lu bytes): no protocol tag: %s\n",
+		if (GNET_PROPERTY(http_debug) > 1)
+			g_message("HTTP req (%lu bytes): no protocol tag: %s",
 				(gulong) len, request);
 		return FALSE;
 	}
 
-	if (http_debug > 4)
-		printf("HTTP req OK (%u.%u)\n", *major, *minor);
+	if (GNET_PROPERTY(http_debug) > 4)
+		g_message("HTTP req OK (%u.%u)", *major, *minor);
 
 	/*
 	 * We don't check trailing chars after the HTTP/x.x indication.
@@ -644,7 +662,7 @@ http_url_parse(const gchar *url, guint16 *port, const gchar **host,
 		return FALSE;
 	}
 
-	if (http_debug > 4) {
+	if (GNET_PROPERTY(http_debug) > 4) {
 		g_message("URL \"%s\" -> host=\"%s\", port=%u, path=\"%s\"\n",
 			url, *host, (unsigned) *port, *path);
 	}
@@ -914,6 +932,7 @@ http_range_parse(
 	gint count = 0;
 
 	g_assert(size > 0);
+	vendor = vendor ? vendor : "unknown";
 
 	if (NULL != (str = is_strprefix(str, unit))) {
 		c = *str;
@@ -968,14 +987,14 @@ http_range_parse(
 			}
 
 			if (!minus_seen) {
-				if (http_debug) g_warning(
+				if (GNET_PROPERTY(http_debug)) g_warning(
 					"weird %s header from <%s>, offset %d (no range?): "
 					"%s", field, vendor, (gint) (str - value) - 1, value);
 				goto reset;
 			}
 
 			if (start == HTTP_OFFSET_MAX && !has_end) {	/* Bad negative range */
-				if (http_debug) g_warning(
+				if (GNET_PROPERTY(http_debug)) g_warning(
 					"weird %s header from <%s>, offset %d "
 					"(incomplete negative range): %s",
 					field, vendor, (gint) (str - value) - 1, value);
@@ -983,7 +1002,7 @@ http_range_parse(
 			}
 
 			if (start > end) {
-				if (http_debug) g_warning(
+				if (GNET_PROPERTY(http_debug)) g_warning(
 					"weird %s header from <%s>, offset %d "
 					"(swapped range?): %s", field, vendor,
 					(gint) (str - value) - 1, value);
@@ -995,7 +1014,7 @@ http_range_parse(
 			count++;
 
 			if (ignored) {
-				if (http_debug) g_warning(
+				if (GNET_PROPERTY(http_debug)) g_warning(
 					"weird %s header from <%s>, offset %d "
 					"(ignored range #%d): %s",
 					field, vendor, (gint) (str - value) - 1, count,
@@ -1010,7 +1029,7 @@ http_range_parse(
 
 		if (c == '-') {
 			if (minus_seen) {
-				if (http_debug) g_warning(
+				if (GNET_PROPERTY(http_debug)) g_warning(
 					"weird %s header from <%s>, offset %d (spurious '-'): %s",
 					field, vendor, (gint) (str - value) - 1, value);
 				goto resync;
@@ -1018,7 +1037,7 @@ http_range_parse(
 			minus_seen = TRUE;
 			if (!has_start) {		/* Negative range */
 				if (!request) {
-					if (http_debug)
+					if (GNET_PROPERTY(http_debug))
 						g_warning("weird %s header from <%s>, offset %d "
 							"(negative range in reply): %s",
 							field, vendor, (gint) (str - value) - 1, value);
@@ -1041,7 +1060,7 @@ http_range_parse(
 			str = dend;		/* Skip number */
 
 			if (has_end) {
-				if (http_debug)
+				if (GNET_PROPERTY(http_debug))
 					g_warning("weird %s header from <%s>, offset %d "
 						"(spurious boundary %s): %s",
 						field, vendor, (gint) (str - value) - 1,
@@ -1059,7 +1078,7 @@ http_range_parse(
 
 			if (has_start) {
 				if (!minus_seen) {
-					if (http_debug)
+					if (GNET_PROPERTY(http_debug))
 						g_warning("weird %s header from <%s>, offset %d "
 							"(no '-' before boundary %s): %s",
 							field, vendor, (gint) (str - value) - 1,
@@ -1080,7 +1099,8 @@ http_range_parse(
 			continue;
 		}
 
-		if (http_debug) g_warning("weird %s header from <%s>, offset %d "
+		if (GNET_PROPERTY(http_debug))
+			g_warning("weird %s header from <%s>, offset %d "
 			"(unexpected char '%c'): %s",
 			field, vendor, (gint) (str - value) - 1, c, value);
 
@@ -1102,14 +1122,16 @@ http_range_parse(
 
 	if (minus_seen) {
 		if (start == HTTP_OFFSET_MAX && !has_end) {	/* Bad negative range */
-			if (http_debug) g_warning("weird %s header from <%s>, offset %d "
+			if (GNET_PROPERTY(http_debug))
+				g_warning("weird %s header from <%s>, offset %d "
 				"(incomplete trailing negative range): %s",
 				field, vendor, (gint) (str - value) - 1, value);
 			goto final;
 		}
 
 		if (start > end) {
-			if (http_debug) g_warning("weird %s header from <%s>, offset %d "
+			if (GNET_PROPERTY(http_debug))
+				g_warning("weird %s header from <%s>, offset %d "
 				"(swapped trailing range?): %s", field, vendor,
 				(gint) (str - value) - 1, value);
 			goto final;
@@ -1119,7 +1141,8 @@ http_range_parse(
 		count++;
 
 		if (ignored)
-			if (http_debug) g_warning("weird %s header from <%s>, offset %d "
+			if (GNET_PROPERTY(http_debug))
+				g_warning("weird %s header from <%s>, offset %d "
 				"(ignored final range #%d): %s",
 				field, vendor, (gint) (str - value) - 1, count,
 				value);
@@ -1127,20 +1150,21 @@ http_range_parse(
 
 final:
 
-	if (http_debug > 4) {
+	if (GNET_PROPERTY(http_debug) > 4) {
 		GSList *l;
-		printf("Saw %d ranges in %s %s: %s\n",
+
+		g_message("Saw %d ranges in %s %s: %s",
 			count, request ? "request" : "reply", field, value);
 		if (ranges)
-			printf("...retained:\n");
+			g_message("...retained:");
 		for (l = ranges; l; l = g_slist_next(l)) {
 			http_range_t *r = (http_range_t *) l->data;
-			printf("...  %s-%s\n",
+			g_message("...  %s-%s",
 				uint64_to_string(r->start), uint64_to_string2(r->end));
 		}
 	}
 
-	if (ranges == NULL && http_debug)
+	if (ranges == NULL && GNET_PROPERTY(http_debug))
 		g_warning("retained no ranges in %s header from <%s>: %s",
 			field, vendor, value);
 
@@ -1258,15 +1282,11 @@ http_range_clone(http_range_t *range)
 GSList *
 http_range_merge(GSList *old_list, GSList *new_list)
 {
-	http_range_t *old_range;
-	http_range_t *new_range;
-	http_range_t *r;
-	GSList *new = new_list;
-	GSList *old = old_list;
+	http_range_t *old_range, *new_range, *r;
+	GSList *new = new_list, *old = old_list;
 	GSList *result_list = NULL;
 	filesize_t highest = 0;
 
-
 	/*
 	 * Build a result list based on the data in the old and new
 	 * lists.
@@ -1274,8 +1294,8 @@ http_range_merge(GSList *old_list, GSList *new_list)
 
 	while (old || new) {
 		if (old && new) {
-			old_range = (http_range_t *) old->data;
-			new_range = (http_range_t *) new->data;
+			old_range = old->data;
+			new_range = new->data;
 
 			/*
 			 * If ranges are identical just copy one.
@@ -1284,8 +1304,8 @@ http_range_merge(GSList *old_list, GSList *new_list)
 			if (new_range->start == old_range->start
 				&& new_range->end == old_range->end) {
 				highest = old_range->end;
-				result_list =
-					g_slist_append(result_list, http_range_clone(old_range));
+				result_list = g_slist_prepend(result_list,
+								http_range_clone(old_range));
 				old = g_slist_next(old);
 				new = g_slist_next(new);
 				continue;
@@ -1313,15 +1333,15 @@ http_range_merge(GSList *old_list, GSList *new_list)
 
 			if (new_range->end < old_range->start) {
 				highest = new_range->end;
-				result_list =
-					g_slist_append(result_list, http_range_clone(new_range));
+				result_list = g_slist_prepend(result_list,
+									http_range_clone(new_range));
 				new = g_slist_next(new);
 				continue;
 			}
 			if (old_range->end < new_range->start) {
 				highest = new_range->end;
-				result_list =
-					g_slist_append(result_list, http_range_clone(old_range));
+				result_list = g_slist_prepend(result_list,
+									http_range_clone(old_range));
 
 				old = g_slist_next(old);
 				continue;
@@ -1343,7 +1363,7 @@ http_range_merge(GSList *old_list, GSList *new_list)
 				else
 					r->end = old_range->end;
 				highest = r->end;
-				result_list = g_slist_append(result_list, r);
+				result_list = g_slist_prepend(result_list, r);
 				old = g_slist_next(old);
 				new = g_slist_next(new);
 				continue;
@@ -1356,7 +1376,7 @@ http_range_merge(GSList *old_list, GSList *new_list)
 				else
 					r->end = old_range->end;
 				highest = r->end;
-				result_list = g_slist_append(result_list, r);
+				result_list = g_slist_prepend(result_list, r);
 				old = g_slist_next(old);
 				new = g_slist_next(new);
 				continue;
@@ -1370,23 +1390,23 @@ http_range_merge(GSList *old_list, GSList *new_list)
 			 */
 
 			if (old) {
-				old_range = (http_range_t *) old->data;
+				old_range = old->data;
 				if (old_range->end > highest)
-					result_list = g_slist_append(result_list,
-								  http_range_clone(old_range));
+					result_list = g_slist_prepend(result_list,
+									http_range_clone(old_range));
 				old = g_slist_next(old);
 			}
 			if (new) {
-				new_range = (http_range_t *) new->data;
+				new_range = new->data;
 				if (new_range->end > highest)
-					result_list = g_slist_append(result_list,
-								  http_range_clone(new_range));
+					result_list = g_slist_prepend(result_list,
+								  	http_range_clone(new_range));
 				new = g_slist_next(new);
 			}
 		}
 	}
 
-	return result_list;
+	return g_slist_reverse(result_list);
 }
 
 
@@ -1456,9 +1476,9 @@ struct http_async {
 	enum http_reqtype type;			/**< Type of request */
 	http_state_t state;				/**< Current request state */
 	guint32 flags;					/**< Operational flags */
-	gchar *url;						/**< Initial URL request (atom) */
-	gchar *path;					/**< Path to request (atom) */
-	gchar *host;					/**< Hostname, if not a numeric IP (atom) */
+	const gchar *url;				/**< Initial URL request (atom) */
+	const gchar *path;				/**< Path to request (atom) */
+	const gchar *host;				/**< Hostname, if not a numeric IP (atom) */
 	struct gnutella_socket *socket;	/**< Attached socket */
 	http_header_cb_t header_ind;	/**< Callback for headers */
 	http_data_cb_t data_ind;		/**< Callback for data */
@@ -1509,7 +1529,7 @@ static GSList *sl_ha_freed = NULL;		/* Pending physical removal */
  */
 const gchar *
 http_async_info(
-	gpointer handle, const gchar **req, gchar **path,
+	struct http_async *handle, const gchar **req, const gchar **path,
 	host_addr_t *addr, guint16 *port)
 {
 	struct http_async *ha = handle;
@@ -1529,9 +1549,10 @@ http_async_info(
  * non-NULL function pointer is given.
  */
 void
-http_async_set_opaque(gpointer handle, gpointer data, http_user_free_t fn)
+http_async_set_opaque(struct http_async *handle, gpointer data,
+	http_user_free_t fn)
 {
-	struct http_async *ha = (struct http_async *) handle;
+	struct http_async *ha = handle;
 
 	g_assert(ha->magic == HTTP_ASYNC_MAGIC);
 	g_assert(data != NULL);
@@ -1544,10 +1565,8 @@ http_async_set_opaque(gpointer handle, gpointer data, http_user_free_t fn)
  * Retrieve user-defined opaque data.
  */
 gpointer
-http_async_get_opaque(gpointer handle)
+http_async_get_opaque(struct http_async *ha)
 {
-	struct http_async *ha = (struct http_async *) handle;
-
 	g_assert(ha->magic == HTTP_ASYNC_MAGIC);
 
 	return ha->user_opaque;
@@ -1566,21 +1585,28 @@ http_async_free_recursive(struct http_async *ha)
 
 	g_assert(sl_outgoing);
 
-	atom_str_free(ha->url);
-	atom_str_free(ha->path);
+	atom_str_free_null(&ha->url);
+	atom_str_free_null(&ha->path);
+	atom_str_free_null(&ha->host);
 
-	if (ha->host)
-		atom_str_free(ha->host);
-	if (ha->io_opaque)
+	if (ha->io_opaque) {
 		io_free(ha->io_opaque);
-	if (ha->bio)
+		ha->io_opaque = NULL;
+	}
+	if (ha->bio) {
 		bsched_source_remove(ha->bio);
+		ha->bio = NULL;
+	}
 	socket_free_null(&ha->socket);
-	if (ha->user_free)
+	if (ha->user_free) {
 		(*ha->user_free)(ha->user_opaque);
-	if (ha->delayed)
+		ha->user_free = NULL;
+		ha->user_opaque = NULL;
+	}
+	if (ha->delayed) {
 		http_buffer_free(ha->delayed);
-
+		ha->delayed = NULL;
+	}
 	sl_outgoing = g_slist_remove(sl_outgoing, ha);
 
 	/*
@@ -1588,7 +1614,7 @@ http_async_free_recursive(struct http_async *ha)
 	 */
 
 	for (l = ha->children; l; l = l->next) {
-		struct http_async *cha = (struct http_async *) l->data;
+		struct http_async *cha = l->data;
 		http_async_free_recursive(cha);
 	}
 
@@ -1634,7 +1660,7 @@ http_async_free_pending(void)
 	GSList *l;
 
 	for (l = sl_ha_freed; l; l = l->next) {
-		struct http_async *ha = (struct http_async *) l->data;
+		struct http_async *ha = l->data;
 
 		g_assert(ha->flags & HA_F_FREED);
 		wfree(ha, sizeof(*ha));
@@ -1648,10 +1674,8 @@ http_async_free_pending(void)
  * Close request.
  */
 void
-http_async_close(gpointer handle)
+http_async_close(struct http_async *ha)
 {
-	struct http_async *ha = (struct http_async *) handle;
-
 	g_assert(ha->magic == HTTP_ASYNC_MAGIC);
 
 	http_async_free(ha);
@@ -1661,13 +1685,11 @@ http_async_close(gpointer handle)
  * Cancel request (internal call).
  */
 static void
-http_async_remove(gpointer handle, http_errtype_t type, gpointer code)
+http_async_remove(struct http_async *ha, http_errtype_t type, gpointer code)
 {
-	struct http_async *ha = (struct http_async *) handle;
-
 	g_assert(ha->magic == HTTP_ASYNC_MAGIC);
 
-	(*ha->error_ind)(handle, type, code);
+	(*ha->error_ind)(ha, type, code);
 	http_async_free(ha);
 }
 
@@ -1675,16 +1697,17 @@ http_async_remove(gpointer handle, http_errtype_t type, gpointer code)
  * Cancel request (user request).
  */
 void
-http_async_cancel(gpointer handle)
+http_async_cancel(struct http_async *handle)
 {
-	http_async_remove(handle, HTTP_ASYNC_ERROR, (gpointer) HTTP_ASYNC_CANCELLED);
+	http_async_remove(handle, HTTP_ASYNC_ERROR,
+		GINT_TO_POINTER(HTTP_ASYNC_CANCELLED));
 }
 
 /**
  * Cancel request (internal error).
  */
 void
-http_async_error(gpointer handle, gint code)
+http_async_error(struct http_async *handle, gint code)
 {
 	http_async_remove(handle, HTTP_ASYNC_ERROR, GINT_TO_POINTER(code));
 }
@@ -1693,7 +1716,7 @@ http_async_error(gpointer handle, gint code)
  * Cancel request (system call error).
  */
 static void
-http_async_syserr(gpointer handle, gint code)
+http_async_syserr(struct http_async *handle, gint code)
 {
 	http_async_remove(handle, HTTP_ASYNC_SYSERR, GINT_TO_POINTER(code));
 }
@@ -1702,7 +1725,7 @@ http_async_syserr(gpointer handle, gint code)
  * Cancel request (header parsing error).
  */
 static void
-http_async_headerr(gpointer handle, gint code)
+http_async_headerr(struct http_async *handle, gint code)
 {
 	http_async_remove(handle, HTTP_ASYNC_HEADER, GINT_TO_POINTER(code));
 }
@@ -1711,8 +1734,8 @@ http_async_headerr(gpointer handle, gint code)
  * Cancel request (HTTP error).
  */
 static void
-http_async_http_error(
-	gpointer handle, struct header *header, gint code, const gchar *message)
+http_async_http_error(struct http_async *handle, struct header *header,
+	gint code, const gchar *message)
 {
 	http_error_t he;
 
@@ -1736,7 +1759,8 @@ http_async_http_error(
  * header.
  */
 static size_t
-http_async_build_request(gpointer unused_handle, gchar *buf, size_t len,
+http_async_build_request(struct http_async *unused_handle,
+	gchar *buf, size_t len,
 	const gchar *verb, const gchar *path, const gchar *host, guint16 port)
 {
 	size_t rw;
@@ -1783,7 +1807,7 @@ http_async_build_request(gpointer unused_handle, gchar *buf, size_t len,
  */
 static struct http_async *
 http_async_create(
-	gchar *url,						/* Either full URL or path */
+	const gchar *url,				/* Either full URL or path */
 	const host_addr_t addr,			/* Optional: 0 means grab from url */
 	guint16 port,					/* Optional, must be given when IP given */
 	enum http_reqtype type,
@@ -1900,9 +1924,9 @@ http_async_newstate(struct http_async *ha, http_state_t state)
  * On error, `error_ind' will be called, and upon return, the request will
  * be automatically cancelled.
  */
-gpointer
+struct http_async *
 http_async_get(
-	gchar *url,
+	const gchar *url,
 	http_header_cb_t header_ind,
 	http_data_cb_t data_ind,
 	http_error_cb_t error_ind)
@@ -1915,9 +1939,9 @@ http_async_get(
  * Same as http_async_get(), but a path on the server is given and the
  * IP and port to contact are given explicitly.
  */
-gpointer
+struct http_async *
 http_async_get_addr(
-	gchar *path,
+	const gchar *path,
 	const host_addr_t addr,
 	guint16 port,
 	http_header_cb_t header_ind,
@@ -1933,10 +1957,8 @@ http_async_get_addr(
  * Redefines the building of the HTTP request.
  */
 void
-http_async_set_op_request(gpointer handle, http_op_request_t op)
+http_async_set_op_request(struct http_async *ha, http_op_request_t op)
 {
-	struct http_async *ha = (struct http_async *) handle;
-
 	g_assert(ha->magic == HTTP_ASYNC_MAGIC);
 	g_assert(op != NULL);
 
@@ -1947,10 +1969,8 @@ http_async_set_op_request(gpointer handle, http_op_request_t op)
  * Defines callback to invoke when the request changes states.
  */
 void
-http_async_on_state_change(gpointer handle, http_state_change_t fn)
+http_async_on_state_change(struct http_async *ha, http_state_change_t fn)
 {
-	struct http_async *ha = (struct http_async *) handle;
-
 	g_assert(ha->magic == HTTP_ASYNC_MAGIC);
 	g_assert(fn != NULL);
 
@@ -1961,10 +1981,8 @@ http_async_on_state_change(gpointer handle, http_state_change_t fn)
  * Whether we should follow HTTP redirections (FALSE by default).
  */
 void
-http_async_allow_redirects(gpointer handle, gboolean allow)
+http_async_allow_redirects(struct http_async *ha, gboolean allow)
 {
-	struct http_async *ha = (struct http_async *) handle;
-
 	g_assert(ha->magic == HTTP_ASYNC_MAGIC);
 
 	ha->allow_redirects = allow;
@@ -1975,11 +1993,9 @@ http_async_allow_redirects(gpointer handle, gboolean allow)
  * Reroute to parent request.
  */
 static gboolean
-http_subreq_header_ind(
-	gpointer handle, struct header *header, gint code, const gchar *message)
+http_subreq_header_ind(struct http_async *ha, struct header *header,
+	gint code, const gchar *message)
 {
-	struct http_async *ha = (struct http_async *) handle;
-
 	g_assert(ha->magic == HTTP_ASYNC_MAGIC);
 	g_assert(ha->parent != NULL);
 	g_assert(ha->parent->header_ind);
@@ -1992,11 +2008,8 @@ http_subreq_header_ind(
  * Reroute to parent request.
  */
 static void
-http_subreq_data_ind(
-	gpointer handle, gchar *data, gint len)
+http_subreq_data_ind(struct http_async *ha, gchar *data, gint len)
 {
-	struct http_async *ha = (struct http_async *) handle;
-
 	g_assert(ha->magic == HTTP_ASYNC_MAGIC);
 	g_assert(ha->parent != NULL);
 	g_assert(ha->parent->data_ind);
@@ -2009,11 +2022,8 @@ http_subreq_data_ind(
  * Reroute to parent request.
  */
 static void
-http_subreq_error_ind(
-	gpointer handle, http_errtype_t error, gpointer val)
+http_subreq_error_ind(struct http_async *ha, http_errtype_t error, gpointer val)
 {
-	struct http_async *ha = (struct http_async *) handle;
-
 	g_assert(ha->magic == HTTP_ASYNC_MAGIC);
 	g_assert(ha->parent != NULL);
 	g_assert(ha->parent->error_ind);
@@ -2152,7 +2162,7 @@ http_got_data(struct http_async *ha, gboolean eof)
 static void
 http_data_read(gpointer data, gint unused_source, inputevt_cond_t cond)
 {
-	struct http_async *ha = (struct http_async *) data;
+	struct http_async *ha = data;
 	struct gnutella_socket *s = ha->socket;
 	ssize_t r;
 
@@ -2203,12 +2213,12 @@ http_got_header(struct http_async *ha, header_t *header)
 	gchar *buf;
 	guint http_major = 0, http_minor = 0;
 
-	if (http_debug > 2) {
-		printf("----Got HTTP reply from %s:\n", host_addr_to_string(s->addr));
-		printf("%s\n", status);
-		header_dump(header, stdout);
-		printf("----\n");
-		fflush(stdout);
+	if (GNET_PROPERTY(http_debug) > 2) {
+		g_message("----Got HTTP reply from %s:\n",
+			host_addr_to_string(s->addr));
+		g_message("%s", status);
+		header_dump(header, stderr);
+		g_message("----");
 	}
 
 	/*
@@ -2265,8 +2275,8 @@ http_got_header(struct http_async *ha, header_t *header)
 			ack_code != 302 ||
 			(ack_code == 302 && (ha->type == HTTP_GET || ha->type == HTTP_HEAD))
 		) {
-			if (http_debug > 2)
-				printf("HTTP %s redirect %d (%s): \"%s\" -> \"%s\"\n",
+			if (GNET_PROPERTY(http_debug) > 2)
+				g_message("HTTP %s redirect %d (%s): \"%s\" -> \"%s\"",
 					http_verb[ha->type], ack_code, ack_message, ha->url, buf);
 
 			/*
@@ -2304,8 +2314,8 @@ http_got_header(struct http_async *ha, header_t *header)
 	g_assert(s->gdk_tag == 0);
 	g_assert(ha->bio == NULL);
 
-	ha->bio = bsched_source_add(bws.in, &s->wio,
-		BIO_F_READ, http_data_read, (gpointer) ha);
+	ha->bio = bsched_source_add(BSCHED_BWS_IN, &s->wio,
+		BIO_F_READ, http_data_read, ha);
 
 	/*
 	 * We may have something left in the input buffer.
@@ -2322,10 +2332,8 @@ http_got_header(struct http_async *ha, header_t *header)
  * Get the state of the HTTP request.
  */
 http_state_t
-http_async_state(gpointer handle)
+http_async_state(struct http_async *ha)
 {
-	struct http_async *ha = (struct http_async *) handle;
-
 	g_assert(ha->magic == HTTP_ASYNC_MAGIC);
 
 	/*
@@ -2339,7 +2347,7 @@ http_async_state(gpointer handle)
 		g_assert(ha->children);
 
 		for (l = ha->children; l; l = g_slist_next(l)) {
-			struct http_async *cha = (struct http_async *) l->data;
+			struct http_async *cha = l->data;
 
 			switch (cha->state) {
 			case HTTP_AS_REDIRECTED:
@@ -2366,7 +2374,7 @@ http_async_state(gpointer handle)
 static void
 call_http_got_header(gpointer obj, header_t *header)
 {
-	struct http_async *ha = (struct http_async *) obj;
+	struct http_async *ha = obj;
 
 	g_assert(ha->magic == HTTP_ASYNC_MAGIC);
 
@@ -2379,9 +2387,9 @@ static struct io_error http_io_error;
  * Called when we start receiving the HTTP headers.
  */
 static void
-http_header_start(gpointer handle)
+http_header_start(gpointer obj)
 {
-	struct http_async *ha = (struct http_async *) handle;
+	struct http_async *ha = obj;
 
 	g_assert(ha->magic == HTTP_ASYNC_MAGIC);
 
@@ -2402,8 +2410,8 @@ http_async_request_sent(struct http_async *ha)
 
 	g_assert(ha->io_opaque == NULL);
 
-	io_get_header(ha, &ha->io_opaque, bws.in, ha->socket, IO_SAVE_FIRST,
-		call_http_got_header, http_header_start, &http_io_error);
+	io_get_header(ha, &ha->io_opaque, BSCHED_BWS_IN, ha->socket,
+		IO_SAVE_FIRST, call_http_got_header, http_header_start, &http_io_error);
 }
 
 /**
@@ -2414,7 +2422,7 @@ static void
 http_async_write_request(gpointer data, gint unused_source,
 	inputevt_cond_t cond)
 {
-	struct http_async *ha = (struct http_async *) data;
+	struct http_async *ha = data;
 	struct gnutella_socket *s = ha->socket;
 	http_buffer_t *r = ha->delayed;
 	ssize_t sent;
@@ -2435,7 +2443,7 @@ http_async_write_request(gpointer data, gint unused_source,
 	rw = http_buffer_unread(r);			/* Data we still have to send */
 	base = http_buffer_read_base(r);	/* And where unsent data start */
 
-	sent = bws_write(bws.out, &s->wio, base, rw);
+	sent = bws_write(BSCHED_BWS_OUT, &s->wio, base, rw);
 	if ((ssize_t) -1 == sent) {
 		g_warning("HTTP request sending to %s failed: %s",
 			host_addr_port_to_string(s->addr, s->port), g_strerror(errno));
@@ -2444,18 +2452,18 @@ http_async_write_request(gpointer data, gint unused_source,
 	} else if ((size_t) sent < rw) {
 		http_buffer_add_read(r, sent);
 		return;
-	} else if (http_debug > 2) {
-		printf("----Sent HTTP request completely to %s (%d bytes):\n%.*s----\n",
+	} else if (GNET_PROPERTY(http_debug) > 2) {
+		g_message("----"
+			"Sent HTTP request completely to %s (%d bytes):\n%.*s\n----\n",
 			host_addr_port_to_string(s->addr, s->port), http_buffer_length(r),
 			http_buffer_length(r), http_buffer_base(r));
-		fflush(stdout);
 	}
 
 	/*
 	 * HTTP request was completely sent.
 	 */
 
-	if (http_debug)
+	if (GNET_PROPERTY(http_debug))
 		g_warning("flushed partially written HTTP request to %s (%d bytes)",
 			host_addr_port_to_string(s->addr, s->port),
 			http_buffer_length(r));
@@ -2473,17 +2481,18 @@ http_async_write_request(gpointer data, gint unused_source,
  * server is made.
  */
 void
-http_async_connected(gpointer handle)
+http_async_connected(struct http_async *ha)
 {
-	struct http_async *ha = (struct http_async *) handle;
-	struct gnutella_socket *s = ha->socket;
+	struct gnutella_socket *s;
 	size_t rw;
 	ssize_t sent;
 	gchar req[2048];
 
 	g_assert(ha->magic == HTTP_ASYNC_MAGIC);
-	g_assert(s);
-	g_assert(s->resource.handle == handle);
+
+	s = ha->socket;
+	socket_check(s);
+	g_assert(s->resource.handle == ha);
 
 	/*
 	 * Build the HTTP request.
@@ -2504,7 +2513,7 @@ http_async_connected(gpointer handle)
 
 	http_async_newstate(ha, HTTP_AS_REQ_SENDING);
 
-	sent = bws_write(bws.out, &s->wio, req, rw);
+	sent = bws_write(BSCHED_BWS_OUT, &s->wio, req, rw);
 	if ((ssize_t) -1 == sent) {
 		g_warning("HTTP request sending to %s failed: %s",
 			host_addr_port_to_string(s->addr, s->port), g_strerror(errno));
@@ -2527,8 +2536,8 @@ http_async_connected(gpointer handle)
 		socket_evt_set(s, INPUT_EVENT_WX, http_async_write_request, ha);
 
 		return;
-	} else if (http_debug > 2) {
-		g_message("----Sent HTTP request to %s (%d bytes):\n%.*s----",
+	} else if (GNET_PROPERTY(http_debug) > 2) {
+		g_message("----Sent HTTP request to %s (%d bytes):\n%.*s\n----",
 			host_addr_port_to_string(s->addr, s->port), (int) rw, (int) rw, req);
 	}
 
@@ -2541,13 +2550,13 @@ http_async_connected(gpointer handle)
  * debugging level is explicitly given.
  */
 void
-http_async_log_error_dbg(
-	gpointer handle, http_errtype_t type, gpointer v, guint32 dbg_level)
+http_async_log_error_dbg(struct http_async *handle,
+	http_errtype_t type, gpointer v, guint32 dbg_level)
 {
 	const gchar *url;
 	const gchar *req;
 	gint error = GPOINTER_TO_INT(v);
-	http_error_t *herror = (http_error_t *) v;
+	http_error_t *herror = v;
 	host_addr_t addr;
 	guint16 port;
 
@@ -2604,9 +2613,9 @@ http_async_log_error_dbg(
  * initial HTTP request and the reported error cause.
  */
 void
-http_async_log_error(gpointer handle, http_errtype_t type, gpointer v)
+http_async_log_error(struct http_async *handle, http_errtype_t type, gpointer v)
 {
-	return http_async_log_error_dbg(handle, type, v, http_debug);
+	return http_async_log_error_dbg(handle, type, v, GNET_PROPERTY(http_debug));
 }
 
 /***
@@ -2616,37 +2625,49 @@ http_async_log_error(gpointer handle, http_errtype_t type, gpointer v)
 static void
 err_line_too_long(gpointer obj)
 {
-	http_async_error(obj, HTTP_ASYNC_HEAD2BIG);
+	struct http_async *ha = obj;
+	g_assert(ha->magic == HTTP_ASYNC_MAGIC);
+	http_async_error(ha, HTTP_ASYNC_HEAD2BIG);
 }
 
 static void
 err_header_error(gpointer obj, gint error)
 {
-	http_async_headerr(obj, error);
+	struct http_async *ha = obj;
+	g_assert(ha->magic == HTTP_ASYNC_MAGIC);
+	http_async_headerr(ha, error);
 }
 
 static void
 err_input_exception(gpointer obj)
 {
-	http_async_error(obj, HTTP_ASYNC_IO_ERROR);
+	struct http_async *ha = obj;
+	g_assert(ha->magic == HTTP_ASYNC_MAGIC);
+	http_async_error(ha, HTTP_ASYNC_IO_ERROR);
 }
 
 static void
 err_input_buffer_full(gpointer obj)
 {
-	http_async_error(obj, HTTP_ASYNC_IO_ERROR);
+	struct http_async *ha = obj;
+	g_assert(ha->magic == HTTP_ASYNC_MAGIC);
+	http_async_error(ha, HTTP_ASYNC_IO_ERROR);
 }
 
 static void
 err_header_read_error(gpointer obj, gint error)
 {
-	http_async_syserr(obj, error);
+	struct http_async *ha = obj;
+	g_assert(ha->magic == HTTP_ASYNC_MAGIC);
+	http_async_syserr(ha, error);
 }
 
 static void
 err_header_read_eof(gpointer obj)
 {
-	http_async_error(obj, HTTP_ASYNC_EOF);
+	struct http_async *ha = obj;
+	g_assert(ha->magic == HTTP_ASYNC_MAGIC);
+	http_async_error(ha, HTTP_ASYNC_EOF);
 }
 
 static struct io_error http_io_error = {
@@ -2670,11 +2691,11 @@ http_timer(time_t now)
 
 retry:
 	for (l = sl_outgoing; l; l = l->next) {
-		struct http_async *ha = (struct http_async *) l->data;
+		struct http_async *ha = l->data;
 		gint elapsed = delta_time(now, ha->last_update);
-		gint timeout = ha->bio ?
-			download_connected_timeout :
-			download_connecting_timeout;
+		gint timeout = ha->bio
+			? GNET_PROPERTY(download_connected_timeout)
+			: GNET_PROPERTY(download_connecting_timeout);
 
 		if (ha->flags & HA_F_SUBREQ)
 			continue;
@@ -2710,8 +2731,7 @@ void
 http_close(void)
 {
 	while (sl_outgoing)
-		http_async_error(
-			(struct http_async *) sl_outgoing->data, HTTP_ASYNC_CANCELLED);
+		http_async_error(sl_outgoing->data, HTTP_ASYNC_CANCELLED);
 }
 
 /* vi: set ts=4 sw=4 cindent: */
