@@ -1,5 +1,5 @@
 /*
- * $Id: huge.h 10829 2006-05-09 05:33:23Z cbiere $
+ * $Id: huge.h 13311 2007-04-14 08:27:24Z cbiere $
  *
  * Copyright (c) 2002-2003, Ch. Tronche & Raphael Manfredi
  *
@@ -44,6 +44,7 @@
 
 struct shared_file;
 struct header;
+struct sha1;
 
 void huge_init(void);		/**< Call this function at the beginning */
 void huge_close(void);		/**< Call this when servent is shutdown */
@@ -54,13 +55,17 @@ void huge_close(void);		/**< Call this when servent is shutdown */
  * the computed value is saved in the cache.
  */
 
+struct tth;
+
 gboolean request_sha1(struct shared_file *);
 gboolean sha1_is_cached(const struct shared_file *sf);
+gboolean huge_update_hashes(struct shared_file *sf,
+	const struct sha1 *sha1, const struct tth *tth);
 
 gboolean huge_improbable_sha1(const gchar *buf, size_t len);
-gboolean huge_sha1_extract32(const gchar *buf, size_t len, gchar *retval,
-	gpointer header, gboolean check_old);
-void huge_collect_locations(const gchar *sha1, struct header *header);
+gboolean huge_sha1_extract32(const gchar *buf, size_t len, struct sha1 *sha1,
+	gconstpointer header);
+void huge_collect_locations(const struct sha1 *sha1, struct header *header);
 
 #endif	/* _core_huge_h_ */
 
