@@ -1,5 +1,5 @@
 /*
- * $Id: ignore.h 11259 2006-07-04 19:00:32Z cbiere $
+ * $Id: ignore.h 13285 2007-04-13 01:03:20Z cbiere $
  *
  * Copyright (c) 2002-2003, Raphael Manfredi
  *
@@ -36,7 +36,7 @@
 #ifndef _core_ignore_h_
 #define _core_ignore_h_
 
-#include <glib.h>
+#include "common.h"
 
 void ignore_init(void);
 void ignore_close(void);
@@ -51,14 +51,16 @@ enum ignore_val {
 	IGNORE_SPAM				/**< Ignore because SHA1 is known spam */
 };
 
+struct sha1;
+
 void ignore_timer(time_t now);
 enum ignore_val ignore_is_requested(
-	const gchar *file, filesize_t size, const gchar *sha1);
+	const gchar *file, filesize_t size, const struct sha1 *sha1);
 const gchar *ignore_reason_to_string(enum ignore_val);
 
 void ignore_add_filesize(const gchar *file, filesize_t size);
-void ignore_add_sha1(const gchar *file, const gchar *sha1);
-gchar *ignore_sha1_filename(const gchar *sha1);
+void ignore_add_sha1(const gchar *file, const struct sha1 *sha1);
+const gchar *ignore_sha1_filename(const struct sha1 *sha1);
 
 #endif	/* _core_ignore_h_ */
 
