@@ -1,5 +1,5 @@
 /*
- * $Id: matching.c,v 1.15 2005/10/26 11:54:56 cbiere Exp $
+ * $Id: matching.c 13996 2007-06-30 22:00:06Z cbiere $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi
  *
@@ -27,12 +27,13 @@
 
 #include "common.h"
 
-RCSID("$Id: matching.c,v 1.15 2005/10/26 11:54:56 cbiere Exp $");
+RCSID("$Id: matching.c 13996 2007-06-30 22:00:06Z cbiere $")
 
 #include "matching.h"
 #include "qrp.h"				/* For qhvec_add() */
 #include "share.h"
 
+#include "lib/atoms.h"
 #include "lib/pattern.h"
 #include "lib/misc.h"
 #include "lib/utf8.h"
@@ -80,15 +81,34 @@ RCSID("$Id: matching.c,v 1.15 2005/10/26 11:54:56 cbiere Exp $");
 
 #define ST_MIN_BIN_SIZE		4
 
+struct shared_file;
+
+struct st_entry {
+	const gchar *string;				/* atom */
+	struct shared_file *sf;
+	guint32 mask;
+};
+
+struct st_bin {
+	gint nslots, nvals;
+	struct st_entry **vals;
+};
+
+struct _search_table {
+	gint nentries, nchars, nbins;
+	struct st_bin **bins;
+	struct st_bin all_entries;
+	guchar index_map[(guchar) -1];
+};
 
 static void
 destroy_entry(struct st_entry *entry)
 {
 	g_assert(entry != NULL);
 
-	G_FREE_NULL(entry->string);
-	shared_file_unref(entry->data);
-	wfree(entry, sizeof(*entry));
+	atom_str_free_null(&entry->string);
+	shared_file_unref(&entry->sf);
+	wfree(entry, sizeof *entry);
 }
 
 /**
@@ -102,8 +122,8 @@ bin_initialize(struct st_bin *bin, gint size)
 	bin->nvals = 0;
 	bin->nslots = size;
 
-	bin->vals = g_malloc(size * sizeof(bin->vals[0]));
-	for (i = 0; i < size; i++)
+	bin->vals = g_malloc(bin->nslots * sizeof bin->vals[0]);
+	for (i = 0; i < bin->nslots; i++)
 		bin->vals[i] = NULL;
 }
 
@@ -113,7 +133,7 @@ bin_initialize(struct st_bin *bin, gint size)
 static struct st_bin *
 bin_allocate(void)
 {
-	struct st_bin *bin = (struct st_bin *) g_malloc(sizeof(struct st_bin));
+	struct st_bin *bin = walloc(sizeof *bin);
 
 	bin_initialize(bin, ST_MIN_BIN_SIZE);
 	return bin;
@@ -128,7 +148,8 @@ static void
 bin_destroy(struct st_bin *bin)
 {
 	G_FREE_NULL(bin->vals);
-	bin->vals = 0;
+	bin->nslots = 0;
+	bin->nvals = 0;
 }
 
 /**
@@ -139,7 +160,7 @@ bin_insert_item(struct st_bin *bin, struct st_entry *entry)
 {
 	if (bin->nvals == bin->nslots) {
 		bin->nslots *= 2;
-		bin->vals = g_realloc(bin->vals, bin->nslots * sizeof(bin->vals[0]));
+		bin->vals = g_realloc(bin->vals, bin->nslots * sizeof bin->vals[0]);
 	}
 	bin->vals[bin->nvals++] = entry;
 }
@@ -151,69 +172,99 @@ static void
 bin_compact(struct st_bin *bin)
 {
 	g_assert(bin->vals != NULL);	/* Or it would not have been allocated */
-	bin->vals = g_realloc(bin->vals, bin->nvals * sizeof(bin->vals[0]));
+
+	bin->vals = g_realloc(bin->vals, bin->nvals * sizeof bin->vals[0]);
 	bin->nslots = bin->nvals;
 }
 
-/**
- * Apply a char map to a string, inplace.
- *
- * @returns length of string.
- */
-size_t
-match_map_string(char_map_t map, gchar *string)
+static guchar map[(guchar) -1];
+
+static void
+setup_map(void)
 {
-	gchar *ptr = string;
-	guchar c;
+	guint i;
 
-	while ((c = (guchar) *ptr))
-		*ptr++ = map[c];
+	for (i = 0; i < G_N_ELEMENTS(map); i++)	{
+		guchar c;
 
-	return ptr - string;
+		if (i > 0 && utf8_byte_is_allowed(i)) {
+			if (is_ascii_upper(i)) {
+				c = ascii_tolower(i);
+			} else if (
+				is_ascii_punct(i) || is_ascii_cntrl(i) || is_ascii_space(i)
+			) {
+				c = ' ';
+			} else { 
+				c = i;
+			}
+		} else {
+			c = 0;
+		}
+		map[i] = c;
+	}
 }
 
 /**
  * Initialize permanent data in search table.
  */
 void
-st_initialize(search_table_t *table, char_map_t map)
+st_initialize(search_table_t *table)
 {
-	gint i;
-	guchar cur_char = '\0', map_char;
+	static guchar fold_map[(guchar) -1];
+	guchar cur_char = '\0';
+	guint i;
 
 	table->nentries = table->nchars = 0;
-
-	for (i = 0; i < 256; i++)
-		table->fold_map[i] = 0;
+	setup_map();
 
 	/*
 	 * The indexing map is used to avoid having 256*256 bins.
 	 */
 
-	for (i = 0; i < 256; i++) {
-		map_char = map[i];
-		if (!table->fold_map[map_char]) {
-			table->fold_map[map_char] = cur_char;
+	for (i = 0; i < G_N_ELEMENTS(table->index_map); i++) {
+		guchar map_char = map[i];
+
+		if (fold_map[map_char]) {
+			table->index_map[i] = fold_map[map_char];
+		} else {
+			fold_map[map_char] = cur_char;
 			table->index_map[i] = cur_char;
 			cur_char++;
-		} else {
-			table->index_map[i] = table->fold_map[map_char];
 		}
 	}
 
-	for (i = 0; i < 256; i++)
-		table->fold_map[i] = map[i];
-
 	table->nchars = cur_char;
 	table->nbins = table->nchars * table->nchars;
-	table->bins = 0;
+	table->bins = NULL;
 	table->all_entries.vals = 0;
 
-	if (dbg > 3)
+	if (GNET_PROPERTY(dbg) > 3)
 		printf("search table will use max of %d bins (%d indexing chars)\n",
 			table->nbins, table->nchars);
 }
 
+/**
+ * Allocates a new search_table_t. Use st_free() to free it.
+ */
+search_table_t *
+st_alloc(void)
+{
+	search_table_t *table = walloc(sizeof *table);
+	return table;
+}
+
+void
+st_free(search_table_t **ptr)
+{
+	g_assert(ptr);
+	if (*ptr) {
+		search_table_t *table = *ptr;
+		wfree(table, sizeof *table);
+		*ptr = NULL;
+	}
+}
+
+
 /**
  * Recreate variable parts of the search table.
  */
@@ -222,9 +273,9 @@ st_create(search_table_t *table)
 {
 	gint i;
 
-	table->bins = g_malloc(table->nbins * sizeof(struct st_bin));
+	table->bins = g_malloc(table->nbins * sizeof table->bins[0]);
 	for (i = 0; i < table->nbins; i++)
-		table->bins[i] = 0;
+		table->bins[i] = NULL;
 
     bin_initialize(&table->all_entries, ST_MIN_BIN_SIZE);
 }
@@ -239,13 +290,14 @@ st_destroy(search_table_t *table)
 
 	if (table->bins) {
 		for (i = 0; i < table->nbins; i++) {
-			if (table->bins[i]) {
-				bin_destroy(table->bins[i]);
-				G_FREE_NULL(table->bins[i]);
+			struct st_bin *bin = table->bins[i];
+
+			if (bin) {
+				bin_destroy(bin);
+				wfree(bin, sizeof *bin);
 			}
 		}
 		G_FREE_NULL(table->bins);
-		table->bins = 0;
 	}
 
 	if (table->all_entries.vals) {
@@ -285,7 +337,7 @@ mask_hash(const gchar *s) {
  * Get key of two-char pair.
  */
 static inline gint
-st_key(search_table_t *table, gchar k[2])
+st_key(search_table_t *table, const gchar k[2])
 {
 	return table->index_map[(guchar) k[0]] * table->nchars +
 		table->index_map[(guchar) k[1]];
@@ -294,39 +346,30 @@ st_key(search_table_t *table, gchar k[2])
 /**
  * Insert an item into the search_table
  * one-char strings are silently ignored.
+ *
+ * @return TRUE if the item was inserted; FALSE otherwise.
  */
-void
-st_insert_item(search_table_t *table, const gchar *s, struct shared_file *sf)
+gboolean
+st_insert_item(search_table_t *table, const gchar *s, gpointer data)
 {
 	size_t i, len;
 	struct st_entry *entry;
 	GHashTable *seen_keys;
-	gchar *string;
 
 	len = utf8_char_count(s);
 	if ((size_t) -1 == len || len < 2)
-		return;
-
-	string = g_strdup(s);
-
-#if 0
-	len = match_map_string(table->fold_map, string);
-	if (len < 2) {
-		G_FREE_NULL(string);
-		return;
-	}
-#endif
+		return FALSE;
 
 	seen_keys = g_hash_table_new(NULL, NULL);
 
 	entry = walloc(sizeof *entry);
-	entry->string = string;
-	entry->data = shared_file_ref(sf);
-	entry->mask = mask_hash(string);
+	entry->string = atom_str_get(s);
+	entry->sf = shared_file_ref(data);
+	entry->mask = mask_hash(entry->string);
 
-	len = strlen(string);
+	len = strlen(entry->string);
 	for (i = 0; i < len - 1; i++) {
-		gint key = st_key(table, &string[i]);
+		gint key = st_key(table, &entry->string[i]);
 
 		/* don't insert item into same bin twice */
 		if (g_hash_table_lookup(seen_keys, GINT_TO_POINTER(key)))
@@ -345,6 +388,7 @@ st_insert_item(search_table_t *table, const gchar *s, struct shared_file *sf)
 	table->nentries++;
 
 	g_hash_table_destroy(seen_keys);
+	return TRUE;
 }
 
 /**
@@ -369,7 +413,7 @@ st_compact(search_table_t *table)
  * Patterns are lazily compiled as needed, using pattern_compile_fast().
  */
 static gboolean
-entry_match(gchar *text, size_t tlen,
+entry_match(const gchar *text, size_t tlen,
 	cpattern_t **pw, word_vec_t *wovec, size_t wn)
 {
 	size_t i;
@@ -402,13 +446,14 @@ entry_match(gchar *text, size_t tlen,
 void
 st_search(
 	search_table_t *table,
-	gchar *search,
-	void (*callback)(gpointer, shared_file_t *),
+	const gchar *search_term,
+	st_search_callback callback,
 	gpointer ctx,
 	gint max_res,
 	query_hashvec_t *qhv)
 {
-	gint key, nres = 0;
+	gchar *search;
+	gint key, nres;
 	guint i, len;
 	struct st_bin *best_bin = NULL;
 	gint best_bin_size = INT_MAX;
@@ -416,12 +461,17 @@ st_search(
 	guint wocnt;
 	cpattern_t **pattern;
 	struct st_entry **vals;
-	gint vcnt;
+	guint vcnt;
 	gint scanned = 0;		/* measure search mask efficiency */
 	guint32 search_mask;
 	size_t minlen;
+	guint random_offset;  /* Randomizer for search returns */
 
-	/* We don't need of the map because the strings are normalized */
+	search = UNICODE_CANONIZE(search_term);
+	if (GNET_PROPERTY(search_debug) && 0 != strcmp(search, search_term)) {
+		g_message(" Original search term: \"%s\"", search_term);
+		g_message("Canonical search term: \"%s\"", search);
+	}
 	len = strlen(search);
 
 	/*
@@ -429,7 +479,7 @@ st_search(
 	 */
 
 	if (len < 2)
-		return;
+		goto finish;
 
 	for (i = 0; i < len - 1; i++) {
 		struct st_bin *bin;
@@ -446,7 +496,7 @@ st_search(
 		}
 	}
 
-	if (dbg > 6)
+	if (GNET_PROPERTY(dbg) > 6)
 		printf("st_search(): str=\"%s\", len=%d, best_bin_size=%d\n",
 			search, len, best_bin_size);
 
@@ -466,7 +516,7 @@ st_search(
 		 */
 
 		if (qhv == NULL)
-			return;
+			goto finish;
 	}
 
 	/*
@@ -489,13 +539,13 @@ st_search(
 	if (wocnt == 0 || best_bin == NULL) {
 		if (wocnt > 0)
 			word_vec_free(wovec, wocnt);
-		return;
+		goto finish;
 	}
 
 	g_assert(best_bin_size > 0);	/* Allocated bin, it must hold something */
 
 
-	pattern = g_malloc0(wocnt * sizeof *pattern);
+	pattern = walloc0(wocnt * sizeof *pattern);
 
 	/*
 	 * Prepare matching optimization, an idea from Mike Green.
@@ -533,30 +583,44 @@ st_search(
 
 	vcnt = best_bin->nvals;
 	vals = best_bin->vals;
+	random_offset = random_raw() % vcnt;
 
-	while (nres < max_res && vcnt-- > 0) {
-		struct st_entry *e = *vals++;
-		struct shared_file *sf = e->data;
+	nres = 0;
+	for (i = 0; i < vcnt; i++) {
+		const struct st_entry *e;
+		struct shared_file *sf;
+		size_t canonic_len;
 
+		/*
+		 * As we only return a limited count of results, pick a random
+		 * offset, so that repeated searches will match different items
+		 * instead of always the first - with some probability.
+		 */
+		e = vals[(i + random_offset) % vcnt];
+		
 		if ((e->mask & search_mask) != search_mask)
 			continue;		/* Can't match */
 
-		if (sf->name_canonic_len < minlen)
+		sf = e->sf;
+
+		canonic_len = shared_file_name_canonic_len(sf);
+		if (canonic_len < minlen)
 			continue;		/* Can't match */
 
 		scanned++;
 
-		if (
-			entry_match(e->string, sf->name_canonic_len, pattern, wovec, wocnt)
-		) {
-			if (dbg > 5)
-				printf("MATCH: %s\n", sf->name_nfc);
+		if (entry_match(e->string, canonic_len, pattern, wovec, wocnt)) {
+			if (GNET_PROPERTY(dbg) > 5)
+				printf("MATCH: %s\n", shared_file_name_nfc(sf));
+
 			callback(ctx, sf);
 			nres++;
+			if (nres >= max_res)
+				break;
 		}
 	}
 
-	if (dbg > 6)
+	if (GNET_PROPERTY(dbg) > 6)
 		printf("st_search(): scanned %d entry from the %d in bin, %d matches\n",
 			scanned, best_bin_size, nres);
 
@@ -564,8 +628,13 @@ st_search(
 		if (pattern[i])					/* Lazily compiled by entry_match() */
 			pattern_free(pattern[i]);
 
-	G_FREE_NULL(pattern);
+	wfree(pattern, wocnt * sizeof *pattern);
 	word_vec_free(wovec, wocnt);
+
+finish:
+	if (search != search_term) {
+		G_FREE_NULL(search);
+	}
 }
 
 /* vi: set ts=4 sw=4 cindent: */
