@@ -1,5 +1,5 @@
 /*
- * $Id: mq_udp.c 11437 2006-08-05 00:24:20Z cbiere $
+ * $Id: mq_udp.c 13130 2007-03-17 19:33:01Z bpringlemeir $
  *
  * Copyright (c) 2002-2003, Raphael Manfredi
  *
@@ -35,7 +35,7 @@
 
 #include "common.h"
 
-RCSID("$Id: mq_udp.c 11437 2006-08-05 00:24:20Z cbiere $")
+RCSID("$Id: mq_udp.c 13130 2007-03-17 19:33:01Z bpringlemeir $")
 
 #include "nodes.h"
 #include "mq.h"
@@ -44,10 +44,12 @@ RCSID("$Id: mq_udp.c 11437 2006-08-05 00:24:20Z cbiere $")
 #include "gmsg.h"
 #include "tx.h"
 #include "gnet_stats.h"
+#include "hosts.h"
 
 #include "lib/walloc.h"
 
 #include "if/gnet_property_priv.h"
+#include "if/core/main.h"
 
 #include "lib/override.h"		/* Must be the last header included */
 
@@ -79,7 +81,7 @@ struct mq_udp_info_extended {
 static void
 mq_udp_pmsg_free(pmsg_t *mb, gpointer arg)
 {
-	struct mq_udp_info *mi = (struct mq_udp_info *) arg;
+	struct mq_udp_info *mi = arg;
 
 	g_assert(pmsg_is_extended(mb));
 
@@ -93,7 +95,7 @@ mq_udp_pmsg_free(pmsg_t *mb, gpointer arg)
 static void
 mq_udp_pmsg_free_extended(pmsg_t *mb, gpointer arg)
 {
-	struct mq_udp_info_extended *mi = (struct mq_udp_info_extended *) arg;
+	struct mq_udp_info_extended *mi = arg;
 
 	g_assert(pmsg_is_extended(mb));
 
@@ -159,6 +161,7 @@ mq_udp_make(gint maxsize, struct gnutella_node *n, struct txdriver *nd)
 	q->maxsize = maxsize;
 	q->lowat = maxsize >> 2;		/* 25% of max size */
 	q->hiwat = maxsize >> 1;		/* 50% of max size */
+	q->qwait = slist_new();
 	q->ops = &mq_udp_ops;
 	q->cops = mq_get_cops();
 
@@ -173,7 +176,7 @@ mq_udp_make(gint maxsize, struct gnutella_node *n, struct txdriver *nd)
 static void
 mq_udp_service(gpointer data)
 {
-	mqueue_t *q = (mqueue_t *) data;
+	mqueue_t *q = data;
 	gint r;
 	GList *l;
 	gint sent;
@@ -190,10 +193,10 @@ mq_udp_service(gpointer data)
 	 */
 
 	for (l = q->qtail; l; /* empty */) {
-		pmsg_t *mb = (pmsg_t *) l->data;
+		pmsg_t *mb = l->data;
 		gchar *mb_start = pmsg_start(mb);
 		gint mb_size = pmsg_size(mb);
-		struct mq_udp_info *mi = (struct mq_udp_info *) pmsg_get_metadata(mb);
+		struct mq_udp_info *mi = pmsg_get_metadata(mb);
 		guint8 function;
 
 		if (!pmsg_check(mb, q)) {
@@ -201,7 +204,7 @@ mq_udp_service(gpointer data)
 			goto skip;
 		}
 
-		r = tx_sendto((txdrv_t *) q->tx_drv, &mi->to, mb_start, mb_size);
+		r = tx_sendto(q->tx_drv, &mi->to, mb_start, mb_size);
 
 		if (r < 0)		/* Error, drop packet and continue */
 			goto skip;
@@ -211,7 +214,7 @@ mq_udp_service(gpointer data)
 
 		if (r != mb_size) {
 			g_warning("partial UDP write (%d bytes) to %s for %d-byte datagram",
-				r, host_addr_port_to_string(mi->to.addr, mi->to.port), mb_size);
+				r, gnet_host_to_string(&mi->to), mb_size);
 			dropped++;
 			goto skip;
 		}
@@ -263,7 +266,7 @@ mq_udp_service(gpointer data)
 
 	if (q->size == 0) {
 		g_assert(q->count == 0);
-		tx_srv_disable((txdrv_t *) q->tx_drv);
+		tx_srv_disable(q->tx_drv);
 		node_tx_service(q->node, FALSE);
 	}
 
@@ -279,18 +282,22 @@ mq_udp_service(gpointer data)
 void
 mq_udp_putq(mqueue_t *q, pmsg_t *mb, const gnet_host_t *to)
 {
-	size_t size = pmsg_size(mb);
-	gchar *mbs = pmsg_start(mb);
-	guint8 function = gmsg_function(mbs);
-	guint8 hops = gmsg_hops(mbs);
-	pmsg_t *mbe;
+	size_t size;
+	gchar *mbs;
+	guint8 function;
+	guint8 hops;
+	pmsg_t *mbe = NULL;		/* Extended message with destination info */
+	gboolean error = FALSE;
 
+again:
 	g_assert(q);
+	g_assert(mb);
 	g_assert(!pmsg_was_sent(mb));
 	g_assert(pmsg_is_unread(mb));
 	g_assert(q->ops == &mq_udp_ops);	/* Is an UDP queue */
 	mq_check(q, 0);
 
+	size = pmsg_size(mb);
 	if (size == 0) {
 		g_warning("mq_putq: called with empty message");
 		goto cleanup;
@@ -301,6 +308,36 @@ mq_udp_putq(mqueue_t *q, pmsg_t *mb, const gnet_host_t *to)
 		goto cleanup;
 	}
 
+	/*
+	 * Protect against recursion: we must not invoke puthere() whilst in
+	 * the middle of another putq() or we would corrupt the qlink array:
+	 * Messages received during recursion are inserted into the qwait list
+	 * and will be stuffed back into the queue when the initial putq() ends.
+	 *		--RAM, 2006-12-29
+	 */
+
+	if (q->putq_entered > 0) {
+		pmsg_t *extended;
+
+		if (debugging(20))
+			g_warning("mq_udp_putq: %s recursion detected (%d already pending)",
+				mq_info(q), slist_length(q->qwait));
+
+		/*
+		 * We insert extended messages into the waiting queue since we need
+		 * the destination information as well.
+		 */
+
+		extended = mq_udp_attach_metadata(mb, to);
+		slist_append(q->qwait, extended);
+		return;
+	}
+	q->putq_entered++;
+
+	mbs = pmsg_start(mb);
+	function = gmsg_function(mbs);
+	hops = gmsg_hops(mbs);
+
 	gnet_stats_count_queued(q->node, function, hops, size);
 
 	/*
@@ -348,8 +385,7 @@ mq_udp_putq(mqueue_t *q, pmsg_t *mb, const gnet_host_t *to)
 
 		if (written > 0) {
 			g_warning("partial UDP write (%d bytes) to %s for %d-byte datagram",
-				(gint) written, host_addr_port_to_string(to->addr, to->port),
-				(gint) size);
+				(gint) written, gnet_host_to_string(to), (gint) size);
 			goto cleanup;
 		}
 
@@ -357,7 +393,9 @@ mq_udp_putq(mqueue_t *q, pmsg_t *mb, const gnet_host_t *to)
 	}
 
 	/*
-	 * Attach the destination information as metadata to the message.
+	 * Attach the destination information as metadata to the message, unless
+	 * it is already known (possible only during unfolding of the queued data
+	 * during re-entrant calls).
 	 *
 	 * This is later extracted via pmsg_get_metadata() on the extended
 	 * message by the message queue to get the destination information.
@@ -365,15 +403,56 @@ mq_udp_putq(mqueue_t *q, pmsg_t *mb, const gnet_host_t *to)
 	 * Then enqueue the extended message.
 	 */
 
-	mbe = mq_udp_attach_metadata(mb, to);
+	if (NULL == mbe)
+		mbe = mq_udp_attach_metadata(mb, to);
 
 	q->cops->puthere(q, mbe, size);
-	mq_check(q, 0);
-	return;
+	mb = NULL;
 
 cleanup:
-	pmsg_free(mb);
+
+	if (mb) {
+		pmsg_free(mb);
+		mb = NULL;
+	}
+
+	/*
+	 * When reaching that point with a zero putq_entered counter, it means
+	 * we triggered an early error condition.  Bail out.
+	 */
+
+	g_assert(q->putq_entered >= 0);
+
+	if (q->putq_entered == 0)
+		error = TRUE;
+	else
+		q->putq_entered--;
+
 	mq_check(q, 0);
+
+	/*
+	 * If we're exiting here with no other putq() registered, then we must
+	 * pop an item off the head of the list and iterate again.
+	 */
+
+	if (0 == q->putq_entered && !error) {
+		mbe = slist_shift(q->qwait);
+		if (mbe) {
+			struct mq_udp_info *mi = pmsg_get_metadata(mbe);
+
+			mb = mbe;		/* An extended message "is-a" message */
+			to = &mi->to;
+
+			if (debugging(20))
+				g_warning(
+					"mq_udp_putq: %s flushing waiting to %s (%d still pending)",
+					mq_info(q), gnet_host_to_string(to),
+					slist_length(q->qwait));
+
+			goto again;
+		}
+	}
+
 	return;
 }
 
@@ -385,9 +464,7 @@ mq_udp_node_putq(mqueue_t *q, pmsg_t *mb, const gnutella_node_t *n)
 {
 	gnet_host_t to;
 
-	to.addr = n->addr;
-	to.port = n->port;
-
+	gnet_host_set(&to, n->addr, n->port);
 	mq_udp_putq(q, mb, &to);
 }
 
