@@ -1,5 +1,5 @@
 /*
- * $Id: mq_udp.h,v 1.2 2005/06/25 01:37:40 daichik Exp $
+ * $Id: mq_udp.h 12496 2006-11-26 09:22:49Z cbiere $
  *
  * Copyright (c) 2002-2003, Raphael Manfredi
  *
@@ -36,6 +36,7 @@
 #ifndef _core_mq_udp_h_
 #define _core_mq_udp_h_
 
+#include "common.h"
 #include "mq.h"
 #include "pmsg.h"
 #include "if/core/hosts.h"		/* For gnet_host_t */
@@ -49,8 +50,8 @@ struct txdriver;
 mqueue_t *mq_udp_make(
 	gint maxsize, struct gnutella_node *n, struct txdriver *nd);
 
-void mq_udp_putq(mqueue_t *q, pmsg_t *mb, gnet_host_t *to);
-void mq_udp_node_putq(mqueue_t *q, pmsg_t *mb, struct gnutella_node *n);
+void mq_udp_putq(mqueue_t *q, pmsg_t *mb, const gnet_host_t *to);
+void mq_udp_node_putq(mqueue_t *q, pmsg_t *mb, const struct gnutella_node *n);
 
 #endif	/* _core_mq_udp_h_ */
 
