@@ -1,5 +1,5 @@
 /*
- * $Id: nodes.h,v 1.40 2005/12/16 14:51:42 cbiere Exp $
+ * $Id: nodes.h 13870 2007-06-18 03:27:25Z cbiere $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi
  *
@@ -44,11 +44,6 @@
 
 #include "lib/header.h"
 
-typedef enum node_protocol_types {
-	PROTOCOL_TYPE_GNUTELLA = 0,
-	PROTOCOL_TYPE_G2
-} node_protocol_types_t;
-
 typedef enum {
 	NODE_MAGIC = 0x67f8e02f
 } node_magic_t;
@@ -89,6 +84,13 @@ struct node_rxfc_mon {
 
 #define CACHE_HOP_IDX(h)	(((h) > MAX_CACHE_HOPS) ? MAX_CACHE_HOPS : (h))
 
+/**
+ * @def NODE_LEAF_MIN_FLOW
+ * The value of the hops-flow under which the leaf is not fully searcheable,
+ * and which will NOT have its QRP included in the inter-UP QRP table.
+ */
+#define NODE_LEAF_MIN_FLOW		3	  /**< Or leaf is not fully searcheable */
+
 /*
  * Throttle periods for ping reception, depending on the peer mode.
  * This is applicable for regular pings, not alive pings.
@@ -96,32 +98,33 @@ struct node_rxfc_mon {
 #define PING_REG_THROTTLE		3		/**< seconds, regular peer */
 #define PING_LEAF_THROTTLE		60		/**< seconds, peer is leaf node */
 
+#define NODE_ID_SELF (node_id_get_self())
+
 typedef struct gnutella_node {
 	node_magic_t magic;			/**< Magic value for consistency checks */
-    gnet_node_t node_handle;    /**< Handle of this node */
 	node_peer_t peermode;		/**< Operating mode (leaf, ultra, normal) */
 	node_peer_t start_peermode;	/**< Operating mode when handshaking begun */
 
 	gchar error_str[256];		/**< To sprintf() error strings with vars */
 	struct gnutella_socket *socket;		/**< Socket of the node */
+	guint16 country;			/**< Country of origin -- encoded ISO3166 */
 	guint8 proto_major;			/**< Handshaking protocol major number */
 	guint8 proto_minor;			/**< Handshaking protocol minor number */
-	node_protocol_types_t protocol_type;
 
 	guint8 qrp_major;			/**< Query routing protocol major number */
 	guint8 qrp_minor;			/**< Query routing protocol minor number */
 	guint8 uqrp_major;			/**< UP Query routing protocol major number */
 	guint8 uqrp_minor;			/**< UP Query routing protocol minor number */
-	gchar *vendor;				/**< Vendor information (always UTF-8) */
-	gint country;				/**< Country of origin -- encoded ISO3166 */
-	union vendor_code vcode;	/**< Vendor code (vcode.u32 == 0 if unknown) */
+	const gchar *vendor;		/**< Vendor information (always UTF-8) */
+	vendor_code_t vcode;		/**< Vendor code (vcode.u32 == 0 if unknown) */
 	gpointer io_opaque;			/**< Opaque I/O callback information */
 
-	struct gnutella_header header;	/**< Header of the current message */
+	gnutella_header_t header;		/**< Header of the current message */
 	extvec_t extvec[MAX_EXTVEC];	/**< GGEP extensions in "fat" messages */
 	gint extcount;					/**< Amount of extensions held */
 
-	guint32 size; /**< How many bytes we need to read for the current message */
+	guint16 size; /**< How many bytes we need to read for the current message */
+	guint16 header_flags;		/**< Header flags (new message architecture) */
 
 	gchar *data;				/**< data of the current message */
 	guint32 pos;				/**< write position in data */
@@ -148,6 +151,8 @@ typedef struct gnutella_node {
 	guint16 n_hard_ttl;			/**< Number of hard_ttl exceeded (bad) */
 	guint32 n_weird;			/**< Number of weird messages from that node */
 	guint32 n_hostile;			/**< Number of messages from hostile IP */
+	guint32 n_spam;				/**< Number of messages rated as spam */
+	guint32 n_evil;				/**< Number of messages with evil filenames */
 
 	guint32 allocated;			/**< Size of allocated buffer data, 0 for none */
 	gboolean have_header;		/**< TRUE if we have got a full message header */
@@ -161,14 +166,13 @@ typedef struct gnutella_node {
 	time_t shutdown_date;		/**< When we entered in shutdown mode */
 	time_t up_date;				/**< When remote server started (0 if unknown) */
 	time_t leaf_flowc_start;	/**< Time when leaf flow-controlled queries */
-	guint32 shutdown_delay;		/**< How long we can stay in shutdown mode */
+	time_delta_t shutdown_delay; /**< How long we can stay in shutdown mode */
 
 	const gchar *remove_msg;	/**< Reason of removing */
 
 	host_addr_t addr;			/**< ip of the node */
 	guint16 port;				/**< port of the node */
 
-	gchar *guid;				/**< GUID of node (atom) for push-proxying */
 	host_addr_t proxy_addr;		/**< ip of the node for push proxyfication */
 	guint16 proxy_port;			/**< port of the node for push proxyfication */
 
@@ -176,16 +180,16 @@ typedef struct gnutella_node {
 	squeue_t *searchq;			/**< TX Search queue */
 	rxdrv_t *rx;				/**< RX stack top */
 
-	gpointer routing_data;		/**< Opaque info, for gnet message routing */
-	gpointer sent_query_table;	/**< Opaque info, query table sent to node */
-	gpointer recv_query_table;	/**< Opaque info, query table recved from node */
-	gpointer qrt_update;		/**< Opaque info, query routing update handle */
-	gpointer qrt_receive;		/**< Opaque info, query routing reception */
+	struct route_data *routing_data;		/**< for gnet message routing */
+	struct routing_table *sent_query_table;	/**< query table sent to node */
+	struct routing_table *recv_query_table;	/**< query table recved from node */
+	struct qrt_update *qrt_update;			/**< query routing update handle */
+	struct qrt_receive *qrt_receive;		/**< query routing reception */
 	qrt_info_t *qrt_info;		/**< Info about received query table */
 
 	gpointer alive_pings;		/**< Opaque info, for alive ping checks */
 	time_t last_alive_ping;		/**< Last time we sent an alive ping */
-	guint alive_period;			/**< Period for sending alive pings (secs) */
+	time_delta_t alive_period;	/**< Period for sending alive pings (secs) */
 
 	wrap_buf_t hello;			/**< Spill buffer for GNUTELLA HELLO */
 
@@ -197,14 +201,14 @@ typedef struct gnutella_node {
 
 	guint32 tcp_rtt;			/**< RTT when exchange takes place over TCP */
 	guint32 udp_rtt;			/**< RTT when exchange takes place over UDP  */
-	gpointer tsync_ev;			/**< Time sync event */
+	cevent_t *tsync_ev;			/**< Time sync event */
 
 	/*
 	 * Data structures used by the ping/pong reduction scheme.
 	 *		--RAM, 02/02/2002
 	 */
 
-	guint32 id;					/**< Unique internal ID */
+	node_id_t id;				/**< Unique internal ID */
 	guint ping_throttle;		/**< Period for accepting new pings (secs) */
 	time_t ping_accept;			/**< Time after which we accept new pings */
 	time_t next_ping;			/**< When to send a ping, for "OLD" clients */
@@ -218,7 +222,7 @@ typedef struct gnutella_node {
 	guint32 gnet_kbytes_count;	/**< Used to answer "Crawling" pings */
 	host_addr_t gnet_pong_addr;	/**< When != 0, last IP we got in pong */
 	host_addr_t gnet_qhit_addr;	/**< When != 0, last IP we got in query hit */
-	gchar *gnet_guid;			/**< GUID of node (atom) seen on the network */
+	const gchar *guid;			/**< GUID of node (atom) seen on the network */
 
 	guint32 n_ping_throttle;  /**< Number of pings we throttled */
 	guint32 n_ping_accepted;  /**< Number of pings we accepted */
@@ -265,57 +269,65 @@ typedef struct gnutella_node {
  * Node flags.
  */
 
-#define NODE_F_HDSK_PING	0x00000001 /**< Expecting handshake ping */
-#define NODE_F_STALE_QRP	0x00000002 /**< Is sending a stale QRP patch */
-#define NODE_F_INCOMING		0x00000004 /**< Incoming (permanent) connection */
-#define NODE_F_ESTABLISHED	0x00000008 /**< Gnutella connection established */
-#define NODE_F_VALID		0x00000010 /**< Handshaked with a Gnutella node */
-#define NODE_F_ALIEN_IP		0x00000020 /**< Pong-IP did not match TCP/IP addr */
-#define NODE_F_WRITABLE		0x00000040 /**< Node is writable */
-#define NODE_F_READABLE		0x00000080 /**< Node is readable, process queries */
-#define NODE_F_BYE_SENT		0x00000100 /**< Bye message was queued */
-#define NODE_F_NODELAY		0x00000200 /**< TCP_NODELAY was activated */
-#define NODE_F_NOREAD		0x00000400 /**< Prevent further reading from node */
-#define NODE_F_EOF_WAIT		0x00000800 /**< At final shutdown, waiting EOF */
-#define NODE_F_CLOSING		0x00001000 /**< Initiated bye or shutdown */
-#define NODE_F_ULTRA		0x00002000 /**< Is one of our ultra nodes */
-#define NODE_F_LEAF			0x00004000 /**< Is one of our leaves */
-#define NODE_F_CRAWLER		0x00008000 /**< Is a Gnutella Crawler */
-#define NODE_F_FAKE_NAME	0x00010000 /**< Was unable to validate GTKG name */
-#define NODE_F_PROXY		0x00020000 /**< Sent a push-proxy request to it */
-#define NODE_F_PROXIED		0x00040000 /**< We are push-proxy for that node */
-#define NODE_F_QRP_SENT		0x00080000 /**< Undergone 1 complete QRP sending */
-#define NODE_F_TLS			0x00100000 /**< TLS-tunneled */
-#define NODE_F_TSYNC_WAIT	0x00200000 /**< Time sync pending via TCP */
-#define NODE_F_TSYNC_TCP	0x00400000 /**< No replies via UDP, use TCP */
-#define NODE_F_GTKG			0x00800000 /**< Node is another gtk-gnutella */
-#define NODE_F_FORCE		0x01000000 /**< Connection is forced */
+enum {
+	NODE_F_HDSK_PING	= 1 << 0,	/**< Expecting handshake ping */
+	NODE_F_STALE_QRP	= 1 << 1,	/**< Is sending a stale QRP patch */
+	NODE_F_INCOMING		= 1 << 2,	/**< Incoming (permanent) connection */
+	NODE_F_ESTABLISHED	= 1 << 3,	/**< Gnutella connection established */
+	NODE_F_VALID		= 1 << 4,	/**< Handshaked with a Gnutella node */
+	NODE_F_ALIEN_IP		= 1 << 5,	/**< Pong-IP did not match TCP/IP addr */
+	NODE_F_WRITABLE		= 1 << 6,	/**< Node is writable */
+	NODE_F_READABLE		= 1 << 7,	/**< Node is readable, process queries */
+	NODE_F_BYE_SENT		= 1 << 8,	/**< Bye message was queued */
+	NODE_F_NODELAY		= 1 << 9,	/**< TCP_NODELAY was activated */
+	NODE_F_NOREAD		= 1 << 10,	/**< Prevent further reading from node */
+	NODE_F_EOF_WAIT		= 1 << 11,	/**< At final shutdown, waiting EOF */
+	NODE_F_CLOSING		= 1 << 12,	/**< Initiated bye or shutdown */
+	NODE_F_ULTRA		= 1 << 13,	/**< Is one of our ultra nodes */
+	NODE_F_LEAF			= 1 << 14,	/**< Is one of our leaves */
+	NODE_F_CRAWLER		= 1 << 15,	/**< Is a Gnutella Crawler */
+	NODE_F_FAKE_NAME	= 1 << 16,	/**< Was unable to validate GTKG name */
+	NODE_F_PROXY		= 1 << 17,	/**< Sent a push-proxy request to it */
+	NODE_F_PROXIED		= 1 << 18,	/**< We are push-proxy for that node */
+	NODE_F_QRP_SENT		= 1 << 19,	/**< Undergone 1 complete QRP sending */
+	NODE_F_TSYNC_WAIT	= 1 << 20,	/**< Time sync pending via TCP */
+	NODE_F_TSYNC_TCP	= 1 << 21,	/**< No replies via UDP, use TCP */
+	NODE_F_GTKG			= 1 << 22,	/**< Node is another gtk-gnutella */
+	NODE_F_FORCE		= 1 << 23,	/**< Connection is forced */
+	NODE_F_NO_OOB_PROXY	= 1 << 24,	/**< Do not OOB proxy the leaf */
+	NODE_F_TLS			= 1 << 25,	/**< TLS-tunneled */
+	NODE_F_CAN_TLS		= 1 << 26	/**< Indicated support for TLS */
+};
 
 /*
  * Node attributes.
  */
 
-#define NODE_A_BYE_PACKET	0x00000001 /**< Supports Bye-Packet */
-#define NODE_A_PONG_CACHING	0x00000002 /**< Supports Pong-Caching */
-#define NODE_A_PONG_ALIEN	0x00000004 /**< Alien Pong-Caching scheme */
-#define NODE_A_QHD_NO_VTAG	0x00000008 /**< Servent has no vendor tag in QHD */
-#define NODE_A_RX_INFLATE	0x00000010 /**< Reading compressed data */
-#define NODE_A_TX_DEFLATE	0x00000020 /**< Sending compressed data */
-#define NODE_A_ULTRA		0x00000040 /**< Node wants to be an Ultrapeer */
-#define NODE_A_NO_ULTRA		0x00000080 /**< Node is NOT ultra capable */
-#define NODE_A_UP_QRP		0x00000100 /**< Supports intra-UP QRP */
-#define NODE_A_LEAF_GUIDE	0x00000200 /**< Supports leaf-guided dyn queries */
-#define NODE_A_TIME_SYNC	0x00000400 /**< Supports time sync */
-#define NODE_A_CRAWLABLE	0x00000800 /**< Node can be UDP-crawled */
-#define NODE_A_DYN_QUERY	0x00001000 /**< Node can perform dynamic queries */
+enum {
+	NODE_A_BYE_PACKET	= 1 << 0,	/**< Supports Bye-Packet */
+	NODE_A_PONG_CACHING	= 1 << 1,	/**< Supports Pong-Caching */
+	NODE_A_PONG_ALIEN	= 1 << 2,	/**< Alien Pong-Caching scheme */
+	NODE_A_QHD_NO_VTAG	= 1 << 3,	/**< Servent has no vendor tag in QHD */
+	NODE_A_RX_INFLATE	= 1 << 4,	/**< Reading compressed data */
+	NODE_A_TX_DEFLATE	= 1 << 5,	/**< Sending compressed data */
+	NODE_A_ULTRA		= 1 << 6,	/**< Node wants to be an Ultrapeer */
+	NODE_A_NO_ULTRA		= 1 << 7,	/**< Node is NOT ultra capable */
+	NODE_A_UP_QRP		= 1 << 8,	/**< Supports intra-UP QRP */
+	NODE_A_GUIDANCE		= 1 << 9,	/**< Can leaf-guide dyn queries */
+	NODE_A_TIME_SYNC	= 1 << 10,	/**< Supports time sync */
+	NODE_A_CRAWLABLE	= 1 << 11,	/**< Node can be UDP-crawled */
+	NODE_A_DYN_QUERY	= 1 << 12,	/**< Node can perform dynamic queries */
+	NODE_A_CAN_SFLAG	= 1 << 13,	/**< Node supports flags in headers */
  
-#define NODE_A_NO_DUPS		0x02000000 /**< For broken old GTKG: no dups! */
-#define NODE_A_CAN_HSEP		0x04000000 /**< Node supports HSEP */
-#define NODE_A_CAN_QRP		0x08000000 /**< Node supports query routing */
-#define NODE_A_CAN_VENDOR	0x10000000 /**< Node supports vendor messages */
-#define NODE_A_CAN_GGEP		0x20000000 /**< Node supports big pongs, etc.. */
-#define NODE_A_CAN_ULTRA	0x40000000 /**< Node is ultra capable */
-#define NODE_A_CAN_INFLATE	0x80000000 /**< Node capable of inflating */
+	NODE_A_NO_KEPT_ZERO	= 1 << 14,	/**< For GTKG < 2006-08-15: no kept=0! */
+	NODE_A_NO_DUPS		= 1 << 15,	/**< For broken old GTKG: no dups! */
+	NODE_A_CAN_HSEP		= 1 << 16,	/**< Node supports HSEP */
+	NODE_A_CAN_QRP		= 1 << 17,	/**< Node supports query routing */
+	NODE_A_CAN_VENDOR	= 1 << 18,	/**< Node supports vendor messages */
+	NODE_A_CAN_ULTRA	= 1 << 19,	/**< Node is ultra capable */
+	NODE_A_CAN_INFLATE	= 1 << 20,	/**< Node capable of inflating */
+	NODE_A_CAN_HEAD		= 1 << 21	/**< Supports HEAD ping (vendor message) */
+};
  
 /*
  * UDP crawling "feature" flags.
@@ -402,9 +414,10 @@ typedef struct gnutella_node {
 
 #define NODE_ID(n)				((n)->id)
 
-#define NODE_CAN_GGEP(n)		((n)->attrs & NODE_A_CAN_GGEP)
+#define NODE_CAN_SFLAG(n)		((n)->attrs & NODE_A_CAN_SFLAG)
 #define NODE_UP_QRP(n)			((n)->attrs & NODE_A_UP_QRP)
-#define NODE_GUIDES_QUERY(n)	((n)->attrs & NODE_A_LEAF_GUIDE)
+#define NODE_LEAF_GUIDE(n)		((n)->attrs & NODE_A_GUIDANCE)
+#define NODE_CAN_INFLATE(n)		((n)->attrs & NODE_A_CAN_INFLATE)
 
 /*
  * Peer inspection macros
@@ -480,9 +493,7 @@ typedef struct gnutella_node {
 #define GNUTELLA_HELLO "GNUTELLA CONNECT/"
 #define GNUTELLA_HELLO_LENGTH	(sizeof(GNUTELLA_HELLO) - 1)
 
-#define NODE_ID_LOCAL	0x0U		/**< ID for "local node" (ourselves) */
-
-extern gchar *start_rfc822_date;
+extern const gchar *start_rfc822_date;
 
 extern GHookList node_added_hook_list;
 extern struct gnutella_node *node_added;
@@ -508,6 +519,7 @@ void node_add_socket(struct gnutella_socket *s, const host_addr_t addr,
 		guint16 port, guint32 flags);
 void node_remove(struct gnutella_node *,
 	const gchar * reason, ...) G_GNUC_PRINTF(2, 3);
+guint node_remove_by_addr(const host_addr_t addr, guint16 port);
 void node_bye(gnutella_node_t *, gint code,
 	const gchar * reason, ...) G_GNUC_PRINTF(3, 4);
 void node_real_remove(gnutella_node_t *);
@@ -534,10 +546,10 @@ gboolean node_bye_pending(void);
 void node_close(void);
 gboolean node_remove_worst(gboolean non_local);
 
-void node_qrt_changed(gpointer query_table);
+void node_qrt_changed(struct routing_table *query_table);
 void node_qrt_discard(struct gnutella_node *n);
-void node_qrt_install(struct gnutella_node *n, gpointer query_table);
-void node_qrt_patched(struct gnutella_node *n, gpointer query_table);
+void node_qrt_install(struct gnutella_node *n, struct routing_table *);
+void node_qrt_patched(struct gnutella_node *n, struct routing_table *);
 
 void send_node_error(struct gnutella_socket *s, int code,
 	const gchar *msg, ...) G_GNUC_PRINTF(3, 4);
@@ -552,6 +564,7 @@ void node_set_hops_flow(gnutella_node_t *n, guint8 hops);
 void node_set_online_mode(gboolean on);
 void node_current_peermode_changed(node_peer_t mode);
 const gchar *node_addr(const gnutella_node_t *n);
+const gchar *node_addr2(const gnutella_node_t *n);
 const gchar *node_gnet_addr(const gnutella_node_t *n);
 
 void node_connect_back(const gnutella_node_t *n, guint16 port);
@@ -559,23 +572,26 @@ void node_connected_back(struct gnutella_socket *s);
 
 void node_mark_bad_vendor(struct gnutella_node *n);
 
-void node_proxying_remove(gnutella_node_t *n, gboolean discard);
-gboolean node_proxying_add(gnutella_node_t *n, gchar *guid);
+void node_proxying_remove(gnutella_node_t *n);
+gboolean node_proxying_add(gnutella_node_t *n, const gchar *guid);
 void node_proxy_add(gnutella_node_t *n, const host_addr_t addr, guint16 port);
 void node_proxy_cancel_all(void);
-void node_http_proxies_add(
-	gchar *buf, gint *retval, gpointer arg, guint32 flags);
-GSList *node_push_proxies(void);
+size_t node_http_proxies_add(gchar *buf, size_t size,
+			gpointer arg, guint32 flags);
+const GSList *node_push_proxies(void);
 const GSList *node_all_nodes(void);
 const GSList *node_all_but_broken_gtkg(void);
-gnutella_node_t *node_active_by_id(guint32 id);
+
+
+gnutella_node_t *node_by_id(const node_id_t node_id);
+gnutella_node_t *node_active_by_id(const node_id_t node_id);
+void node_set_leaf_guidance(const node_id_t node_id, gboolean supported);
 
 void node_became_firewalled(void);
 void node_became_udp_firewalled(void);
 void node_set_socket_rx_size(gint rx_size);
 
-mqueue_t *node_udp_get_outq(void);
-void node_udp_enable(void);
+mqueue_t *node_udp_get_outq(enum net_type net);
 void node_udp_disable(void);
 void node_udp_process(struct gnutella_socket *s);
 gnutella_node_t *node_udp_get_addr_port(const host_addr_t addr, guint16 port);
@@ -590,9 +606,29 @@ guint feed_host_cache_from_headers(header_t *headers,
 	host_type_t sender, gboolean gnet, const host_addr_t peer);
 
 gnutella_node_t *node_browse_prepare(
-	gnet_host_t *host, const gchar *vendor, struct gnutella_header *header,
+	gnet_host_t *host, const gchar *vendor, gnutella_header_t *header,
 	gchar *data, guint32 size);
 void node_browse_cleanup(gnutella_node_t *n);
+void node_kill_hostiles(void);
+
+node_id_t node_id_get_self(void);
+gboolean node_id_self(const node_id_t node_id);
+
+static inline void
+node_check(const struct gnutella_node * const n)
+{
+	g_assert(n);
+	g_assert(NODE_MAGIC == n->magic);
+}
+
+static inline const gchar *
+node_guid(const struct gnutella_node * const n)
+{
+	return n->guid;
+}
+
+gboolean node_set_guid(struct gnutella_node *n, const gchar *guid);
+struct gnutella_node *node_by_guid(const gchar *guid);
 
 #endif /* _core_nodes_h_ */
 
