@@ -1,5 +1,5 @@
 /*
- * $Id: pcache.c 12147 2006-10-27 19:55:09Z cbiere $
+ * $Id: pcache.c 13893 2007-06-19 17:33:36Z cbiere $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi
  *
@@ -35,26 +35,27 @@
 
 #include "common.h"
 
-RCSID("$Id: pcache.c 12147 2006-10-27 19:55:09Z cbiere $")
+RCSID("$Id: pcache.c 13893 2007-06-19 17:33:36Z cbiere $")
 
-#include "sockets.h"
-#include "hosts.h"
-#include "hcache.h"
-#include "pcache.h"
-#include "nodes.h"
-#include "share.h" /* For shared_files_scanned() and shared_kbytes_scanned(). */
-#include "routing.h"
-#include "gmsg.h"
 #include "alive.h"
-#include "inet.h"
-#include "gnet_stats.h"
-#include "hostiles.h"
-#include "settings.h"
-#include "udp.h"
-#include "uhc.h"
 #include "extensions.h"
 #include "ggep.h"
 #include "ggep_type.h"
+#include "gmsg.h"
+#include "gnet_stats.h"
+#include "hcache.h"
+#include "hostiles.h"
+#include "hosts.h"
+#include "inet.h"
+#include "nodes.h"
+#include "pcache.h"
+#include "routing.h"
+#include "settings.h"
+#include "share.h" /* For shared_files_scanned() and shared_kbytes_scanned(). */
+#include "sockets.h"
+#include "tls_cache.h"
+#include "udp.h"
+#include "uhc.h"
 #include "version.h"
 
 #include "if/core/hosts.h"
@@ -112,7 +113,7 @@ send_ping(struct gnutella_node *n, guint8 ttl)
 
 		if (NODE_IS_WRITABLE(n)) {
 			n->n_ping_sent++;
-			gmsg_sendto_one_ggep(n, m, size, sizeof *m);
+			gmsg_sendto_one(n, m, size);
 		}
 	} else {
 		const GSList *sl_nodes = node_all_nodes();
@@ -124,13 +125,13 @@ send_ping(struct gnutella_node *n, guint8 ttl)
 		 */
 
 		for (sl = sl_nodes; sl; sl = g_slist_next(sl)) {
-			n = (struct gnutella_node *) sl->data;
+			n = sl->data;
 			if (!NODE_IS_WRITABLE(n))
 				continue;
 			n->n_ping_sent++;
 		}
 
-		gmsg_sendto_all_ggep(sl_nodes, m, size, sizeof *m);
+		gmsg_sendto_all(sl_nodes, m, size);
 	}
 }
 
@@ -187,7 +188,11 @@ build_ping_msg(const gchar *muid, guint8 ttl, gboolean uhc, guint32 *size)
 		ggep = cast_to_gpointer(&m[1]);
 		ggep_stream_init(&gs, ggep, sizeof msg_init.buf - sizeof *m);
 		
-		spp = (current_peermode == NODE_P_LEAF) ? 0x0 : 0x1;
+		spp = GNET_PROPERTY(current_peermode) == NODE_P_LEAF ? 0x0 : 0x1;
+#ifdef HAS_GNUTLS
+		spp |= 0x02;
+#endif	/* HAS_GNUTLS */
+		
 		ok = ggep_stream_pack(&gs, GGEP_NAME(SCP), &spp, sizeof spp, 0);
 		g_assert(ok);
 
@@ -312,9 +317,9 @@ build_pong_msg(host_addr_t sender_addr, guint16 sender_port,
 		}
 
 		if (meta->flags & PONG_META_HAS_DU) {	/* Daily average uptime */
-			gchar uptime[sizeof meta->daily_uptime];
+			gchar uptime[sizeof(guint64)];
 			guint32 value = MIN(meta->daily_uptime, 86400);
-			gint len;
+			guint len;
 
 			len = ggept_du_encode(value, uptime);
 			ggep_stream_pack(&gs, GGEP_NAME(DU), uptime, len, 0);
@@ -325,6 +330,9 @@ build_pong_msg(host_addr_t sender_addr, guint16 sender_port,
 				host_addr_ipv6(&meta->ipv6_addr), 16, 0);
 		}
 
+		if (meta->flags & PONG_META_HAS_TLS) {
+			ggep_stream_pack(&gs, GGEP_NAME(TLS), NULL, 0, 0);
+		}
 	}
 
 	/*
@@ -345,9 +353,10 @@ build_pong_msg(host_addr_t sender_addr, guint16 sender_port,
 		hcount = hcache_fill_caught_array(HOST_ULTRA, host, PCACHE_UHC_MAX_IP);
 
 		if (hcount > 0) {
-			gint i;
+			guchar tls_bytes[(G_N_ELEMENTS(host) + 7) / 8];
+			guint tls_index, tls_length;
 			gboolean ok;
-			gchar addr[6];
+			gint i;
 
 			/*
 			 * The binary data that makes up IPP does not deflate well.
@@ -356,20 +365,40 @@ build_pong_msg(host_addr_t sender_addr, guint16 sender_port,
 			 * pain and the CPU overhead.
 			 */
 
+			memset(tls_bytes, 0, sizeof tls_bytes);
+			tls_index = 0;
+			tls_length = 0;
+
 			ok = ggep_stream_begin(&gs, GGEP_NAME(IPP), 0);
 
 			for (i = 0; ok && i < hcount; i++) {
-				if (NET_TYPE_IPV4 == host_addr_net(host[i].addr)) {
-					guint32 ip;
+				gchar addr_buf[6];
+				host_addr_t addr;
+				guint16 port;
 
-					ip = host_addr_ipv4(host[i].addr); /* @todo TODO: IPv6 */
-					poke_be32(&addr[0], ip);
-					poke_le16(&addr[4], host[i].port);
-					ok = ggep_stream_write(&gs, addr, sizeof addr);
+				/* @todo TODO: IPv6 */
+				if (NET_TYPE_IPV4 != gnet_host_get_net(&host[i]))
+					continue;
+				
+				addr = gnet_host_get_addr(&host[i]);
+				port = gnet_host_get_port(&host[i]);
+				poke_be32(&addr_buf[0], host_addr_ipv4(addr));
+				poke_le16(&addr_buf[4], port);
+
+				ok = ggep_stream_write(&gs, addr_buf, sizeof addr_buf);
+
+				if (tls_cache_lookup(addr, port)) {
+					tls_bytes[tls_index >> 3] |= 0x80U >> (tls_index & 7);
+					tls_length = (tls_index >> 3) + 1;
 				}
+				tls_index++;
 			}
 
 			ok = ok && ggep_stream_end(&gs);
+			if (ok && tls_length > 0) {
+				ok = ggep_stream_pack(&gs, GGEP_NAME(IPP_TLS),
+						tls_bytes, tls_length, 0);
+			}
 		}
 	}
 
@@ -377,7 +406,7 @@ build_pong_msg(host_addr_t sender_addr, guint16 sender_port,
 		gchar ip_port[6];
 
 		/* Ip Port (not UHC IPP!)*/
-		if (pcache_debug || ggep_debug)
+		if (GNET_PROPERTY(pcache_debug) || GNET_PROPERTY(ggep_debug))
 			g_message("adding GGEP IP for %s",
 				host_addr_port_to_string(sender_addr, sender_port));
 
@@ -432,7 +461,7 @@ send_pong(
 	else if (control)
 		gmsg_ctrl_sendto_one(n, r, sizeof *r);
 	else
-		gmsg_sendto_one_ggep(n, r, size, sizeof *r);
+		gmsg_sendto_one(n, r, size);
 }
 
 /**
@@ -462,7 +491,8 @@ ping_type(const gnutella_node_t *n)
 
 				paylen = ext_paylen(e);
 				if (paylen >= 1) {
-					guint8 mask = ext_payload(e)[0];
+					const guchar *payload = ext_payload(e);
+					guint8 mask = payload[0];
 					flags |= (mask & 0x1) ? PING_F_UHC_ULTRA : PING_F_UHC_LEAF;
 				} else {
 					flags |= PING_F_UHC_ANY;
@@ -487,7 +517,7 @@ ping_type(const gnutella_node_t *n)
 
 	}
 
-	if ((flags & PING_F_UHC) && ggep_debug > 1)
+	if ((flags & PING_F_UHC) && GNET_PROPERTY(ggep_debug) > 1)
 		printf("%s: UHC ping requesting %s slots from %s\n",
 			gmsg_infostr(&n->header),
 			(flags & PING_F_UHC_ANY) ?	"unspecified" :
@@ -527,7 +557,7 @@ send_personal_info(struct gnutella_node *n, gboolean control,
 
 	kbytes = MIN(shared_kbytes_scanned(), ~((guint32) 0U));
 
-	if (current_peermode == NODE_P_ULTRA) {
+	if (GNET_PROPERTY(current_peermode) == NODE_P_ULTRA) {
 		guint32 next, prev;
 		
 		next = next_pow2(kbytes);
@@ -556,8 +586,8 @@ send_personal_info(struct gnutella_node *n, gboolean control,
 	 * every 12 hours, it makes no sense to advertise a high daily uptime...
 	 */
 
-	ip_uptime = delta_time(tm_time(), current_ip_stamp);
-	ip_uptime = MAX(ip_uptime, average_ip_uptime);
+	ip_uptime = delta_time(tm_time(), GNET_PROPERTY(current_ip_stamp));
+	ip_uptime = MAX(ip_uptime, GNET_PROPERTY(average_ip_uptime));
 	avg_uptime = get_average_servent_uptime(tm_time());
 	local_meta.daily_uptime = MIN(avg_uptime, ip_uptime);
 
@@ -565,7 +595,7 @@ send_personal_info(struct gnutella_node *n, gboolean control,
 	 * Activate "UP" only if we're an ultrapeer right now.
 	 */
 
-	if (current_peermode == NODE_P_ULTRA) {
+	if (GNET_PROPERTY(current_peermode) == NODE_P_ULTRA) {
 		local_meta.flags |= PONG_META_HAS_UP;
 		local_meta.up_slots = MIN(node_missing(), 255);
 		local_meta.leaf_slots = MIN(node_leaves_missing(), 255);
@@ -576,13 +606,20 @@ send_personal_info(struct gnutella_node *n, gboolean control,
 		local_meta.sender_port = n->port;
 	}
 
-	if (NET_TYPE_IPV6 == host_addr_net(listen_addr6())) {
+	if (
+		NET_TYPE_IPV6 == host_addr_net(listen_addr6()) &&
+		is_host_addr(listen_addr6())
+	) {
 		local_meta.ipv6_addr = listen_addr6();
 		local_meta.flags |= PONG_META_HAS_IPV6;
 	}
 
+#ifdef HAS_GNUTLS
+	local_meta.flags |= PONG_META_HAS_TLS;
+#endif	/* HAS_GNUTLS */
+
 	send_pong(n, control, flags, 0,
-		MIN((guint) gnutella_header_get_hops(&n->header) + 1, max_ttl),
+		MIN(gnutella_header_get_hops(&n->header) + 1U, GNET_PROPERTY(max_ttl)),
 		gnutella_header_get_muid(&n->header), &info, &local_meta);
 
 	/* Reset flags that must be recomputed each time */
@@ -604,7 +641,7 @@ send_neighbouring_info(struct gnutella_node *n)
 	g_assert(gnutella_header_get_ttl(&n->header) == 2);	/* "Crawler" ping */
 
 	for (sl = node_all_nodes(); sl; sl = g_slist_next(sl)) {
-		struct gnutella_node *cn = (struct gnutella_node *) sl->data;
+		struct gnutella_node *cn = sl->data;
 		struct pong_info info;
 
 		if (!NODE_IS_WRITABLE(cn))
@@ -664,12 +701,12 @@ send_neighbouring_info(struct gnutella_node *n)
  */
 
 static time_t pcache_expire_time = 0;
-static gpointer udp_pings = NULL;
+static struct aging *udp_pings;
 
 struct cached_pong {		/**< A cached pong */
 	gint refcount;			/**< How many lists reference us? */
-	guint32 node_id;		/**< The node ID from which we got that pong */
-	guint32 last_sent_id;	/**< Node ID to which we last sent this pong */
+	node_id_t node_id;		/**< The node ID from which we got that pong */
+	node_id_t last_sent_id; /**< Node ID we last sent this pong to */
 	struct pong_info info;	/**< Values from the pong message */
 	pong_meta_t *meta;		/**< Optional meta data */
 };
@@ -747,8 +784,7 @@ pcache_init(void)
 	 */
 
 	udp_pings = aging_make(UDP_PING_FREQ,
-			host_addr_hash_func, host_addr_eq_func, wfree_host_addr,
-			NULL, NULL, NULL);
+			host_addr_hash_func, host_addr_eq_func, wfree_host_addr);
 
 	/*
 	 * The `local_meta' structure collects our meta data that we may send
@@ -776,7 +812,7 @@ pcache_init(void)
 #define GET_LANG(x)										\
 G_STMT_START {											\
 	if (lang == NULL) {									\
-		lang = getenv(STRINGIFY(x));					\
+		lang = getenv(#x);								\
 		if (lang != NULL) {								\
 			if (strlen(lang) >= 3 && lang[2] != '_')	\
 				lang = NULL;							\
@@ -844,6 +880,9 @@ free_cached_pong(struct cached_pong *cp)
 
 	if (cp->meta)
 		wfree(cp->meta, sizeof(*cp->meta));
+
+	node_id_unref(cp->node_id);
+	node_id_unref(cp->last_sent_id);
 	wfree(cp, sizeof(*cp));
 }
 
@@ -925,7 +964,7 @@ found:
 	*addr =last_addr = cp->info.addr;
 	*port = last_port = cp->info.port;
 
-	if (pcache_debug > 8)
+	if (GNET_PROPERTY(pcache_debug) > 8)
 		printf("returning recent %s PONG %s\n",
 			host_type_to_string(type),
 			host_addr_port_to_string(cp->info.addr, cp->info.port));
@@ -957,7 +996,7 @@ add_recent_pong(host_type_t type, struct cached_pong *cp)
 
 	if (rec->recent_pong_count == RECENT_PING_SIZE) {		/* Full */
 		GList *lnk = g_list_last(rec->recent_pongs);
-		struct cached_pong *p = (struct cached_pong *) lnk->data;
+		struct cached_pong *p = lnk->data;
 
 		rec->recent_pongs = g_list_remove_link(rec->recent_pongs, lnk);
 		g_hash_table_remove(rec->ht_recent_pongs, p);
@@ -1007,7 +1046,7 @@ pcache_clear_recent(host_type_t type)
 	rec = &recent_pongs[type];
 
 	for (l = rec->recent_pongs; l; l = g_list_next(l)) {
-		struct cached_pong *cp = (struct cached_pong *) l->data;
+		struct cached_pong *cp = l->data;
 
 		g_hash_table_remove(rec->ht_recent_pongs, cp);
 		free_cached_pong(cp);
@@ -1033,8 +1072,11 @@ pcache_outgoing_connection(struct gnutella_node *n)
 {
 	g_assert(NODE_IS_CONNECTED(n));
 
-	if (connected_nodes() < up_connections || hcache_is_low(HOST_ANY))
-		send_ping(n, my_ttl);		/* Regular ping, get fresh pongs */
+	if (
+		connected_nodes() < GNET_PROPERTY(up_connections) ||
+		hcache_is_low(HOST_ANY)
+	)
+		send_ping(n, GNET_PROPERTY(my_ttl)); /* Regular ping, get fresh pongs */
 	else
 		send_ping(n, 1);			/* Handshaking ping */
 }
@@ -1054,7 +1096,7 @@ pcache_expire(void)
 
 		for (sl = cl->pongs; sl; sl = g_slist_next(sl)) {
 			entries++;
-			free_cached_pong((struct cached_pong *) sl->data);
+			free_cached_pong(sl->data);
 		}
 		g_slist_free(cl->pongs);
 
@@ -1062,7 +1104,7 @@ pcache_expire(void)
 		cl->cursor = NULL;
 	}
 
-	if (pcache_debug > 4)
+	if (GNET_PROPERTY(pcache_debug) > 4)
 		printf("Pong CACHE expired (%d entr%s, %d in reserve)\n",
 			entries, entries == 1 ? "y" : "ies", hcache_size(HOST_ANY));
 }
@@ -1115,7 +1157,7 @@ ping_all_neighbours(time_t now)
 	 */
 
 	for (sl = node_all_nodes(); sl; sl = g_slist_next(sl)) {
-		struct gnutella_node *n = (struct gnutella_node *) sl->data;
+		struct gnutella_node *n = sl->data;
 
 		if (!NODE_IS_WRITABLE(n) || NODE_IS_LEAF(n))
 			continue;
@@ -1136,7 +1178,7 @@ ping_all_neighbours(time_t now)
 	}
 
 	for (sl = may_ping, left = ping_cnt; sl; sl = g_slist_next(sl), left--) {
-		struct gnutella_node *n = (struct gnutella_node *) sl->data;
+		struct gnutella_node *n = sl->data;
 
 		if (
 			ping_cnt <= MIN_UP_PING ||
@@ -1149,12 +1191,12 @@ ping_all_neighbours(time_t now)
 	}
 
 	for (sl = to_ping; sl; sl = g_slist_next(sl)) {
-		struct gnutella_node *n = (struct gnutella_node *) sl->data;
+		struct gnutella_node *n = sl->data;
 
 		if (!(n->attrs & NODE_A_PONG_CACHING))
-			n->next_ping = now + OLD_PING_PERIOD;
+			n->next_ping = time_advance(now, OLD_PING_PERIOD);
 
-		send_ping(n, my_ttl);
+		send_ping(n, GNET_PROPERTY(my_ttl));
 	}
 
 	g_slist_free(may_ping);
@@ -1170,7 +1212,8 @@ pcache_possibly_expired(time_t now)
 {
 	if (delta_time(now, pcache_expire_time) >= 0) {
 		pcache_expire();
-		pcache_expire_time = now + cache_lifespan(current_peermode);
+		pcache_expire_time = time_advance(now,
+							cache_lifespan(GNET_PROPERTY(current_peermode)));
 		ping_all_neighbours(now);
 	}
 }
@@ -1236,7 +1279,7 @@ setup_pong_demultiplexing(struct gnutella_node *n, guint8 ttl)
 		guchar amount = (guchar) (remains / (MAX_CACHE_HOPS + 1 - h));
 		n->pong_needed[h] = amount;
 		remains -= amount;
-		if (pcache_debug > 7)
+		if (GNET_PROPERTY(pcache_debug) > 7)
 			printf("pong_needed[%d] = %d, remains = %d\n", h, amount, remains);
 	}
 
@@ -1262,7 +1305,7 @@ iterate_on_cached_line(
 
 	sl = start;
 	for (; sl && sl != end && n->pong_missing; sl = g_slist_next(sl)) {
-		struct cached_pong *cp = (struct cached_pong *) sl->data;
+		struct cached_pong *cp = sl->data;
 
 		cl->cursor = sl;
 
@@ -1281,11 +1324,13 @@ iterate_on_cached_line(
 		 * only if they strictly match the needed TTL.
 		 */
 
-		if (n->id == cp->node_id)
+		if (node_id_eq(NODE_ID(n), cp->node_id))
 			continue;
-		if (n->id == cp->last_sent_id)
+		if (node_id_eq(NODE_ID(n), cp->last_sent_id))
 			continue;
-		cp->last_sent_id = n->id;
+
+		node_id_unref(cp->last_sent_id);
+		cp->last_sent_id = node_id_ref(NODE_ID(n));
 
 		/*
 		 * When sending a cached pong, don't forget that its cached hop count
@@ -1301,7 +1346,7 @@ iterate_on_cached_line(
 
 		n->pong_missing--;
 
-		if (pcache_debug > 7)
+		if (GNET_PROPERTY(pcache_debug) > 7)
 			printf("iterate: sent cached pong %s (hops=%d, TTL=%d) to %s, "
 				"missing=%d %s\n",
 				host_addr_port_to_string(cp->info.addr, cp->info.port),
@@ -1363,9 +1408,8 @@ send_cached_pongs(
 static void
 send_demultiplexed_pongs(gnutella_node_t *n)
 {
-	gint h;
-	guint8 ttl;
 	enum ping_flag flags;
+	guint h, ttl;
 
 	/*
 	 * Look whether the "ping" they sent bore the "SCP" extension, meaning
@@ -1397,7 +1441,8 @@ send_demultiplexed_pongs(gnutella_node_t *n)
 	 * vector that is filled, and we'll have to send the pong afterwards.
 	 */
 
-	ttl = MIN((guint) gnutella_header_get_hops(&n->header) + 1, max_ttl);
+	ttl = gnutella_header_get_hops(&n->header) + 1U;
+	ttl = MIN(ttl, GNET_PROPERTY(max_ttl));
 
 	for (h = 0; n->pong_missing; h++) {
 		struct cache_line *cl;
@@ -1445,7 +1490,7 @@ pong_all_neighbours_but_one(
 	const GSList *sl;
 
 	for (sl = node_all_nodes(); sl; sl = g_slist_next(sl)) {
-		struct gnutella_node *cn = (struct gnutella_node *) sl->data;
+		struct gnutella_node *cn = sl->data;
 
 		if (cn == n)
 			continue;
@@ -1487,7 +1532,7 @@ pong_all_neighbours_but_one(
 		send_pong(cn, FALSE, PING_F_NONE,
 			hops + 1, ttl, cn->ping_guid, &cp->info, cp->meta);
 
-		if (pcache_debug > 7)
+		if (GNET_PROPERTY(pcache_debug) > 7)
 			printf("pong_all: sent cached pong %s (hops=%d, TTL=%d) to %s "
 				"missing=%d\n",
 				host_addr_port_to_string(cp->info.addr, cp->info.port),
@@ -1506,10 +1551,10 @@ pong_random_leaf(struct cached_pong *cp, guint8 hops, guint8 ttl)
 	gint leaves;
 	struct gnutella_node *leaf = NULL;
 
-	g_assert(current_peermode == NODE_P_ULTRA);
+	g_assert(GNET_PROPERTY(current_peermode) == NODE_P_ULTRA);
 
 	for (sl = node_all_nodes(), leaves = 0; sl; sl = g_slist_next(sl)) {
-		struct gnutella_node *cn = (struct gnutella_node *) sl->data;
+		struct gnutella_node *cn = sl->data;
 		gint threshold;
 
 		if (cn->pong_missing)	/* A job for pong_all_neighbours_but_one() */
@@ -1547,7 +1592,7 @@ pong_random_leaf(struct cached_pong *cp, guint8 hops, guint8 ttl)
 		send_pong(leaf, FALSE, PING_F_NONE, hops + 1, ttl, leaf->ping_guid,
 			&cp->info, cp->meta);
 
-		if (pcache_debug > 7)
+		if (GNET_PROPERTY(pcache_debug) > 7)
 			printf("pong_random_leaf: sent pong %s (hops=%d, TTL=%d) to %s\n",
 				host_addr_port_to_string(cp->info.addr, cp->info.port),
 				hops, ttl, node_addr(leaf));
@@ -1576,8 +1621,8 @@ pong_extract_metadata(struct gnutella_node *n)
 
 	for (i = 0; i < n->extcount; i++) {
 		extvec_t *e = &n->extvec[i];
+		const guchar *payload;
 		guint16 paylen;
-		const gchar *payload;
 
 		switch (e->ext_token) {
 		case EXT_T_GGEP_DU:
@@ -1601,9 +1646,12 @@ pong_extract_metadata(struct gnutella_node *n)
 			 */
 
 			ALLOCATE(GUE);
-			meta->guess = 0x1;
-			if (ext_paylen(e) >= 1)
-				meta->guess = *ext_payload(e);
+			if (ext_paylen(e) > 0) {
+				payload = ext_payload(e);
+				meta->guess = payload[0];
+			} else {
+				meta->guess = 0x1;
+			}
 			break;
 		case EXT_T_GGEP_LOC:
 			/*
@@ -1621,13 +1669,13 @@ pong_extract_metadata(struct gnutella_node *n)
 
 			paylen = ext_paylen(e);
 
-			if (paylen >= 2) {
+			if (paylen > 1) {
 				payload = ext_payload(e);
 				ALLOCATE(LOC);
 				memcpy(meta->language, payload, 2);
 				meta->country[0] = '\0';		/* Signals no country code */
-				if (paylen >= 5 && payload[2] == '_')
-					memcpy(meta->country, payload + 3, 2);
+				if (paylen > 4 && payload[2] == '_')
+					memcpy(meta->country, &payload[3], 2);
 			}
 			break;
 		case EXT_T_GGEP_UP:
@@ -1640,7 +1688,7 @@ pong_extract_metadata(struct gnutella_node *n)
 
 			paylen = ext_paylen(e);
 
-			if (paylen >= 3) {
+			if (paylen > 2) {
 				payload = ext_payload(e);
 				ALLOCATE(UP);
 				meta->version_up = payload[0];
@@ -1674,8 +1722,11 @@ pong_extract_metadata(struct gnutella_node *n)
 				}
 			}
 			break;
+		case EXT_T_GGEP_TLS:
+			ALLOCATE(TLS);
+			break;
 		default:
-			if (ggep_debug > 1 && e->ext_type == EXT_GGEP) {
+			if (GNET_PROPERTY(ggep_debug) > 1 && e->ext_type == EXT_GGEP) {
 				paylen = ext_paylen(e);
 				g_warning("%s: unhandled GGEP \"%s\" (%d byte%s)",
 					gmsg_infostr(&n->header), ext_ggep_id_str(e),
@@ -1711,8 +1762,8 @@ record_fresh_pong(
 	cp = walloc(sizeof *cp);
 
 	cp->refcount = 1;
-	cp->node_id = n->id;
-	cp->last_sent_id = n->id;
+	cp->node_id = node_id_ref(NODE_ID(n));
+	cp->last_sent_id = node_id_ref(NODE_ID(n));
 	cp->info.addr = addr;
 	cp->info.port = port;
 	cp->info.files_count = files_count;
@@ -1741,8 +1792,11 @@ pcache_udp_ping_received(struct gnutella_node *n)
 	 * noticed that we got an unsolicited UDP message.
 	 */
 
-	if (guid_eq(servent_guid, gnutella_header_get_muid(&n->header))) {
-		if (udp_debug > 19)
+	if (
+		guid_eq(GNET_PROPERTY(servent_guid),
+			gnutella_header_get_muid(&n->header))
+	) {
+		if (GNET_PROPERTY(udp_debug) > 19)
 			printf("UDP got unsolicited PING matching our GUID!\n");
 		return;
 	}
@@ -1813,7 +1867,7 @@ pcache_ping_received(struct gnutella_node *n)
 		if (gnutella_header_get_ttl(&n->header) == 1)
 			send_personal_info(n, TRUE, PING_F_NONE);	/* Prioritary */
 		else if (gnutella_header_get_ttl(&n->header) == 2) {
-			if (current_peermode != NODE_P_LEAF)
+			if (GNET_PROPERTY(current_peermode) != NODE_P_LEAF)
 				send_neighbouring_info(n);
 		} else
 			alive_ack_first(n->alive_pings,
@@ -1833,7 +1887,7 @@ pcache_ping_received(struct gnutella_node *n)
 		(n->attrs & (NODE_A_PONG_CACHING|NODE_A_PONG_ALIEN)) ==
 			NODE_A_PONG_CACHING
 	) {
-		if (pcache_debug || dbg)
+		if (GNET_PROPERTY(pcache_debug) || GNET_PROPERTY(dbg))
 			g_warning("node %s (%s) [%d.%d] claimed ping reduction, "
 				"got ping with hops=%d", node_addr(n),
 				node_vendor(n),
@@ -1879,14 +1933,17 @@ pcache_ping_received(struct gnutella_node *n)
 
 	if (
 		n->n_ping_accepted == 1 ||
-		((is_firewalled || node_missing() > 0) && inet_can_answer_ping())
+		(
+		 	(GNET_PROPERTY(is_firewalled) || node_missing() > 0) &&
+			inet_can_answer_ping()
+		)
 	) {
 		send_personal_info(n, FALSE, PING_F_NONE);
 		if (!NODE_IS_CONNECTED(n))	/* Can be removed if send queue is full */
 			return;
 	}
 
-	if (current_peermode == NODE_P_LEAF)
+	if (GNET_PROPERTY(current_peermode) == NODE_P_LEAF)
 		return;
 
 	/*
@@ -1902,10 +1959,22 @@ pcache_ping_received(struct gnutella_node *n)
 static void
 pcache_udp_pong_received(struct gnutella_node *n)
 {
+	host_addr_t ipv4_addr;
+	host_addr_t ipv6_addr;
+	gboolean supports_tls;
+	guint16 port;
 	gint i;
 
 	g_assert(NODE_IS_UDP(n));
 
+	if (!udp_ping_is_registered(gnutella_header_get_muid(&n->header)))
+		return;
+
+	port = peek_le16(&n->data[0]);
+	ipv4_addr = host_addr_peek_ipv4(&n->data[2]);
+	ipv6_addr = zero_host_addr;
+	supports_tls = FALSE;
+	
 	/*
 	 * We pretty much ignore pongs we get from UDP, unless they bear
 	 * the GGEP "IPP" extension, containing a packed set of IP:port.
@@ -1925,11 +1994,19 @@ pcache_udp_pong_received(struct gnutella_node *n)
 				g_warning("%s (UDP): bad length for GGEP \"%s\" (%d byte%s)",
 					gmsg_infostr(&n->header), ext_ggep_id_str(e),
 					paylen, paylen == 1 ? "" : "s");
-			} else
-				uhc_ipp_extract(n, payload, paylen);
+			} else {
+				uhc_ipp_extract(n, payload, paylen); 
+			}
+			break;
+		case EXT_T_GGEP_GTKG_IPV6:
+			ggept_gtkg_ipv6_extract(e, &ipv6_addr);
+			break;
+		case EXT_T_GGEP_TLS:
+		case EXT_T_GGEP_GTKG_TLS:
+			supports_tls = TRUE;
 			break;
 		default:
-			if (ggep_debug > 1 && e->ext_type == EXT_GGEP) {
+			if (GNET_PROPERTY(ggep_debug) > 1 && e->ext_type == EXT_GGEP) {
 				paylen = ext_paylen(e);
 				g_warning("%s (UDP): unhandled GGEP \"%s\" (%d byte%s)",
 					gmsg_infostr(&n->header), ext_ggep_id_str(e),
@@ -1938,6 +2015,32 @@ pcache_udp_pong_received(struct gnutella_node *n)
 			break;
 		}
 	}
+
+	if (port == n->port) {
+		host_addr_t addr;
+
+		switch (host_addr_net(n->addr)) {
+		case NET_TYPE_IPV4:
+			addr = ipv4_addr;
+			break;
+		case NET_TYPE_IPV6:
+			addr = ipv6_addr;
+			break;
+		default:
+			addr = zero_host_addr;
+		}
+
+		if (
+			host_addr_equal(addr, n->addr) &&
+			host_is_valid(addr, port) &&
+        	!hcache_node_is_bad(addr)
+		) {
+			host_add(addr, port, TRUE);
+			if (supports_tls) {
+				tls_cache_insert(addr, port);
+			}
+		}
+	}
 }
 
 /**
@@ -1976,7 +2079,7 @@ pcache_pong_received(struct gnutella_node *n)
 	 */
 
 	port = peek_le16(&n->data[0]);
-	addr = host_addr_get_ipv4(peek_be32(&n->data[2]));
+	addr = host_addr_peek_ipv4(&n->data[2]);
 	files_count = peek_le32(&n->data[6]);
 	kbytes_count = peek_le32(&n->data[10]);
 	
@@ -2004,14 +2107,15 @@ pcache_pong_received(struct gnutella_node *n)
 		swapped_count = swap_guint32(files_count);
 
 		if (swapped_count > PCACHE_MAX_FILES) {
-			if (pcache_debug && host_addr_equal(addr, n->addr))
+			if (GNET_PROPERTY(pcache_debug) && host_addr_equal(addr, n->addr))
 				g_warning("node %s (%s) sent us a pong with "
 					"large file count %u (0x%x), dropped",
 					node_addr(n), node_vendor(n), files_count, files_count);
 			n->rx_dropped++;
 			return;
 		} else {
-			if (pcache_debug && host_addr_equal(addr, n->addr)) g_warning(
+			if (GNET_PROPERTY(pcache_debug) && host_addr_equal(addr, n->addr))
+				g_warning(
 				"node %s (%s) sent us a pong with suspect file count %u "
 				"(fixed to %u)",
 				node_addr(n), node_vendor(n), files_count, swapped_count);
@@ -2045,7 +2149,7 @@ pcache_pong_received(struct gnutella_node *n)
 				n->gnet_addr = addr;	/* Signals: we have figured it out */
 				n->gnet_port = port;
 			} else if (!(n->flags & NODE_F_ALIEN_IP)) {
-				if (pcache_debug) g_warning(
+				if (GNET_PROPERTY(pcache_debug)) g_warning(
 					"node %s (%s) sent us a pong for itself with alien IP %s",
 					node_addr(n), node_vendor(n), host_addr_to_string(addr));
 				n->flags |= NODE_F_ALIEN_IP;	/* Probably firewalled */
@@ -2076,7 +2180,7 @@ pcache_pong_received(struct gnutella_node *n)
 			is_host_addr(n->gnet_pong_addr) &&
 			!host_addr_equal(addr, n->gnet_pong_addr)
 		) {
-			if (pcache_debug && n->n_ping_sent > 2) g_warning(
+			if (GNET_PROPERTY(pcache_debug) && n->n_ping_sent > 2) g_warning(
 				"node %s (%s) sent us a pong for new IP %s (used %s before)",
 				node_addr(n), node_vendor(n),
 				host_addr_port_to_string(addr, port),
@@ -2089,8 +2193,7 @@ pcache_pong_received(struct gnutella_node *n)
 		 * If it was an acknowledge for one of our alive pings, don't cache.
 		 */
 
-		if (alive_ack_ping(n->alive_pings,
-				gnutella_header_get_muid(&n->header)))
+		if (alive_ack_ping(n->alive_pings, gnutella_header_get_muid(&n->header)))
 			return;
 	}
 
@@ -2117,7 +2220,7 @@ pcache_pong_received(struct gnutella_node *n)
 	 * (tests) or someone is trying to fool us.
 	 */
 
-	if (is_my_address(addr, port))
+	if (is_my_address_and_port(addr, port))
 		return;
 
 	/*
@@ -2135,7 +2238,7 @@ pcache_pong_received(struct gnutella_node *n)
 	if (!(n->attrs & NODE_A_PONG_CACHING)) {
 		gint ratio = (gint) random_value(100);
 		if (ratio >= OLD_CACHE_RATIO) {
-			if (pcache_debug > 7)
+			if (GNET_PROPERTY(pcache_debug) > 7)
 				printf("NOT CACHED pong %s (hops=%d, TTL=%d) from OLD %s\n",
 					host_addr_port_to_string(addr, port),
 					gnutella_header_get_hops(&n->header),
@@ -2159,7 +2262,7 @@ pcache_pong_received(struct gnutella_node *n)
 	if (ptype == HOST_ULTRA)
 		add_recent_pong(HOST_ULTRA, cp);
 
-	if (pcache_debug > 6)
+	if (GNET_PROPERTY(pcache_debug) > 6)
 		printf("CACHED %s pong %s (hops=%d, TTL=%d) from %s %s\n",
 			ptype == HOST_ULTRA ? "ultra" : "normal",
 			host_addr_port_to_string(addr, port),
@@ -2172,7 +2275,7 @@ pcache_pong_received(struct gnutella_node *n)
 	 * received it from, provided they need more pongs of this hop count.
 	 */
 
-	if (current_peermode != NODE_P_LEAF)
+	if (GNET_PROPERTY(current_peermode) != NODE_P_LEAF)
 		pong_all_neighbours_but_one(n, cp, ptype,
 			CACHE_HOP_IDX(gnutella_header_get_hops(&n->header)),
 			MAX(1, gnutella_header_get_ttl(&n->header)));
@@ -2183,7 +2286,7 @@ pcache_pong_received(struct gnutella_node *n)
 	 */
 
 	if (
-		current_peermode == NODE_P_ULTRA &&
+		GNET_PROPERTY(current_peermode) == NODE_P_ULTRA &&
 		ptype == HOST_ULTRA && random_value(99) < 33
 	)
 		pong_random_leaf(cp,
