@@ -1,5 +1,5 @@
 /*
- * $Id: pproxy.c 12147 2006-10-27 19:55:09Z cbiere $
+ * $Id: pproxy.c 13815 2007-06-14 05:41:30Z cbiere $
  *
  * Copyright (c) 2003, Raphael Manfredi
  *
@@ -35,7 +35,7 @@
 
 #include "common.h"
 
-RCSID("$Id: pproxy.c 12147 2006-10-27 19:55:09Z cbiere $")
+RCSID("$Id: pproxy.c 13815 2007-06-14 05:41:30Z cbiere $")
 
 #include "pproxy.h"
 #include "http.h"
@@ -93,18 +93,12 @@ static void pproxy_error_remove(struct pproxy *pp, int code,
 static void
 pproxy_free_resources(struct pproxy *pp)
 {
-	if (pp->guid != NULL) {
-		atom_guid_free(pp->guid);
-		pp->guid = NULL;
-	}
+	atom_guid_free_null(&pp->guid);
 	if (pp->io_opaque != NULL) {
 		io_free(pp->io_opaque);
 		g_assert(pp->io_opaque == NULL);
 	}
-	if (pp->user_agent != NULL) {
-		atom_str_free(pp->user_agent);
-		pp->user_agent = NULL;
-	}
+	atom_str_free_null(&pp->user_agent);
 	if (pp->socket != NULL) {
 		g_assert(pp->socket->resource.pproxy == pp);
 		socket_free_null(&pp->socket);
@@ -200,8 +194,8 @@ pproxy_remove_v(struct pproxy *pp, const gchar *reason, va_list ap)
 		}
 	}
 
-	if (dbg > 1) {
-		printf("push-proxy: ending request from %s (%s): %s\n",
+	if (GNET_PROPERTY(push_proxy_debug) > 0) {
+		g_message("push-proxy: ending request from %s (%s): %s",
 			pp->socket ? host_addr_to_string(pp->socket->addr) : "<no socket>",
 			pproxy_vendor_str(pp),
 			logreason);
@@ -276,7 +270,7 @@ pproxy_timer(time_t now)
 
 		if (
 			delta_time(now, pp->last_update) >
-				(time_delta_t) upload_connecting_timeout
+				(time_delta_t) GNET_PROPERTY(upload_connecting_timeout)
 		) {
 			to_remove = g_slist_prepend(to_remove, pp);
 		}
@@ -319,7 +313,7 @@ pproxy_create(struct gnutella_socket *s)
  */
 static gboolean
 get_params(struct pproxy *pp, const gchar *request,
-	gchar **guid_atom, guint32 *file_idx)
+	const gchar **guid_atom, guint32 *file_idx, gboolean *supports_tls)
 {
 	static const struct {
 		const gchar *req;
@@ -341,6 +335,7 @@ get_params(struct pproxy *pp, const gchar *request,
 	g_assert(request);
 	g_assert(guid_atom);
 	g_assert(file_idx);
+	g_assert(supports_tls);
 	
 	/*
 	 * Move to the start of the requested path.  Note that sizeof("GET")
@@ -381,7 +376,7 @@ get_params(struct pproxy *pp, const gchar *request,
 	for (i = 0; i < G_N_ELEMENTS(req_types); i++) {
 		gchar *q;
 
-		if (NULL != (q = is_strprefix(req_types[i].req, uri))) {
+		if (NULL != (q = is_strprefix(uri, req_types[i].req))) {
 			attr = req_types[i].attr;
 			uri = q;
 			break;
@@ -431,8 +426,8 @@ get_params(struct pproxy *pp, const gchar *request,
 			goto error;
 		}
 
-		if (dbg > 4)
-			printf("PUSH-PROXY: decoding %s=%s as base32\n", attr, value);
+		if (GNET_PROPERTY(push_proxy_debug) > 0)
+			g_message("PUSH-PROXY: decoding %s=%s as base32", attr, value);
 
 		guid = base32_to_guid(value);
 		if (guid == NULL) {
@@ -456,8 +451,8 @@ get_params(struct pproxy *pp, const gchar *request,
 			goto error;
 		}
 
-		if (dbg > 4)
-			printf("PUSH-PROXY: decoding %s=%s as hexadecimal\n", attr, value);
+		if (GNET_PROPERTY(push_proxy_debug) > 0)
+			g_message("PUSH-PROXY: decoding %s=%s as hexadecimal", attr, value);
 
 		if (!hex_to_guid(value, guid)) {
 			pproxy_error_remove(pp, 400, "Malformed push-proxy request: "
@@ -475,11 +470,20 @@ get_params(struct pproxy *pp, const gchar *request,
 	 */
 
 	value = url_params_get(up, "file");
-	{
+	if (value) {
 		gint error;
 
 		/* Ignore errors; parse_uint32() returns 0 on error. */
-		*file_idx = value ? parse_uint32(value, NULL, 10, &error) : 0;
+		*file_idx = parse_uint32(value, NULL, 10, &error);
+	} else {
+		*file_idx = 0;
+	}
+
+	value = url_params_get(up, "tls");
+	if (value) {
+		*supports_tls = 0 == ascii_strcasecmp(value, "true");
+	} else {
+		*supports_tls = FALSE;
 	}
 
 	url_params_free(up);
@@ -503,10 +507,10 @@ error:
  * @param port the port number the receiving peer should connect to.
  * @param file_idx the file index this push is for.
  * @return	A pointer to a static buffer holding the created Gnutella PUSH
- *			packet on success, NULL on failure.
+ *			packet on success, an empty array on failure.
  */
-const gchar *
-build_push(size_t *size_ptr, guint8 ttl, guint8 hops, const gchar *guid,
+struct array
+build_push(guint8 ttl, guint8 hops, const gchar *guid,
 	host_addr_t addr_v4, host_addr_t addr_v6, guint16 port,
 	guint32 file_idx, gboolean supports_tls)
 {
@@ -518,7 +522,6 @@ build_push(size_t *size_ptr, guint8 ttl, guint8 hops, const gchar *guid,
 	size_t len = 0, size = sizeof packet;
 	ggep_stream_t gs;
 
-	g_assert(size_ptr);
 	g_assert(guid);
 	g_assert(0 != port);
 
@@ -541,19 +544,21 @@ build_push(size_t *size_ptr, guint8 ttl, guint8 hops, const gchar *guid,
 	ggep_stream_init(&gs, p, size);
 
 #ifdef HAS_GNUTLS
-	if (is_host_addr(addr_v4)) {
-		supports_tls |= is_my_address(addr_v4, port);
-	}
-	if (is_host_addr(addr_v6)) {
-		supports_tls |= is_my_address(addr_v6, port);
-	}
+	supports_tls = supports_tls
+		|| is_my_address_and_port(addr_v4, port)
+		|| is_my_address_and_port(addr_v6, port);
 #endif /* HAS_GNUTLS */
 	
 	if (supports_tls) {
+		if (!ggep_stream_pack(&gs, GGEP_NAME(TLS), NULL, 0, 0)) {
+			g_warning("could not write GGEP \"TLS\" extension into PUSH");
+			ggep_stream_close(&gs);
+			return zero_array;
+		}
 		if (!ggep_stream_pack(&gs, GGEP_GTKG_NAME(TLS), NULL, 0, 0)) {
 			g_warning("could not write GGEP \"GTKG.TLS\" extension into PUSH");
 			ggep_stream_close(&gs);
-			return NULL;
+			return zero_array;
 		}
 	}
 
@@ -563,7 +568,7 @@ build_push(size_t *size_ptr, guint8 ttl, guint8 hops, const gchar *guid,
 		if (!ggep_stream_pack(&gs, GGEP_GTKG_NAME(IPV6), ipv6, 16, 0)) {
 			g_warning("could not write GGEP \"GTKG.IPV6\" extension into PUSH");
 			ggep_stream_close(&gs);
-			return NULL;
+			return zero_array;
 		}
 	}
 
@@ -590,8 +595,7 @@ build_push(size_t *size_ptr, guint8 ttl, guint8 hops, const gchar *guid,
 		gnutella_header_get_muid(gnutella_msg_push_request_header(&packet.m)),
 		GTA_MSG_PUSH_REQUEST, NULL);
 
-	*size_ptr = p - packet.data;
-	return packet.data;
+	return array_init(packet.data, p - packet.data);
 }
 
 /**
@@ -599,10 +603,10 @@ build_push(size_t *size_ptr, guint8 ttl, guint8 hops, const gchar *guid,
  *
  * @return atom, or NULL.
  */
-static gchar *
+static const gchar *
 validate_vendor(gchar *vendor, gchar *token, const host_addr_t addr)
 {
-	gchar *result = NULL;
+	const gchar *result;
 
 	if (vendor) {
 		gboolean faked = !version_check(vendor, token, addr);
@@ -615,6 +619,8 @@ validate_vendor(gchar *vendor, gchar *token, const host_addr_t addr)
 			result = atom_str_get(name);
 		} else
 			result = atom_str_get(vendor);
+	} else {
+		result = NULL;
 	}
 
 	return result;
@@ -634,15 +640,13 @@ pproxy_request(struct pproxy *pp, header_t *header)
 	gchar *token;
 	gchar *user_agent;
 	GSList *nodes;
-	gboolean supports_tls;
+	gboolean supports_tls = FALSE;
 
-	if (dbg > 2) {
-		printf("----Push-proxy request from %s:\n",
-			host_addr_to_string(s->addr));
-		printf("%s\n", request);
-		header_dump(header, stdout);
-		printf("----\n");
-		fflush(stdout);
+	if (GNET_PROPERTY(push_proxy_debug) > 0) {
+		g_message("----Push-proxy request from %s:\n%s",
+			host_addr_to_string(s->addr), request);
+		header_dump(header, stderr);
+		g_message("----");
 	}
 
 	/*
@@ -658,16 +662,16 @@ pproxy_request(struct pproxy *pp, header_t *header)
 	 * Determine the servent ID.
 	 */
 
-	if (!get_params(pp, request, &pp->guid, &pp->file_idx))
+	if (!get_params(pp, request, &pp->guid, &pp->file_idx, &supports_tls))
 		return;				/* Already reported the error in get_params() */
 
-	if (dbg > 2)
-		printf("PUSH-PROXY: %s requesting a push to %s for file #%d\n",
+	supports_tls |= header_get_feature("tls", header, NULL, NULL);
+
+	if (GNET_PROPERTY(push_proxy_debug) > 0)
+		g_message("PUSH-PROXY: %s requesting a push to %s for file #%d",
 			host_addr_to_string(s->addr), guid_hex_str(pp->guid),
 			pp->file_idx);
 
-	supports_tls = header_get_feature("tls", header, NULL, NULL);
-			
 	/*
 	 * Make sure they provide an X-Node header so we know whom to set up
 	 * as the originator of the push.  Then validate the address.
@@ -760,25 +764,24 @@ pproxy_request(struct pproxy *pp, header_t *header)
 	n = route_proxy_find(pp->guid);
 
 	if (n != NULL) {
-		const gchar *packet;
-		size_t size;
+		struct array packet;
 
 		/*
  		 * We set TTL=max_ttl-1 and hops=1 since
 		 * it does not come from our node really.
 		 */
 
-		packet = build_push(&size, max_ttl - 1, 1, pp->guid,
-					pp->addr_v4, pp->addr_v6, pp->port, pp->file_idx,
-					supports_tls);
+		packet = build_push(GNET_PROPERTY(max_ttl) - 1, 1, pp->guid,
+					pp->addr_v4, pp->addr_v6, pp->port,
+					pp->file_idx, supports_tls);
 
-		if (NULL == packet) {
+		if (NULL == packet.data) {
 			g_warning("Failed to send push for %s/%s (index=%lu)",
 				host_addr_port_to_string(pp->addr_v4, pp->port),
 				host_addr_port_to_string2(pp->addr_v6, pp->port),
 				(gulong) pp->file_idx);
 		} else {
-			gmsg_sendto_one(n, &packet, size);
+			gmsg_sendto_one(n, packet.data, packet.size);
 			gnet_stats_count_general(GNR_PUSH_PROXY_RELAYED, 1);
 
 			http_send_status(pp->socket, 202, FALSE, NULL, 0,
@@ -797,19 +800,18 @@ pproxy_request(struct pproxy *pp, header_t *header)
 	 */
 
 	if (NULL != (nodes = route_towards_guid(pp->guid))) {
-		const gchar *packet;
-		size_t size;
+		struct array packet;
 
 		/*
  		 * We set TTL=max_ttl-1 and hops=1 since
 		 * it does not come from our node really.
 		 */
 
-		packet = build_push(&size, max_ttl - 1, 1,
-					pp->guid, pp->addr_v4, pp->addr_v6, pp->port,
+		packet = build_push(GNET_PROPERTY(max_ttl) - 1, 1, pp->guid,
+					pp->addr_v4, pp->addr_v6, pp->port,
 					pp->file_idx, supports_tls);
 
-		if (NULL == packet) {
+		if (NULL == packet.data) {
 			g_warning("Failed to send push to %s/%s (index=%lu)",
 				host_addr_port_to_string(pp->addr_v4, pp->port),
 				host_addr_port_to_string2(pp->addr_v6, pp->port),
@@ -817,7 +819,7 @@ pproxy_request(struct pproxy *pp, header_t *header)
 		} else {
 			gint cnt;
 
-			gmsg_sendto_all(nodes, packet, size);
+			gmsg_sendto_all(nodes, packet.data, packet.size);
 			gnet_stats_count_general(GNR_PUSH_PROXY_BROADCASTED, 1);
 
 			cnt = g_slist_length(nodes);
@@ -841,7 +843,7 @@ pproxy_request(struct pproxy *pp, header_t *header)
 	 * sending a GIV back.
 	 */
 
-	if (guid_eq(pp->guid, servent_guid)) {
+	if (guid_eq(pp->guid, GNET_PROPERTY(servent_guid))) {
 		upload_send_giv(pp->addr_v4, pp->port, 0, 1, 0,
 			"<from push-proxy>", FALSE, pp->flags);
 
@@ -957,7 +959,7 @@ pproxy_add(struct gnutella_socket *s)
 	 * Read HTTP headers, then call pproxy_request() when done.
 	 */
 
-	io_get_header(pp, &pp->io_opaque, bws.in, s, IO_HEAD_ONLY,
+	io_get_header(pp, &pp->io_opaque, BSCHED_BWS_IN, s, IO_HEAD_ONLY,
 		call_pproxy_request, NULL, &pproxy_io_error);
 }
 
@@ -983,13 +985,6 @@ pproxy_close(void)
  *** Client-side of push-proxy
  ***/
 
-/* XXX needed only because of the temporary hack below -- RAM, 05/08/2003 */
-static gboolean cproxy_http_header_ind(
-	gpointer handle, header_t *header, gint code, const gchar *message);
-static size_t cproxy_build_request(gpointer handle, gchar *buf, size_t len,
-	const gchar *verb, const gchar *path, const gchar *host, guint16 port);
-static void cproxy_http_newstate(gpointer handle, http_state_t newstate);
-
 #define CPROXY_MAGIC	0xc8301U
 
 /**
@@ -1000,18 +995,12 @@ cproxy_free(struct cproxy *cp)
 {
 	g_assert(cp->magic == CPROXY_MAGIC);
 
-	if (cp->guid != NULL) {
-		atom_guid_free(cp->guid);
-		cp->guid = NULL;
-	}
+	atom_guid_free_null(&cp->guid);
 	if (cp->http_handle != NULL) {
 		http_async_cancel(cp->http_handle);
 		cp->http_handle = NULL;
 	}
-	if (cp->server != NULL) {
-		atom_str_free(cp->server);
-		cp->server = NULL;
-	}
+	atom_str_free_null(&cp->server);
 
 	cp->magic = 0;
 	wfree(cp, sizeof *cp);
@@ -1021,9 +1010,10 @@ cproxy_free(struct cproxy *cp)
  * HTTP async callback for error notifications.
  */
 static void
-cproxy_http_error_ind(gpointer handle, http_errtype_t type, gpointer v)
+cproxy_http_error_ind(struct http_async *handle,
+	http_errtype_t type, gpointer v)
 {
-	struct cproxy *cp = (struct cproxy *) http_async_get_opaque(handle);
+	struct cproxy *cp = http_async_get_opaque(handle);
 
 	g_assert(cp != NULL);
 	g_assert(cp->magic == CPROXY_MAGIC);
@@ -1047,17 +1037,22 @@ cproxy_http_error_ind(gpointer handle, http_errtype_t type, gpointer v)
  * @returns whether processing can continue.
  */
 static gboolean
-cproxy_http_header_ind(gpointer handle, header_t *header,
+cproxy_http_header_ind(struct http_async *handle, header_t *header,
 	gint code, const gchar *message)
 {
 	struct cproxy *cp = http_async_get_opaque(handle);
 	gchar *token;
 	gchar *server;
+	gchar *to_free;
 
 	g_assert(cp != NULL);
 	g_assert(cp->d != NULL);
 	g_assert(cp->magic == CPROXY_MAGIC);
 
+	/* message is not valid anymore after http_async_cancel() */
+	to_free = g_strdup(message);
+	message = to_free;
+
 	/*
 	 * Extract vendor information.
 	 */
@@ -1113,12 +1108,14 @@ cproxy_http_header_ind(gpointer handle, header_t *header,
 		break;
 	}
 
-	if (dbg > 2 && cp->sent)
-		printf("PUSH-PROXY at %s (%s) sent PUSH for %s file #%u %s\n",
+	if (GNET_PROPERTY(push_proxy_debug) > 0 && cp->sent)
+		g_message("PUSH-PROXY at %s (%s) sent PUSH for %s file #%u %s",
 			host_addr_port_to_string(cp->addr, cp->port), cproxy_vendor_str(cp),
 			guid_hex_str(cp->guid), cp->file_idx,
 			cp->directly ? "directly" : "via Gnet");
 
+	G_FREE_NULL(to_free);
+
 	return FALSE;		/* Don't continue -- handle invalid now anyway */
 }
 
@@ -1131,7 +1128,7 @@ cproxy_http_header_ind(gpointer handle, header_t *header,
  * @return length of generated request.
  */
 static size_t
-cproxy_build_request(gpointer unused_handle, gchar *buf, size_t len,
+cproxy_build_request(struct http_async *unused_handle, gchar *buf, size_t len,
 	const gchar *verb, const gchar *path, const gchar *unused_host,
 	guint16 unused_port)
 {
@@ -1152,14 +1149,14 @@ cproxy_build_request(gpointer unused_handle, gchar *buf, size_t len,
 	addr = listen_addr();
 	if (is_host_addr(addr)) {
 		gm_snprintf(addr_buf, sizeof addr_buf, "X-Node: %s\r\n",
-			host_addr_port_to_string(addr, listen_port));
+			host_addr_port_to_string(addr, GNET_PROPERTY(listen_port)));
 	} else {
 		addr_buf[0] = '\0';
 	}
 	addr = listen_addr6();
 	if (is_host_addr(addr)) {
 		gm_snprintf(addr_v6_buf, sizeof addr_v6_buf, "X-Node: %s\r\n",
-			host_addr_port_to_string(addr, listen_port));
+			host_addr_port_to_string(addr, GNET_PROPERTY(listen_port)));
 	} else {
 		addr_v6_buf[0] = '\0';
 	}
@@ -1181,7 +1178,7 @@ cproxy_build_request(gpointer unused_handle, gchar *buf, size_t len,
  * Invoked when the state of the HTTP async request changes.
  */
 static void
-cproxy_http_newstate(gpointer handle, http_state_t newstate)
+cproxy_http_newstate(struct http_async *handle, http_state_t newstate)
 {
 	struct cproxy *cp = http_async_get_opaque(handle);
 
@@ -1200,14 +1197,18 @@ cproxy_http_newstate(gpointer handle, http_state_t newstate)
  */
 struct cproxy *
 cproxy_create(struct download *d, const host_addr_t addr, guint16 port,
-	gchar *guid, guint32 file_idx)
+	const gchar *guid, guint32 file_idx)
 {
+	struct http_async *handle;
 	struct cproxy *cp;
-	gpointer handle;
-	static gchar path[128];
+	gchar path[128];
 
-	(void) gm_snprintf(path, sizeof(path),
-		"/gnutella/push-proxy?ServerId=%s", guid_base32_str(guid));
+	concat_strings(path, sizeof path,
+		"/gnutella/push-proxy?ServerId=", guid_base32_str(guid),
+#ifdef HAS_GNUTLS
+		"&tls=true",
+#endif /* HAS_GNUTLS */
+		(void *) 0);
 
 	/*
 	 * Try to connect immediately: if we can't connect, no need to continue.
