@@ -1,5 +1,5 @@
 /*
- * $Id: qhit.c 12242 2006-11-06 20:41:55Z cbiere $
+ * $Id: qhit.c 13893 2007-06-19 17:33:36Z cbiere $
  *
  * Copyright (c) 2004, Raphael Manfredi
  *
@@ -35,23 +35,25 @@
 
 #include "common.h"
 
-RCSID("$Id: qhit.c 12242 2006-11-06 20:41:55Z cbiere $")
+RCSID("$Id: qhit.c 13893 2007-06-19 17:33:36Z cbiere $")
 
-#include "qhit.h"
-#include "gnutella.h"
+#include "bsched.h"
+#include "dmesh.h"		/* For dmesh_fill_alternate() */
 #include "ggep.h"
 #include "ggep_type.h"
 #include "gmsg.h"
-#include "share.h"
+#include "gnutella.h"
 #include "nodes.h"
-#include "bsched.h"
-#include "dmesh.h"		/* For dmesh_fill_alternate() */
-#include "uploads.h"	/* For count_uploads */
+#include "qhit.h"
 #include "settings.h"	/* For listen_ip() */
+#include "share.h"
+#include "tls_cache.h"
+#include "uploads.h"	/* For count_uploads */
 
 #include "if/gnet_property_priv.h"
 #include "if/core/main.h"			/* For main_get_build() */
 
+#include "lib/array.h"
 #include "lib/getdate.h"
 #include "lib/endian.h"
 #include "lib/misc.h"
@@ -83,11 +85,12 @@ struct found_struct {
 	size_t pos;					/**< current write position */
 	size_t files;				/**< amount of file entries */
 	size_t max_size;			/**< max query hit size */
-	gboolean use_ggep_h;		/**< whether to use GGEP "H" to send SHA1 */
 	const gchar *muid;			/**< the MUID to put in all query hits */
+	const struct array *token;	/**< Optional secure OOB token */
 	qhit_process_t process;		/**< processor once query hit is built */
 	gpointer udata;				/**< processor argument */
 	gboolean open;				/**< Set if found_open() was used */
+	gboolean use_ggep_h;        /**< whether to use GGEP "H" to send SHA-1 */
 };
 
 static struct found_struct *
@@ -115,7 +118,7 @@ found_add_files(size_t n)
 	found_get()->files += n;
 }
 
-static gboolean 
+static gboolean
 found_ggep_h(void)
 {
 	return found_get()->use_ggep_h;
@@ -217,14 +220,22 @@ found_set_header(void)
 	 * Compute connection speed dynamically if requested.
 	 */
 
-	connect_speed = connection_speed;
-	if (compute_connection_speed) {
-		connect_speed = max_uploads == 0 ?
-			0 : (MAX(bsched_avg_bps(bws.out), bsched_bwps(bws.out)) * 8 / 1024);
-		if (max_uploads > 0 && connect_speed == 0)
-			connect_speed = 32;		/* No b/w limit set and no traffic yet */
+	connect_speed = GNET_PROPERTY(connection_speed);
+	if (GNET_PROPERTY(compute_connection_speed)) {
+		if (GNET_PROPERTY(max_uploads) > 0) {
+			connect_speed = bsched_avg_bps(BSCHED_BWS_OUT);
+			connect_speed = MAX(connect_speed,
+								bsched_bw_per_second(BSCHED_BWS_OUT));
+			connect_speed /= 1024 / 8;
+			if (connect_speed == 0) {
+				/* No b/w limit set and no traffic yet */
+				connect_speed = 32;
+			}
+		} else {
+			connect_speed = 0;
+		}
 	}
-	connect_speed /= MAX(1, max_uploads);	/* Upload speed expected per slot */
+	connect_speed /= MAX(1, GNET_PROPERTY(max_uploads));	/* Upload speed expected per slot */
 
 	gnutella_msg_search_results_set_host_port(msg, socket_listen_port());
 	gnutella_msg_search_results_set_host_ip(msg, host_addr_ipv4(listen_addr()));
@@ -251,15 +262,23 @@ found_process(void)
 	f->process(f->data, f->pos, f->udata);
 }
 
+static const struct array *
+found_token(void)
+{
+	struct found_struct *f = found_get();
+	return f->token;
+}
+
 static void
 found_init(size_t max_size, const gchar *xuid, gboolean ggep_h,
-	qhit_process_t proc, gpointer udata)
+	qhit_process_t proc, gpointer udata, const struct array *token)
 {
 	struct found_struct *f = found_get();
 
 	g_assert(max_size <= INT_MAX);
 	g_assert(xuid != NULL);
 	g_assert(proc != NULL);
+	g_assert(token != NULL);
 
 	f->max_size = max_size;
 	f->muid = xuid;
@@ -267,6 +286,7 @@ found_init(size_t max_size, const gchar *xuid, gboolean ggep_h,
 	f->process = proc;
 	f->udata = udata;
 	f->open = FALSE;
+	f->token = token;
 }
 
 static time_t release_date;
@@ -279,9 +299,10 @@ qhit_send_node(gpointer data, size_t len, gpointer udata)
 {
 	gnutella_node_t *n = udata;
 	gnutella_header_t *packet_head = data;
+	guint ttl;
 
-	if (dbg > 3) {
-		g_message("flushing query hit (%u entr%s, %u bytes sofar) to %s\n",
+	if (GNET_PROPERTY(dbg) > 3) {
+		g_message("flushing query hit (%u entr%s, %u bytes sofar) to %s",
 			(guint) found_file_count(),
 			found_file_count() == 1 ? "y" : "ies",
 			(guint) found_size(),
@@ -305,8 +326,9 @@ qhit_send_node(gpointer data, size_t len, gpointer udata)
 		gnutella_header_set_hops(&n->header, 1);
 	}
 
-	gnutella_header_set_ttl(packet_head,
-		MIN((guint) gnutella_header_get_hops(&n->header) + 5, hard_ttl_limit));
+	ttl = gnutella_header_get_hops(&n->header) + 5U;
+	ttl = MIN(ttl, GNET_PROPERTY(hard_ttl_limit));
+	gnutella_header_set_ttl(packet_head, ttl);
 
 	gmsg_sendto_one(n, data, len);
 }
@@ -323,7 +345,7 @@ flush_match(void)
 	gchar *trailer_start;
 
 	/*
-	 * Build Gtk-Gnutella trailer.
+	 * Build gtk-gnutella trailer.
 	 * It is compatible with BearShare's one in the "open data" section.
 	 */
 
@@ -332,11 +354,11 @@ flush_match(void)
 	trailer[5] = 0x04 | 0x08 | 0x20;	/* Valid flags we set */
 	trailer[6] = 0x01;				/* Our flags (valid firewall bit) */
 
-	if (ul_running >= max_uploads)
+	if (GNET_PROPERTY(ul_running) >= GNET_PROPERTY(max_uploads))
 		trailer[6] |= 0x04;			/* Busy flag */
-	if (total_uploads > 0)
+	if (GNET_PROPERTY(total_uploads) > 0)
 		trailer[6] |= 0x08;			/* One file uploaded, at least */
-	if (is_firewalled)
+	if (GNET_PROPERTY(is_firewalled))
 		trailer[5] |= 0x01;			/* Firewall bit set in enabling byte */
 
 	/*
@@ -398,34 +420,52 @@ flush_match(void)
 			g_warning("could not write GGEP \"GTKGV1\" extension in query hit");
 	}
 
+	{
+		const struct array *token = found_token();
+		
+		if (
+			token->data &&
+			!ggep_stream_pack(&gs, GGEP_NAME(SO), token->data, token->size, 0)
+		)
+			g_warning("could not add GGEP \"SO\" extension to query hit");
+	}
+
 	/*
 	 * Look whether we'll need a "PUSH" GGEP extension to give out
 	 * our current push proxies.  Prepare payload in `proxies'.
 	 */
 
-	if (is_firewalled) {
-		GSList *nodes = node_push_proxies();
+	if (GNET_PROPERTY(is_firewalled)) {
+		const GSList *nodes = node_push_proxies();
 
 		if (nodes != NULL) {
-			GSList *l;
-			gint count;
-			gchar proxy[6];
+			guchar tls_bytes[(QHIT_MAX_PROXIES + 7) / 8];
+			guint tls_index, tls_length;
+			const GSList *iter;
 			gboolean ok;
 
 			ok = ggep_stream_begin(&gs, GGEP_NAME(PUSH), 0);
+			memset(tls_bytes, 0, sizeof tls_bytes);
+			tls_index = 0;
+			tls_length = 0;
 
-			for (
-				l = nodes, count = 0;
-				ok && l && count < QHIT_MAX_PROXIES;
-				l = g_slist_next(l), count++
-			) {
-				struct gnutella_node *n = l->data;
+			for (iter = nodes; ok && NULL != iter; iter = g_slist_next(iter)) {
+				const struct gnutella_node *n = iter->data;
+				gchar proxy[6];
 
-				if (NET_TYPE_IPV4 == host_addr_net(n->proxy_addr)) {
-					WRITE_GUINT32_BE(host_addr_ipv4(n->proxy_addr), &proxy[0]);
-					WRITE_GUINT16_LE(n->proxy_port, &proxy[4]);
-					ok = ggep_stream_write(&gs, proxy, sizeof proxy);
+				if (NET_TYPE_IPV4 != host_addr_net(n->proxy_addr))
+					continue;
+
+				poke_be32(&proxy[0], host_addr_ipv4(n->proxy_addr));
+				poke_le16(&proxy[4], n->proxy_port);
+				ok = ggep_stream_write(&gs, proxy, sizeof proxy);
+				if (NODE_F_CAN_TLS & n->flags) {
+					tls_bytes[tls_index >> 3] |= 0x80U >> (tls_index & 7);
+					tls_length = (tls_index >> 3) + 1;
 				}
+				tls_index++;
+				if (tls_index >= QHIT_MAX_PROXIES)
+					break;
 			}
 
 			ok = ok && ggep_stream_end(&gs);
@@ -433,6 +473,13 @@ flush_match(void)
 			if (!ok)
 				g_warning("could not write GGEP \"PUSH\" extension "
 					"in query hit");
+
+			if (ok && tls_length > 0) {
+				ok = ggep_stream_pack(&gs, GGEP_NAME(PUSH_TLS),
+						tls_bytes, tls_length, 0);
+				if (!ok)
+					g_warning("could not write GGEP \"PUSH_TLS\" extension");
+			}
 		}
 	}
 
@@ -441,11 +488,16 @@ flush_match(void)
 	 * server's hostname.
 	 */
 
-	if (!is_firewalled && give_server_hostname && '\0' != server_hostname[0]) {
+	if (
+		!GNET_PROPERTY(is_firewalled) &&
+		'\0' != GNET_PROPERTY(server_hostname)[0]
+	) {
 		gboolean ok;
 
 		ok = ggep_stream_pack(&gs, GGEP_NAME(HNAME),
-				server_hostname, strlen(server_hostname), 0);
+				GNET_PROPERTY(server_hostname),
+				strlen(GNET_PROPERTY(server_hostname)),
+				0);
 
 		if (!ok)
 			g_warning("could not write GGEP \"HNAME\" extension "
@@ -465,6 +517,8 @@ flush_match(void)
 	}
 
 #ifdef HAS_GNUTLS
+	if (!ggep_stream_pack(&gs, GGEP_NAME(TLS), NULL, 0, 0))
+		g_warning("could not write GGEP \"TLS\" extension into query hit");
 	if (!ggep_stream_pack(&gs, GGEP_GTKG_NAME(TLS), NULL, 0, 0))
 		g_warning("could not write GGEP \"GTKG.TLS\" extension into query hit");
 #endif /* HAS_GNUTLS */
@@ -475,7 +529,7 @@ flush_match(void)
 	 * enabled.
      */
 
-	if (browse_host_enabled) {
+	if (GNET_PROPERTY(browse_host_enabled)) {
 		if (!ggep_stream_pack(&gs, GGEP_NAME(BH), NULL, 0, 0))
 			g_warning("could not write GGEP \"BH\" extension into query hit");
 	}
@@ -490,7 +544,7 @@ flush_match(void)
 	 * Store the GUID in the last 16 bytes of the query hit.
 	 */
 
-	if (!found_write(servent_guid, 16))
+	if (!found_write(GNET_PROPERTY(servent_guid), GUID_RAW_SIZE))
 		goto failure;
 
 	found_set_header();
@@ -527,8 +581,7 @@ add_file(const struct shared_file *sf)
 
 	needed = 8 + 2 + shared_file_name_nfc_len(sf);	/* size of hit entry */
 
-	sha1_available = SHARE_F_HAS_DIGEST ==
-		(shared_file_flags(sf) & (SHARE_F_HAS_DIGEST | SHARE_F_RECOMPUTING));
+	sha1_available = sha1_hash_available(sf);
 
 	/*
 	 * In case we emit the SHA1 as a GGEP "H", we'll grow the buffer
@@ -541,22 +594,9 @@ add_file(const struct shared_file *sf)
 	 */
 
 	if (sha1_available) {
-		gint i, j;
-
 		needed += 9 + SHA1_BASE32_SIZE;
-		hcnt = dmesh_fill_alternate(shared_file_sha1(sf), hvec, QHIT_MAX_ALT);
-
-		/* Scrap all non-IPv4 alt-locs. Unfortunately there is no
-		 * GGEP item to hold IPv6 addresses as of yet.
-		 */
-		for (i = 0, j = 0; i < hcnt; i++) {
-			if (NET_TYPE_IPV4 == host_addr_net(hvec[i].addr) && j != i) {
-				hvec[j] = hvec[i];
-				j++;
-			}
-		}
-		hcnt = j;
-
+		hcnt = dmesh_fill_alternate(shared_file_sha1(sf),
+					hvec, G_N_ELEMENTS(hvec));
 		needed += hcnt * 6 + 6;
 	}
 
@@ -566,7 +606,7 @@ add_file(const struct shared_file *sf)
 
 	if (
 		found_size() + needed + QHIT_MIN_TRAILER_LEN
-			> search_answers_forward_size
+			> GNET_PROPERTY(search_answers_forward_size)
 	)
 		return FALSE;
 
@@ -585,10 +625,10 @@ add_file(const struct shared_file *sf)
 
 	fs32 = shared_file_size(sf) >= (1U << 31) ? ~0U : shared_file_size(sf);
 
-	WRITE_GUINT32_LE(shared_file_index(sf), &idx_le);
+	poke_le32(&idx_le, shared_file_index(sf));
 	if (!found_write(&idx_le, sizeof idx_le))
 		return FALSE;
-	WRITE_GUINT32_LE(fs32, &fs32_le);
+	poke_le32(&fs32_le, fs32);
 	if (!found_write(&fs32_le, sizeof fs32_le))
 		return FALSE;
 	if (!found_write(shared_file_name_nfc(sf), shared_file_name_nfc_len(sf)))
@@ -610,12 +650,14 @@ add_file(const struct shared_file *sf)
 	if (sha1_available && !found_ggep_h()) {
 		static const gchar urnsha1[] = "urn:sha1:";
 		const gchar *b32 = sha1_base32(shared_file_sha1(sf));
-		/* Good old way: ASCII URN */
 
+		/* Good old way: ASCII URN */
 		if (!found_write(urnsha1, CONST_STRLEN(urnsha1)))
 			return FALSE;
 		if (!found_write(b32, SHA1_BASE32_SIZE))
 			return FALSE;
+		if (!found_write("\x1c", 1))
+			return FALSE;
 	}
 
 	/*
@@ -668,27 +710,84 @@ add_file(const struct shared_file *sf)
 	 */
 
 	if (hcnt > 0) {
+		guchar tls_bytes[(G_N_ELEMENTS(hvec) + 7) / 8];
+		guint tls_index, tls_length;
 		gint i;
 
 		g_assert(hcnt <= QHIT_MAX_ALT);
+		memset(tls_bytes, 0, sizeof tls_bytes);
+		tls_index = 0;
+		tls_length = 0;
 
 		ok = ggep_stream_begin(&gs, GGEP_NAME(ALT), GGEP_W_COBS);
 
 		for (i = 0; ok && i < hcnt; i++) {
+			host_addr_t addr;
+			guint16 port;
+			gchar alt[6];
+
 			g_assert(start == gs.outbuf);
-			if (NET_TYPE_IPV4 == host_addr_net(hvec[i].addr)) {
-				gchar alt[6];
-			
-				poke_be32(&alt[0], host_addr_ipv4(hvec[i].addr));
-				poke_le16(&alt[4], hvec[i].port);
-				ok = ggep_stream_write(&gs, &alt, sizeof alt);
+			if (NET_TYPE_IPV4 != gnet_host_get_net(&hvec[i]))
+				continue;
+
+			addr = gnet_host_get_addr(&hvec[i]);
+			port = gnet_host_get_port(&hvec[i]);
+			poke_be32(&alt[0], host_addr_ipv4(addr));
+			poke_le16(&alt[4], port);
+			ok = ggep_stream_write(&gs, &alt, sizeof alt);
+
+			if (tls_cache_lookup(addr, port)) {
+				tls_bytes[tls_index >> 3] |= 0x80U >> (tls_index & 7);
+				tls_length = (tls_index >> 3) + 1;
 			}
+			tls_index++;
 		}
 
 		ok = ok && ggep_stream_end(&gs);
 
 		if (!ok)
 			g_warning("could not write GGEP \"ALT\" extension in query hit");
+
+		if (ok && tls_length > 0) {
+			ok = ggep_stream_pack(&gs, GGEP_NAME(ALT_TLS),
+					tls_bytes, tls_length, GGEP_W_COBS);
+			if (!ok)
+				g_warning("could not write GGEP \"ALT_TLS\" extension");
+		}
+	}
+
+	{
+		const gchar *rp = shared_file_relative_path(sf);
+		
+		if (rp) {
+			ok = ggep_stream_pack(&gs, GGEP_NAME(PATH), rp, strlen(rp), 0);
+			if (!ok)
+				g_warning("could not add GGEP \"PATH\" extension to query hit");
+		}
+	}
+
+	{
+		time_t mtime;	
+
+		/* FIXME: Create time != modification time */
+		mtime = shared_file_modification_time(sf);
+		if ((time_t) -1 != mtime) {
+			gchar buf[sizeof(guint64)];
+			gint len;
+
+			/*
+			 * Suppress negative values (if time_t is signed) as this would
+			 * be interpreted as a date far in this future.
+			 */
+			mtime = MAX(0, mtime);
+
+			len = ggept_ct_encode(mtime, buf);
+			g_assert(len >= 0 && len <= (gint) sizeof buf);
+
+			ok = ggep_stream_pack(&gs, GGEP_NAME(CT), buf, len, GGEP_W_COBS);
+			if (!ok)
+				g_warning("could not write GGEP \"CT\" extension in query hit");
+		}
 	}
 
 	/*
@@ -727,8 +826,6 @@ add_file(const struct shared_file *sf)
  *
  * @param max_size the maximum size in bytes of individual query hits
  *
- * @param use_ggep_h whether GGEP "H" can be used to send the SHA1 of files
- *
  * @param muid is the MUID that should be put in all the generated hits.
  * This must point to a memory location that is guaranteed to stay accurate
  * during all the processing.
@@ -741,12 +838,12 @@ add_file(const struct shared_file *sf)
  * have to be sent out-of-bound
  */
 static void
-found_reset(size_t max_size, const gchar *muid,
-	gboolean use_ggep_h, qhit_process_t process, gpointer udata)
+found_reset(size_t max_size, const gchar *muid, gboolean ggep_h,
+	qhit_process_t process, gpointer udata, const struct array *token)
 {
 	g_assert(process != NULL);
 	g_assert(max_size <= INT_MAX);
-	found_init(max_size, muid, use_ggep_h, process, udata);
+	found_init(max_size, muid, ggep_h, process, udata, token);
 	found_clear();
 }
 
@@ -758,12 +855,10 @@ found_reset(size_t max_size, const gchar *muid,
  * @param files			the list of shared_file_t entries that make up results
  * @param count			the amount of results
  * @param muid			the query's MUID
- * @param use_ggep_h	whether GGEP "H" can be used to send the SHA1 of files
  */
 void
-qhit_send_results(
-	struct gnutella_node *n, GSList *files, gint count,
-	const gchar *muid, gboolean use_ggep_h)
+qhit_send_results(struct gnutella_node *n, GSList *files, gint count,
+	const gchar *muid, gboolean ggep_h)
 {
 	GSList *sl;
 	gint sent = 0;
@@ -776,13 +871,14 @@ qhit_send_results(
 	 * to forward to other nodes).
 	 */
 
-	found_reset(QHIT_SIZE_THRESHOLD, muid, use_ggep_h, qhit_send_node, n);
+	found_reset(QHIT_SIZE_THRESHOLD, muid, ggep_h, qhit_send_node, n,
+		&zero_array);
 
 	for (sl = files; sl; sl = g_slist_next(sl)) {
 		shared_file_t *sf = sl->data;
 		if (add_file(sf))
 			sent++;
-		shared_file_unref(sf);
+		shared_file_unref(&sf);
 	}
 
 	if (0 != found_file_count())	/* Still some unflushed results */
@@ -790,8 +886,8 @@ qhit_send_results(
 
 	g_slist_free(files);
 
-	if (dbg > 3)
-		g_message("sent %d/%d hits to %s\n", sent, count, node_addr(n));
+	if (GNET_PROPERTY(dbg) > 3)
+		g_message("sent %d/%d hits to %s", sent, count, node_addr(n));
 }
 
 /**
@@ -815,20 +911,19 @@ qhit_send_results(
  * @param cb			the processor callback to invoke on each built hit
  * @param udata			argument to pass to callback
  * @param muid			the MUID to use on each generated hit
- * @param use_ggep_h	whether GGEP "H" can be used to send the SHA1 of files
  */
 void
-qhit_build_results(
-	const GSList *files, gint count,
-	size_t max_msgsize,
-	qhit_process_t cb, gpointer udata,
-	const gchar *muid, gboolean use_ggep_h)
+qhit_build_results(const GSList *files, gint count, size_t max_msgsize,
+	qhit_process_t cb, gpointer udata, const gchar *muid, gboolean ggep_h,
+	const struct array *token)
 {
 	const GSList *sl;
 	gint sent;
 
 	g_assert(cb != NULL);
-	found_reset(max_msgsize, muid, use_ggep_h, cb, udata);
+	g_assert(token);
+
+	found_reset(max_msgsize, muid, ggep_h, cb, udata, token);
 
 	for (sl = files, sent = 0; sl && sent < count; sl = g_slist_next(sl)) {
 		const shared_file_t *sf = sl->data;
