@@ -1,5 +1,5 @@
 /*
- * $Id: qhit.h,v 1.7 2005/08/28 22:31:07 rmanfredi Exp $
+ * $Id: qhit.h 13164 2007-03-20 20:20:00Z cbiere $
  *
  * Copyright (c) 2001-2004, Raphael Manfredi
  *
@@ -36,7 +36,7 @@
 #ifndef _core_qhit_h_
 #define _core_qhit_h_
 
-#include <glib.h>
+#include "common.h"
 
 typedef void (*qhit_process_t)(gpointer data, size_t len, gpointer udata);
 
@@ -45,18 +45,16 @@ typedef void (*qhit_process_t)(gpointer data, size_t len, gpointer udata);
  */
 
 struct gnutella_node;
+struct array; 
 
 void qhit_init(void);
 void qhit_close(void);
 
-void qhit_send_results(
-	struct gnutella_node *n, GSList *files, gint count,
-	const gchar *muid, gboolean use_ggep_h);
-void qhit_build_results(
-	GSList *files, gint count,
-	size_t max_msgsize,
-	qhit_process_t cb, gpointer udata,
-	const gchar *muid, gboolean use_ggep_h);
+void qhit_send_results(struct gnutella_node *n, GSList *files, gint count,
+	const gchar *muid, gboolean ggep_h);
+void qhit_build_results(const GSList *files, gint count, size_t max_msgsize,
+	qhit_process_t cb, gpointer udata, const gchar *muid, gboolean ggep_h,
+	const struct array *token);
 
 #endif /* _core_qhit_h_ */
 
