@@ -1,10 +1,8 @@
 /*
- * $Id: rx_inflate.h,v 1.2 2004/09/03 15:05:07 cbiere Exp $
+ * $Id: rx_inflate.h 12496 2006-11-26 09:22:49Z cbiere $
  *
  * Copyright (c) 2002-2003, Raphael Manfredi
  *
- * Network RX -- multiplexed decompressing stage.
- *
  *----------------------------------------------------------------------
  * This file is part of gtk-gnutella.
  *
@@ -25,13 +23,41 @@
  *----------------------------------------------------------------------
  */
 
+/**
+ * @ingroup core
+ * @file
+ *
+ * Network RX -- multiplexed decompressing stage.
+ *
+ * @author Raphael Manfredi
+ * @date 2002-2003
+ */
+
 #ifndef _core_rx_inflate_h_
 #define _core_rx_inflate_h_
 
+#include "common.h"
+
 #include "rx.h"
 
 const struct rxdrv_ops* rx_inflate_get_ops(void);
 
+/**
+ * Callbacks used by the inflating layer.
+ */
+struct rx_inflate_cb {
+	void (*add_rx_inflated)(gpointer owner, gint amount);
+	void (*inflate_error)(gpointer owner,
+			const gchar *reason, ...) PRINTF_FUNC_PTR(2, 3);
+};
+
+/**
+ * Arguments to be passed when the layer is intantiated.
+ */
+struct rx_inflate_args {
+	const struct rx_inflate_cb *cb;		/**< Callbacks */
+};
+
 #endif	/* _core_rx_inflate_h_ */
 
 /* vi: set ts=4 sw=4 cindent: */
