@@ -1,5 +1,5 @@
 /*
- * $Id: search.h 12147 2006-10-27 19:55:09Z cbiere $
+ * $Id: search.h 13045 2007-03-11 03:06:23Z cbiere $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi
  *
@@ -36,6 +36,8 @@
 #ifndef _core_search_h_
 #define _core_search_h_
 
+#include "common.h"
+
 #include "nodes.h"
 
 /*
@@ -49,9 +51,15 @@
 #define QUERY_SPEED_GGEP_H		0x0008	/**< Recipient understands GGEP "H" */
 #define QUERY_SPEED_OOB_REPLY	0x0004	/**< Out-of-band reply possible */
 #define QUERY_SPEED_FW_TO_FW	0x0002	/**< Can do fw to fw transfers */
+/**
+ * NOTE: At this point all of the bits are exhausted. 0x0001 is not available
+ * 		 because the next 9 bits are not available for flags.
+ */
 
 #define QUERY_FW2FW_FILE_INDEX	0x7FFFFFFD	/**< Magic index for fw-fw reqs */
 
+struct download;
+
 /*
  * Global Functions
  */
@@ -62,14 +70,14 @@ void search_shutdown(void);
 gboolean search_results(gnutella_node_t *n, gint *results);
 gboolean search_query_allowed(gnet_search_t sh);
 guint32 search_get_id(gnet_search_t sh, gpointer *search);
-void search_notify_sent(gpointer search, guint32 id, guint32 node_id);
+void search_notify_sent(gpointer search, guint32 id, const node_id_t node_id);
 void search_add_kept(gnet_search_t sh, guint32 kept);
 gboolean search_get_kept_results(const gchar *muid, guint32 *kept);
 guint32 search_get_kept_results_by_handle(gnet_search_t sh);
-void search_oob_pending_results(
-	gnutella_node_t *n, const gchar *muid, gint hits, gboolean udp_firewalled);
+void search_oob_pending_results(gnutella_node_t *n, const gchar *muid,
+	gint hits, gboolean udp_firewalled, gboolean secure);
 
-void search_dissociate_browse(gnet_search_t sh, gpointer download);
+void search_dissociate_browse(gnet_search_t sh, struct download *d);
 void search_browse_results(gnutella_node_t *n, gnet_search_t sh);
 
 #endif /* _core_search_h_ */
