@@ -1,5 +1,5 @@
 /*
- * $Id: settings.c 12232 2006-11-06 03:42:50Z cbiere $
+ * $Id: settings.c 13777 2007-05-29 00:23:11Z cbiere $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi, Richard Eckart
  *
@@ -27,7 +27,7 @@
  * @ingroup core
  * @file
  *
- * Gtk-Gnutella configuration.
+ * gtk-gnutella configuration.
  *
  * @author Raphael Manfredi
  * @author Richard Eckart
@@ -42,15 +42,16 @@
 
 #include "lib/eval.h"
 
-#include "settings.h"
-#include "search.h"
-#include "hosts.h"
-#include "upload_stats.h"
-#include "sockets.h"
-#include "inet.h"
-#include "hcache.h"
+#include "bsched.h"
 #include "downloads.h"
+#include "hcache.h"
+#include "hosts.h"
+#include "inet.h"
+#include "search.h"
+#include "settings.h"
 #include "share.h"
+#include "sockets.h"
+#include "upload_stats.h"
 
 #include "if/gnet_property.h"
 #include "if/gnet_property_priv.h"
@@ -60,15 +61,14 @@
 #include "if/bridge/c2ui.h"
 
 #include "lib/bit_array.h"
+#include "lib/getphysmemsize.h"
 #include "lib/cq.h"
 #include "lib/file.h"
 #include "lib/glib-missing.h"
 #include "lib/tm.h"
 #include "lib/override.h"		/* Must be the last header included */
 
-RCSID("$Id: settings.c 12232 2006-11-06 03:42:50Z cbiere $")
-
-#define debug dbg
+RCSID("$Id: settings.c 13777 2007-05-29 00:23:11Z cbiere $")
 
 static const gchar config_file[] = "config_gnet";
 static const gchar ul_stats_file[] = "upload_stats";
@@ -99,6 +99,8 @@ static prop_set_t *properties = NULL;
 
 static const gchar pidfile[] = "gtk-gnutella.pid";
 
+static gboolean settings_init_running;
+
 static void settings_callbacks_init(void);
 static void settings_callbacks_shutdown(void);
 static void update_uptimes(void);
@@ -112,7 +114,9 @@ host_addr_t
 listen_addr(void)
 {
 	if (s_tcp_listen) {
-		return force_local_ip ? forced_local_ip : local_ip;
+		return GNET_PROPERTY(force_local_ip)
+				? GNET_PROPERTY(forced_local_ip)
+				: GNET_PROPERTY(local_ip);
 	} else {
 		return zero_host_addr;
 	}
@@ -125,19 +129,37 @@ host_addr_t
 listen_addr6(void)
 {
 	if (s_tcp_listen6) {
-		return force_local_ip6 ? forced_local_ip6 : local_ip6;
+		return GNET_PROPERTY(force_local_ip6)
+				? GNET_PROPERTY(forced_local_ip6)
+				: GNET_PROPERTY(local_ip6);
 	} else {
 		return zero_host_addr;
 	}
 }
 
-gboolean
-is_my_address(const host_addr_t addr, guint16 port)
+host_addr_t
+listen_addr_by_net(enum net_type net)
 {
-	return port == listen_port && (
-		host_addr_equal(addr, listen_addr()) ||
-		host_addr_equal(addr, listen_addr6())
-	);
+	switch (net) {
+	case NET_TYPE_IPV4: return listen_addr();
+	case NET_TYPE_IPV6: return listen_addr6();
+	case NET_TYPE_LOCAL:
+	case NET_TYPE_NONE:
+		break;
+	}
+	return zero_host_addr;
+}
+	
+gboolean
+is_my_address(const host_addr_t addr)
+{
+	return host_addr_equal(addr, listen_addr_by_net(host_addr_net(addr)));
+}
+
+gboolean
+is_my_address_and_port(const host_addr_t addr, guint16 port)
+{
+	return port == GNET_PROPERTY(listen_port) && is_my_address(addr);
 }
 
 
@@ -301,91 +323,6 @@ save_pid(gint fd)
 
 /* ----------------------------------------- */
 
-/**
- * @return the amount of physical RAM in bytes, or zero in case of failure
- */
-static guint64
-settings_getphysmemsize(void)
-#if defined (_SC_PHYS_PAGES)
-{
-	size_t pagesize = compat_pagesize();
-	glong pages;
-
-	errno = 0;
-	pages = sysconf(_SC_PHYS_PAGES);
-	if ((glong) -1 == pages && 0 != errno) {
-		g_warning("sysconf(_SC_PHYS_PAGES) failed: %s", g_strerror(errno));
-		return 0;
-	}
-	return pagesize * (guint64) (gulong) pages;
-}
-#elif defined (HAS_SYSCTL) && defined (CTL_HW) && defined (HW_USERMEM64)
-{
-/* There's also HW_PHYSMEM but HW_USERMEM is better for our needs. */
-	int mib[2] = { CTL_HW, HW_USERMEM64 };
-	guint64 amount = 0;
-	size_t len = sizeof amount;
-
-	if (-1 == sysctl(mib, 2, &amount, &len, NULL, 0)) {
-		g_warning(
-			"settings_getphysmemsize: sysctl() for HW_USERMEM64 failed: %s",
-			g_strerror(errno));
-		return 0;
-	}
-
-	return amount;
-}
-#elif defined (HAS_SYSCTL) && defined (CTL_HW) && defined (HW_USERMEM)
-{
-/* There's also HW_PHYSMEM but HW_USERMEM is better for our needs. */
-	int mib[2] = { CTL_HW, HW_USERMEM };
-	guint32 amount = 0;
-	size_t len = sizeof amount;
-
-	if (-1 == sysctl(mib, 2, &amount, &len, NULL, 0)) {
-		g_warning(
-			"settings_getphysmemsize: sysctl() for HW_USERMEM failed: %s",
-			g_strerror(errno));
-		return 0;
-	}
-
-	return amount;
-}
-#elif defined (HAS_GETINVENT)
-{
-	inventory_t *inv;
-	long physmem = 0;
-
-	if (-1 == setinvent()) {
-		g_warning("settings_getphysmemsize: setinvent() failed: %s",
-			g_strerror(errno));
-		return 0;
-	}
-
-	errno = 0;
-	while ((inv = getinvent()) != NULL) {
-		if (inv->inv_class == INV_MEMORY && inv->inv_type == INV_MAIN_MB) {
-			physmem = inv->inv_state;
-			break;
-		}
-	}
-	endinvent();
-
-	if (-1L == physmem && 0 != errno) {
-		g_warning("settings_getphysmemsize: "
-			"getinvent() for INV_MEMORY faild: %s", g_strerror(errno));
-		return 0;
-	}
-
-	return (guint64) physmem * 1024 * 1024;
-}
-#else /* ! _SC_PHYS_PAGES && ! HAS_SYSCTL && ! HAS_GETINVENT */
-{
-	g_warning("Unable to determine amount of physical RAM");
-	return 0;
-}
-#endif /* _SC_PHYS_PAGES */
-
 /**
  * Initializes "config_dir" and "home_dir".
  */
@@ -394,10 +331,20 @@ settings_early_init(void)
 {
 	config_dir = g_strdup(getenv("GTK_GNUTELLA_DIR"));
 	home_dir = g_strdup(eval_subst("~"));
-	if (!home_dir)
-		g_warning(_("Can't find your home directory!"));
-	if (!config_dir && home_dir)
+	if (home_dir) {
+		if (!is_absolute_path(home_dir)) {
+			g_error("$HOME must point to an absolute path!");
+		}
+	} else {
+		g_error(_("Can't find your home directory!"));
+	}
+	if (config_dir) {
+		if (!is_absolute_path(config_dir)) {
+			g_error("$GTK_GNUTELLA_DIR must point to an absolute path!");
+		}
+	} else { 
 		config_dir = make_pathname(home_dir, ".gtk-gnutella");
+	}
 }
 
 /**
@@ -441,10 +388,12 @@ settings_ensure_unicity(gboolean check_only)
 void
 settings_init(void)
 {
-	guint64 memory = settings_getphysmemsize();
+	guint64 memory = getphysmemsize();
 	guint64 amount = memory / 1024;
 	guint max_fd;
 
+	settings_init_running = TRUE;
+
 #ifdef RLIMIT_DATA 
 	{
 		struct rlimit lim;
@@ -462,7 +411,7 @@ settings_init(void)
 	gnet_prop_set_guint32_val(PROP_SYS_NOFILE, max_fd);
 	gnet_prop_set_guint64_val(PROP_SYS_PHYSMEM, amount);
 
-	memset(deconstify_gpointer(servent_guid), 0, sizeof servent_guid);
+	memset(deconstify_gpointer(GNET_PROPERTY(servent_guid)), 0, GUID_RAW_SIZE);
 
 	if (NULL == config_dir || '\0' == config_dir[0])
 		goto no_config_dir;
@@ -487,9 +436,9 @@ settings_init(void)
 
 	if (debugging(0)) {
 		g_message("detected amount of physical RAM: %s",
-			short_size(memory, display_metric_units));
+			short_size(memory, GNET_PROPERTY(display_metric_units)));
 		g_message("process can use at maximum: %s",
-			short_kb_size(amount, display_metric_units));
+			short_kb_size(amount, GNET_PROPERTY(display_metric_units)));
 		g_message("process can use %u file descriptors", max_fd);
 		g_message("max I/O vector size is %d items", MAX_IOV_COUNT);
 	}
@@ -505,21 +454,22 @@ settings_init(void)
 
 	/* watch for filter_file defaults */
 
-	if (hard_ttl_limit < max_ttl) {
-		*(guint32 *) &hard_ttl_limit = max_ttl;
+	if (GNET_PROPERTY(hard_ttl_limit) < GNET_PROPERTY(max_ttl)) {
+		*(guint32 *) &GNET_PROPERTY(hard_ttl_limit) = GNET_PROPERTY(max_ttl);
 		g_warning("hard_ttl_limit was too small, adjusted to %u",
-			hard_ttl_limit);
+			GNET_PROPERTY(hard_ttl_limit));
 	}
 
 	/* Flow control depends on this being not too small */
-	if (node_sendqueue_size < 1.5 * settings_max_msg_size()) {
-		*(guint32 *) &node_sendqueue_size =
+	if (GNET_PROPERTY(node_sendqueue_size) < 1.5 * settings_max_msg_size()) {
+		*(guint32 *) &GNET_PROPERTY(node_sendqueue_size) =
 			(guint32) (1.5 * settings_max_msg_size());
 		g_warning("node_sendqueue_size was too small, adjusted to %u",
-			node_sendqueue_size);
+			GNET_PROPERTY(node_sendqueue_size));
 	}
 
     settings_callbacks_init();
+	settings_init_running = FALSE;
 	return;
 
 no_config_dir:
@@ -582,15 +532,15 @@ settings_local_socket_path(void)
 enum net_type
 settings_dns_net(void)
 {
-	switch (network_protocol) {
+	switch (GNET_PROPERTY(network_protocol)) {
 	case NET_USE_BOTH: return NET_TYPE_NONE;
 	case NET_USE_IPV4: return NET_TYPE_IPV4;
 	case NET_USE_IPV6:
-#ifdef USE_IPV6
+#ifdef HAS_IPV6
 	return NET_TYPE_IPV6;
 #else
 	return NET_TYPE_NONE;
-#endif /* USE_IPV6 */
+#endif /* HAS_IPV6 */
 	}
 	g_assert_not_reached();
 	return NET_TYPE_NONE;
@@ -621,10 +571,12 @@ addr_ipv4_changed(const host_addr_t new_addr, const host_addr_t peer)
 	static host_addr_t peers[3], last_addr_seen;
 	guint i;
 
-	g_return_if_fail(!force_local_ip); /* Must be called when IP isn't forced */
 	g_return_if_fail(NET_TYPE_IPV4 == host_addr_net(new_addr));
 	g_return_if_fail(NET_TYPE_IPV4 == host_addr_net(peer));
 
+	if (GNET_PROPERTY(force_local_ip))
+		return;
+
 	/*
 	 * Accept updates for private addresses only from peer in the same /16
 	 * network; addresses are in host byte order.
@@ -661,7 +613,7 @@ addr_ipv4_changed(const host_addr_t new_addr, const host_addr_t peer)
 		peers[i] = zero_host_addr;
 	}
 
-	if (host_addr_equal(new_addr, local_ip))
+	if (host_addr_equal(new_addr, GNET_PROPERTY(local_ip)))
 		return;
 
     gnet_prop_set_ip_val(PROP_LOCAL_IP, new_addr);
@@ -674,10 +626,12 @@ addr_ipv6_changed(const host_addr_t new_addr, const host_addr_t peer)
 	static host_addr_t peers[3], last_addr_seen;
 	guint i;
 
-	g_return_if_fail(!force_local_ip6); /* Must not be called if IP is forced */
 	g_return_if_fail(NET_TYPE_IPV6 == host_addr_net(new_addr));
 	g_return_if_fail(NET_TYPE_IPV6 == host_addr_net(peer));
 
+	if (GNET_PROPERTY(force_local_ip6))
+		return;
+
 	/*
 	 * Accept updates for private addresses only from peer in the same /64
 	 * network; addresses are in host byte order.
@@ -714,7 +668,7 @@ addr_ipv6_changed(const host_addr_t new_addr, const host_addr_t peer)
 		peers[i] = zero_host_addr;
 	}
 
-	if (host_addr_equal(new_addr, local_ip6))
+	if (host_addr_equal(new_addr, GNET_PROPERTY(local_ip6)))
 		return;
 
     gnet_prop_set_ip_val(PROP_LOCAL_IP6, new_addr);
@@ -791,34 +745,13 @@ settings_max_msg_size(void)
 
 	guint32 maxsize;
 
-	maxsize = MAX(search_queries_kick_size, search_answers_kick_size);
-	maxsize = MAX(maxsize, other_messages_kick_size);
+	maxsize = GNET_PROPERTY(search_queries_kick_size);
+	maxsize = MAX(maxsize, GNET_PROPERTY(search_answers_kick_size));
+	maxsize = MAX(maxsize, GNET_PROPERTY(other_messages_kick_size));
 
 	return maxsize;
 }
 
-/**
- * Ask them to set a property to be able to run.
- */
-void
-settings_ask_for_property(const gchar *name, const gchar *value)
-{
-	fprintf(stderr, "\n*** ANCIENT VERSION DETECTED! ***\n\n");
-	fprintf(stderr,
-		"Sorry, this program is too ancient to run without\n"
-		"an explicit user action: If it's not possible to upgrade\n"
-		"you may edit the file\n\n"
-		"\t%s%s%s\n\n"
-		"and set the variable \"%s\" to\n\"%s\".\n\n"
-		"You will then be able to run this version forever, but\n"
-		"please consider upgrading, as Gnutella is an evolving\n"
-		"network in which ancient software is less useful or even\n"
-		"harmful!\n\n",
-		config_dir, G_DIR_SEPARATOR_S, config_file, name, value);
-	fprintf(stderr, "*** EXITING ***\n\n");
-	exit(EXIT_FAILURE);
-}
-
 /**
  * Called at exit time to flush the property files.
  */
@@ -853,6 +786,15 @@ settings_close(void)
 	G_FREE_NULL(config_dir);
 }
 
+static void
+bw_stats(gnet_bw_stats_t *s, gboolean enabled, bsched_bws_t bws)
+{
+	s->enabled = enabled;
+	s->current = bsched_bps(bws);
+	s->average = bsched_avg_bps(bws);
+	s->limit = bsched_bw_per_second(bws);
+}
+
 void
 gnet_get_bw_stats(gnet_bw_source type, gnet_bw_stats_t *s)
 {
@@ -860,54 +802,31 @@ gnet_get_bw_stats(gnet_bw_source type, gnet_bw_stats_t *s)
 
     switch (type) {
     case BW_GNET_IN:
-        s->enabled  = bws_gin_enabled;
-        s->current  = bsched_bps(bws.gin);
-        s->average  = bsched_avg_bps(bws.gin);
-        s->limit    = bws.gin->bw_per_second;
-        break;
+		bw_stats(s, GNET_PROPERTY(bws_gin_enabled), BSCHED_BWS_GIN);
+		return;
     case BW_GNET_OUT:
-        s->enabled  = bws_gout_enabled;
-        s->current  = bsched_bps(bws.gout);
-        s->average  = bsched_avg_bps(bws.gout);
-        s->limit    = bws.gout->bw_per_second;
-        break;
+		bw_stats(s, GNET_PROPERTY(bws_gout_enabled), BSCHED_BWS_GOUT);
+		return;
     case BW_GNET_UDP_IN:
-        s->enabled  = bws_gin_enabled;
-        s->current  = bsched_bps(bws.gin_udp);
-        s->average  = bsched_avg_bps(bws.gin_udp);
-        s->limit    = bws.gin_udp->bw_per_second;
-        break;
+		bw_stats(s, GNET_PROPERTY(bws_gin_enabled), BSCHED_BWS_GIN_UDP);
+		return;
     case BW_GNET_UDP_OUT:
-        s->enabled  = bws_gout_enabled;
-        s->current  = bsched_bps(bws.gout_udp);
-        s->average  = bsched_avg_bps(bws.gout_udp);
-        s->limit    = bws.gout_udp->bw_per_second;
-        break;
+		bw_stats(s, GNET_PROPERTY(bws_gout_enabled), BSCHED_BWS_GOUT_UDP);
+		return;
     case BW_HTTP_IN:
-        s->enabled  = bws_in_enabled;
-        s->current  = bsched_bps(bws.in);
-        s->average  = bsched_avg_bps(bws.in);
-        s->limit    = bws.in->bw_per_second;
-        break;
+		bw_stats(s, GNET_PROPERTY(bws_in_enabled), BSCHED_BWS_IN);
+		return;
     case BW_HTTP_OUT:
-        s->enabled  = bws_out_enabled;
-        s->current  = bsched_bps(bws.out);
-        s->average  = bsched_avg_bps(bws.out);
-        s->limit    = bws.out->bw_per_second;
-        break;
+		bw_stats(s, GNET_PROPERTY(bws_out_enabled), BSCHED_BWS_OUT);
+		return;
     case BW_LEAF_IN:
-        s->enabled  = bws_glin_enabled;
-        s->current  = bsched_bps(bws.glin);
-        s->average  = bsched_avg_bps(bws.glin);
-        s->limit    = bws.glin->bw_per_second;
-        break;
+		bw_stats(s, GNET_PROPERTY(bws_glin_enabled), BSCHED_BWS_GLIN);
+		return;
     case BW_LEAF_OUT:
-        s->enabled  = bws_glout_enabled;
-        s->current  = bsched_bps(bws.glout);
-        s->average  = bsched_avg_bps(bws.glout);
-        s->limit    = bws.glout->bw_per_second;
-        break;
+		bw_stats(s, GNET_PROPERTY(bws_glout_enabled), BSCHED_BWS_GLOUT);
+		return;
     }
+	g_assert_not_reached();
 }
 
 /***
@@ -926,12 +845,12 @@ get_average_ip_lifetime(time_t now, enum net_type net)
 
 	switch (net) {
 	case NET_TYPE_IPV4: 
-		stamp = current_ip_stamp;
-		average = average_ip_uptime;
+		stamp = GNET_PROPERTY(current_ip_stamp);
+		average = GNET_PROPERTY(average_ip_uptime);
 		break;
 	case NET_TYPE_IPV6: 
-		stamp = current_ip6_stamp;
-		average = average_ip6_uptime;
+		stamp = GNET_PROPERTY(current_ip6_stamp);
+		average = GNET_PROPERTY(average_ip6_uptime);
 		break;
 	default:
 		return 0;
@@ -968,7 +887,7 @@ update_address_lifetime(void)
 	addr = listen_addr();
 	if (!is_host_addr(old_addr)) {				/* First time */
 		old_addr = addr;
-		if (0 == current_ip_stamp) {
+		if (0 == GNET_PROPERTY(current_ip_stamp)) {
 			gnet_prop_set_timestamp_val(PROP_CURRENT_IP_STAMP, now);
 		}
 	}
@@ -979,7 +898,7 @@ update_address_lifetime(void)
 		 */
 
 		old_addr = addr;
-		if (current_ip_stamp) {
+		if (GNET_PROPERTY(current_ip_stamp)) {
 			gnet_prop_set_guint32_val(PROP_AVERAGE_IP_UPTIME,
 				get_average_ip_lifetime(now, host_addr_net(addr)));
 		}
@@ -989,7 +908,7 @@ update_address_lifetime(void)
 	addr = listen_addr6();
 	if (!is_host_addr(old_addr_v6)) {				/* First time */
 		old_addr_v6 = addr;
-		if (0 == current_ip6_stamp) {
+		if (0 == GNET_PROPERTY(current_ip6_stamp)) {
 			gnet_prop_set_timestamp_val(PROP_CURRENT_IP6_STAMP, now);
 		}
 	}
@@ -1000,7 +919,7 @@ update_address_lifetime(void)
 		 */
 
 		old_addr_v6 = addr;
-		if (current_ip6_stamp) {
+		if (GNET_PROPERTY(current_ip6_stamp)) {
 			gnet_prop_set_guint32_val(PROP_AVERAGE_IP6_UPTIME,
 				get_average_ip_lifetime(now, host_addr_net(addr)));
 		}
@@ -1019,7 +938,7 @@ get_average_servent_uptime(time_t now)
 	time_delta_t d;
 	glong uptime;
 
-	d = delta_time(now, start_stamp);
+	d = delta_time(now, GNET_PROPERTY(start_stamp));
 	uptime = MAX(0, d);
 
 	/*
@@ -1027,7 +946,7 @@ get_average_servent_uptime(time_t now)
 	 * The smoothing factor sm=2/(7+1) is therefore 0.25.
 	 */
 
-	avg = average_servent_uptime;
+	avg = GNET_PROPERTY(average_servent_uptime);
 	avg += (uptime >> 2) - (avg >> 2);
 
 	return avg;
@@ -1059,8 +978,9 @@ up_connections_changed(property_t prop)
 {
 	g_assert(PROP_UP_CONNECTIONS == prop);
 
-    if (up_connections > max_connections) {
-        gnet_prop_set_guint32_val(PROP_MAX_CONNECTIONS, up_connections);
+    if (GNET_PROPERTY(up_connections) > GNET_PROPERTY(max_connections)) {
+        gnet_prop_set_guint32_val(PROP_MAX_CONNECTIONS,
+			GNET_PROPERTY(up_connections));
 	}
     return FALSE;
 }
@@ -1070,8 +990,9 @@ max_connections_changed(property_t prop)
 {
 	g_assert(PROP_MAX_CONNECTIONS == prop);
 
-    if (up_connections > max_connections) {
-        gnet_prop_set_guint32_val(PROP_UP_CONNECTIONS, max_connections);
+    if (GNET_PROPERTY(up_connections) > GNET_PROPERTY(max_connections)) {
+        gnet_prop_set_guint32_val(PROP_UP_CONNECTIONS,
+			GNET_PROPERTY(max_connections));
 	}
     return FALSE;
 }
@@ -1112,20 +1033,25 @@ get_bind_addr(enum net_type net)
 	
 	switch (net) {
 	case NET_TYPE_IPV4:
-		addr = listen_addr();
+		addr = GNET_PROPERTY(force_local_ip)
+				? GNET_PROPERTY(forced_local_ip)
+				: GNET_PROPERTY(local_ip);
+
 		if (
-			!force_local_ip ||
-			!bind_to_forced_local_ip ||
+			!GNET_PROPERTY(force_local_ip) ||
+			!GNET_PROPERTY(bind_to_forced_local_ip) ||
 			!host_addr_initialized(addr)
 		) {
 			addr = ipv4_unspecified;
 		}
 		break;
 	case NET_TYPE_IPV6:
-		addr = listen_addr6();
+		addr = GNET_PROPERTY(force_local_ip6)
+				? GNET_PROPERTY(forced_local_ip6)
+				: GNET_PROPERTY(local_ip6);
 		if (
-			!force_local_ip6 ||
-			!bind_to_forced_local_ip6 ||
+			!GNET_PROPERTY(force_local_ip6) ||
+			!GNET_PROPERTY(bind_to_forced_local_ip6) ||
 			!host_addr_initialized(addr)
 		) {
 			addr = ipv6_unspecified;
@@ -1146,20 +1072,26 @@ enable_udp_changed(property_t prop)
     gnet_prop_get_boolean_val(prop, &enabled);
 	if (enabled) {
 		if (s_tcp_listen) {
+			g_assert(!s_udp_listen);
 			s_udp_listen = socket_udp_listen(get_bind_addr(NET_TYPE_IPV4),
-								listen_port);
+								GNET_PROPERTY(listen_port));
 			if (!s_udp_listen) {
 				gcu_statusbar_warning(_("Failed to create IPv4 UDP socket"));
 			}
 		}
 		if (s_tcp_listen6) {
+			g_assert(!s_udp_listen6);
 			s_udp_listen6 = socket_udp_listen(get_bind_addr(NET_TYPE_IPV6),
-								listen_port);
-			if (!s_udp_listen) {
+								GNET_PROPERTY(listen_port));
+			if (!s_udp_listen6) {
 				gcu_statusbar_warning(_("Failed to create IPv6 UDP socket"));
 			}
 		}
 	} else {
+		/* Also takes care of freeing s_udp_listen and s_udp_listen6 */
+		node_udp_disable();
+
+		/* We have to free them anyway during startup*/
 		socket_free_null(&s_udp_listen);
 		socket_free_null(&s_udp_listen6);
 	}
@@ -1202,14 +1134,19 @@ enable_local_socket_changed(property_t prop)
 static void
 request_new_sockets(guint16 port, gboolean check_firewalled)
 {
+	/* Also takes care of freeing s_udp_listen and s_udp_listen6 */
+	node_udp_disable();
+
+	/* We have to free them anyway during startup*/
+	socket_free_null(&s_udp_listen);
+	socket_free_null(&s_udp_listen6);
+
 	/*
-	 * Close old ports.
+	 * Close sockets at the old port.
 	 */
 
 	socket_free_null(&s_tcp_listen);
 	socket_free_null(&s_tcp_listen6);
-	socket_free_null(&s_udp_listen);
-	socket_free_null(&s_udp_listen6);
 
 	/*
 	 * If the new port != 0, open the new port
@@ -1218,22 +1155,30 @@ request_new_sockets(guint16 port, gboolean check_firewalled)
 	if (0 == port)
 		return;
 
-	if (NET_USE_BOTH == network_protocol || NET_USE_IPV4 == network_protocol) {
+	if (
+		NET_USE_BOTH == GNET_PROPERTY(network_protocol) ||
+		NET_USE_IPV4 == GNET_PROPERTY(network_protocol)
+	) {
 		host_addr_t bind_addr = get_bind_addr(NET_TYPE_IPV4);
 
 		s_tcp_listen = socket_tcp_listen(bind_addr, port);
-		if (enable_udp) {
+		if (GNET_PROPERTY(enable_udp)) {
+			g_assert(!s_udp_listen);
 			s_udp_listen = socket_udp_listen(bind_addr, port);
 			if (!s_udp_listen) {
 				socket_free_null(&s_tcp_listen);
 			}
 		}
 	}
-	if (NET_USE_BOTH == network_protocol || NET_USE_IPV6 == network_protocol) {
+	if (
+		NET_USE_BOTH == GNET_PROPERTY(network_protocol) ||
+		NET_USE_IPV6 == GNET_PROPERTY(network_protocol)
+	) {
 		host_addr_t bind_addr = get_bind_addr(NET_TYPE_IPV6);
 
 		s_tcp_listen6 = socket_tcp_listen(bind_addr, port);
-		if (enable_udp) {
+		if (GNET_PROPERTY(enable_udp)) {
+			g_assert(!s_udp_listen6);
 			s_udp_listen6 = socket_udp_listen(bind_addr, port);
 			if (!s_udp_listen6) {
 				socket_free_null(&s_tcp_listen6);
@@ -1245,7 +1190,7 @@ request_new_sockets(guint16 port, gboolean check_firewalled)
 	 * If UDP is enabled, also listen on the same UDP port.
 	 */
 
-	if (enable_udp) {
+	if (GNET_PROPERTY(enable_udp)) {
 		node_update_udp_socket();
 	}
 
@@ -1264,21 +1209,24 @@ listen_port_changed(property_t prop)
 	 * If port did not change values, do nothing.
 	 */
 
-	if (listen_port == old_port && listen_port != 0)
+	if (
+		GNET_PROPERTY(listen_port) == old_port &&
+		GNET_PROPERTY(listen_port) != 0
+	)
 		return FALSE;
-	old_port = listen_port;
+	old_port = GNET_PROPERTY(listen_port);
 
 	/*
 	 * 1 is a magic port number for us, which means "pick a random port"
 	 * whereas 0 means "don't listen on any port".
 	 */
 
-	if (1 != listen_port) {
-		request_new_sockets(listen_port, FALSE);
+	if (1 != GNET_PROPERTY(listen_port)) {
+		request_new_sockets(GNET_PROPERTY(listen_port), FALSE);
 	} else {
 		bit_array_t tried[BIT_ARRAY_SIZE(65536)];
 		guint num_tried = 0;
-    	guint32 port = listen_port;
+    	guint32 port = GNET_PROPERTY(listen_port);
 
 		/* Mark ports below 1024 as already tried, these ports can
 		 * be configured manually but we don't want to pick one of
@@ -1316,19 +1264,21 @@ listen_port_changed(property_t prop)
 		gnet_prop_set_guint32_val(prop, port);
 	}
 
-	inet_firewalled();
-	inet_udp_firewalled();
+	if (GNET_PROPERTY(is_firewalled) || !settings_init_running) {
+		inet_firewalled();
+		inet_udp_firewalled();
+	}
 
 	/*
      * If socket allocation failed, reset the property
      */
 
-    if (s_tcp_listen == NULL && listen_port != 0) {
+    if (s_tcp_listen == NULL && GNET_PROPERTY(listen_port) != 0) {
 		gcu_statusbar_warning(_("Failed to create listening sockets"));
 		old_port = (guint32) -1;
         return TRUE;
     } else {
-		old_port = listen_port;
+		old_port = GNET_PROPERTY(listen_port);
 	}
 
     return FALSE;
@@ -1339,93 +1289,59 @@ network_protocol_changed(property_t prop)
 {
 
 	(void) prop;
-	request_new_sockets(listen_port, TRUE);
+	request_new_sockets(GNET_PROPERTY(listen_port),
+		GNET_PROPERTY(is_firewalled) || !settings_init_running);
 	return FALSE;
 }
 
 
 static gboolean
-bw_http_in_enabled_changed(property_t prop)
+bw_switch(property_t prop, bsched_bws_t bs)
 {
     gboolean val;
 
     gnet_prop_get_boolean_val(prop, &val);
     if (val)
-        bsched_enable(bws.in);
+        bsched_enable(bs);
     else
-        bsched_disable(bws.in);
+        bsched_disable(bs);
+	return FALSE;
+}
 
-    return FALSE;
+static gboolean
+bw_http_in_enabled_changed(property_t prop)
+{
+	return bw_switch(prop, BSCHED_BWS_IN);
 }
 
 static gboolean
 bw_http_out_enabled_changed(property_t prop)
 {
-    gboolean val;
-
-    gnet_prop_get_boolean_val(prop, &val);
-    if (val)
-        bsched_enable(bws.out);
-    else
-        bsched_disable(bws.out);
-
-    return FALSE;
+	return bw_switch(prop, BSCHED_BWS_OUT);
 }
 
 static gboolean
 bw_gnet_in_enabled_changed(property_t prop)
 {
-    gboolean val;
-
-    gnet_prop_get_boolean_val(prop, &val);
-    if (val)
-        bsched_enable(bws.gin);
-    else
-        bsched_disable(bws.gin);
-
-    return FALSE;
+	return bw_switch(prop, BSCHED_BWS_GIN);
 }
 
 static gboolean
 bw_gnet_out_enabled_changed(property_t prop)
 {
-    gboolean val;
-
-    gnet_prop_get_boolean_val(prop, &val);
-    if (val)
-        bsched_enable(bws.gout);
-    else
-        bsched_disable(bws.gout);
-
-    return FALSE;
+	return bw_switch(prop, BSCHED_BWS_GOUT);
 }
 
 static gboolean
 bw_gnet_lin_enabled_changed(property_t prop)
 {
-    gboolean val;
-
-    gnet_prop_get_boolean_val(prop, &val);
-    if (val)
-        bsched_enable(bws.glin);
-    else
-        bsched_disable(bws.glin);
-
-    return FALSE;
+	return bw_switch(prop, BSCHED_BWS_GLIN);
 }
 
 static gboolean
 bw_gnet_lout_enabled_changed(property_t prop)
 {
-    gboolean val;
-
-    gnet_prop_get_boolean_val(prop, &val);
-    if (val)
-        bsched_enable(bws.glout);
-    else
-        bsched_disable(bws.glout);
-
-    return FALSE;
+	return bw_switch(prop, BSCHED_BWS_GLOUT);
 }
 
 static gboolean
@@ -1434,7 +1350,7 @@ node_sendqueue_size_changed(property_t unused_prop)
     guint32 min = 1.5 * settings_max_msg_size();
 
 	(void) unused_prop;
-    if (node_sendqueue_size < min) {
+    if (GNET_PROPERTY(node_sendqueue_size) < min) {
         gnet_prop_set_guint32_val(PROP_NODE_SENDQUEUE_SIZE, min);
         return TRUE;
     }
@@ -1464,7 +1380,7 @@ file_path_changed(property_t prop)
 	if (!is_directory(s)) {
 		g_message("Attempt to create directory \"%s\"", s);
 
-		if (0 != create_directory(s))
+		if (0 != create_directory(s, DEFAULT_DIRECTORY_MODE))
 			g_message("Attempt failed: \"%s\"", g_strerror(errno));
 	}
 
@@ -1505,8 +1421,8 @@ hard_ttl_limit_changed(property_t prop)
 {
 	g_assert(PROP_HARD_TTL_LIMIT == prop);
 
-    if (hard_ttl_limit < max_ttl) {
-        gnet_prop_set_guint32_val(PROP_MAX_TTL, hard_ttl_limit);
+    if (GNET_PROPERTY(hard_ttl_limit) < GNET_PROPERTY(max_ttl)) {
+        gnet_prop_set_guint32_val(PROP_MAX_TTL, GNET_PROPERTY(hard_ttl_limit));
 	}
     return FALSE;
 }
@@ -1516,8 +1432,8 @@ max_ttl_changed(property_t prop)
 {
 	g_assert(PROP_MAX_TTL == prop);
 
-    if (hard_ttl_limit < max_ttl) {
-        gnet_prop_set_guint32_val(PROP_HARD_TTL_LIMIT, max_ttl);
+    if (GNET_PROPERTY(hard_ttl_limit) < GNET_PROPERTY(max_ttl)) {
+        gnet_prop_set_guint32_val(PROP_HARD_TTL_LIMIT, GNET_PROPERTY(max_ttl));
 	}
     return FALSE;
 }
@@ -1529,8 +1445,8 @@ bw_http_in_changed(property_t prop)
 
 	g_assert(PROP_BW_HTTP_IN == prop);
     gnet_prop_get_guint32_val(prop, &val);
-    bsched_set_bandwidth(bws.in, val);
-	bsched_set_peermode(current_peermode);
+    bsched_set_bandwidth(BSCHED_BWS_IN, val);
+	bsched_set_peermode(GNET_PROPERTY(current_peermode));
 
     return FALSE;
 }
@@ -1541,8 +1457,8 @@ bw_http_out_changed(property_t prop)
     guint32 val;
 
     gnet_prop_get_guint32_val(prop, &val);
-    bsched_set_bandwidth(bws.out, val);
-	bsched_set_peermode(current_peermode);
+    bsched_set_bandwidth(BSCHED_BWS_OUT, val);
+	bsched_set_peermode(GNET_PROPERTY(current_peermode));
 
     return FALSE;
 }
@@ -1553,9 +1469,9 @@ bw_gnet_in_changed(property_t prop)
     guint32 val;
 
     gnet_prop_get_guint32_val(prop, &val);
-    bsched_set_bandwidth(bws.gin, val / 2);
-    bsched_set_bandwidth(bws.gin_udp, val / 2);
-	bsched_set_peermode(current_peermode);
+    bsched_set_bandwidth(BSCHED_BWS_GIN, val / 2);
+    bsched_set_bandwidth(BSCHED_BWS_GIN_UDP, val / 2);
+	bsched_set_peermode(GNET_PROPERTY(current_peermode));
 
     return FALSE;
 }
@@ -1566,9 +1482,9 @@ bw_gnet_out_changed(property_t prop)
     guint32 val;
 
     gnet_prop_get_guint32_val(prop, &val);
-    bsched_set_bandwidth(bws.gout, val / 2);
-    bsched_set_bandwidth(bws.gout_udp, val / 2);
-	bsched_set_peermode(current_peermode);
+    bsched_set_bandwidth(BSCHED_BWS_GOUT, val / 2);
+    bsched_set_bandwidth(BSCHED_BWS_GOUT_UDP, val / 2);
+	bsched_set_peermode(GNET_PROPERTY(current_peermode));
 
     return FALSE;
 }
@@ -1579,8 +1495,8 @@ bw_gnet_lin_changed(property_t prop)
     guint32 val;
 
     gnet_prop_get_guint32_val(prop, &val);
-    bsched_set_bandwidth(bws.glin, val);
-	bsched_set_peermode(current_peermode);
+    bsched_set_bandwidth(BSCHED_BWS_GLIN, val);
+	bsched_set_peermode(GNET_PROPERTY(current_peermode));
 
     return FALSE;
 }
@@ -1591,8 +1507,8 @@ bw_gnet_lout_changed(property_t prop)
     guint32 val;
 
     gnet_prop_get_guint32_val(prop, &val);
-    bsched_set_bandwidth(bws.glout, val);
-	bsched_set_peermode(current_peermode);
+    bsched_set_bandwidth(BSCHED_BWS_GLOUT, val);
+	bsched_set_peermode(GNET_PROPERTY(current_peermode));
 
     return FALSE;
 }
@@ -1638,9 +1554,10 @@ static gboolean
 forced_local_ip_changed(property_t prop)
 {
 	(void) prop;
-	if (force_local_ip || force_local_ip6) {
+	if (GNET_PROPERTY(force_local_ip) || GNET_PROPERTY(force_local_ip6)) {
 		update_address_lifetime();
-		request_new_sockets(listen_port, TRUE);
+		request_new_sockets(GNET_PROPERTY(listen_port),
+			GNET_PROPERTY(is_firewalled) || !settings_init_running);
 	}
     return FALSE;
 }
@@ -1679,15 +1596,12 @@ local_addr_changed(property_t prop)
 		host_addr_t old_addr = addr;
 
 		addr = zero_host_addr;
-		sl_addrs = host_addr_get_interface_addrs();
+		sl_addrs = host_addr_get_interface_addrs(net);
 		for (sl = sl_addrs; NULL != sl; sl = g_slist_next(sl)) {
 			host_addr_t *addr_ptr;
 
 			addr_ptr = sl->data;
-			if (
-				net == host_addr_net(*addr_ptr) &&
-				host_addr_is_routable(*addr_ptr)
-			) {
+			if (host_addr_is_routable(*addr_ptr)) {
 				addr = *addr_ptr;
 				break;
 			}
@@ -1729,7 +1643,7 @@ configured_peermode_changed(property_t prop)
 	switch (val) {
 	case NODE_P_NORMAL:
 	case NODE_P_ULTRA:
-		if (is_firewalled) {
+		if (GNET_PROPERTY(is_firewalled)) {
 			val = NODE_P_AUTO;
 			forced = TRUE;
 			g_warning("must run as a leaf when TCP-firewalled");
@@ -1757,10 +1671,10 @@ configured_peermode_changed(property_t prop)
 static gboolean
 current_peermode_changed(property_t prop)
 {
-    guint32 val;
+	(void) prop;
 
-    gnet_prop_get_guint32_val(prop, &val);
-	node_current_peermode_changed((node_peer_t) val);
+	node_current_peermode_changed(GNET_PROPERTY(current_peermode));
+	bsched_set_peermode(GNET_PROPERTY(current_peermode));
 
     return FALSE;
 }
@@ -1794,8 +1708,8 @@ node_rx_size_changed(property_t prop)
  * future.
  */
 
-static gpointer ev_file_descriptor_shortage = NULL;
-static gpointer ev_file_descriptor_runout = NULL;
+static cevent_t *ev_file_descriptor_shortage;
+static cevent_t *ev_file_descriptor_runout;
 
 #define RESET_PROP_TM	(10*60*1000)	/**< 10 minutes in ms */
 
@@ -1829,7 +1743,7 @@ static gboolean
 file_descriptor_x_changed(property_t prop)
 {
 	gboolean state;
-	gpointer *ev = NULL;
+	cevent_t **ev = NULL;
 
 	gnet_prop_get_boolean_val(prop, &state);
 	if (!state)
@@ -2136,7 +2050,7 @@ settings_callbacks_init(void)
     for (n = 0; n < GNET_PROPERTY_NUM; n ++)
         init_list[n] = FALSE;
 
-    if (debug >= 2) {
+    if (GNET_PROPERTY(dbg) >= 2) {
         printf("settings_callbacks_init: property_map size: %u\n",
             (guint) PROPERTY_MAP_SIZE);
     }
@@ -2156,13 +2070,13 @@ settings_callbacks_init(void)
                 property_map[n].prop,
                 property_map[n].cb,
                 property_map[n].init);
-        } else if (debug >= 10) {
+        } else if (GNET_PROPERTY(dbg) >= 10) {
             printf("settings_callbacks_init: property ignored: %s\n",
 				gnet_prop_name(prop));
         }
     }
 
-    if (debug >= 1) {
+    if (GNET_PROPERTY(dbg) >= 1) {
         for (n = 0; n < GNET_PROPERTY_NUM; n++) {
             if (!init_list[n])
                 printf("settings_callbacks_init: unmapped property: %s\n",
@@ -2176,14 +2090,8 @@ settings_callbacks_shutdown(void)
 {
     guint n;
 
-	if (ev_file_descriptor_shortage != NULL) {
-		cq_cancel(callout_queue, ev_file_descriptor_shortage);
-		ev_file_descriptor_shortage = NULL;
-	}
-	if (ev_file_descriptor_runout != NULL) {
-		cq_cancel(callout_queue, ev_file_descriptor_runout);
-		ev_file_descriptor_runout = NULL;
-	}
+	cq_cancel(callout_queue, &ev_file_descriptor_shortage);
+	cq_cancel(callout_queue, &ev_file_descriptor_runout);
 
     for (n = 0; n < PROPERTY_MAP_SIZE; n ++) {
         if (property_map[n].cb) {
