@@ -1,5 +1,5 @@
 /*
- * $Id: settings.h 11820 2006-08-30 02:10:23Z cbiere $
+ * $Id: settings.h 13163 2007-03-20 16:03:43Z cbiere $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi, Richard Eckart
  *
@@ -27,7 +27,7 @@
  * @ingroup core
  * @file
  *
- * Gtk-Gnutella configuration.
+ * gtk-gnutella configuration.
  *
  * @author Raphael Manfredi
  * @author Richard Eckart
@@ -53,7 +53,8 @@ extern struct in_addr *local_netmasks;
  * Global Functions
  */
 
-gboolean is_my_address(const host_addr_t addr, guint16 port);
+gboolean is_my_address(const host_addr_t addr);
+gboolean is_my_address_and_port(const host_addr_t addr, guint16 port);
 
 void settings_early_init(void);
 gint settings_ensure_unicity(gboolean silent);
@@ -63,8 +64,6 @@ void settings_shutdown(void);
 void settings_addr_changed(const host_addr_t new_addr, const host_addr_t peer);
 guint32 settings_max_msg_size(void);
 void settings_close(void);
-void settings_ask_for_property(const gchar *name,
-	const gchar *value) G_GNUC_NORETURN;
 
 guint32 get_average_servent_uptime(time_t now);
 guint32 get_average_ip_lifetime(time_t now, enum net_type net);
