@@ -1,5 +1,5 @@
 /*
- * $Id: share.h 12147 2006-10-27 19:55:09Z cbiere $
+ * $Id: share.h 14032 2007-07-03 23:17:08Z cbiere $
  *
  * Copyright (c) 2001-2005, Raphael Manfredi
  * Copyright (c) 2000 Daniel Walker (dwalker@cats.ucsc.edu)
@@ -45,39 +45,45 @@
 
 typedef struct shared_file shared_file_t;
 
-struct extension {
-	gchar *str;					/**< Extension string (e.g. "html") */
-	size_t len;					/**< Extension length (e.g. 4) */
-};
-
 /**
  * shared_file flags
  */
 
-#define SHARE_F_HAS_DIGEST	0x00000001		/**< Digest is set */
-#define SHARE_F_RECOMPUTING	0x00000002		/**< Digest being recomputed */
+enum {
+	SHARE_F_HAS_DIGEST	=	(1 << 0),		/**< Digest is set */
+	SHARE_F_RECOMPUTING	=	(1 << 1),		/**< Digest being recomputed */
+	SHARE_F_SPECIAL		=	(1 << 2),		/**< Special (robots.txt, favicon)*/
+	SHARE_F_BASENAME	=	(1 << 3),		/**< File is in basename index */
+	SHARE_F_INDEXED		=	(1 << 4),		/**< File is in file_table index */
+};
 
 /**
  * Known MIME content types
  */
 
 enum share_mime_type {
-	SHARE_M_APPLICATION_BINARY = 0,
-	SHARE_M_IMAGE_PNG,
-	SHARE_M_TEXT_PLAIN,
+#define MIME_TYPE(id, name) id,
+#include "mime_types.h"
+#undef MIME_TYPE
+
+	SHARE_M_NUM
 };
 
-#define SHARE_REBUILDING	((struct shared_file *) 0x1)
+#define SHARE_REBUILDING shared_file_dummy()
+
+static inline struct shared_file *
+shared_file_dummy(void)
+{
+	static struct shared_file *dummy;
+	if (!dummy) {
+		dummy = deconstify_gpointer(vmm_trap_page());
+	}
+	return dummy;
+}
 
 struct gnutella_node;
 struct query_hashvec;
 
-/**
- * Global Data.
- */
-
-extern GSList *extensions, *shared_dirs;
-
 /*
  * Special return value from shared_file() during library rebuild time.
  * This is needed because we no longer block the GUI whilst scanning.
@@ -95,9 +101,9 @@ const gchar *share_mime_type(enum share_mime_type type);
 shared_file_t *shared_file(guint idx);
 shared_file_t *shared_file_by_name(const gchar *filename);
 shared_file_t * shared_file_ref(shared_file_t *sf);
-shared_file_t *shared_file_by_sha1(const gchar *sha1_digest);
+shared_file_t *shared_file_by_sha1(const struct sha1 *sha1);
 shared_file_t *shared_special(const gchar *path);
-void shared_file_unref(shared_file_t *sf);
+void shared_file_unref(shared_file_t **sf_ptr);
 void shared_file_remove(shared_file_t *sf);
 
 gboolean search_request(struct gnutella_node *n, struct query_hashvec *qhv);
@@ -110,9 +116,12 @@ size_t compact_query(gchar *search);
 void query_strip_oob_flag(const struct gnutella_node *n, gchar *data);
 void query_set_oob_flag(const struct gnutella_node *n, gchar *data);
 
-void shared_file_set_sha1(struct shared_file *, const gchar *sha1_digest);
+void shared_file_set_sha1(struct shared_file *, const struct sha1 *sha1);
+void shared_file_set_tth(struct shared_file *, const struct tth *tth);
 void shared_file_set_modification_time(struct shared_file *sf, time_t mtime);
+void shared_file_set_path(struct shared_file *sf, const gchar *pathname);
 
+void shared_file_check(const struct shared_file *sf);
 gboolean sha1_hash_available(const struct shared_file *sf);
 gboolean sha1_hash_is_uptodate(struct shared_file *sf);
 gboolean shared_file_is_partial(const struct shared_file *sf);
@@ -120,9 +129,11 @@ filesize_t shared_file_size(const shared_file_t *sf);
 guint32 shared_file_index(const shared_file_t *sf);
 time_t shared_file_modification_time(const struct shared_file *sf);
 const gchar *shared_file_path(const shared_file_t *sf);
-const gchar *shared_file_sha1(const shared_file_t *sf);
+const struct sha1 *shared_file_sha1(const shared_file_t *sf);
+const struct tth *shared_file_tth(const shared_file_t *sf);
 const gchar *shared_file_name_nfc(const shared_file_t *sf);
 const gchar *shared_file_name_canonic(const shared_file_t *sf);
+const gchar *shared_file_relative_path(const shared_file_t *sf);
 size_t shared_file_name_nfc_len(const shared_file_t *sf);
 size_t shared_file_name_canonic_len(const shared_file_t *sf);
 guint32 shared_file_flags(const shared_file_t *sf);
@@ -130,6 +141,7 @@ fileinfo_t *shared_file_fileinfo(const shared_file_t *sf);
 const gchar *shared_file_content_type(const shared_file_t *sf);
 void shared_file_from_fileinfo(fileinfo_t *fi);
 
+void record_query_string(const gchar muid[GUID_RAW_SIZE], const gchar *query);
 const gchar *map_muid_to_query_string(const gchar muid[GUID_RAW_SIZE]);
 
 #endif /* _core_share_h_ */
