@@ -1,5 +1,5 @@
 /*
- * $Id: spam.h 11170 2006-06-25 14:21:29Z cbiere $
+ * $Id: spam.h 13599 2007-05-11 18:15:46Z cbiere $
  *
  * Copyright (c) 2003, Markus Goetz & Raphael Manfredi
  *
@@ -40,9 +40,9 @@
 #define _core_spam_h_
 
 #include "common.h"
-#include "lib/host_addr.h"
+#include "spam_sha1.h"
 
-gboolean spam_check(const gchar *sha1);
+gboolean spam_check_filename_and_size(const gchar *filename, filesize_t size);
 void spam_init(void);
 void spam_close(void);
 
