@@ -1,5 +1,5 @@
 /*
- * $Id: tsync.c,v 1.17 2005/09/10 08:17:29 daichik Exp $
+ * $Id: tsync.c 13777 2007-05-29 00:23:11Z cbiere $
  *
  * Copyright (c) 2004, Raphael Manfredi
  *
@@ -35,7 +35,7 @@
 
 #include "common.h"
 
-RCSID("$Id: tsync.c,v 1.17 2005/09/10 08:17:29 daichik Exp $");
+RCSID("$Id: tsync.c 13777 2007-05-29 00:23:11Z cbiere $")
 
 #include "nodes.h"
 #include "tsync.h"
@@ -60,17 +60,17 @@ typedef enum {
  * along with the event that will expire those entries.
  */
 struct tsync {
-	tsync_magic_t magic;	/**< Magic of this structure for consistency checks */
-	tm_t sent;				/**< Time at which we sent the synchronization */
-	guint32 node_id;		/**< Node to which we sent the request */
-	gpointer expire_ev;		/**< Expiration callout queue callback */
-	gboolean udp;			/**< Whether request was sent using UDP */
+	tsync_magic_t magic; /**< Magic of this structure for consistency checks */
+	tm_t sent;			 /**< Time at which we sent the synchronization */
+	node_id_t node_id;	 /**< Node to which we sent the request */
+	cevent_t *expire_ev; /**< Expiration callout queue callback */
+	gboolean udp;		 /**< Whether request was sent using UDP */
 };
 
 /*
  * Table recording the "tsync" structures, indexed by sent time.
  */
-static GHashTable *tsync_by_time = NULL;	/**< tm_t -> tsync */
+static GHashTable *tsync_by_time;	/**< tm_t -> tsync */
 
 /**
  * Free a tsync structure.
@@ -81,9 +81,8 @@ tsync_free(struct tsync *ts)
 	g_assert(ts);
 	g_assert(ts->magic == TSYNC_MAGIC);
 
-	if (ts->expire_ev)
-		cq_cancel(callout_queue, ts->expire_ev);
-
+	cq_cancel(callout_queue, &ts->expire_ev);
+	node_id_unref(ts->node_id);
 	ts->magic = 0;
 	wfree(ts, sizeof(*ts));
 }
@@ -102,7 +101,7 @@ tsync_expire(cqueue_t *unused_cq, gpointer obj)
 	g_assert(ts);
 	g_assert(ts->magic == TSYNC_MAGIC);
 
-	if (dbg > 1)
+	if (GNET_PROPERTY(dbg) > 1)
 		printf("TSYNC expiring time %d.%d\n",
 			(gint) ts->sent.tv_sec, (gint) ts->sent.tv_usec);
 
@@ -130,7 +129,7 @@ tsync_expire(cqueue_t *unused_cq, gpointer obj)
  * we're sending the time synchronization request.
  */
 void
-tsync_send(struct gnutella_node *n, guint32 node_id)
+tsync_send(struct gnutella_node *n, const node_id_t node_id)
 {
 	struct tsync *ts;
 
@@ -142,7 +141,7 @@ tsync_send(struct gnutella_node *n, guint32 node_id)
 	ts->magic = TSYNC_MAGIC;
 	tm_now_exact(&ts->sent);
 	ts->sent.tv_sec = clock_loc2gmt(ts->sent.tv_sec);
-	ts->node_id = node_id;
+	ts->node_id = node_id_ref(node_id);
 	ts->udp = NODE_IS_UDP(n);
 
 	/*
@@ -150,12 +149,11 @@ tsync_send(struct gnutella_node *n, guint32 node_id)
 	 * the next TSYNC_EXPIRE_MS millisecs.
 	 */
 
-	ts->expire_ev = cq_insert(callout_queue, TSYNC_EXPIRE_MS,
-		tsync_expire, ts);
+	ts->expire_ev = cq_insert(callout_queue, TSYNC_EXPIRE_MS, tsync_expire, ts);
 
 	g_hash_table_insert(tsync_by_time, &ts->sent, ts);
 
-	vmsg_send_time_sync_req(n, ntp_detected, &ts->sent);
+	vmsg_send_time_sync_req(n, GNET_PROPERTY(ntp_detected), &ts->sent);
 }
 
 /**
@@ -171,7 +169,7 @@ tsync_send_timestamp(tm_t *orig, tm_t *final)
 {
 	struct tsync *ts;
 
-	if (dbg > 1) {
+	if (GNET_PROPERTY(dbg) > 1) {
 		tm_t elapsed = *final;
 		tm_sub(&elapsed, orig);
 		printf("TSYNC request %d.%d sent at %d.%d (delay = %.6f secs)\n",
@@ -182,7 +180,7 @@ tsync_send_timestamp(tm_t *orig, tm_t *final)
 
 	ts = g_hash_table_lookup(tsync_by_time, orig);
 	if (ts == NULL) {
-		if (dbg > 1)
+		if (GNET_PROPERTY(dbg) > 1)
 			printf("TSYNC request %d.%d not found, expired already?\n",
 				(gint) orig->tv_sec, (gint) orig->tv_usec);
 		return;
@@ -214,7 +212,7 @@ tsync_send_timestamp(tm_t *orig, tm_t *final)
 void
 tsync_got_request(struct gnutella_node *n, tm_t *got)
 {
-	vmsg_send_time_sync_reply(n, ntp_detected, got);
+	vmsg_send_time_sync_reply(n, GNET_PROPERTY(ntp_detected), got);
 }
 
 /**
@@ -242,7 +240,7 @@ tsync_got_reply(struct gnutella_node *n,
 
 	rtt = tm2f(&delay);
 
-	if (dbg > 1)
+	if (GNET_PROPERTY(dbg) > 1)
 		printf("TSYNC RTT for %d.%d with %s via %s is: %.6f secs\n",
 			(gint) sent->tv_sec, (gint) sent->tv_usec,
 			node_addr(n), NODE_IS_UDP(n) ? "UDP" : "TCP", (double) rtt);
@@ -255,7 +253,7 @@ tsync_got_reply(struct gnutella_node *n,
 	ts = g_hash_table_lookup(tsync_by_time, sent);
 
 	if (ts == NULL) {
-		if (dbg > 1)
+		if (GNET_PROPERTY(dbg) > 1)
 			printf("TSYNC sending time %d.%d not found (expired?)\n",
 				(gint) sent->tv_sec, (gint) sent->tv_usec);
 	} else {
@@ -277,7 +275,7 @@ tsync_got_reply(struct gnutella_node *n,
 
 		clock_offset = tm2f(&offset) / 2;
 
-		if (dbg > 1)
+		if (GNET_PROPERTY(dbg) > 1)
 			printf("TSYNC offset between %s clock at %s and ours: %.6f secs\n",
 				ntp ? "NTP" : "regular", node_addr(n), (double) clock_offset);
 
