@@ -1,5 +1,5 @@
 /*
- * $Id: udp.c 12242 2006-11-06 20:41:55Z cbiere $
+ * $Id: udp.c 13901 2007-06-20 22:32:33Z cbiere $
  *
  * Copyright (c) 2004, Raphael Manfredi
  *
@@ -35,26 +35,30 @@
 
 #include "common.h"
 
-RCSID("$Id: udp.c 12242 2006-11-06 20:41:55Z cbiere $")
+RCSID("$Id: udp.c 13901 2007-06-20 22:32:33Z cbiere $")
 
-#include "udp.h"
-#include "gmsg.h"
-#include "inet.h"
-#include "nodes.h"
-#include "sockets.h"
+#include "bogons.h"
 #include "bsched.h"
+#include "gmsg.h"
 #include "gnet_stats.h"
 #include "gnutella.h"
+#include "inet.h"
 #include "mq_udp.h"
-#include "routing.h"
-#include "pcache.h"
+#include "nodes.h"
 #include "ntp.h"
-#include "bogons.h"
+#include "pcache.h"
+#include "routing.h"
+#include "sockets.h"
+#include "udp.h"
 
 #include "if/gnet_property_priv.h"
 
+#include "lib/atoms.h"
 #include "lib/endian.h"
+#include "lib/hashlist.h"
 #include "lib/misc.h"
+#include "lib/walloc.h"
+
 #include "lib/override.h"		/* Must be the last header included */
 
 /**
@@ -68,6 +72,8 @@ udp_is_valid_gnet(struct gnutella_socket *s, gboolean truncated)
 	const gchar *msg;
 	guint16 size;				/**< Payload size, from the Gnutella message */
 
+	g_return_val_if_fail(n, FALSE);
+
 	if (s->pos < GTA_HEADER_SIZE) {
 		msg = "Too short";
 		goto not;
@@ -160,7 +166,7 @@ not:
 	/* FALL THROUGH */
 
 log:
-	if (udp_debug) {
+	if (GNET_PROPERTY(udp_debug)) {
 		g_warning("got invalid Gnutella packet from UDP (%s): %s",
 			host_addr_port_to_string(s->addr, s->port), msg);
 		if (s->pos)
@@ -226,7 +232,7 @@ udp_received(struct gnutella_socket *s, gboolean truncated)
 	if (bogons_check(s->addr)) {
 		bogus = TRUE;
 
-		if (udp_debug) {
+		if (GNET_PROPERTY(udp_debug)) {
 			g_warning("UDP %sdatagram (%d byte%s) received from bogus IP %s",
 				truncated ? "truncated " : "",
 				(gint) s->pos, s->pos == 1 ? "" : "s",
@@ -243,7 +249,7 @@ udp_received(struct gnutella_socket *s, gboolean truncated)
 	 * another node, only we'll use a special "pseudo UDP node" as origin.
 	 */
 
-	if (udp_debug > 19)
+	if (GNET_PROPERTY(udp_debug) > 19)
 		g_message("UDP got %s from %s%s", gmsg_infostr_full(s->buf),
 			bogus ? "BOGUS " : "", host_addr_port_to_string(s->addr, s->port));
 
@@ -258,10 +264,26 @@ void
 udp_send_msg(const gnutella_node_t *n, gconstpointer buf, gint len)
 {
 	g_assert(NODE_IS_UDP(n));
+	g_return_if_fail(n->outq);
 
 	mq_udp_node_putq(n->outq, gmsg_to_pmsg(buf, len), n);
 }
 
+/**
+ * Send a message to specified UDP node.
+ *
+ * It is up to the caller to clone the message if needed, otherwise the
+ * node's queue becomes the sole owner of the message and will pmsg_free() it.
+ */
+void
+udp_send_mb(const gnutella_node_t *n, pmsg_t *mb)
+{
+	g_assert(NODE_IS_UDP(n));
+	g_return_if_fail(n->outq);
+
+	mq_udp_node_putq(n->outq, mb, n);
+}
+
 /**
  * Send a Gnutella ping to the specified host via UDP, using the
  * specified MUID.
@@ -269,37 +291,159 @@ udp_send_msg(const gnutella_node_t *n, gconstpointer buf, gint len)
 void
 udp_connect_back(const host_addr_t addr, guint16 port, const gchar *muid)
 {
-	struct gnutella_node *n = node_udp_get_addr_port(addr, port);
-	gnutella_msg_init_t *m;
-	guint32 size;
+	if (udp_send_ping(muid, addr, port, FALSE)) {
+		if (GNET_PROPERTY(udp_debug) > 19)
+			g_message("UDP queued connect-back PING %s to %s\n",
+				guid_hex_str(muid), host_addr_port_to_string(addr, port));
+	}
+}
 
-	if (!udp_active() || !n->outq)
-		return;
+struct udp_ping {
+	gchar muid[GUID_RAW_SIZE];	/* MUST be at offset zero */
+	time_t added;				/**< Timestamp of insertion */
+};
 
-	m = build_ping_msg(muid, 1, FALSE, &size);
+static const time_delta_t UDP_PING_TIMEOUT	    = 30;	/**< seconds */
+static const size_t		  UDP_PING_MAX 			= 1024;	/**< amount to track */
+static const gint 		  UDP_PING_PERIODIC_MS	= 5000;	/**< milliseconds */
 
-	mq_udp_node_putq(n->outq, gmsg_to_pmsg(m, size), n);
+static hash_list_t *udp_pings;	/**< Tracks send/forwarded UDP Pings */
+static cevent_t *udp_ping_ev;	/**< Monitoring event */
 
-	if (udp_debug > 19)
-		printf("UDP queued connect-back PING %s (%u bytes) to %s\n",
-			guid_hex_str(muid), size, host_addr_port_to_string(addr, port));
+static inline void
+udp_ping_free(struct udp_ping *ping)
+{
+	wfree(ping, sizeof *ping);
+}
+
+static void
+udp_ping_expire(gboolean forced)
+{
+	time_t now;
+
+	g_return_if_fail(udp_pings);
+
+	now = tm_time();
+	for (;;) {
+		struct udp_ping *ping;
+		time_delta_t d;
+
+		ping = hash_list_head(udp_pings);
+		if (!ping) {
+			break;
+		}
+		if (!forced) {
+			d = delta_time(now, ping->added);
+			if (d > 0 && d <= UDP_PING_TIMEOUT) {
+				break;
+			}
+		}
+		hash_list_remove(udp_pings, ping);
+		udp_ping_free(ping);
+	}
+}
+
+/**
+ * Callout queue callback to perform periodic monitoring of the
+ * registered files.
+ */
+static void
+udp_ping_timer(cqueue_t *cq, gpointer unused_udata)
+{
+	(void) unused_udata;
+
+	/*
+	 * Re-install timer for next time.
+	 */
+
+	udp_ping_ev = cq_insert(cq, UDP_PING_PERIODIC_MS, udp_ping_timer, NULL);
+	udp_ping_expire(FALSE);
+}
+
+gboolean
+udp_ping_register(const gchar *muid)
+{
+	static gboolean initialized;
+	struct udp_ping *ping;
+	guint length;
+
+	g_assert(muid);
+
+	if (!initialized) {
+		initialized = TRUE;
+		udp_pings = hash_list_new(guid_hash, guid_eq);
+		udp_ping_timer(callout_queue, NULL);
+	}
+	g_return_val_if_fail(udp_pings, FALSE);
+
+	if (hash_list_contains(udp_pings, muid, NULL)) {
+		/* Probably a duplicate */
+		return FALSE;
+	}
+
+	/* random early drop */
+	length = hash_list_length(udp_pings);
+	if (length >= UDP_PING_MAX) {
+		return FALSE;
+	} else if (length > (UDP_PING_MAX / 4) * 3) {
+		if ((random_raw() % UDP_PING_MAX) < length)
+			return FALSE;
+	}
+
+	ping = walloc(sizeof *ping);
+	memcpy(ping->muid, muid, GUID_RAW_SIZE);
+	ping->added = tm_time();
+	hash_list_append(udp_pings, ping);
+	return TRUE;
+}
+
+gboolean
+udp_ping_is_registered(const gchar *muid)
+{
+	g_assert(muid);
+
+	if (udp_pings) {
+		struct udp_ping *ping;
+
+		ping = hash_list_remove(udp_pings, muid);
+		if (ping) {
+			udp_ping_free(ping);
+			return TRUE;
+		}
+	}
+	return FALSE;
 }
 
 /**
  * Send a Gnutella ping to the specified host.
  */
-void
-udp_send_ping(const host_addr_t addr, guint16 port)
+gboolean
+udp_send_ping(const gchar *muid, const host_addr_t addr, guint16 port,
+	gboolean uhc_ping)
 {
 	struct gnutella_node *n = node_udp_get_addr_port(addr, port);
-	gnutella_msg_init_t *m;
-	guint32 size;
 
-	if (!udp_active() || !n->outq)
-		return;
+	if (n) {
+		gnutella_msg_init_t *m;
+		guint32 size;
 
-	m = build_ping_msg(NULL, 1, FALSE, &size);
-	udp_send_msg(n, m, size);
+		m = build_ping_msg(muid, 1, uhc_ping, &size);
+		if (udp_ping_register(gnutella_header_get_muid(m))) {
+			udp_send_msg(n, m, size);
+			return TRUE;
+		}
+	}
+	return FALSE;
+}
+
+/**
+ * Final cleanup when application terminates.
+ */
+void
+udp_close(void)
+{
+	udp_ping_expire(TRUE);
+	hash_list_free(&udp_pings);
 }
 
 /* vi: set ts=4 sw=4 cindent: */
