@@ -1,5 +1,5 @@
 /*
- * $Id: verify_tth.h 12058 2006-09-27 19:33:55Z cbiere $
+ * $Id: verify_tth.h 13446 2007-04-28 04:22:50Z cbiere $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi
  *
@@ -36,12 +36,23 @@
 #ifndef _core_verify_tth_h_
 #define _core_verify_tth_h_
 
-struct shared_file;
+#include "common.h"
+#include "verify.h"
 
-void tt_verify_init(void);
-void tt_verify_close(void);
+struct tth;
 
-void request_tigertree(const struct shared_file *sf);
-void tt_compute_close(void);
+void verify_tth_append(const char *pathname,
+	filesize_t offset, filesize_t amount,
+	verify_callback callback, void *user_data);
+void verify_tth_prepend(const char *pathname,
+	filesize_t offset, filesize_t amount,
+	verify_callback callback, void *user_data);
+
+const struct tth *verify_tth_digest(const struct verify *);
+
+void verify_tth_init(void);
+void verify_tth_close(void);
+
+void request_tigertree(struct shared_file *sf, gboolean high_priority);
 
 #endif /* _core_verify_tth_h_ */
