@@ -1,5 +1,5 @@
 /*
- * $Id: version.c 12111 2006-10-15 17:10:46Z cbiere $
+ * $Id: version.c 13938 2007-06-23 04:52:29Z cbiere $
  *
  * Copyright (c) 2002-2003, Raphael Manfredi
  *
@@ -35,11 +35,7 @@
 
 #include "common.h"
 
-RCSID("$Id: version.c 12111 2006-10-15 17:10:46Z cbiere $")
-
-#ifdef I_SYS_UTSNAME
-#include <sys/utsname.h>		/* For uname() */
-#endif
+RCSID("$Id: version.c 13938 2007-06-23 04:52:29Z cbiere $")
 
 #include "version.h"
 #include "token.h"
@@ -60,8 +56,8 @@ RCSID("$Id: version.c 12111 2006-10-15 17:10:46Z cbiere $")
 
 #define SECS_PER_DAY	86400
 
-gchar *version_string = NULL;
-gchar *version_short_string = NULL;
+const gchar *version_string = NULL;
+const gchar *version_short_string = NULL;
 
 static version_t our_version;
 static version_t last_rel_version;
@@ -324,7 +320,7 @@ version_parse(const gchar *str, version_t *ver)
 			ver->build = (guint32) atoi(p);
 	}
 
-	if (dbg > 6)
+	if (GNET_PROPERTY(dbg) > 6)
 		version_dump(str, ver, "#");
 
 	return TRUE;
@@ -472,7 +468,7 @@ version_check(const gchar *str, const gchar *token, const host_addr_t addr)
 
 	cmp = version_cmp(target_version, &their_version);
 
-	if (dbg > 6)
+	if (GNET_PROPERTY(dbg) > 6)
 		version_dump(str, &their_version,
 			cmp == 0 ? "=" :
 			cmp > 0 ? "-" : "+");
@@ -483,7 +479,7 @@ version_check(const gchar *str, const gchar *token, const host_addr_t addr)
 
 	version_stamp(str, &their_version);
 
-	if (dbg > 6)
+	if (GNET_PROPERTY(dbg) > 6)
 		printf("VERSION time=%d\n", (gint) their_version.timestamp);
 
 	/*
@@ -495,7 +491,7 @@ version_check(const gchar *str, const gchar *token, const host_addr_t addr)
 		tok_error_t error;
 
 		if (token == NULL) {
-            if (dbg) {
+            if (GNET_PROPERTY(dbg)) {
                 g_message("got GTKG vendor string \"%s\" without token!", str);
             }
 			return FALSE;	/* Can't be correct */
@@ -518,7 +514,7 @@ version_check(const gchar *str, const gchar *token, const host_addr_t addr)
 			error = TOK_OK;		/* Our keys have expired, cannot validate */
 
 		if (error != TOK_OK) {
-            if (dbg) {
+            if (GNET_PROPERTY(dbg)) {
                 g_message("vendor string \"%s\" [%s] has wrong token "
                     "\"%s\": %s ", str, host_addr_to_string(addr), token,
                     tok_strerror(error));
@@ -548,11 +544,11 @@ version_check(const gchar *str, const gchar *token, const host_addr_t addr)
 			|| their_version.build > target_version->build) &&
 		target_version == &last_rel_version
 	) {
-		if (dbg > 6)
+		if (GNET_PROPERTY(dbg) > 6)
 			printf("VERSION is a SVN update of a release\n");
 
 		if (version_build_cmp(&last_dev_version, &their_version) > 0) {
-			if (dbg > 6)
+			if (GNET_PROPERTY(dbg) > 6)
 				printf("VERSION is less recent than latest dev we know\n");
 			return TRUE;
 		}
@@ -583,7 +579,7 @@ version_check(const gchar *str, const gchar *token, const host_addr_t addr)
 	 * We found a more recent version than the last version seen.
 	 */
 
-	if (dbg > 4)
+	if (GNET_PROPERTY(dbg) > 4)
 		printf("more recent %s VERSION \"%s\"\n",
 			target_version == &last_dev_version ? "dev" : "rel", str);
 
@@ -639,7 +635,7 @@ version_build_string(void)
 				sysname = un.sysname;
 				machine = un.machine;
 			} else {
-				g_warning("uname() failed: %s:", g_strerror(errno));
+				g_warning("uname() failed: %s", g_strerror(errno));
 			}
 		}
 #endif /* HAS_UNAME */
@@ -733,38 +729,9 @@ version_init(void)
 		delta_time(now, our_version.timestamp) > VERSION_ANCIENT_WARN ||
 		(our_version.tag &&
 			delta_time(now, our_version.timestamp) > VERSION_UNSTABLE_WARN)
-	)
-		*(guint32 *) &ancient_version = TRUE;
-}
-
-/**
- * Called when the version has expired since the indicated amount.
- * If that amount is greater than our grace period, refuse to run unless
- * they set the "ancient_version_force" property explicitly.
- */
-static void
-version_maybe_refuse(gint overtime)
-{
-	gchar *force;
-	property_t prop = PROP_ANCIENT_VERSION_FORCE;
-
-	if (overtime < VERSION_ANCIENT_GRACE)
-		return;
-
-	force = gnet_prop_get_string(prop, NULL, 0);
-
-	if (0 == strcmp(force, version_string)) {
-		G_FREE_NULL(force);
-		return;
+	) {
+		*deconstify_gboolean(&GNET_PROPERTY(ancient_version)) = TRUE;
 	}
-
-	/*
-	 * Sorry, they must explicitly allow us to run, we're too ancient.
-	 */
-
-	settings_ask_for_property(gnet_prop_name(prop), version_string);
-
-	/* NOTREACHED */
 }
 
 /**
@@ -797,14 +764,12 @@ version_ancient_warn(void)
 	elapsed = delta_time(now, our_version.timestamp);
 
 	if (elapsed > VERSION_ANCIENT_WARN || tok_is_ancient(now)) {
-		version_maybe_refuse(elapsed - VERSION_ANCIENT_WARN);
 		g_warning("version of gtk-gnutella is too old, you should upgrade!");
         gnet_prop_set_boolean_val(PROP_ANCIENT_VERSION, TRUE);
 		return;
 	}
 
 	if (our_version.tag && elapsed > VERSION_UNSTABLE_WARN) {
-		version_maybe_refuse(elapsed - VERSION_UNSTABLE_WARN);
 		g_warning("unstable version of gtk-gnutella is aging, please upgrade!");
         gnet_prop_set_boolean_val(PROP_ANCIENT_VERSION, TRUE);
 		return;
