@@ -1,5 +1,5 @@
 /*
- * $Id: c2ui.h,v 1.10 2005/12/11 13:52:51 rmanfredi Exp $
+ * $Id: c2ui.h 13629 2007-05-13 04:30:01Z cbiere $
  *
  * Copyright (c) 2004, Emile Roberts
  *
@@ -36,6 +36,8 @@
 #ifndef _if_bridge_c2ui_h_
 #define _if_bridge_c2ui_h_
 
+#include "common.h"
+
 /*
  *	SECTION 1 - Interface includes
  */
@@ -50,16 +52,9 @@
 /* Property table includes */
 #include "if/gui_property.h"
 
-/* Other includes */
-#include <glib.h>
-
-/* upload interface functions */
-void gcu_uploads_gui_set_gnet_addr(
-	gnet_upload_t u, host_addr_t addr, guint16 port);
-
 /* download interface functions */
 void gcu_download_enable_start_now(guint32 running_downloads,
-	guint32 max_downloads);
+	guint32 max_num_downloads);
 void gcu_gui_update_download(struct download *d, gboolean force);
 void gcu_gui_update_download_server(struct download *d);
 void gcu_gui_update_download_range(struct download *d);
@@ -68,7 +63,6 @@ void gcu_gui_update_download_host(struct download *d);
 void gcu_gui_update_download_abort_resume(void);
 void gcu_gui_update_download_clear(void);
 void gcu_gui_update_download_clear_now(void);
-void gcu_gui_update_queue_frozen(void);
 void gcu_download_gui_add(struct download *d);
 void gcu_download_gui_remove(struct download *d);
 
@@ -85,8 +79,8 @@ void gcu_statusbar_warning(const gchar *message);
 void gcu_statusbar_message(const gchar *message);
 
 /* upload interface functions */
-void gcu_upload_stats_gui_add(struct ul_stats *);
-void gcu_upload_stats_gui_update(const gchar *name, guint64 size);
+void gcu_upload_stats_gui_add(const struct ul_stats *);
+void gcu_upload_stats_gui_update(const struct ul_stats *);
 void gcu_upload_stats_gui_clear_all(void);
 
 /** bitzi results */
