@@ -1,5 +1,5 @@
 /*
- * $Id: ui2c.c 12175 2006-11-02 00:43:15Z cbiere $
+ * $Id: ui2c.c 13925 2007-06-22 05:24:10Z cbiere $
  *
  * Copyright (c) 2004, Emile Roberts
  *
@@ -57,7 +57,6 @@
 #include "if/core/search.h"
 #include "if/core/share.h"
 #include "if/core/settings.h"
-#include "if/core/upload_stats.h"
 #include "if/core/uploads.h"
 #include "if/core/upload_stats.h"
 #include "if/core/version.h"
@@ -79,15 +78,15 @@ guc_adns_resolve(const gchar *hostname,
 /*	bitzi interface functions (UI -> Core)*/
 
 void
-guc_query_bitzi_by_sha1(const gchar *sha1)
+guc_query_bitzi_by_sha1(const struct sha1 *sha1, filesize_t filesize)
 {
-    bitzi_query_by_sha1(sha1);
+    bitzi_query_by_sha1(sha1, filesize);
 }
 
-bitzi_data_t *
-guc_query_cache_bitzi_by_sha1(const gchar *sha1)
+gboolean
+guc_bitzi_has_cached_ticket(const struct sha1 *sha1)
 {
-    return bitzi_query_cache_by_sha1(sha1);
+	return bitzi_has_cached_ticket(sha1);
 }
 
 /*	download and src interface functions (UI -> Core)*/
@@ -103,6 +102,18 @@ guc_file_info_build_magnet(gnet_fi_t handle)
 	return file_info_build_magnet(handle);
 }
 
+gchar *
+guc_file_info_get_file_url(gnet_fi_t handle)
+{
+	return file_info_get_file_url(handle);
+}
+
+const gchar *
+guc_file_info_status_to_string(const gnet_fi_status_t *status)
+{
+	return file_info_status_to_string(status);
+}
+
 gint
 guc_download_get_http_req_percent(const struct download *d)
 {
@@ -117,18 +128,20 @@ guc_download_fallback_to_push(struct download *d, gboolean on_timeout,
 }
 
 gint
-guc_download_remove_all_from_peer(gchar *guid, const host_addr_t addr,
+guc_download_remove_all_from_peer(const gchar *guid, const host_addr_t addr,
 	guint16 port, gboolean unavailable)
 {
 	return download_remove_all_from_peer(guid, addr, port, unavailable);
 }
 
-gint guc_download_remove_all_named(const gchar *name)
+gint
+guc_download_remove_all_named(const gchar *name)
 {
 	return download_remove_all_named(name);
 }
 
-gint guc_download_remove_all_with_sha1(const gchar *sha1)
+gint
+guc_download_remove_all_with_sha1(const struct sha1 *sha1)
 {
 	return download_remove_all_with_sha1(sha1);
 }
@@ -157,6 +170,12 @@ guc_download_start(struct download *d, gboolean check_allowed)
 	download_start(d, check_allowed);
 }
 
+void
+guc_download_pause(struct download *d)
+{
+	download_pause(d);
+}
+
 gboolean
 guc_download_remove(struct download *d)
 {
@@ -200,25 +219,66 @@ guc_download_clear_stopped(gboolean complete,
 	download_clear_stopped(complete, failed, unavailable, now);
 }
 
-void
-guc_download_auto_new(const gchar *file, filesize_t size,
-	guint32 record_index, const host_addr_t addr, guint16 port,
-	const gchar *guid, gchar *hostname, gchar *sha1, time_t stamp,
-	gboolean file_size_known, fileinfo_t *fi,
-	gnet_host_vec_t *proxies, guint32 flags)
+guint
+guc_download_handle_magnet(const gchar *url)
 {
-	download_auto_new(file, size, record_index, addr, port, guid, hostname,
-		sha1, stamp, file_size_known, fi, proxies, flags);
+	return download_handle_magnet(url);
 }
 
 gboolean
-guc_download_new_unknown_size(const gchar *file,
-	guint32 record_index, const host_addr_t addr, guint16 port,
-	const gchar *guid, gchar *hostname, gchar *sha1, time_t stamp,
-	fileinfo_t *fi, gnet_host_vec_t *proxies, guint32 flags)
+guc_download_new(const gchar *filename,
+	const gchar *uri,
+	filesize_t size,
+	const host_addr_t addr,
+	guint16 port,
+	const gchar *guid,
+	const gchar *hostname,
+	const struct sha1 *sha1,
+	time_t stamp,
+	fileinfo_t *fi,
+	gnet_host_vec_t *proxies,
+	guint32 flags,
+	const gchar *parq_id)
 {
-	return download_new_unknown_size(file, record_index, addr, port, guid,
-		hostname, sha1, stamp, fi, proxies, flags);
+	return download_new(filename,
+			uri,
+			size,
+			addr,
+			port,
+		   	guid,
+			hostname,
+			sha1,
+			stamp,
+			fi,
+			proxies,
+			flags,
+			parq_id);
+}
+
+void
+guc_download_auto_new(const gchar *filename,
+	filesize_t size,
+	const host_addr_t addr,
+	guint16 port,
+	const gchar *guid,
+	const gchar *hostname,
+	const struct sha1 *sha1,
+	time_t stamp,
+	fileinfo_t *fi,
+	gnet_host_vec_t *proxies,
+	guint32 flags)
+{
+	download_auto_new(filename,
+		size,
+		addr,
+		port,
+		guid,
+		hostname,
+		sha1,
+		stamp,
+		fi,
+		proxies,
+		flags);
 }
 
 const gchar *
@@ -251,31 +311,9 @@ guc_download_something_to_clear(void)
 	return download_something_to_clear();
 }
 
-gboolean
-guc_download_new(const gchar *file, filesize_t size,
-			guint32 record_index, const host_addr_t addr, guint16 port,
-			const gchar *guid, gchar *hostname, gchar *sha1, time_t stamp,
-			fileinfo_t *fi, gnet_host_vec_t *proxies, guint32 flags)
-{
-	return download_new(file, size, record_index, addr, port, guid, hostname,
-			sha1, stamp, fi, proxies, flags);
-}
-
-gboolean
-guc_download_new_uri(const gchar *file, const gchar *uri, filesize_t size,
-	const host_addr_t addr, guint16 port,
-	const gchar *guid, gchar *hostname, gchar *sha1, time_t stamp,
-	fileinfo_t *fi, gnet_host_vec_t *proxies, guint32 flags)
-{
-	return download_new_uri(file, uri, size,
-			addr, port, guid, hostname,
-			sha1, stamp, fi, proxies, flags);
-}
-
-
 void
 guc_download_index_changed(const host_addr_t addr, guint16 port,
-	gchar *guid, filesize_t from, filesize_t to)
+	const gchar *guid, filesize_t from, filesize_t to)
 {
 	download_index_changed(addr, port, guid, from, to);
 }
@@ -399,6 +437,11 @@ guc_gnet_get_bw_stats(gnet_bw_source type, gnet_bw_stats_t *stats)
 	gnet_get_bw_stats(type, stats);
 }
 
+const gchar *
+guc_gnet_stats_drop_reason_to_string(msg_drop_reason_t reason)
+{
+	return gnet_stats_drop_reason_to_string(reason);
+}
 
 /*	hcache interface functions (UI -> Core)*/
 void
@@ -523,27 +566,27 @@ guc_node_add(const host_addr_t addr, guint16 port, guint32 flags)
 }
 
 void
-guc_node_remove_by_handle(gnet_node_t n)
+guc_node_remove_by_id(const node_id_t node_id)
 {
-	node_remove_by_handle(n);
+	node_remove_by_id(node_id);
 }
 
 void
-guc_node_remove_nodes_by_handle(GSList *node_list)
+guc_node_remove_nodes_by_id(const GSList *node_list)
 {
-	node_remove_nodes_by_handle(node_list);
+	node_remove_nodes_by_id(node_list);
 }
 
-void
-guc_node_get_status(const gnet_node_t n, gnet_node_status_t *s)
+gboolean
+guc_node_get_status(const node_id_t node_id, gnet_node_status_t *s)
 {
-	node_get_status(n, s);
+	return node_get_status(node_id, s);
 }
 
 gnet_node_info_t *
-guc_node_get_info(const gnet_node_t n)
+guc_node_get_info(const node_id_t node_id)
 {
-	return node_get_info(n);
+	return node_get_info(node_id);
 }
 
 void
@@ -558,16 +601,16 @@ guc_node_free_info(gnet_node_info_t *info)
 	node_free_info(info);
 }
 
-void
-guc_node_fill_flags(gnet_node_t n, gnet_node_flags_t *flags)
+gboolean
+guc_node_fill_flags(const node_id_t node_id, gnet_node_flags_t *flags)
 {
-	node_fill_flags(n, flags);
+	return node_fill_flags(node_id, flags);
 }
 
-void
-guc_node_fill_info(const gnet_node_t n, gnet_node_info_t *info)
+gboolean
+guc_node_fill_info(const node_id_t node_id, gnet_node_info_t *info)
 {
-	node_fill_info(n, info);
+	return node_fill_info(node_id, info);
 }
 
 const gchar *
@@ -608,6 +651,12 @@ guc_get_parq_dl_retry_delay(const struct download *d)
 }
 
 /*	search interface functions (UI -> Core)*/
+guint
+guc_search_handle_magnet(const gchar *url)
+{
+	return search_handle_magnet(url);
+}
+
 void
 guc_search_update_items(gnet_search_t sh, guint32 items)
 {
@@ -644,10 +693,10 @@ guc_search_set_create_time(gnet_search_t sh, time_t t)
 	search_set_create_time(sh, t);
 }
 
-gboolean
-guc_search_is_passive(gnet_search_t sh)
+const gchar *
+guc_search_query(gnet_search_t sh)
 {
-	return search_is_passive(sh);
+	return search_query(sh);
 }
 
 gboolean
@@ -656,6 +705,24 @@ guc_search_is_active(gnet_search_t sh)
 	return search_is_active(sh);
 }
 
+gboolean
+guc_search_is_browse(gnet_search_t sh)
+{
+	return search_is_browse(sh);
+}
+
+gboolean
+guc_search_is_local(gnet_search_t sh)
+{
+	return search_is_local(sh);
+}
+
+gboolean
+guc_search_is_passive(gnet_search_t sh)
+{
+	return search_is_passive(sh);
+}
+
 gboolean
 guc_search_is_expired(gnet_search_t sh)
 {
@@ -727,14 +794,7 @@ guc_listen_port(void)
 host_addr_t
 guc_listen_addr(enum net_type net)
 {
-	switch (net) {
-	case NET_TYPE_IPV4: return listen_addr();
-	case NET_TYPE_IPV6: return listen_addr6();
-	case NET_TYPE_LOCAL:
-	case NET_TYPE_NONE:
-		break;
-	}
-	return zero_host_addr;
+	return listen_addr_by_net(net);
 }
 
 const gchar *
