@@ -1,5 +1,5 @@
 /*
- * $Id: downloads.h 11882 2006-09-05 21:05:36Z cbiere $
+ * $Id: downloads.h 13880 2007-06-18 22:45:58Z cbiere $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi, Richard Eckart
  *
@@ -85,7 +85,7 @@ struct vernum {
 };
 
 struct dl_key {
-	gchar *guid;				/**< GUID of server (atom) */
+	const gchar *guid;			/**< GUID of server (atom) */
 	host_addr_t addr;			/**< IP address of server */
 	guint16 port;				/**< Port of server */
 };
@@ -94,50 +94,57 @@ enum dl_server_magic { DL_SERVER_MAGIC = 0x5e45e4ffU };
 
 struct dl_server {
 	enum dl_server_magic magic;	/**< Magic number */
-	gint refcnt;			 /**< Reference count */
-	struct dl_key *key;		 /**< Key properties */
-	list_t *list[DL_LIST_SZ]; /**< Download lists */
-	gchar *vendor;	 		/**< Remote server vendor string (atom) */
-	gchar *hostname;	 	/**< Remote hostname, if known (atom) */
-	gint country;			 /**< Country of origin -- encoded ISO3166 */
-	time_t retry_after;		 /**< Time at which we may retry from this host */
-	time_t dns_lookup;		/**< Last DNS lookup for hostname */
-	struct vernum parq_version; /**< Supported queueing version */
-	guint32 attrs;
+	gint refcnt;				/**< Reference count */
+	struct dl_key *key;			/**< Key properties */
+	list_t *list[DL_LIST_SZ];	/**< Download lists */
+	const gchar *vendor;		/**< Remote server vendor string (atom) */
+	const gchar *hostname;		/**< Remote hostname, if known (atom) */
 	GSList *proxies;		/**< Known push proxies (struct gnutella_host) */
-	time_t proxies_stamp;	/**< Time when list was last updated */
 	GHashTable *sha1_counts;
+	time_t retry_after;		/**< Time at which we may retry from this host */
+	time_t dns_lookup;		/**< Last DNS lookup for hostname */
+	time_t proxies_stamp;	/**< Time when list was last updated */
+	struct vernum parq_version; /**< Supported queueing version */
+	guint speed_avg;			/**< Average (EMA) upload speed, in bytes/sec */
+	guint32 attrs;
+	guint16 country;			/**< Country of origin -- encoded ISO3166 */
 };
 
-#define dl_server_valid(x)	((x) != NULL && (x)->magic == DL_SERVER_MAGIC)
+static inline gboolean
+dl_server_valid(const struct dl_server *s)
+{
+	return s != NULL && s->magic == DL_SERVER_MAGIC;
+}
 
 /**
  * Download states.
  */
 
 typedef enum {
-    GTA_DL_QUEUED           = 1,  /**< Download queued, will start later */
-    GTA_DL_CONNECTING       = 2,  /**< We are connecting to the server */
-    GTA_DL_PUSH_SENT        = 3,  /**< Sent a push, waiting connection */
-    GTA_DL_FALLBACK         = 4,  /**< Direct request failed, using push */
-    GTA_DL_REQ_SENT         = 5,  /**< Request sent, waiting for HTTP headers */
-    GTA_DL_HEADERS          = 6,  /**< We are receiving the HTTP headers */
-    GTA_DL_RECEIVING        = 7,  /**< We are receiving the data of the file */
-    GTA_DL_COMPLETED        = 8,  /**< Download is completed */
-    GTA_DL_ERROR            = 9,  /**< Download is stopped due to error */
-    GTA_DL_ABORTED          = 10, /**< User used the 'Abort Download' button */
-    GTA_DL_TIMEOUT_WAIT     = 11, /**< Waiting to try connecting again */
-    GTA_DL_REMOVED          = 12, /**< Download was removed, pending free */
-    GTA_DL_VERIFY_WAIT      = 13, /**< Waiting to verify SHA1 */
-    GTA_DL_VERIFYING        = 14, /**< Computing SHA1 */
-    GTA_DL_VERIFIED         = 15, /**< Verify of SHA1 done */
-    GTA_DL_MOVE_WAIT        = 16, /**< Waiting to be moved to "done/bad" dir */
-    GTA_DL_MOVING           = 17, /**< Being moved to "done/bad" dir */
-    GTA_DL_DONE             = 18, /**< All done! */
-    GTA_DL_SINKING          = 19, /**< Sinking HTML reply */
-    GTA_DL_ACTIVE_QUEUED    = 20, /**< Actively queued */
-    GTA_DL_PASSIVE_QUEUED   = 21, /**< Passively queued */
-    GTA_DL_REQ_SENDING      = 22  /**< Sending HTTP request */
+    GTA_DL_INVALID,			/**< Never used */
+    GTA_DL_QUEUED,			/**< Download queued, will start later */
+    GTA_DL_CONNECTING,  	/**< We are connecting to the server */
+    GTA_DL_PUSH_SENT,		/**< Sent a push, waiting connection */
+    GTA_DL_FALLBACK,		/**< Direct request failed, using push */
+    GTA_DL_REQ_SENT,		/**< Request sent, waiting for HTTP headers */
+    GTA_DL_HEADERS,			/**< We are receiving the HTTP headers */
+    GTA_DL_RECEIVING,		/**< We are receiving the data of the file */
+    GTA_DL_COMPLETED,		/**< Download is completed */
+    GTA_DL_ERROR,			/**< Download is stopped due to error */
+    GTA_DL_ABORTED,			/**< User used the 'Abort Download' button */
+    GTA_DL_TIMEOUT_WAIT,	/**< Waiting to try connecting again */
+    GTA_DL_REMOVED,			/**< Download was removed, pending free */
+    GTA_DL_VERIFY_WAIT,		/**< Waiting to verify SHA1 */
+    GTA_DL_VERIFYING,		/**< Computing SHA1 */
+    GTA_DL_VERIFIED,		/**< Verify of SHA1 done */
+    GTA_DL_MOVE_WAIT,		/**< Waiting to be moved to "done/bad" dir */
+    GTA_DL_MOVING,			/**< Being moved to "done/bad" dir */
+    GTA_DL_DONE,			/**< All done! */
+    GTA_DL_SINKING,			/**< Sinking HTML reply */
+    GTA_DL_ACTIVE_QUEUED,	/**< Actively queued */
+    GTA_DL_PASSIVE_QUEUED,	/**< Passively queued */
+    GTA_DL_REQ_SENDING,		/**< Sending HTTP request */
+	GTA_DL_IGNORING,		/**< Ignoring received data on resuming mismatch */
 } download_status_t;
 
 typedef struct download download_t;
@@ -182,8 +189,8 @@ struct download {
 
 	struct dl_file_info *file_info;
 	guint32 record_index;		/**< Index of the file on the Gnutella server */
-	gchar *file_name;			/**< Name of the file on the Gnutella server */
-	gchar *escaped_name;		/**< As file_name, with control chars escaped */
+	const gchar *file_name;		/**< Name of the file on the Gnutella server */
+	const gchar *escaped_name;	/**< As file_name, with control chars escaped */
 	filesize_t file_size;		/**< Total size of the file, in bytes */
 
 	filesize_t size;			/**< Total size of the next request, in bytes */
@@ -202,16 +209,20 @@ struct download {
 	time_t last_gui_update;		/**< Last stats update on the GUI */
 	time_t record_stamp;		/**< Stamp of the query hit that launched us */
 	time_t retry_after;			/**< Time at which we may retry this download */
+	time_t head_ping_sent;		/**< Time at which last HEAD ping was sent */
 	tm_t header_sent;			/**< When we sent headers, for latency */
 
 	guint32 retries;
 	guint32 timeout_delay;
 	guint32 served_reqs;		/**< Amount of served requests on connection */
+	guint32 mismatches;			/**< Amount of resuming data mismatches */
+	guint32 header_read_eof;	/**< EOF errors with empty headers */
+	guint32 data_timeouts;		/**< # of timeouts after getting headers */
 
 	const gchar *remove_msg;
 
-	gchar *sha1;				/**< Known SHA1 (binary atom), NULL if none */
-	gchar *uri;					/**< Uri if not dealing with regular gnutella
+	const struct sha1 *sha1;	/**< Known SHA1 (binary atom), NULL if none */
+	const gchar *uri;			/**< Uri if not dealing with regular gnutella
 								 **< file download */
 	guint32 last_dmesh;			/**< Time when last download mesh was sent */
 
@@ -230,41 +241,53 @@ struct download {
 	gboolean unavailable;		/**< Set on Timout, Push route lost */
 
 	struct cproxy *cproxy;		/**< Push proxy being used currently */
-	gpointer queue_status;		/**< Queuing status */
+	struct parq_dl_queued *parq_dl;	/**< Queuing status */
 	struct browse_ctx *browse;	/**< For browse-host requests */
+	struct thex_download *thex;	/**< For THEX downloads */
 };
 
 /*
  * Download flags.
  */
 
-#define DL_F_URIRES			0x00000001	/**< We sent a "/uri-res/N2R?" request */
-#define DL_F_PUSH_IGN		0x00000002	/**< Trying to ignore push flag */
-#define DL_F_OVERLAPPED		0x00000004	/**< We went through overlap checking */
-#define DL_F_REPLIED		0x00000008	/**< Servent replied to last request */
-#define DL_F_CHUNK_CHOSEN	0x00000010	/**< Retrying with specific chunk */
-#define DL_F_SHRUNK_REPLY	0x00000020	/**< Server sending less than we asked */
-#define DL_F_SUNK_DATA		0x00000040	/**< Whether we previously sunk data */
-#define DL_F_ACTIVE_QUEUED	0x00000080	/**< Download is actively queued */
-#define DL_F_PASSIVE_QUEUED	0x00000100	/**< Download is passively queued */
-#define DL_F_DNS_LOOKUP		0x00000200	/**< Attempted DNS lookup */
-#define DL_F_BROWSE			0x00000400	/**< Browse host type (requests "/") */
-#define DL_F_TRANSIENT		0x20000000	/**< Transient, don't persist */
-#define DL_F_SUSPENDED		0x40000000	/**< Suspended, do not schedule */
-#define DL_F_MARK			0x80000000	/**< Marked in traversal */
+enum {
+	DL_F_URIRES			= 1 << 0,	/**< We sent a "/uri-res/N2R?" request */
+	DL_F_PUSH_IGN		= 1 << 1,	/**< Trying to ignore push flag */
+	DL_F_OVERLAPPED		= 1 << 2,	/**< We went through overlap checking */
+	DL_F_REPLIED		= 1 << 3,	/**< Servent replied to last request */
+	DL_F_CHUNK_CHOSEN	= 1 << 4,	/**< Retrying with specific chunk */
+	DL_F_SHRUNK_REPLY	= 1 << 5,	/**< Server sending less than we asked */
+	DL_F_SUNK_DATA		= 1 << 6,	/**< Whether we previously sunk data */
+	DL_F_ACTIVE_QUEUED	= 1 << 7,	/**< Download is actively queued */
+	DL_F_PASSIVE_QUEUED	= 1 << 8,	/**< Download is passively queued */
+	DL_F_DNS_LOOKUP		= 1 << 9,	/**< Attempted DNS lookup */
+	DL_F_BROWSE			= 1 << 10,	/**< Browse host type (requests "/") */
+	DL_F_TRANSIENT		= 1 << 11,	/**< Transient, don't persist */
+	DL_F_SUSPENDED		= 1 << 12,	/**< Suspended, do not schedule */
+	DL_F_MARK			= 1 << 13,	/**< Marked in traversal */
+	DL_F_PREFIX_HEAD	= 1 << 14,	/**< Sent HEAD request before GET */
+	DL_F_INITIAL		= 1 << 15,	/**< First request on this connection */
+	DL_F_PAUSED			= 1 << 16,	/**< Paused by user */
+	DL_F_THEX			= 1 << 17,	/**< THEX download */
+	DL_F_UDP_PUSH		= 1 << 18,	/**< UDP push already attempted */
+	DL_F_FETCH_TTH		= 1 << 19,	/**< Tigertree data is being fetched */
+	DL_F_GOT_TTH		= 1 << 20	/**< Tigertree fetching finished */
+};
 
 /*
  * Server attributes.
  */
-
-#define DLS_A_UNUSED_1		0x00000001	/**< UNUSED */
-#define DLS_A_PUSH_IGN		0x00000002	/**< Ignore pushes, connect directly */
-#define DLS_A_UNUSED_2		0x00000004	/**< UNUSED */
-#define DLS_A_NO_HTTP_1_1	0x00000008	/**< Server does NOT support HTTP/1.1 */
-#define DLS_A_MINIMAL_HTTP	0x00000010	/**< Use minimalist HTTP with server */
-#define DLS_A_BANNING		0x00000020	/**< Server might be banning us */
-#define DLS_A_DNS_LOOKUP	0x00000080	/**< Perform DNS lookup if possible */
-#define DLS_A_REMOVED		0x80000000	/**< Server marked for removal */
+enum {
+	DLS_A_UNUSED_1		= 1 << 0,	/**< UNUSED */
+	DLS_A_PUSH_IGN		= 1 << 1,	/**< Ignore pushes, connect directly */
+	DLS_A_UNUSED_2		= 1 << 2,	/**< UNUSED */
+	DLS_A_NO_HTTP_1_1	= 1 << 3,	/**< Server does NOT support HTTP/1.1 */
+	DLS_A_MINIMAL_HTTP	= 1 << 4,	/**< Use minimalist HTTP with server */
+	DLS_A_BANNING		= 1 << 5,	/**< Server might be banning us */
+	DLS_A_DNS_LOOKUP	= 1 << 6,	/**< Perform DNS lookup if possible */
+	DLS_A_REMOVED		= 1 << 7,	/**< Server marked for removal */
+	DLS_A_FOOBAR		= 1 << 8	/**< Server is foobar */
+};
 
 /*
  * Access macros.
@@ -280,12 +303,16 @@ struct download {
 #define download_vendor_str(d) \
 	((d)->server->vendor ? (d)->server->vendor : "")
 
-#define download_path(d)		((d)->file_info->path)
-#define download_outname(d)		((d)->file_info->file_name)
 #define download_filesize(d)	((d)->file_info->size)
 #define download_filedone(d)	((d)->file_info->done + (d)->file_info->buffered)
 #define download_buffered(d)	((d)->buffers == NULL ? 0 : (d)->buffers->held)
 
+/*
+ * Sorted list of http_range_t objects, telling us about the available ranges
+ * on the remote size, in case the file is partial.
+ */
+#define download_ranges(d)		((d)->ranges)
+
 /*
  * State inspection macros.
  */
@@ -310,7 +337,8 @@ struct download {
 	|| (d)->status == GTA_DL_DONE       )
 
 #define DOWNLOAD_IS_ACTIVE(d)			\
-	(  (d)->status == GTA_DL_RECEIVING	)
+	(  (d)->status == GTA_DL_RECEIVING	\
+	|| (d)->status == GTA_DL_IGNORING	)
 
 #define DOWNLOAD_IS_WAITING(d)			\
 	(  (d)->status == GTA_DL_TIMEOUT_WAIT)
@@ -323,6 +351,7 @@ struct download {
 	|| (d)->status == GTA_DL_REQ_SENDING	\
 	|| (d)->status == GTA_DL_ACTIVE_QUEUED	\
 	|| (d)->status == GTA_DL_SINKING	\
+	|| (d)->status == GTA_DL_IGNORING	\
 	|| (d)->status == GTA_DL_HEADERS	)
 
 #define DOWNLOAD_IS_EXPECTING_GIV(d)	\
@@ -336,13 +365,19 @@ struct download {
 #define DOWNLOAD_IS_IN_PUSH_MODE(d) (d->push)
 #define DOWNLOAD_IS_VISIBLE(d)		(d->visible)
 
+gboolean download_has_blank_guid(const struct download *d);
+
 static inline void
 download_check(const struct download * const d)
 {
 	g_assert(d);
 	g_assert(DOWNLOAD_MAGIC == d->magic);
+	g_assert(!d->always_push || !download_has_blank_guid(d));
 }
 
+const gchar *download_pathname(const struct download *d);
+const gchar *download_basename(const struct download *d);
+
 /*
  * Public interface, visible only from the bridge.
  */
@@ -353,38 +388,52 @@ download_check(const struct download * const d)
  *        actually needs to be in downloads.h and should be called from
  *        search.h and not from search_gui.h.
  */
-void download_index_changed(const host_addr_t, guint16, gchar *,
+void download_index_changed(const host_addr_t, guint16, const gchar *,
 		guint32, guint32);
 
-gboolean download_new(const gchar *,
-	filesize_t, guint32, const host_addr_t addr, guint16,
-	const gchar *, gchar *, gchar *, time_t,
-    struct dl_file_info *, gnet_host_vec_t *, guint32 flags);
-gboolean download_new_uri(const gchar *file, const gchar *uri, filesize_t size,
-	  const host_addr_t addr, guint16 port, const gchar *guid, gchar *hostname,
-	  gchar *sha1, time_t stamp,
-	  struct dl_file_info *fi, gnet_host_vec_t *proxies, guint32 flags);
-void download_auto_new(const gchar *,
- 	filesize_t, guint32, const host_addr_t, guint16, const gchar *,
-	const gchar *, const gchar *, time_t,
-    gboolean, struct dl_file_info *, gnet_host_vec_t *,
+gboolean download_new(const gchar *filename,
+	const gchar *uri,
+	filesize_t size,
+	const host_addr_t addr,
+	guint16 port,
+	const gchar *guid,
+	const gchar *hostname,
+	const struct sha1 *sha1,
+	time_t stamp,
+    struct dl_file_info *fi,
+	const gnet_host_vec_t *proxies,
+	guint32 flags,
+	const gchar *parq_id);
+
+void download_auto_new(const gchar *filename,
+ 	filesize_t size,
+	const host_addr_t addr,
+	guint16 port,
+	const gchar *guid,
+	const gchar *hostname,
+	const struct sha1 *sha1,
+	time_t stamp,
+	struct dl_file_info *fi,
+	gnet_host_vec_t *proxies,
 	guint32 flags);
 
 void src_add_listener(src_listener_t, gnet_src_ev_t, frequency_t, guint32);
 void src_remove_listener(src_listener_t, gnet_src_ev_t);
 struct download *src_get_download(gnet_src_t src_handle);
 
+guint download_handle_magnet(const gchar *url);
 gchar *download_build_url(const struct download *d);
 gint download_get_http_req_percent(const struct download *d);
 void download_fallback_to_push(struct download *, gboolean, gboolean);
-gint download_remove_all_from_peer(gchar *guid, host_addr_t addr, guint16 port,
-	gboolean unavailable);
+gint download_remove_all_from_peer(const gchar *guid,
+		host_addr_t addr, guint16 port, gboolean unavailable);
 gint download_remove_all_named(const gchar *name);
-gint download_remove_all_with_sha1(const gchar *sha1);
+gint download_remove_all_with_sha1(const struct sha1 *sha1);
 void download_remove_file(struct download *d, gboolean reset);
 gboolean download_file_exists(const struct download *d);
 void download_requeue(struct download *);
 void download_start(struct download *, gboolean);
+void download_pause(struct download *);
 gboolean download_remove(struct download *);
 void download_abort(struct download *);
 void download_resume(struct download *);
@@ -392,10 +441,6 @@ void download_freeze_queue(void);
 void download_thaw_queue(void);
 gboolean download_queue_is_frozen(void);
 void download_clear_stopped(gboolean, gboolean, gboolean, gboolean);
-gboolean download_new_unknown_size(const gchar *file, guint32 record_index,
-	 const host_addr_t addr, guint16 port, const gchar *guid, gchar *hostname,
-	gchar *sha1, time_t stamp,
-	struct dl_file_info *fi, gnet_host_vec_t *proxies, guint32 flags);
 const gchar *download_get_hostname(const struct download *d);
 gdouble download_source_progress(const struct download *);
 gdouble download_total_progress(const struct download *);
@@ -404,6 +449,7 @@ struct download *src_get_download(gnet_src_t src_handle);
 void src_add_listener(src_listener_t cb, gnet_src_ev_t ev,
 	frequency_t t, guint32 interval);
 void src_remove_listener(src_listener_t cb, gnet_src_ev_t ev);
+guint download_speed_avg(struct download *d);
 
 #endif /* CORE_SOURCES */
 #endif /* _if_core_downloads_h_ */
