@@ -1,5 +1,5 @@
 /*
- * $Id: main.h 11741 2006-08-26 13:43:13Z rmanfredi $
+ * $Id: main.h 12761 2007-01-16 16:07:30Z cbiere $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi
  *
@@ -26,7 +26,7 @@
 #ifndef _if_core_main_h_
 #define _if_core_main_h_
 
-#include <glib.h>
+#include "common.h"
 
 guint32 main_get_build(void);
 
