@@ -1,5 +1,5 @@
 /*
- * $Id: search.h 12221 2006-11-04 04:48:55Z cbiere $
+ * $Id: search.h 13870 2007-06-18 03:27:25Z cbiere $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi, Richard Eckart
  *
@@ -48,24 +48,74 @@ enum {
 	SEARCH_F_LOCAL		= 1 << 4	/**< Search in local files */
 };
 
+typedef struct {
+	guint8 data[4 + 2];		/* IPv4 address (BE) + Port (LE) */
+} gnet_ipv4_host_t;
+
+typedef struct {
+	guint8 data[16 + 2];	/* IPv6 address + Port (LE) */
+} gnet_ipv6_host_t;
+
 /*
  * Host vectors held in query hits.
  */
 typedef struct gnet_host_vec {
-	gnet_host_t *hvec;		/**< Vector of alternate locations */
-	gint hvcnt;				/**< Amount of hosts in vector */
+	gnet_ipv4_host_t *hvec_v4;	/**< Vector of alternate IPv4 locations */
+	gnet_ipv6_host_t *hvec_v6;	/**< Vector of alternate IPv6 locations */
+	guint8 n_ipv4;				/**< Amount of hosts in IPv4 vector */
+	guint8 n_ipv6;				/**< Amount of hosts in IPv6 vector */
 } gnet_host_vec_t;
 
+static inline gint
+gnet_host_vec_count(const gnet_host_vec_t *hvec)
+{
+	return (gint) hvec->n_ipv4 + hvec->n_ipv6; 
+}
+
+static inline gnet_host_t
+gnet_host_vec_get(const gnet_host_vec_t *hvec, guint i)
+{
+	gnet_host_t host;
+	host_addr_t addr;
+	guint16 port;
+
+	g_assert(i < (guint) gnet_host_vec_count(hvec));
+
+	if (i < hvec->n_ipv4) {
+		addr = host_addr_peek_ipv4(hvec->hvec_v4[i].data);
+		port = peek_le16(&hvec->hvec_v4[i].data[4]);
+	} else {
+		i -= hvec->n_ipv4;
+		addr = host_addr_peek_ipv6(hvec->hvec_v6[i].data);
+		port = peek_le16(&hvec->hvec_v6[i].data[16]);
+	}
+
+	gnet_host_set(&host, addr, port);
+	return host;
+}
+
+gnet_host_vec_t *gnet_host_vec_alloc(void);
+void gnet_host_vec_free(gnet_host_vec_t **vec_ptr);
+gnet_host_vec_t *gnet_host_vec_copy(const gnet_host_vec_t *vec);
+void gnet_host_vec_add(gnet_host_vec_t *vec, host_addr_t addr, guint16 port);
+gnet_host_vec_t *gnet_host_vec_create(gnet_host_t *hvec, gint hcnt);
+
 /*
  * Result sets `status' flags.
  */
 enum {
-	 ST_LOCAL				= (1 << 16), /**< Local search result */
-	 ST_FW2FW				= (1 << 15), /**< Firewall-to-Firewall support */
-	 ST_HOSTILE				= (1 << 14), /**< From an hostile host */
-	 ST_UNREQUESTED			= (1 << 13), /**< Unrequested (OOB) result */
-	 ST_EVIL				= (1 << 12), /**< Carries evil filename */
-	 ST_SPAM				= (1 << 11), /**< Carries spam */
+	 ST_GOOD_TOKEN			= (1 << 22), /**< OOB v3 matched */
+	 ST_BROWSE				= (1 << 21), /**< Browse Host "search" result */
+	 ST_LOCAL				= (1 << 20), /**< Local search result */
+	 ST_FW2FW				= (1 << 19), /**< Firewall-to-Firewall support */
+	 ST_HOSTILE				= (1 << 18), /**< From an hostile host */
+	 ST_UNREQUESTED			= (1 << 17), /**< Unrequested (OOB) result */
+	 ST_EVIL				= (1 << 16), /**< Carries evil filename */
+	 ST_ALT_SPAM			= (1 << 15), /**< Carries alt-loc spam */
+	 ST_DUP_SPAM			= (1 << 14), /**< Carries spam known by URN */
+	 ST_NAME_SPAM			= (1 << 13), /**< Carries alt-loc spam */
+	 ST_URL_SPAM			= (1 << 12), /**< Carries action URL spam */
+	 ST_URN_SPAM			= (1 << 11), /**< Carries spam known by URN */
 	 ST_TLS					= (1 << 10), /**< Indicated support for TLS */
 	 ST_BH					= (1 << 9),	 /**< Browse Host support */
 	 ST_KNOWN_VENDOR		= (1 << 8),	 /**< Found known vendor code */
@@ -76,7 +126,9 @@ enum {
 	 ST_GGEP				= (1 << 3),	 /**< Trailer has a GGEP extension */
 	 ST_UPLOADED			= (1 << 2),	 /**< Is "stable", people downloaded */
 	 ST_BUSY				= (1 << 1),	 /**< Has currently no slots */
-	 ST_FIREWALL			= (1 << 0)	 /**< Is behind a firewall */
+	 ST_FIREWALL			= (1 << 0),	 /**< Is behind a firewall */
+
+	 ST_SPAM	= (ST_ALT_SPAM|ST_DUP_SPAM|ST_NAME_SPAM|ST_URL_SPAM|ST_URN_SPAM)
 };
 
 /*
@@ -100,22 +152,22 @@ typedef struct gnet_results_set {
 	host_addr_t addr;
 	host_addr_t last_hop;		/**< IP of delivering node */
 
-	gchar *guid;				/**< Servent's GUID (atom) */
-	gchar *hostname;			/**< Optional: server's hostname */
-	gchar *version;				/**< Version information (atom) */
-	gchar *query;				/**< Optional: Original query string (atom) */
+	const gchar *guid;			/**< Servent's GUID (atom) */
+	const gchar *hostname;		/**< Optional: server's hostname */
+	const gchar *version;		/**< Version information (atom) */
+	const gchar *query;			/**< Optional: Original query string (atom) */
 	gnet_host_vec_t *proxies;	/**< Optional: known push proxies */
 	GSList *records;
-	time_t  stamp;				/**< Reception time of the hit */
 
-	gint country;				/**< Country code -- encoded ISO3166 */
-	union vendor_code vcode;	/**< Vendor code */
+	time_t  stamp;				/**< Reception time of the hit */
+	vendor_code_t vcode;		/**< Vendor code */
 	guint32 speed;
 	guint32 num_recs;
-	
 	guint32 status;				/**< Parsed status bits from trailer */
+
     flag_t  flags;
 	guint16 port;
+	guint16 country;			/**< Country code -- encoded ISO3166 */
 	guint8 hops;
 	guint8 ttl;
 } gnet_results_set_t;
@@ -138,13 +190,14 @@ enum {
  * as identified by the parent results_set structure that contains this hit.
  */
 typedef struct gnet_record {
-	gchar  *name;				/**< File name */
-	gchar  *sha1;				/**< SHA1 URN (binary form, atom) */
-	gchar  *tag;				/**< Optional tag data string (atom) */
-	gchar  *xml;				/**< Optional XML data string (atom) */
-	gchar  *path;				/**< Optional path (atom) */
+	const gchar  *name;			/**< File name */
+	const struct sha1 *sha1;	/**< SHA1 URN (binary form, atom) */
+	const gchar  *tag;			/**< Optional tag data string (atom) */
+	const gchar  *xml;			/**< Optional XML data string (atom) */
+	const gchar  *path;			/**< Optional path (atom) */
 	gnet_host_vec_t *alt_locs;	/**< Optional: known alternate locations */
 	filesize_t size;			/**< Size of file, in bytes */
+	time_t  create_time;		/**< Create Time of file; zero if unknown */
 	guint32 file_index;			/**< Index for GET command */
     flag_t  flags;
 } gnet_record_t;
@@ -176,10 +229,14 @@ gboolean search_is_stopped(gnet_search_t sh);
 void search_reissue(gnet_search_t sh);
 void search_add_kept(gnet_search_t sh, guint32 kept);
 
-gboolean search_is_passive(gnet_search_t sh);
+const gchar *search_query(gnet_search_t sh);
+
 gboolean search_is_active(gnet_search_t sh);
-gboolean search_is_frozen(gnet_search_t sh);
+gboolean search_is_browse(gnet_search_t sh);
 gboolean search_is_expired(gnet_search_t sh);
+gboolean search_is_frozen(gnet_search_t sh);
+gboolean search_is_local(gnet_search_t sh);
+gboolean search_is_passive(gnet_search_t sh);
 
 void search_set_reissue_timeout(gnet_search_t sh, guint32 timeout);
 guint32 search_get_reissue_timeout(gnet_search_t sh);
@@ -195,6 +252,7 @@ gboolean search_browse(gnet_search_t sh,
 	const gchar *hostname, host_addr_t addr, guint16 port,
 	const gchar *guid, const gnet_host_vec_t *proxies, guint32 flags);
 gboolean search_locally(gnet_search_t sh, const gchar *query);
+guint search_handle_magnet(const gchar *url);
 
 #endif /* CORE_SOURCES */
 #endif /* _if_core_search_h_ */
