@@ -1,5 +1,5 @@
 /*
- * $Id: uploads.h 11727 2006-08-26 04:20:24Z cbiere $
+ * $Id: uploads.h 13870 2007-06-18 03:27:25Z cbiere $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi, Richard Eckart
  *
@@ -57,6 +57,7 @@ typedef struct gnet_upload_status {
 	guint32 bps;         /**< Current transfer rate */
 	guint32 avg_bps;     /**< Average transfer rate */
 	time_t  last_update;
+	guint reqnum;		 /**< Count of uploaded chunks */
 
 	gboolean parq_quick;
 	guint	parq_position;
@@ -69,24 +70,25 @@ typedef struct gnet_upload_status {
 typedef struct gnet_upload_info {
 	gnet_upload_t upload_handle;
 
-	gchar  *name;        /**< Name of requested file (converted to UTF-8) */
-
 	host_addr_t addr;		/**< remote IP address */
+	host_addr_t gnet_addr;	/**< Advertised Gnutella address for connecting */
 
 	filesize_t file_size;	/**< Size of requested file */
 	filesize_t range_start;	/**< First byte to send, inclusive */
 	filesize_t range_end;	/**< Last byte to send, inclusive */
 
+	const gchar  *name;		/**< Name of requested file (converted to UTF-8) */
+	const gchar  *user_agent;	/**< Remote user agent (converted to UTF-8) */
+
 	time_t  start_date;
+	time_t  last_update;
 
-	gchar  *user_agent;	/**< Remote user agent (converted to UTF-8) */
-	gint   country;  	/**< Contry of origin */
 	gboolean push;		/**< Whether we're pushing or not */
 	gboolean partial;	/**< Whether it's a partial file */
 	gboolean encrypted; /**< Whether the connection is (TLS) encrypted */
 	
-	host_addr_t gnet_addr;	/**< Advertised Gnutella address for connecting */
 	guint16 gnet_port;		/**< Advertised Gnutella listening port */
+	guint16 country;  		/**< Contry of origin */
 } gnet_upload_info_t;
 
 /*
@@ -115,7 +117,7 @@ typedef struct gnet_upload_info {
 	((u)->status == GTA_UL_HEADERS || (u)->status == GTA_UL_WAITING)
 
 struct ul_stats {
-	gchar  *filename;	/**< Atom */
+	const gchar  *filename;	/**< Atom */
 	filesize_t size;
 	guint32 attempts;
 	guint32 complete;
