@@ -1,5 +1,5 @@
 /*
- * $Id: search.h 6494 2004-08-25 05:40:41Z rmanfredi $
+ * $Id: search.h 12761 2007-01-16 16:07:30Z cbiere $
  *
  * Copyright (c) 2004, Raphael Manfredi
  *
@@ -26,7 +26,7 @@
 #ifndef _if_ui_gtk_search_h_
 #define _if_ui_gtk_search_h_
 
-#include <glib.h>
+#include "common.h"
 
 /*
  * Public interface, visible from the bridge.
