@@ -1,5 +1,5 @@
 /*
- * $Id: upload_stats.h 6531 2004-08-27 15:52:45Z rmanfredi $
+ * $Id: upload_stats.h 13057 2007-03-11 23:04:51Z cbiere $
  *
  * Copyright (c) 2004, Raphael Manfredi
  *
@@ -26,6 +26,8 @@
 #ifndef _if_ui_gtk_upload_stats_h_
 #define _if_ui_gtk_upload_stats_h_
 
+#include "common.h"
+
 /*
  * Public interface, visible from the bridge.
  */
@@ -35,9 +37,9 @@
 struct ul_stats;
 
 void upload_stats_gui_init(void);
-void upload_stats_gui_update(const gchar *name, guint64 size);
 void upload_stats_gui_clear_all(void);
-void upload_stats_gui_add(struct ul_stats *stat);
+void upload_stats_gui_add(const struct ul_stats *s);
+void upload_stats_gui_update(const struct ul_stats *s);
 
 #endif /* GUI_SOURCES */
 #endif /* _if_ui_gtk_upload_stats_h_ */
