@@ -1,5 +1,5 @@
 /*
- * $Id: cobs.h,v 1.8 2005/08/16 12:42:13 rmanfredi Exp $
+ * $Id: cobs.h 12500 2006-11-26 10:53:21Z cbiere $
  *
  * Copyright (c) 2002-2003, Raphael Manfredi
  *
@@ -36,7 +36,7 @@
 #ifndef _cobs_h_
 #define _cobs_h_
 
-#include <glib.h>
+#include "common.h"
 
 enum cobs_magic { COBS_MAGIC = 0xc0befU };
 
