@@ -1,5 +1,5 @@
 /*
- * $Id: cq.c,v 1.13 2005/11/03 08:37:39 rmanfredi Exp $
+ * $Id: cq.c 13015 2007-03-08 19:07:02Z cbiere $
  *
  * Copyright (c) 2002-2003, Raphael Manfredi
  *
@@ -35,7 +35,7 @@
 
 #include "common.h"
 
-RCSID("$Id: cq.c,v 1.13 2005/11/03 08:37:39 rmanfredi Exp $");
+RCSID("$Id: cq.c 13015 2007-03-08 19:07:02Z cbiere $")
 
 #include "cq.h"
 #include "misc.h"
@@ -43,6 +43,57 @@ RCSID("$Id: cq.c,v 1.13 2005/11/03 08:37:39 rmanfredi Exp $");
 #include "walloc.h"
 #include "override.h"		/* Must be the last header included */
 
+/**
+ * Callout queue event.
+ */
+struct cevent {
+	struct cevent *ce_bnext;	/**< Next item in hash bucket */
+	struct cevent *ce_bprev;	/**< Prev item in hash bucket */
+	cq_service_t ce_fn;			/**< Callback routine */
+	gpointer ce_arg;			/**< Argument to pass to said callback */
+	cq_time_t ce_time;			/**< Absolute trigger time (virtual cq time) */
+	guint ce_magic;				/**< Magic number */
+};
+
+/**
+ * @struct cqueue
+ *
+ * Callout queue descriptor.
+ *
+ * A callout queue is really a sorted linked list of events that are to
+ * happen in the near future, most recent coming first.
+ *
+ * Naturally, the insertion/deletion of items has to be relatively efficient.
+ * We don't want to go through all the items in the list to find the proper
+ * position for insertion.
+ *
+ * To do that, we maintain a parallel hash list of all the events, each event
+ * being inserted in the bucket i, where i is computed by abs_time % size,
+ * abs_time being the absolute time where the event is to be triggered
+ * and size being the size of the hash list. All the items under the bucket
+ * list are further sorted by increasing trigger time.
+ *
+ * To be completely generic, the callout queue "absolute time" is a mere
+ * unsigned long value. It can represent an amount of ms, or an amount of
+ * yet-to-come messages, or whatever. We don't care, and we don't want to care.
+ * The notion of "current time" is simply given by calling cq_clock() at
+ * regular intervals and giving it the "elasped time" since the last call.
+ */
+
+struct chash {
+	cevent_t *ch_head;			/**< Bucket list head */
+	cevent_t *ch_tail;			/**< Bucket list tail */
+};
+
+struct cqueue {
+	struct chash *cq_hash;		/**< Array of buckets for hash list */
+	cq_time_t cq_time;			/**< "current time" */
+	gint cq_ticks;				/**< Number of cq_clock() calls processed */
+	gint cq_items;				/**< Amount of recorded events */
+	gint cq_last_bucket;		/**< Last bucket slot we were at */
+	struct chash *cq_current;	/**< Current bucket scanned in cq_clock() */
+};
+
 #define HASH_SIZE	1024			/**< Hash list size, must be power of 2 */
 #define HASH_MASK	(HASH_SIZE - 1)
 #define EV_MAGIC	0xc0110172U		/**< Magic number for event marking */
@@ -56,7 +107,7 @@ RCSID("$Id: cq.c,v 1.13 2005/11/03 08:37:39 rmanfredi Exp $");
 #define EV_HASH(x) (((x) >> 5) & HASH_MASK)
 #define EV_OVER(x) (((x) >> 5) & ~HASH_MASK)
 
-#define valid_ptr(a)	((gpointer) (a) > GUINT_TO_POINTER(100U))
+#define valid_ptr(a)	(cast_to_gconstpointer(a) > GUINT_TO_POINTER(100U))
 
 cqueue_t *callout_queue;
 
@@ -65,17 +116,17 @@ cqueue_t *callout_queue;
  * initialize the "current time". Use zero if you don't care...
  */
 cqueue_t *
-cq_make(time_t now)
+cq_make(cq_time_t now)
 {
 	cqueue_t *cq;
 
-	cq = (cqueue_t *) g_malloc(sizeof(*cq));
+	cq = g_malloc(sizeof *cq);
 
 	/*
 	 * The cq_hash hash list is used to speed up insert/delete operations.
 	 */
 
-	cq->cq_hash = (struct chash *) g_malloc0(HASH_SIZE * sizeof(struct chash));
+	cq->cq_hash = g_malloc0(HASH_SIZE * sizeof *cq->cq_hash);
 	cq->cq_items = 0;
 	cq->cq_ticks = 0;
 	cq->cq_time = now;
@@ -101,7 +152,7 @@ cq_free(cqueue_t *cq)
 	for (ch = cq->cq_hash, i = 0; i < HASH_SIZE; i++, ch++) {
 		for (ev = ch->ch_head; ev; ev = ev_next) {
 			ev_next = ev->ce_bnext;
-			wfree(ev, sizeof(*ev));
+			wfree(ev, sizeof *ev);
 		}
 	}
 
@@ -116,7 +167,7 @@ static void
 ev_link(cqueue_t *cq, cevent_t *ev)
 {
 	struct chash *ch;		/* Hashing bucket */
-	time_t trigger;			/* Trigger time */
+	cq_time_t trigger;		/* Trigger time */
 	cevent_t *hev;			/* To loop through the hash bucket */
 
 	g_assert(valid_ptr(cq));
@@ -133,7 +184,7 @@ ev_link(cqueue_t *cq, cevent_t *ev)
 	 * cq_clock() run.
 	 */
 
-	if (trigger  <= cq->cq_time)
+	if (trigger <= cq->cq_time)
 		ch = cq->cq_current;
 	else
 		ch = &cq->cq_hash[EV_HASH(trigger)];
@@ -242,9 +293,14 @@ ev_unlink(cqueue_t *cq, cevent_t *ev)
  * time we shall call fn(cq, arg), where cq is the callout queue from
  * where we triggered, and arg is an additional argument.
  *
+ * @param cq		The callout queue
+ * @param delay		The delay, expressed in cq's "virtual time" (see cq_clock)
+ * @param fn		The callback function
+ * @param arg		The argument to be passed to the callback function
+ *
  * @returns the handle, or NULL on error.
  */
-gpointer
+cevent_t *
 cq_insert(cqueue_t *cq, gint delay, cq_service_t fn, gpointer arg)
 {
 	cevent_t *ev;				/* Event to insert */
@@ -253,7 +309,7 @@ cq_insert(cqueue_t *cq, gint delay, cq_service_t fn, gpointer arg)
 	g_assert(valid_ptr(cast_func_to_gpointer((func_ptr_t) fn)));
 	g_assert(delay > 0);
 
-	ev = (cevent_t *) walloc(sizeof(*ev));
+	ev = walloc(sizeof *ev);
 
 	ev->ce_magic = EV_MAGIC;
 	ev->ce_time = cq->cq_time + delay;
@@ -275,18 +331,21 @@ cq_insert(cqueue_t *cq, gint delay, cq_service_t fn, gpointer arg)
  * the list before firing it off.
  */
 void
-cq_cancel(cqueue_t *cq, gpointer handle)
+cq_cancel(cqueue_t *cq, cevent_t **handle_ptr)
 {
-	cevent_t *ev = (cevent_t *) handle;
+	cevent_t *ev = *handle_ptr;
 
-	g_assert(valid_ptr(cq));
-	g_assert(valid_ptr(handle));
-	g_assert(ev->ce_magic == EV_MAGIC);
-	g_assert(cq->cq_items > 0);
+	if (ev) {
+		g_assert(valid_ptr(cq));
+		g_assert(valid_ptr(ev));
+		g_assert(ev->ce_magic == EV_MAGIC);
+		g_assert(cq->cq_items > 0);
 
-	ev_unlink(cq, ev);
-	ev->ce_magic = 0;			/* Prevent further use as a valid event */
-	wfree(ev, sizeof(*ev));
+		ev_unlink(cq, ev);
+		ev->ce_magic = 0;			/* Prevent further use as a valid event */
+		wfree(ev, sizeof *ev);
+		*handle_ptr = NULL;
+	}
 }
 
 /**
@@ -295,9 +354,9 @@ cq_cancel(cqueue_t *cq, gpointer handle)
  * expired, i.e. that the event has not triggered yet.
  */
 void
-cq_resched(cqueue_t *cq, gpointer handle, gint delay)
+cq_resched(cqueue_t *cq, cevent_t *handle, gint delay)
 {
-	cevent_t *ev = (cevent_t *) handle;
+	cevent_t *ev = handle;
 
 	g_assert(valid_ptr(cq));
 	g_assert(valid_ptr(handle));
@@ -340,7 +399,7 @@ cq_expire(cqueue_t *cq, cevent_t *ev)
 	g_assert(ev->ce_magic == EV_MAGIC);
 	g_assert(valid_ptr(cast_func_to_gpointer((func_ptr_t) fn)));
 
-	cq_cancel(cq, ev);			/* Remove event from queue before firing */
+	cq_cancel(cq, &ev);			/* Remove event from queue before firing */
 
 	/*
 	 * All the callout queue data structures were updated.
@@ -356,15 +415,19 @@ cq_expire(cqueue_t *cq, cevent_t *ev)
  *
  * Called to notify us about the elapsed "time" so that we can expire timeouts
  * and maintain our notion of "current time".
+ *
+ * NB: The time maintained by the callout queue is "virtual".  It's the
+ * elapased delay given by regular calls to cq_clock() that define its unit.
+ * For gtk-gnutella, the time unit is the millisecond.
  */
 void
 cq_clock(cqueue_t *cq, gint elapsed)
 {
-	time_t now;
 	gint bucket;
 	gint last_bucket;
 	struct chash *ch;
 	cevent_t *ev;
+	cq_time_t now;
 
 	g_assert(valid_ptr(cq));
 	g_assert(elapsed >= 0);
@@ -439,21 +502,21 @@ done:
  * Called every CALLOUT_PERIOD to heartbeat the callout queue.
  */
 static gboolean
-callout_timer(gpointer p)
+callout_timer(gpointer unused_p)
 {
-	static tm_t last_period = { 0L, 0L };
+	static tm_t last_period;
 	GTimeVal tv;
 	gint delay;
 
-	(void) p;
+	(void) unused_p;
 	tm_now_exact(&tv);
 
 	/*
 	 * How much elapsed since last call?
 	 */
 
-	delay = (gint) ((tv.tv_sec - last_period.tv_sec) * 1000 +
-		(tv.tv_usec - last_period.tv_usec) / 1000);
+	delay = (tv.tv_sec - last_period.tv_sec) * 1000 +
+		(tv.tv_usec - last_period.tv_usec) / 1000;
 
 	last_period = tv;		/* struct copy */
 
@@ -480,8 +543,13 @@ callout_timer(gpointer p)
 gdouble
 callout_queue_coverage(gint old_ticks)
 {
-	return 
-		(callout_queue->cq_ticks - old_ticks) * CALLOUT_PERIOD / 1000.0;
+	return (callout_queue->cq_ticks - old_ticks) * CALLOUT_PERIOD / 1000.0;
+}
+
+gint
+cq_ticks(cqueue_t *cq)
+{
+	return cq->cq_ticks;
 }
 
 /**
@@ -504,4 +572,4 @@ cq_close(void)
 	callout_queue = NULL;
 }
 
-/* vi: set ts=4: */
+/* vi: set ts=4 sw=4 cindent: */
