@@ -1,5 +1,5 @@
 /*
- * $Id: crc.h 9433 2005-09-07 17:35:30Z cbiere $
+ * $Id: crc.h 12500 2006-11-26 10:53:21Z cbiere $
  *
  * Copyright (c) 2003, Raphael Manfredi
  *
@@ -36,7 +36,7 @@
 #ifndef _crc_h_
 #define _crc_h_
 
-#include <glib.h>
+#include "common.h"
 
 void crc_init(void);
 guint32 crc32_update_crc(guint32 crc_accum, gconstpointer data, size_t len);
