@@ -1,5 +1,5 @@
 /*
- * $Id: file.c,v 1.11 2005/09/08 21:45:58 cbiere Exp $
+ * $Id: file.c 13659 2007-05-15 03:01:23Z cbiere $
  *
  * Copyright (c) 2002-2003, Raphael Manfredi
  *
@@ -35,13 +35,15 @@
 
 #include "common.h"
 
-RCSID("$Id: file.c,v 1.11 2005/09/08 21:45:58 cbiere Exp $");
+RCSID("$Id: file.c 13659 2007-05-15 03:01:23Z cbiere $")
 
 #include "file.h"
 #include "misc.h"
 #include "tm.h"
 #include "override.h"		/* Must be the last header included */
 
+static guint32 common_dbg = 0;	/**< XXX -- need to init lib's props --RAM */
+
 static const gchar orig_ext[] = "orig";
 static const gchar new_ext[] = "new";
 static const gchar instead_str[] = " instead";
@@ -99,17 +101,12 @@ open_read(
 	g_assert(fvcnt >= 1);
 
 	path = make_pathname(fv->dir, fv->name);
-	g_return_val_if_fail(NULL != path, NULL);
 	if (!is_absolute_path(path)) {
 		G_FREE_NULL(path);
 		return NULL;
 	}
 
 	path_orig = g_strdup_printf("%s.%s", path, orig_ext);
-	if (NULL == path_orig)
-        G_FREE_NULL(path);
-    g_return_val_if_fail(NULL != path_orig, NULL);
-
 	in = fopen(path, "r");
 	if (in) {
 		if (renaming && -1 == rename(path, path_orig))
@@ -122,8 +119,8 @@ open_read(
 			g_warning("[%s] failed to retrieve from \"%s\": %s", what, path,
 				g_strerror(errno));
 		}
-        if (fvcnt > 1)
-            g_warning("[%s] trying to load from alternate locations...", what);
+        if (fvcnt > 1 && common_dbg > 0)
+            g_message("[%s] trying to load from alternate locations...", what);
     }
 
 	/*
@@ -164,21 +161,21 @@ open_read(
 		}
 	}
 
-	if (in)
-		g_warning("[%s] retrieving from \"%s\"%s", what, path, instead);
-	else if (instead == instead_str)
+	if (in) {
+		if (common_dbg > 0)
+			g_message("[%s] retrieving from \"%s\"%s", what, path, instead);
+	} else if (instead == instead_str) {
 		g_warning("[%s] unable to retrieve: tried %d alternate location%s",
 			what, fvcnt, fvcnt == 1 ? "" : "s");
-    else
+    } else {
 		g_warning("[%s] unable to retrieve: no alternate locations known",
 			what);
+	}
 
 out:
 
-	if (NULL != path)
-		G_FREE_NULL(path);
-	if (NULL != path_orig)
-		G_FREE_NULL(path_orig);
+	G_FREE_NULL(path);
+	G_FREE_NULL(path_orig);
 	if (in != NULL && chosen != NULL)
 		*chosen = idx;
 
@@ -274,6 +271,7 @@ file_config_close(FILE *out, const file_path_t *fv)
 {
 	char *path = NULL;
 	char *path_new = NULL;
+	gboolean success = FALSE;
 
 	if (0 != fclose(out)) {
 		g_warning("could not flush \"%s\": %s", fv->name, g_strerror(errno));
@@ -292,17 +290,13 @@ file_config_close(FILE *out, const file_path_t *fv)
 		goto failed;
 	}
 
-	G_FREE_NULL(path_new);
-	G_FREE_NULL(path);
-	return TRUE;
+	success = TRUE;
 
 failed:
 
-	if (NULL != path_new)
-		G_FREE_NULL(path_new);
-	if (NULL != path)
-		G_FREE_NULL(path);
-	return FALSE;
+	G_FREE_NULL(path_new);
+	G_FREE_NULL(path);
+	return success;
 }
 
 /**
@@ -325,7 +319,11 @@ file_config_preamble(FILE *out, const gchar *what)
 void
 file_path_set(file_path_t *fp, const char *dir, const char *name)
 {
-	g_assert(NULL != fp);
+	g_assert(fp);
+	g_assert(dir);
+	g_assert(name);
+	g_assert(is_absolute_path(dir));
+
 	fp->dir = dir;
 	fp->name = name;
 }
@@ -338,8 +336,8 @@ file_path_set(file_path_t *fp, const char *dir, const char *name)
 static gint
 do_open(const gchar *path, gint flags, gint mode, gboolean missing)
 {
-	gint fd;
 	const gchar *what;
+	gint fd;
 
 	if (!is_absolute_path(path)) {
 		errno = EPERM;
@@ -351,34 +349,37 @@ do_open(const gchar *path, gint flags, gint mode, gboolean missing)
 #endif /* O_NOCTTY */
 
 	fd = open(path, flags, mode);
-	if (fd >= 0)
-		return fd;
+	if (fd < 0) {
+		if (flags & O_CREAT)
+			what = "create";
+		else if (flags & O_RDONLY)
+			what = "read";
+		else if (flags & O_WRONLY)
+			what = "write into";
+		else
+			what = "open";
 
-	if (flags & O_CREAT)
-		what = "create";
-	else if (flags & O_RDONLY)
-		what = "read";
-	else if (flags & O_WRONLY)
-		what = "write into";
-	else
-		what = "open";
+		/*
+		 * If we ran out of file descriptors, try to reclaim one from the
+		 * banning pool and retry.
+		 */
 
-	/*
-	 * If we ran out of file descriptors, try to reclaim one from the
-	 * banning pool and retry.
-	 */
-
-	if (
-		(errno == EMFILE || errno == ENFILE) &&
-		reclaim_fd != NULL && (*reclaim_fd)()
-	) {
-		fd = open(path, flags, mode);
-		if (fd >= 0) {
-			g_warning("had to close a banned fd to %s file", what);
-			return fd;
+		if (
+			(errno == EMFILE || errno == ENFILE) &&
+			reclaim_fd != NULL && (*reclaim_fd)()
+		) {
+			fd = open(path, flags, mode);
+			if (fd >= 0) {
+				g_warning("do_open(): had to close a banned fd to %s file",
+					what);
+			}
 		}
 	}
 
+	if (fd >= 0) {
+		return get_non_stdio_fd(fd);
+	}
+
 	/*
 	 * Hack for broken libc, which can return -1 with errno = 0!
 	 * This happens when compiling with gcc-3.x and linking with -lpthread
@@ -387,12 +388,15 @@ do_open(const gchar *path, gint flags, gint mode, gboolean missing)
 	 */
 
 	if (errno == 0) {
-		g_warning("open() returned -1 with errno = 0, assuming ENOENT");
+		g_warning("do_open(): "
+			"open() returned -1 with errno = 0, assuming ENOENT");
 		errno = ENOENT;
 	}
 
-	if (!missing || errno != ENOENT)
-		g_warning("can't %s file \"%s\": %s", what, path, g_strerror(errno));
+	if (!missing || errno != ENOENT) {
+		g_warning("do_open(): can't %s file \"%s\": %s",
+			what, path, g_strerror(errno));
+	}
 
 	return -1;
 }
@@ -428,6 +432,17 @@ file_create(const gchar *path, gint flags, gint mode)
 	return do_open(path, flags | O_CREAT, mode, FALSE);
 }
 
+/**
+ * Create file, returning file descriptor or -1 on error with errno set.
+ * Errors are logged as a warning, unless the error is ENOENT which means
+ * the directory does not exist.
+ */
+gint
+file_create_missing(const gchar *path, gint flags, gint mode)
+{
+	return do_open(path, flags | O_CREAT, mode, TRUE);
+}
+
 /**
  * Open file, returning FILE pointer if success or NULL on error.
  * Errors are logged as a warning, unless error is ENOENT and `missing'
@@ -502,4 +517,15 @@ file_fopen_missing(const gchar *path, const gchar *mode)
 	return do_fopen(path, mode, TRUE);
 }
 
+void
+file_set_nonblocking(gint fd)
+{
+	gint ret, flags;
+
+	ret = fcntl(fd, F_GETFL, 0);
+	flags = ret | VAL_O_NONBLOCK;
+	if (flags != ret)
+		fcntl(fd, F_SETFL, flags);
+}
+
 /* vi: set ts=4: */
