@@ -1,5 +1,5 @@
 /*
- * $Id: file.h 8376 2005-06-25 01:37:45Z daichik $
+ * $Id: file.h 13659 2007-05-15 03:01:23Z cbiere $
  *
  * Copyright (c) 2002-2003, Raphael Manfredi
  *
@@ -67,8 +67,11 @@ void file_path_set(file_path_t *fp, const char *dir, const char *name);
 gint file_open(const gchar *path, gint flags);
 gint file_open_missing(const gchar *path, gint flags);
 gint file_create(const gchar *path, gint flags, gint mode);
+gint file_create_missing(const gchar *path, gint flags, gint mode);
 FILE *file_fopen(const gchar *path, const gchar *mode);
 FILE *file_fopen_missing(const gchar *path, const gchar *mode);
 
+void file_set_nonblocking(gint fd);
+
 #endif /* _file_ */
 /* vi: set ts=4 sw=4 cindent: */
