@@ -1,5 +1,5 @@
 /*
- * $Id: hashlist.h 11315 2006-07-16 08:17:19Z cbiere $
+ * $Id: hashlist.h 13918 2007-06-22 00:43:29Z cbiere $
  *
  * Copyright (c) 2003, Christian Biere
  *
@@ -26,33 +26,37 @@
 #ifndef _hashlist_h_
 #define _hashlist_h_
 
+#include "common.h"
 #include "glib-missing.h"
 
 typedef struct hash_list_iter hash_list_iter_t;
 typedef struct hash_list hash_list_t;
 
 hash_list_t *hash_list_new(GHashFunc hash_func, GEqualFunc eq_func);
-void hash_list_free(hash_list_t *hl);
-void hash_list_remove(hash_list_t *hl, gpointer key);
-void hash_list_append(hash_list_t *hl, gpointer key);
-void hash_list_prepend(hash_list_t *hl, gpointer key);
-void hash_list_insert_sorted(hash_list_t *hl, gpointer key, GCompareFunc func);
-void hash_list_moveto_head(hash_list_t *hl, gpointer key);
-void hash_list_moveto_tail(hash_list_t *hl, gpointer key);
-gpointer hash_list_first(const hash_list_t *hl);
-gpointer hash_list_last(const hash_list_t *hl);
+void hash_list_free(hash_list_t **hl_ptr);
+gpointer hash_list_remove(hash_list_t *hl, gconstpointer key);
+gpointer hash_list_shift(hash_list_t *hl);
+void hash_list_append(hash_list_t *hl, gconstpointer key);
+void hash_list_prepend(hash_list_t *hl, gconstpointer key);
+void hash_list_insert_sorted(hash_list_t *hl, gconstpointer key,
+	GCompareFunc func);
+void hash_list_moveto_head(hash_list_t *hl, gconstpointer key);
+void hash_list_moveto_tail(hash_list_t *hl, gconstpointer key);
+gpointer hash_list_head(const hash_list_t *hl);
+gpointer hash_list_tail(const hash_list_t *hl);
+gpointer hash_list_next(hash_list_t *hl, gconstpointer key);
 guint hash_list_length(const hash_list_t *hl);
+
 hash_list_iter_t *hash_list_iterator(hash_list_t *hl);
-hash_list_iter_t *hash_list_iterator_last(hash_list_t *hl);
-void hash_list_release(hash_list_iter_t *i);
-gboolean hash_list_has_next(const hash_list_iter_t *i);
-gboolean hash_list_has_previous(const hash_list_iter_t *i);
-gboolean hash_list_has_follower(const hash_list_iter_t *i);
-gpointer hash_list_next(hash_list_iter_t *i);
-gpointer hash_list_previous(hash_list_iter_t *i);
-gpointer hash_list_follower(hash_list_iter_t *i);
+hash_list_iter_t *hash_list_iterator_tail(hash_list_t *hl);
+void hash_list_iter_release(hash_list_iter_t **iter_ptr);
+gboolean hash_list_iter_has_next(const hash_list_iter_t *i);
+gpointer hash_list_iter_next(hash_list_iter_t *i);
+gpointer hash_list_iter_previous(hash_list_iter_t *i);
+gpointer hash_list_iter_follower(hash_list_iter_t *i);
+
 gboolean hash_list_contains(hash_list_t *hl,
-		gconstpointer key, gpointer *orig_key);
+		gconstpointer key, gconstpointer *orig_key);
 void hash_list_foreach(const hash_list_t *hl, GFunc func, gpointer user_data);
 
 #endif	/* _hashlist_h_ */
