@@ -1,5 +1,5 @@
 /*
- * $Id: inputevt.c,v 1.43 2005/09/30 22:07:43 cbiere Exp $
+ * $Id: inputevt.c 13371 2007-04-19 23:30:54Z cbiere $
  *
  * Copyright (c) 2002, ko (ko-@wanadoo.fr)
  * Copyright (c) 2005, Christian Biere
@@ -44,24 +44,62 @@
 
 #include "common.h"
 
-RCSID("$Id: inputevt.c,v 1.43 2005/09/30 22:07:43 cbiere Exp $");
+RCSID("$Id: inputevt.c 13371 2007-04-19 23:30:54Z cbiere $")
 
 #ifdef HAS_KQUEUE
 #include <sys/event.h>
+/*
+ * Some kqueue() implementations have a "struct kevent" with "udata"
+ * being of type (void *) while others have "udata" of type "intptr_t".
+ * To prevent incorrect casts and compiler warnings the two macros below
+ * should be used to access this struct member.
+ */
+#if defined(HAS_KEVENT_INT_UDATA)
+#define KEVENT_UDATA_TO_PTR(x) cast_uintptr_to_ptr(x)
+#define PTR_TO_KEVENT_UDATA(x) cast_ptr_to_uintptr(x)
+#else
+#define KEVENT_UDATA_TO_PTR(x) (x)
+#define PTR_TO_KEVENT_UDATA(x) (x)
+#endif /* HAVE_KEVENT_INT_UDATA */
 
 /* In case any system has both, kqueue() is preferred */
-#ifdef HAS_EPOLL
 #undef HAS_EPOLL
-#endif /* HAS_EPOLL */
+#undef HAS_DEV_POLL
+
+struct inputevt_array {
+	struct kevent *ev;
+};
 
 #endif /* HAS_KQUEUE */
 
 #ifdef HAS_EPOLL
 #include <sys/epoll.h>
+
+struct inputevt_array {
+	struct epoll_event *ev;
+};
+#undef HAS_DEV_POLL
+
 #endif /* HAS_EPOLL */
 
+#ifdef HAS_DEV_POLL
+#include <stropts.h>	/* ioctl() */
+#include <sys/devpoll.h>
+
+struct inputevt_array {
+	struct pollfd *ev;
+};
+#endif /* HAS_DEV_POLL */
+
+#if !(defined(HAS_DEV_POLL) || defined(HAS_EPOLL) || defined(HAS_KQUEUE))
+#define USE_POLL 1
+#endif	/* */
+
 #include "inputevt.h"
+#include "misc.h"
 #include "walloc.h"
+#include "tm.h"
+#include "bit_array.h"
 #include "override.h"		/* Must be the last header included */
 
 /*
@@ -79,7 +117,7 @@ const gchar *
 inputevt_cond_to_string(inputevt_cond_t cond)
 {
 	switch (cond) {
-#define CASE(x) case x: return STRINGIFY(x)
+#define CASE(x) case x: return #x
 	CASE(INPUT_EVENT_EXCEPTION);
 	CASE(INPUT_EVENT_R);
 	CASE(INPUT_EVENT_W);
@@ -111,21 +149,15 @@ typedef struct relay_list {
 
 static const inputevt_handler_t zero_handler;
 
-#if defined(HAS_EPOLL) || defined(HAS_KQUEUE)
-
+#if !defined(USE_POLL)
 struct poll_ctx {
-#ifdef HAS_KQUEUE
-	struct kevent *ev; 			/**< Used by kevent() */
-	
-#else /* HAS_KQUEUE */
-	struct epoll_event *ev; 	/**< Used by epoll_wait() */
-#endif /* !HAS_KQUEUE */
-
+	struct inputevt_array ev_arr;
 	inputevt_relay_t **relay;	/**< The relay contexts */
-	gulong *used;				/**< A bit array, which ID slots are used */
+	bit_array_t *used;			/**< A bit array, which ID slots are used */
 	GSList *removed;			/**< List of removed IDs */
 	GHashTable *ht;				/**< Records file descriptors */
 	guint num_ev;				/**< Length of the "ev" and "relay" arrays */
+	guint num_ready;			/**< Used for /dev/poll only */
 	gint fd;					/**< The ``master'' fd for epoll or kqueue */
 	gboolean initialized;		/**< TRUE if the context has been initialized */
 	gboolean dispatching;		/**< TRUE if dispatching events */
@@ -138,23 +170,30 @@ get_global_poll_ctx(void)
 	return &poll_ctx;
 }
 
-static inline gpointer
-get_poll_event_udata(gpointer p)
-#ifdef HAS_KQUEUE 
+#endif /* !USE_POLL */
+
+#ifndef HAS_KQUEUE
+size_t
+inputevt_data_available(void)
+{
+	return 0;
+}
+#endif	/* !HAS_KQUEUE */
+
+
+#if defined(HAS_KQUEUE)
+
+static guint data_available;	/** Used by inputevt_data_available(). */
+
+static inline gint
+get_poll_event_fd(gpointer p)
 {
 	struct kevent *ev = p;
-	return (gpointer) (gulong) ev->udata;
+	return GPOINTER_TO_INT(KEVENT_UDATA_TO_PTR(ev->udata));
 }
-#else /* !HAS_KQUEUE */
-{
-	struct epoll_event *ev = p;
-	return ev->data.ptr;
-}
-#endif /* HAS_KQUEUE */
 
 static inline inputevt_cond_t 
 get_poll_event_cond(gpointer p)
-#ifdef HAS_KQUEUE 
 {
 	struct kevent *ev = p;
 	inputevt_cond_t cond;
@@ -172,43 +211,58 @@ get_poll_event_cond(gpointer p)
 	}
 	return cond;
 }
-#else /* !HAS_KQUEUE */
+
+/**
+ * @return A positive value indicates how much data is available for reading.
+ *		   If zero is returned the amount of available data is unknown.
+ */
+size_t
+inputevt_data_available(void)
 {
-	struct epoll_event *ev = p;
-	return ((EPOLLIN | EPOLLPRI | EPOLLHUP) & ev->events ? INPUT_EVENT_R : 0)
-		| (EPOLLOUT & ev->events ? INPUT_EVENT_W : 0)
-		| (EPOLLERR & ev->events ? INPUT_EVENT_EXCEPTION : 0);
+	return data_available;
+}
+
+static inline void
+poll_event_set_data_avail(gpointer p)
+{
+	struct kevent *ev = p;
+
+	data_available = EVFILT_READ == ev->filter ? MIN(INT_MAX, ev->data) : 0;
+}
+
+static int
+create_poll_fd(void)
+{
+	return kqueue();
 }
-#endif /* HAS_KQUEUE */
 
 static gint
 update_poll_event(struct poll_ctx *poll_ctx, gint fd,
 	inputevt_cond_t old, inputevt_cond_t cur)
-#ifdef HAS_KQUEUE
 {
 	static const struct timespec zero_ts;
 	struct kevent kev[2];
 	size_t i;
-	gulong udata;
+	gpointer udata;
 	gint ret;
 
 	if ((INPUT_EVENT_RW & old) == (INPUT_EVENT_RW & cur))
 		return 0;
 
 	i = 0;
-	udata = (gulong) GINT_TO_POINTER(fd);
+	udata = GINT_TO_POINTER(fd);
 
 	if ((INPUT_EVENT_R & old) != (INPUT_EVENT_R & cur)) {
 		EV_SET(&kev[i], fd, EVFILT_READ,
 			(INPUT_EVENT_R & cur) ? EV_ADD : (EV_DELETE | EV_DISABLE),
-			0, 0, udata);
+			0, 0, PTR_TO_KEVENT_UDATA(udata));
 		i++;
 	}
 
 	if ((INPUT_EVENT_W & old) != (INPUT_EVENT_W & cur)) {
 		EV_SET(&kev[i], fd, EVFILT_WRITE,
 			(INPUT_EVENT_W & cur) ? EV_ADD : (EV_DELETE | EV_DISABLE),
-			0, 0, udata);
+			0, 0, PTR_TO_KEVENT_UDATA(udata));
 		i++;
 	}
 
@@ -217,7 +271,54 @@ update_poll_event(struct poll_ctx *poll_ctx, gint fd,
 
 	return ret;
 }
-#else /* !HAS_KQUEUE */
+
+static int
+check_poll_events(struct poll_ctx *poll_ctx)
+{
+	static const struct timespec zero_ts;
+	
+	g_assert(poll_ctx);
+	g_assert(poll_ctx->fd >= 0);
+
+	return kevent(poll_ctx->fd, NULL, 0,
+			poll_ctx->ev_arr.ev, poll_ctx->num_ev, &zero_ts);
+}
+
+#endif /* HAS_KQUEUE */
+
+#if defined(HAS_EPOLL)
+
+static inline gint
+get_poll_event_fd(gpointer p)
+{
+	struct epoll_event *ev = p;
+	return GPOINTER_TO_INT(ev->data.ptr);
+}
+
+static inline inputevt_cond_t 
+get_poll_event_cond(gpointer p)
+{
+	struct epoll_event *ev = p;
+	return ((EPOLLIN | EPOLLPRI | EPOLLHUP) & ev->events ? INPUT_EVENT_R : 0)
+		| (EPOLLOUT & ev->events ? INPUT_EVENT_W : 0)
+		| (EPOLLERR & ev->events ? INPUT_EVENT_EXCEPTION : 0);
+}
+
+static inline void
+poll_event_set_data_avail(gpointer p)
+{
+	(void) p;
+}
+
+static int
+create_poll_fd(void)
+{
+	return epoll_create(1024 /* Just an arbitrary value as hint */);
+}
+
+static gint
+update_poll_event(struct poll_ctx *poll_ctx, gint fd,
+	inputevt_cond_t old, inputevt_cond_t cur)
 {
 	static const struct epoll_event zero_ev;
 	struct epoll_event ev;
@@ -245,119 +346,251 @@ update_poll_event(struct poll_ctx *poll_ctx, gint fd,
 
 	return epoll_ctl(poll_ctx->fd, op, fd, &ev);
 }
-#endif /* HAS_KQUEUE */
-
-static int
-create_poll_fd(void)
-#ifdef HAS_KQUEUE 
-{
-	return kqueue();
-}
-#else /* !HAS_KQUEUE */
-{
-	return epoll_create(1024 /* Just an arbitrary value as hint */);
-}
-#endif /* HAS_KQUEUE */
 
 static int
 check_poll_events(struct poll_ctx *poll_ctx)
-#ifdef HAS_KQUEUE 
-{
-	static const struct timespec zero_ts;
-	
-	g_assert(poll_ctx);
-	g_assert(poll_ctx->fd >= 0);
-	
-	return kevent(poll_ctx->fd, NULL, 0,
-			poll_ctx->ev, poll_ctx->num_ev, &zero_ts);
-}
-#else /* !HAS_KQUEUE */
 {
 	g_assert(poll_ctx);
 	g_assert(poll_ctx->fd >= 0);
 	
-	return epoll_wait(poll_ctx->fd, poll_ctx->ev, poll_ctx->num_ev, 0);
+	return epoll_wait(poll_ctx->fd, poll_ctx->ev_arr.ev, poll_ctx->num_ev, 0);
 }
-#endif /* HAS_KQUEUE */
 
-#endif /* HAS_EPOLL || HAS_KQUEUE */
+#endif	/* HAS_EPOLL */
 
-/*
- * Functions for handling arrays of bits. On BSD systems, the * macros from
- * <bitstring.h> could be used for better efficiency. So far, the following
- * implementation does not eliminate loop overhead by handling all bits
- * of a "gulong" at once where possible.
- */
-
-/* @todo TODO: Move these functions to bit_array.h */
-
-static inline gulong *
-bit_array_realloc(gulong *base, size_t n)
+#if defined(HAS_DEV_POLL)
+static int
+create_poll_fd(void)
 {
-	size_t size;
-	
-	size = (n / 8) + (n % (8 * sizeof base[0]) ? sizeof base[0] : 0);
-	return g_realloc(base, size);
+	return get_non_stdio_fd(open("/dev/poll", O_RDWR));
 }
 
-#define BIT_ARRAY_BYTE(base, i) base[i / (8 * sizeof base[0])]
-#define BIT_ARRAY_BIT(base, i) (1UL << (i % (8 * sizeof base[0]))) 
+static gint
+update_poll_event(struct poll_ctx *poll_ctx, gint fd,
+	inputevt_cond_t old, inputevt_cond_t cur)
+{
+	old &= INPUT_EVENT_RW;
+	cur &= INPUT_EVENT_RW;
+	if (cur != old) {
+		static const struct pollfd zero_pfd;
+		struct pollfd pfd[2];
+		gint i = 0;
+
+		if (0 != old) {
+			pfd[i] = zero_pfd;
+			pfd[i].fd = fd;
+			pfd[i].events = POLLREMOVE;
+			i++;
+		}
+		if (0 != cur) {
+			pfd[i] = zero_pfd;
+			pfd[i].fd = fd;
+			pfd[i].events = 0
+				| (INPUT_EVENT_R & cur ? (POLLIN | POLLPRI) : 0)
+				| (INPUT_EVENT_W & cur ? POLLOUT : 0);
+			i++;
+		}
+
+		{
+			ssize_t ret;
+			size_t size;
+			size = i * sizeof pfd[0];
+			ret = write(poll_ctx->fd, &pfd, size);
+			if ((ssize_t) -1 == ret) {
+				g_error("update_poll_event(): write() failed: %s",
+						g_strerror(errno));
+			}
+			g_assert((size_t) ret == size);
+		}
+	}
+	return 0;
+}
+
+static int
+check_poll_events(struct poll_ctx *poll_ctx)
+{
+	int ret;
+
+	ret = poll_ctx->num_ready;
+	poll_ctx->num_ready = 0;
+	return ret;
+}
+
+static inline inputevt_cond_t 
+get_poll_event_cond(gpointer p)
+{
+	struct pollfd *ev = p;
+	return ((POLLIN | POLLHUP) & ev->revents ? INPUT_EVENT_R : 0)
+		| (POLLOUT & ev->revents ? INPUT_EVENT_W : 0)
+		| ((POLLERR | POLLNVAL) & ev->revents ? INPUT_EVENT_EXCEPTION : 0);
+}
+
+static inline gint
+get_poll_event_fd(gpointer p)
+{
+	const struct pollfd *pfd = p;
+	return pfd->fd;
+}
 
 static inline void
-bit_array_set(gulong *base, size_t i)
+poll_event_set_data_avail(gpointer p)
 {
-	BIT_ARRAY_BYTE(base, i) |= BIT_ARRAY_BIT(base, i);
+	(void) p;
 }
 
-static inline void 
-bit_array_clear(gulong *base, size_t i)
+static inline short
+poll_events_from_gio_cond(gushort events)
 {
-	BIT_ARRAY_BYTE(base, i) &= ~BIT_ARRAY_BIT(base, i);
+	return 0
+		| ((G_IO_IN   & events) ? POLLIN   : 0)
+		| ((G_IO_OUT  & events) ? POLLOUT  : 0)
+		| ((G_IO_PRI  & events) ? POLLPRI  : 0)
+		| ((G_IO_HUP  & events) ? POLLHUP  : 0)
+		| ((G_IO_ERR  & events) ? POLLERR  : 0)
+		| ((G_IO_NVAL & events) ? POLLNVAL : 0);
 }
 
-static inline void 
-bit_array_flip(gulong *base, size_t i)
+static inline gushort
+poll_events_to_gio_cond(short events)
 {
-	BIT_ARRAY_BYTE(base, i) ^= BIT_ARRAY_BIT(base, i);
+	return 0
+		| ((POLLIN   & events) ? G_IO_IN   : 0)
+		| ((POLLOUT  & events) ? G_IO_OUT  : 0)
+		| ((POLLPRI  & events) ? G_IO_PRI  : 0)
+		| ((POLLHUP  & events) ? G_IO_HUP  : 0)
+		| ((POLLERR  & events) ? G_IO_ERR  : 0)
+		| ((POLLNVAL & events) ? G_IO_NVAL : 0);
 }
 
-static inline gboolean
-bit_array_get(const gulong *base, size_t i)
+static void
+check_dev_poll(struct poll_ctx *poll_ctx, gint *timeout_ms_ptr)
 {
-	return 0 != (BIT_ARRAY_BYTE(base, i) & BIT_ARRAY_BIT(base, i));
-}
+	struct dvpoll dvp; 
+	tm_t before, after, elapsed;
+	time_delta_t d;
+	gint ret, timeout_ms;
 
-static inline void 
-bit_array_clear_range(gulong *base, size_t from, size_t to)
-{
-	g_assert(from <= to);
+	g_assert(poll_ctx);
+	g_assert(timeout_ms_ptr);
+#if 0
+	g_assert(0 == poll_ctx->num_ready);
+#endif
 
-	if (from <= to) {
-		size_t i = from;
-	
-		do
-			bit_array_clear(base, i);
-		while (i++ != to);
-	}
-}
-
-static inline size_t
-bit_array_first_clear(const gulong *base, size_t from, size_t to)
-{
-	g_assert(from <= to);
-
-	if (from <= to) {
-		size_t i = from;
-	
-		do
-			if (0 == bit_array_get(base, i))
-				return i;
-		while (i++ != to);
+	if (poll_ctx->num_ev <= 0) {
+		poll_ctx->num_ready = 0;
+		return;
 	}
 
-	return (size_t) -1;
+	timeout_ms = *timeout_ms_ptr;
+	timeout_ms = MAX(0, timeout_ms);
+
+	dvp.dp_timeout = timeout_ms;
+	dvp.dp_nfds = poll_ctx->num_ev;
+	dvp.dp_fds = poll_ctx->ev_arr.ev;
+
+	tm_now_exact(&before);
+	ret = ioctl(poll_ctx->fd, DP_POLL, &dvp);
+	if (-1 == ret && !is_temporary_error(errno)) {
+		g_warning("check_dev_poll(): ioctl() failed: %s", g_strerror(errno));
+	}
+	tm_now_exact(&after);
+	tm_elapsed(&elapsed, &after, &before);
+	d = tm2ms(&elapsed);
+	if (d >= timeout_ms || ret > 0) {
+		timeout_ms = 0;
+	} else {
+		timeout_ms -= d;
+	}
+	poll_ctx->num_ready = MAX(0, ret);
+
+	/* If the original timeout was negative (=INFINITE) and no event
+	 * has occured, the timeout isn't touched.
+	 */
+	if (*timeout_ms_ptr >= 0 || ret > 0) {
+		*timeout_ms_ptr = timeout_ms;
+	} 
 }
 
+/**
+ * This code is only used for /dev/poll. This is necessary because the
+ * device does not support polling...
+ */
+static gint
+poll_func(GPollFD *gfds, guint n, gint timeout_ms)
+{
+	struct poll_ctx *poll_ctx;
+	struct pollfd local_pfds[16], *pfds;
+	size_t w_size = 0;
+	gpointer w_buf = NULL;
+	gboolean do_check = FALSE;
+	gint ret;
+
+	poll_ctx = get_global_poll_ctx();
+	g_assert(poll_ctx);
+	g_assert(poll_ctx->initialized);
+	g_assert(poll_ctx->fd >= 0);
+
+	if (n > 0) {
+		guint i;
+
+		if (n > G_N_ELEMENTS(local_pfds)) {
+			w_size = n * sizeof *pfds;
+			w_buf = walloc(w_size);
+			pfds = w_buf;
+		} else {
+			pfds = local_pfds;
+		}
+
+		for (i = 0; i < n; i++) {
+			pfds[i].fd = gfds[i].fd;
+			if (gfds[i].fd != poll_ctx->fd) {
+				pfds[i].events = poll_events_from_gio_cond(
+									gfds[i].events & ~(G_IO_ERR | G_IO_NVAL));
+			} else {
+				pfds[i].events = 0;
+				do_check = TRUE;
+			}
+			pfds[i].revents = 0;
+			gfds[i].revents = 0;
+		}
+	} else {
+		pfds = NULL;
+	}
+
+	if (do_check) {
+		check_dev_poll(poll_ctx, &timeout_ms);
+	}
+
+	ret = poll(pfds, n, timeout_ms);
+	if (-1 == ret && !is_temporary_error(errno)) {
+		g_warning("poll() failed: %s", g_strerror(errno));
+	}
+	if (do_check || ret > 0) {
+		guint i;
+
+		g_assert(ret < 0 || (guint) ret <= n);
+
+		for (i = 0; i < n; i++) {
+			g_assert(gfds[i].fd == pfds[i].fd);
+			if (gfds[i].fd != poll_ctx->fd) {
+				if (ret > 0) {
+					gfds[i].revents = poll_events_to_gio_cond(pfds[i].revents);
+				}
+			} else if (poll_ctx->num_ready > 0) {
+				gfds[i].revents = G_IO_IN;
+			}
+		}
+	}
+
+	if (do_check && poll_ctx->num_ready > 0) {
+		ret = 1 + MAX(0, ret);
+	}
+
+	WFREE_NULL(w_buf, w_size);
+	return ret;
+}
+#endif	/* HAS_DEV_POLL */
+
 /**
  * Frees the relay structure when its time comes.
  */
@@ -416,7 +649,7 @@ inputevt_add_source_with_glib(inputevt_relay_t *relay)
 	return id;
 }
 
-#if defined(HAS_EPOLL) || defined(HAS_KQUEUE)
+#if !defined(USE_POLL)
 
 static void
 inputevt_timer(struct poll_ctx *poll_ctx)
@@ -425,13 +658,14 @@ inputevt_timer(struct poll_ctx *poll_ctx)
 
 	g_assert(poll_ctx);
 	g_assert(poll_ctx->initialized);
-	g_assert(-1 != poll_ctx->fd);
+	g_assert(poll_ctx->fd >= 0);
+	g_assert(poll_ctx->ht);
 
 	/* Maybe this must safely fail for general use, thus no assertion */
 	g_return_if_fail(!poll_ctx->dispatching);
 
 	n = check_poll_events(poll_ctx);
-	if (-1 == n && EINTR != errno) {
+	if (-1 == n && !is_temporary_error(errno)) {
 		g_warning("check_poll_events(%d) failed: %s",
 			poll_ctx->fd, g_strerror(errno));
 	}
@@ -451,8 +685,8 @@ inputevt_timer(struct poll_ctx *poll_ctx)
 		GSList *sl;
 		gint fd;
 
-		cond = get_poll_event_cond(&poll_ctx->ev[i]);
-		fd = GPOINTER_TO_INT(get_poll_event_udata(&poll_ctx->ev[i]));
+		cond = get_poll_event_cond(&poll_ctx->ev_arr.ev[i]);
+		fd = get_poll_event_fd(&poll_ctx->ev_arr.ev[i]);
 		g_assert(fd >= 0);
 		
 		rl = g_hash_table_lookup(poll_ctx->ht, GINT_TO_POINTER(fd));
@@ -476,8 +710,10 @@ inputevt_timer(struct poll_ctx *poll_ctx)
 			if (zero_handler == relay->handler)
 				continue;
 
-			if (relay->condition & cond)
+			if (relay->condition & cond) {
+				poll_event_set_data_avail(&poll_ctx->ev_arr.ev[i]);
 				relay->handler(relay->data, fd, cond);
+			}
 		}
 	}
 	
@@ -563,7 +799,7 @@ inputevt_remove(guint id)
 	g_assert(poll_ctx->initialized);
 	g_assert(0 != id);
 
-	if (-1 == poll_ctx->fd) {
+	if (poll_ctx->fd < 0) {
 		g_source_remove(id);
 	} else {
 		inputevt_relay_t *relay;
@@ -571,6 +807,7 @@ inputevt_remove(guint id)
 		inputevt_cond_t old, cur;
 		gint fd;
 
+		g_assert(poll_ctx->ht);
 		g_assert(id < poll_ctx->num_ev);
 		g_assert(0 != bit_array_get(poll_ctx->used, id));
 
@@ -638,11 +875,9 @@ inputevt_get_free_id(const struct poll_ctx *poll_ctx)
 	
 	return bit_array_first_clear(poll_ctx->used, 0, poll_ctx->num_ev - 1);
 }
-#endif /* HAS_EPOLL || HAS_KQUEUE*/
 
 static guint 
 inputevt_add_source(inputevt_relay_t *relay)
-#if defined(HAS_EPOLL) || defined(HAS_KQUEUE)
 {
 	struct poll_ctx *poll_ctx;
 	guint id;
@@ -651,7 +886,7 @@ inputevt_add_source(inputevt_relay_t *relay)
 	g_assert(poll_ctx->initialized);
 	g_assert(relay->fd >= 0);
 	
-	if (-1 == poll_ctx->fd) {
+	if (poll_ctx->fd < 0) {
 		/*
 		 * Linux systems with 2.4 kernels usually have all epoll stuff
 		 * in their headers but the system calls just return ENOSYS.
@@ -661,6 +896,8 @@ inputevt_add_source(inputevt_relay_t *relay)
 		inputevt_cond_t old;
 		guint f;
 
+		g_assert(poll_ctx->ht);
+
 		f = inputevt_get_free_id(poll_ctx);
 		g_assert((guint) -1 == f || f < poll_ctx->num_ev);
 
@@ -677,8 +914,8 @@ inputevt_add_source(inputevt_relay_t *relay)
 
 			poll_ctx->num_ev = 0 != n ? n << 1 : 32;
 
-			size = poll_ctx->num_ev * sizeof poll_ctx->ev[0];
-			poll_ctx->ev = g_realloc(poll_ctx->ev, size);
+			size = poll_ctx->num_ev * sizeof poll_ctx->ev_arr.ev[0];
+			poll_ctx->ev_arr.ev = g_realloc(poll_ctx->ev_arr.ev, size);
 			
 			poll_ctx->used = bit_array_realloc(poll_ctx->used,
 								poll_ctx->num_ev);
@@ -756,11 +993,63 @@ inputevt_add_source(inputevt_relay_t *relay)
 	g_assert(0 != id);	
 	return id;
 }
-#else /* !(HAS_EPOLL || HAS_KQUEUE) */
+
+/**
+ * Performs module initialization.
+ */
+void
+inputevt_init(void)
+{
+	struct poll_ctx *poll_ctx;
+	
+	poll_ctx = get_global_poll_ctx();
+	g_assert(!poll_ctx->initialized);
+	
+	poll_ctx->initialized = TRUE;
+
+	poll_ctx->fd = create_poll_fd();
+	if (poll_ctx->fd < 0) {
+		g_warning("create_poll_fd() failed: %s", g_strerror(errno));
+		/* This is no hard error, we fall back to the GLib source watcher */
+	} else {
+		GIOChannel *ch;
+
+#ifdef HAS_DEV_POLL
+		g_main_set_poll_func(poll_func);
+#endif	/* HAS_DEV_POLL */
+
+		poll_ctx->ht = g_hash_table_new(NULL, NULL);
+		ch = g_io_channel_unix_new(poll_ctx->fd);
+
+#if GLIB_CHECK_VERSION(2, 0, 0)
+		g_io_channel_set_encoding(ch, NULL, NULL); /* binary data */
+#endif /* GLib >= 2.0 */
+
+		(void) g_io_add_watch(ch, READ_CONDITION, dispatch_poll, poll_ctx);
+	}
+}
+
+#endif /* !USE_POLL */
+
+#ifdef USE_POLL
+static guint 
+inputevt_add_source(inputevt_relay_t *relay)
 {
 	return inputevt_add_source_with_glib(relay);
 }
-#endif /* HAS_EPOLL || HAS_KQUEUE */
+
+void
+inputevt_remove(guint id)
+{
+	g_source_remove(id);
+}
+
+void
+inputevt_init(void)
+{
+	/* Nothing to do */
+}
+#endif /* USE_POLL */
 
 /**
  * Adds an event source to the main GLIB monitor queue.
@@ -801,38 +1090,6 @@ inputevt_add(gint fd, inputevt_cond_t cond,
 	return inputevt_add_source(relay);
 }
 
-/**
- * Performs module initialization.
- */
-void
-inputevt_init(void)
-{
-#if defined(HAS_EPOLL) || defined(HAS_KQUEUE)
-	struct poll_ctx *poll_ctx;
-	
-	poll_ctx = get_global_poll_ctx();
-	g_assert(!poll_ctx->initialized);
-	
-	poll_ctx->initialized = TRUE;
-
-	if (-1 == (poll_ctx->fd = create_poll_fd())) {
-		g_warning("create_poll_fd() failed: %s", g_strerror(errno));
-		/* This is no hard error, we fall back to the GLib source watcher */
-	} else {
-		GIOChannel *ch;
-
-		poll_ctx->ht = g_hash_table_new(NULL, NULL);
-		ch = g_io_channel_unix_new(poll_ctx->fd);
-
-#if GLIB_CHECK_VERSION(2, 0, 0)
-		g_io_channel_set_encoding(ch, NULL, NULL); /* binary data */
-#endif /* GLib >= 2.0 */
-
-		(void) g_io_add_watch(ch, READ_CONDITION, dispatch_poll, poll_ctx);
-	}
-#endif /* HAS_EPOLL || HAS_KQUEUE */
-}
-
 /**
  * Performs module cleanup.
  */
