@@ -1,5 +1,5 @@
 /*
- * $Id: listener.h 9452 2005-09-10 08:58:40Z daichik $
+ * $Id: listener.h 12500 2006-11-26 10:53:21Z cbiere $
  *
  * Copyright (c) 2001-2003, Richard Eckart
  *
@@ -36,7 +36,7 @@
 #ifndef _listener_h_
 #define _listener_h_
 
-#include <glib.h>
+#include "common.h"
 
 /**
  * To use the macros below with a "node_added" signal for example,
