@@ -1,5 +1,5 @@
 /*
- * $Id: misc.h 12267 2006-11-08 20:57:11Z cbiere $
+ * $Id: misc.h 13970 2007-06-24 20:17:48Z cbiere $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi
  *
@@ -50,19 +50,14 @@
 #ifndef _misc_h_
 #define _misc_h_
 
-#include "config.h"
-
-#include <time.h>
-#include <stdio.h>
-#include <ctype.h>
-
-#include <glib.h>
+#include "common.h"
 
 #include "vmm.h"
 
 #define SIZE_FIELD_MAX 64		/**< Max size of sprintf-ed size quantity */
-#define GUID_RAW_SIZE	16		/**< Binary representation of 128 bits */
-#define GUID_HEX_SIZE	32		/**< Hexadecimal GUID representation */
+#define GUID_RAW_SIZE		16	/**< Binary representation of 128 bits */
+#define GUID_HEX_SIZE		32	/**< Hexadecimal GUID representation */
+#define GUID_BASE32_SIZE	26	/**< base32 GUID representation */
 
 typedef struct short_string {
 	gchar str[SIZE_FIELD_MAX];
@@ -94,6 +89,20 @@ size_t w_concat_strings(gchar **dst,
 gint ascii_strcasecmp(const gchar *s1, const gchar *s2);
 gint ascii_strncasecmp(const gchar *s1, const gchar *s2, size_t len);
 
+/**
+ * Converts an integer to a single hexadecimal ASCII digit. The are no checks,
+ * this is just a convenience function.
+ *
+ * @param x An integer between 0 and 15.
+ * @return The ASCII character corresponding to the hex digit [0-9a-f].
+ */
+static inline guchar
+hex_digit(guchar x)
+{
+	extern const char hex_alphabet_lower[];
+	return hex_alphabet_lower[x & 0xf]; 
+}
+
 /**
  * Converts a hexadecimal char (0-9, A-F, a-f) to an integer.
  *
@@ -264,6 +273,21 @@ skip_ascii_non_spaces(const gchar *s)
 	return deconstify_gchar(s);
 }
 
+/**
+ * Skips over all characters which are ASCII alphanumerical characters
+ * starting at ``s''.
+ *
+ * @return a pointer to the first space or NUL character starting from s.
+ */
+static inline WARN_UNUSED_RESULT gchar *
+skip_ascii_alnum(const gchar *s)
+{
+	while (is_ascii_alnum(*s))
+		s++;
+
+	return deconstify_gchar(s);
+}
+
 /**
  * Skips over all ASCII blank characters starting at ``s''.
  *
@@ -278,6 +302,15 @@ skip_ascii_blanks(const gchar *s)
 	return deconstify_gchar(s);
 }
 
+static inline WARN_UNUSED_RESULT gchar *
+skip_dir_separators(const gchar *s)
+{
+	while ('/' == s[0] || G_DIR_SEPARATOR == s[0])
+		s++;
+
+	return deconstify_gchar(s);
+}
+
 /*
  * Determine the length of string literals
  */
@@ -430,25 +463,42 @@ short_string_t short_rate_get_string(guint64 rate, gboolean metric);
 /*
  * SHA1<->base32 string conversion
  */
-gchar *sha1_to_base32_buf(const gchar *sha1, gchar *dst, size_t size);
-gchar *sha1_base32(const gchar *sha1);
-gchar *base32_sha1(const gchar *base32);
+typedef struct sha1 {
+	gchar data[SHA1_RAW_SIZE];
+} sha1_t;
+
+const gchar *sha1_to_string(const struct sha1 sha1);
+const gchar *sha1_to_urn_string(const struct sha1 *sha1);
+gchar *sha1_to_base32_buf(const struct sha1 *sha1, gchar *dst, size_t size);
+const gchar *sha1_base32(const struct sha1 *sha1);
+const struct sha1 *base32_sha1(const gchar *base32);
+
+static inline int
+sha1_cmp(const struct sha1 *a, const struct sha1 *b)
+{
+	return memcmp(a, b, SHA1_RAW_SIZE);
+}
 
 /*
  * TTH <-> base32 string conversion
  */
 typedef struct tth {
-	guchar data[TTH_RAW_SIZE];
+	gchar data[TTH_RAW_SIZE];
 } tth_t;
 
 const gchar *tth_base32(const struct tth *tth);
 const struct tth *base32_tth(const gchar *base32);
 
+
+const gchar *bitprint_to_urn_string(const struct sha1 *, const struct tth *);
+
 /*
  * GUID<->hex string conversion
  */
 const gchar *guid_hex_str(const gchar *guid);
 gboolean hex_to_guid(const gchar *hexguid, gchar *guid);
+size_t guid_to_string_buf(const gchar *guid, gchar *dst, size_t size);
+const gchar *guid_to_string(const gchar *guid);
 
 /*
  * GUID<->base32 string conversion
@@ -467,6 +517,7 @@ gboolean file_exists(const gchar *pathname);
 gboolean file_does_not_exist(const gchar *pathname);
 guint32 next_pow2(guint32 n);
 
+#define IS_POWER_OF_2(x) ((x) && 0 == ((x) & ((x) - 1)))
 /**
  * Checks whether the given value is a power of 2.
  *
@@ -481,7 +532,7 @@ is_pow2(guint32 value)
 }
 #else /* !HAS_BUILTIN_POPCOUNT */
 {
-	return value && 0 == (value & (value - 1));
+	return IS_POWER_OF_2(value);
 }
 #endif /* HAS_BUILTIN_POPCOUNT */
 
@@ -504,7 +555,9 @@ void dump_hex(FILE *, const gchar *, gconstpointer, gint);
 void locale_strlower(gchar *, const gchar *);
 void ascii_strlower(gchar *dst, const gchar *src);
 gint strcmp_delimit(const gchar *a, const gchar *b, const gchar *delimit);
-gint strcasecmp_delimit(const gchar *a, const gchar *b, const gchar *delimit);
+gint ascii_strcasecmp_delimit(const gchar *a, const gchar *b,
+		const gchar *delimit);
+size_t filename_shrink(const gchar *filename, gchar *buf, size_t size);
 char *unique_filename(const gchar *path, const gchar *file, const gchar *ext,
 		gboolean (*name_is_uniq)(const gchar *pathname));
 gchar *hex_escape(const gchar *name, gboolean strict);
@@ -512,10 +565,21 @@ gchar *control_escape(const gchar *s);
 const gchar *lazy_string_to_printf_escape(const gchar *src);
 gint highest_bit_set(guint32 n) G_GNUC_CONST;
 gfloat force_range(gfloat value, gfloat min, gfloat max);
+gchar *absolute_pathname(const gchar *file);
 gchar *make_pathname(const gchar *dir, const gchar *file);
 gchar *short_filename(gchar *fullname);
 gchar *data_hex_str(const gchar *data, size_t len);
-gint create_directory(const gchar *dir);
+
+#if defined(S_IROTH) && defined(S_IXOTH)
+/* 0755 */
+#define DEFAULT_DIRECTORY_MODE (S_IRWXU | S_IRGRP | S_IXGRP | S_IROTH | S_IXOTH)
+#else
+/* 0750 */
+#define DEFAULT_DIRECTORY_MODE (S_IRWXU | S_IRGRP | S_IXGRP)
+#endif /* S_IROTH && S_IXOTH */
+
+gint create_directory(const gchar *dir, mode_t mode);
+gint compat_mkdir(const gchar *path, mode_t mode);
 gboolean filepath_exists(const gchar *dir, const gchar *file);
 const gchar * filepath_basename(const gchar *pathname);
 gchar * filepath_directory(const gchar *pathname);
@@ -534,6 +598,7 @@ const gchar *uint64_to_string(guint64 v);
 const gchar *uint64_to_string2(guint64 v);
 const gchar *off_t_to_string(off_t v);
 const gchar *time_t_to_string(time_t v);
+const gchar *filesize_to_string(filesize_t v);
 gint parse_major_minor(const gchar *src, gchar const **endptr,
 	guint *major, guint *minor);
 gchar *is_strprefix(const gchar *s, const gchar *prefix) WARN_UNUSED_RESULT;
@@ -542,31 +607,65 @@ size_t html_escape(const gchar *src, gchar *dst, size_t dst_size);
 guint32 html_decode_entity(const gchar *src, const gchar **endptr);
 gint canonize_path(gchar *dst, const gchar *path);
 guint compat_max_fd(void);
-gint compat_mkdir(const gchar *path, mode_t mode);
+void close_file_descriptors(const int first_fd);
 gboolean compat_is_superuser(void);
 int compat_daemonize(const char *directory);
+void set_close_on_exec(gint fd);
+void compat_fadvise_sequential(int fd, off_t offset, off_t size);
+void *compat_memmem(const void *data, size_t data_size,
+		const void *pattern, size_t pattern_size);
+
+int get_non_stdio_fd(int fd);
 
 typedef void (*signal_handler_t)(gint signo);
 signal_handler_t set_signal(gint signo, signal_handler_t handler);
 
 gchar *ascii_strcasestr(const gchar *haystack, const gchar *needle);
+gchar *normalize_dir_separators(const gchar *s);
+size_t memcmp_diff(const void *a, const void *b, size_t n);
+guint32 cpu_noise(void);
+
+static inline guint
+pointer_hash_func(const void *p)
+{
+	size_t v = (size_t) p;
+	return (((guint64) 0x4F1BBCDCUL * v) >> 32) ^ v;
+}
+
+static inline guint
+str_case_hash_func(gconstpointer key)
+{
+	const guchar *s = key;
+	gulong c, hash = 0;
+	
+	while ((c = ascii_tolower(*s++))) {
+		hash ^= (hash << 8) | c;
+	}
+	return hash ^ (((guint64) 1048573 * hash) >> 32);
+}
+
+static inline gint
+str_case_eq_func(gconstpointer a, gconstpointer b)
+{
+	return 0 == ascii_strcasecmp(a, b);
+}
 
 /**
  * Determines the length of a NUL-terminated string looking only at the first
- * "src_size" bytes. If src[0..size] contains no NUL byte, (size_t) -1 is
+ * "src_size" bytes. If src[0..size] contains no NUL byte, "src_size" is
  * returned. Otherwise, the returned value is identical to strlen(str). Thus,
  * it is safe to pass a possibly non-terminated buffer.
  * 
- * @return (size_t) -1, if the string as long or longer than "src_size".
- *         Otherwise, the result is identical to strlen(str).
+ * @return The number of bytes in "src" before the first found NUL or src_size
+ *		   if there is no NUL.
  */
 static inline size_t
-str_len_capped(const gchar *src, size_t src_size)
+clamp_strlen(const gchar *src, size_t src_size)
 {
 	const gchar *p;
 	
 	p = memchr(src, '\0', src_size);
-	return p ? (size_t) (p - src) : (size_t) -1;
+	return (p ? p : &src[src_size]) - src;
 }
 
 static inline const gchar *
@@ -591,12 +690,15 @@ EMPTY_STRING(const gchar *s)
 static inline G_GNUC_CONST guint32
 swap_guint32(guint32 i)
 {
-	gint a = i & 0x000000ff;
-	gint b = (i & 0x0000ff00) >> 8;
-	gint c = (i & 0x00ff0000) >> 16;
-	gint d = (i & 0xff000000) >> 24;
-
-	return d + (c << 8) + (b << 16) + (a << 24);
+	guint32 a;
+	guint32 b;
+                                  /* i -> ABCD */
+	a = (i & 0x00ff00ff) << 8;    /* a -> B0D0 */
+	b = (i & 0xff00ff00) >> 8;    /* b -> 0A0C */
+	i = a | b;                    /* i -> BADC */
+	i = (i << 16) | (i >> 16);    /* i -> DCBA */
+    
+	return i;
 }
 
 /**
@@ -625,6 +727,19 @@ netmask_to_cidr(guint32 netmask)
 }
 #endif /* HAVE_BUILTIN_POPCOUNT */
 
+/**
+ * Converts the CIDR prefix length to a IPv4 netmask in host byte order.
+ * No checks are performed.
+ *
+ * @param bits A value between 1..32.
+ * @return The equivalent netmask in host byte order.
+ */
+static inline G_GNUC_CONST WARN_UNUSED_RESULT guint32
+cidr_to_netmask(guint bits)
+{
+	return (guint32)-1 << (32 - bits);
+}
+
 /**
  * Rounds ``n'' up so that it matches the given alignment ``align''.
  */
@@ -662,6 +777,10 @@ is_temporary_error(gint error)
 /* Wrapper around lseek() to handle filesize -> off_t conversion. */
 gint seek_to_filepos(gint fd, filesize_t pos);
 
+guint filesize_per_100(filesize_t size, filesize_t part);
+guint filesize_per_1000(filesize_t size, filesize_t part);
+guint filesize_per_10000(filesize_t size, filesize_t part);
+
 /*
  * CIDR split of IP range.
  */
@@ -731,6 +850,23 @@ G_STMT_START { \
 	} \
 } G_STMT_END
 
+static inline const char *
+print_number(char *dst, size_t size, unsigned long value)
+{
+	char *p = &dst[size];
+
+	if (size > 0) {
+		*--p = '\0';
+	}
+	while (p != dst) {
+		*--p = (value % 10) + '0';
+		value /= 10;
+		if (0 == value)
+			break;
+	}
+	return p;
+}
+
 #endif /* _misc_h_ */
 
 /* vi: set ts=4 sw=4 cindent: */
