@@ -1,5 +1,5 @@
 /*
- * $Id: prop.c 12177 2006-11-02 01:09:54Z cbiere $
+ * $Id: prop.c 13096 2007-03-15 17:12:47Z cbiere $
  *
  * Copyright (c) 2001-2003, Richard Eckart
  *
@@ -25,7 +25,7 @@
 
 #include "common.h"
 
-RCSID("$Id: prop.c 12177 2006-11-02 01:09:54Z cbiere $")
+RCSID("$Id: prop.c 13096 2007-03-15 17:12:47Z cbiere $")
 
 #include "prop.h"
 #include "file.h"
@@ -59,7 +59,7 @@ const struct {
 G_STMT_START {														\
 	if (!(x)) {														\
 		g_error("assertion failed for property \"%s\": %s",			\
-			PROP(ps, prop).name, STRINGIFY(x));						\
+			PROP(ps, prop).name, #x);								\
 	}																\
 } G_STMT_END
 
@@ -1593,8 +1593,6 @@ prop_save_to_file(prop_set_t *ps, const gchar *dir, const gchar *filename)
 		return;
 
 	pathname = make_pathname(dir, filename);
-	g_return_if_fail(NULL != pathname);
-
 	if (-1 == stat(pathname, &sb)) {
 		g_warning("could not stat \"%s\": %s", pathname, g_strerror(errno));
 	} else {
@@ -1627,7 +1625,7 @@ prop_save_to_file(prop_set_t *ps, const gchar *dir, const gchar *filename)
 		goto end;
 
 	fprintf(config,
-			"#\n# Gtk-Gnutella %s%s (%s) by Olrick & Co.\n# %s\n#\n",
+			"#\n# gtk-gnutella %s%s (%s) by Olrick & Co.\n# %s\n#\n",
 			GTA_VERSION_NUMBER,
 #ifdef GTA_REVISION
 			" " GTA_REVISION,
@@ -1939,8 +1937,6 @@ prop_load_from_file(prop_set_t *ps, const gchar *dir, const gchar *filename)
 		return;
 
 	path = make_pathname(dir, filename);
-	g_return_if_fail(NULL != path);
-
 	config = file_fopen(path, "r");
 	if (!config) {
 		G_FREE_NULL(path);
