@@ -1,5 +1,5 @@
 /*
- * $Id: stats.c 11437 2006-08-05 00:24:20Z cbiere $
+ * $Id: stats.c 12789 2007-01-30 21:33:15Z cbiere $
  *
  * Copyright (c) 2004, Raphael Manfredi
  *
@@ -35,9 +35,11 @@
 
 #include "common.h"
 
-RCSID("$Id: stats.c 11437 2006-08-05 00:24:20Z cbiere $")
+RCSID("$Id: stats.c 12789 2007-01-30 21:33:15Z cbiere $")
 
+#ifdef I_MATH
 #include <math.h>
+#endif	/* I_MATH */
 
 #include "stats.h"
 #include "walloc.h"
