@@ -1,5 +1,5 @@
 /*
- * $Id: tigertree.h 12210 2006-11-04 01:28:01Z cbiere $
+ * $Id: tigertree.h 13627 2007-05-13 04:17:17Z cbiere $
  *
  * Copyright (c) 2003, Jeroen Asselman
  *
@@ -56,36 +56,47 @@
 #ifndef _tigertree_h_
 #define _tigertree_h_
 
-#include "tiger.h"
+#include "common.h"
+
+#include "lib/tiger.h"
+#include "lib/misc.h"
 
 /* tiger hash result size, in bytes */
-#define TIGERSIZE 24
+#define TIGERSIZE	24
 
-/* size of each block independently tiger-hashed, not counting leaf 0x00 prefix */
-#define BLOCKSIZE 1024
+/* Maximum depth to preserve */
+#define TTH_MAX_DEPTH	11
+#define TTH_MAX_LEAVES	(1 << TTH_MAX_DEPTH)
 
-/* size of input to each non-leaf hash-tree node, not counting node 0x01 prefix */
-#define NODESIZE (TIGERSIZE*2)
+/*
+ * size of each block independently tiger-hashed,
+ * not counting leaf 0x00 prefix
+ */
+#define TTH_BLOCKSIZE	1024
 
-/* default size of interim values stack, in TIGERSIZE
- * blocks. If this overflows (as it will for input
- * longer than 2^64 in size), havoc may ensue. */
-#define TTH_STACKSIZE (TIGERSIZE*56)
 
-typedef struct tt_context {
-  guint64 count;                   /* total blocks processed */
-  guchar leaf[1+BLOCKSIZE]; /* leaf in progress */
-  guchar *block;            /* leaf data */
-  guchar node[1+NODESIZE]; /* node scratch space */
-  gint idx;                      /* index into block */
-  guchar *top;             /* top (next empty) stack slot */
-  guchar nodes[TTH_STACKSIZE]; /* stack of interim node values */
-} TT_CONTEXT;
+struct TTH_CONTEXT;
+typedef struct TTH_CONTEXT TTH_CONTEXT;
 
+size_t tt_size(void);
 void tt_check(void);
-void tt_init(TT_CONTEXT *ctx);
-void tt_update(TT_CONTEXT *ctx, gconstpointer data, size_t len);
-void tt_digest(TT_CONTEXT *ctx, guchar hash[TIGERSIZE]);
+
+void tt_init(TTH_CONTEXT *ctx, filesize_t filesize);
+void tt_update(TTH_CONTEXT *ctx, gconstpointer data, size_t len);
+void tt_digest(TTH_CONTEXT *ctx, struct tth *tth);
+
+const struct tth *tt_leaves(TTH_CONTEXT *ctx);
+size_t tt_leave_count(TTH_CONTEXT *ctx);
+struct tth tt_root_hash(const struct tth *src, size_t n_leaves);
+size_t tt_compute_parents(struct tth *dst,
+		const struct tth *src, size_t src_leaves);
+
+filesize_t tt_node_count_at_depth(filesize_t filesize, unsigned depth);
+size_t tt_good_node_count(filesize_t filesize);
+filesize_t tt_good_slice_size(filesize_t filesize);
+
+unsigned tt_full_depth(filesize_t filesize);
+unsigned tt_good_depth(filesize_t filesize);
 
 #endif /* _tigertree_h_ */
 /* vi: set ts=4 sw=4 cindent: */
