@@ -1,5 +1,5 @@
 /*
- * $Id: urn.c 11437 2006-08-05 00:24:20Z cbiere $
+ * $Id: urn.c 13970 2007-06-24 20:17:48Z cbiere $
  *
  * Copyright (c) 2002-2004, Raphael Manfredi
  *
@@ -35,7 +35,7 @@
 
 #include "common.h"
 
-RCSID("$Id: urn.c 11437 2006-08-05 00:24:20Z cbiere $")
+RCSID("$Id: urn.c 13970 2007-06-24 20:17:48Z cbiere $")
 
 #include "base32.h"
 #include "misc.h"
@@ -47,53 +47,27 @@ RCSID("$Id: urn.c 11437 2006-08-05 00:24:20Z cbiere $")
  * of a SHA1 hash, and write decoded value in `retval'.
  *
  * The SHA1 typically comes from HTTP, in a X-Gnutella-Content-URN header.
- * Therefore, we unconditionally accept both old and new encodings.
  *
  * @return TRUE if the SHA1 was valid and properly decoded, FALSE on error.
  */
 gboolean
-urn_get_http_sha1(const gchar *buf, gchar *retval)
+parse_base32_sha1(const gchar *buf, size_t size, struct sha1 *sha1)
 {
-	gint i;
-	gchar raw[SHA1_RAW_SIZE];
+	struct sha1 raw;
+	size_t len;
 
-	if (!retval)
-		retval = raw;
-
-	/*
-	 * Make sure we have at least SHA1_BASE32_SIZE characters before the
-	 * end of the string.
-	 */
-
-	for (i = 0; i < SHA1_BASE32_SIZE; i++) {
-		if ('\0' == buf[i])
-			goto invalid;
+	if (!sha1) {
+		sha1 = &raw;
 	}
 
-	if (base32_decode_into(buf, SHA1_BASE32_SIZE, retval, SHA1_RAW_SIZE))
-		return TRUE;
+	if (size < SHA1_BASE32_SIZE)
+		return FALSE;
+		
+	len = base32_decode(sha1->data, SHA1_RAW_SIZE, buf, SHA1_BASE32_SIZE);
+	if (SHA1_RAW_SIZE != len)
+		return FALSE;
 
-	/*
-	 * When extracting SHA1 from HTTP headers, we want to get the proper
-	 * hash value: some servents were deployed with the old base32 encoding
-	 * (using the digits 8 and 9 in the alphabet instead of letters L and O
-	 * in the new one).
-	 *
-	 * Among the 32 groups of 5 bits, equi-probable, there is a 2/32 chance
-	 * of having a 8 or a 9 encoded in the old alphabet.  Therefore, the
-	 * probability of not having a 8 or a 9 in the first letter is 30/32.
-	 * The probability of having no 8 or 9 in the 32 letters is (30/32)^32.
-	 * So the probability of having at least an 8 or a 9 is 1-(30/32)^32,
-	 * which is 87.32%.
-	 */
-
-	if (base32_decode_old_into(buf, SHA1_BASE32_SIZE, retval, SHA1_RAW_SIZE))
-		return TRUE;
-
-invalid:
-	g_warning("ignoring invalid SHA1 base32 encoding: %s", buf);
-
-	return FALSE;
+	return TRUE;
 }
 
 /**
@@ -103,9 +77,10 @@ invalid:
  * @return whether we successfully extracted the SHA1.
  */
 gboolean
-urn_get_sha1(const gchar *buf, gchar *digest)
+urn_get_sha1(const gchar *buf, struct sha1 *sha1)
 {
-	const gchar *sha1;
+	const gchar *p;
+	size_t len;
 
 	/*
 	 * We handle both "urn:sha1:" and "urn:bitprint:".  In the latter case,
@@ -113,12 +88,70 @@ urn_get_sha1(const gchar *buf, gchar *digest)
 	 */
 
 	if (
-		NULL == (sha1 = is_strcaseprefix(buf, "urn:sha1:")) &&
-		NULL == (sha1 = is_strcaseprefix(buf, "urn:bitprint:"))
+		NULL == (p = is_strcaseprefix(buf, "urn:sha1:")) &&
+		NULL == (p = is_strcaseprefix(buf, "urn:bitprint:"))
 	)
 		return FALSE;
 
-	return urn_get_http_sha1(sha1, digest);
+	len = clamp_strlen(p, SHA1_BASE32_SIZE);
+	return parse_base32_sha1(p, len, sha1);
+}
+
+gboolean
+urn_get_bitprint(const gchar *buf, size_t size,
+	struct sha1 *sha1, struct tth *tth)
+{
+	static const char prefix[] = "urn:bitprint:";
+	size_t len;
+	const gchar *p;
+
+	g_assert(0 == size || NULL != buf);
+	g_assert(sha1);
+	g_assert(tth);
+
+	if (size < CONST_STRLEN(prefix) + BITPRINT_BASE32_SIZE) {
+		return FALSE;
+	}
+	p = is_strcaseprefix(buf, prefix);
+	if (NULL == p) {
+		return FALSE;
+	}
+	if (!parse_base32_sha1(p, SHA1_BASE32_SIZE, sha1)) {
+		return FALSE;
+	}
+	p += SHA1_BASE32_SIZE;
+	if ('.' != *p++) {
+		return FALSE;
+	}
+	len = base32_decode(tth->data, TTH_RAW_SIZE, p, TTH_BASE32_SIZE);
+	if (len != TTH_RAW_SIZE) {
+		return FALSE;
+	}
+	return TRUE;
+}
+
+gboolean
+urn_get_tth(const gchar *buf, size_t size, struct tth *tth)
+{
+	static const char prefix[] = "urn:tree:tiger:";
+	size_t len;
+	const gchar *p;
+
+	g_assert(0 == size || NULL != buf);
+	g_assert(tth);
+
+	if (size < CONST_STRLEN(prefix) + TTH_BASE32_SIZE) {
+		return FALSE;
+	}
+	p = is_strcaseprefix(buf, prefix);
+	if (NULL == p) {
+		return FALSE;
+	}
+	len = base32_decode(tth->data, TTH_RAW_SIZE, p, TTH_BASE32_SIZE);
+	if (len != TTH_RAW_SIZE) {
+		return FALSE;
+	}
+	return TRUE;
 }
 
 /**
@@ -133,9 +166,10 @@ urn_get_sha1(const gchar *buf, gchar *digest)
  * @return whether we successfully extracted the SHA1.
  */
 gboolean
-urn_get_sha1_no_prefix(const gchar *buf, gchar *digest)
+urn_get_sha1_no_prefix(const gchar *buf, struct sha1 *sha1)
 {
-	const gchar *sha1;
+	const gchar *p;
+	size_t len;
 
 	/*
 	 * We handle both "sha1:" and "bitprint:".  In the latter case,
@@ -143,12 +177,13 @@ urn_get_sha1_no_prefix(const gchar *buf, gchar *digest)
 	 */
 
 	if (
-		NULL == (sha1 = is_strcaseprefix(buf, "sha1:")) &&
-		NULL == (sha1 = is_strcaseprefix(buf, "bitprint:"))
+		NULL == (p = is_strcaseprefix(buf, "sha1:")) &&
+		NULL == (p = is_strcaseprefix(buf, "bitprint:"))
 	)
 		return FALSE;
 
-	return urn_get_http_sha1(sha1, digest);
+	len = clamp_strlen(p, SHA1_BASE32_SIZE);
+	return parse_base32_sha1(p, len, sha1);
 }
 
 /*
