@@ -1,5 +1,5 @@
 /*
- * $Id: vendors.h 12147 2006-10-27 19:55:09Z cbiere $
+ * $Id: vendors.h 12648 2006-12-23 04:58:29Z cbiere $
  *
  * Copyright (c) 2001-2003, Richard Eckart
  *
@@ -36,7 +36,7 @@
 #ifndef _vendors_h_
 #define _vendors_h_
 
-#include <glib.h>
+#include "common.h"
 
 /***
  *** Known gnutella vendor codes
@@ -132,13 +132,13 @@
  */
 #define VENDOR_CODE_CMP(a, b) CMP(a, b)
 
-union vendor_code {
+typedef struct vendor_code {
 	guint32 be32;	/**< Always big-endian order; for convenient '=' and '!=' */
-};
+} vendor_code_t;
 
 const gchar *vendor_code_str(guint32 code);
-const gchar *lookup_vendor_name(union vendor_code code);
-gboolean is_vendor_known(union vendor_code code);
+const gchar *lookup_vendor_name(vendor_code_t code);
+gboolean is_vendor_known(vendor_code_t code);
 
 void vendor_init(void);
 
