@@ -1,5 +1,5 @@
 /*
- * $Id: zalloc.h,v 1.5 2005/08/02 12:12:59 daichik Exp $
+ * $Id: zalloc.h 12500 2006-11-26 10:53:21Z cbiere $
  *
  * Copyright (c) 2002-2003, Raphael Manfredi
  *
@@ -36,7 +36,7 @@
 #ifndef _zalloc_h_
 #define _zalloc_h_
 
-#include <glib.h>
+#include "common.h" 
 
 #define ZALLOC_ALIGNBYTES	MEM_ALIGNBYTES
 
@@ -47,30 +47,8 @@
 #define zalloc_round(s) \
 	((gulong) (((gulong) (s) + ZALLOC_MASK) & ~ZALLOC_MASK))
 
-/**
- * @struct zone
- *
- * Zone structure.
- *
- * Zone structures can be linked together to form one big happy chain.
- * During allocation/free, only the first zone structure of the list is
- * updated. The other zone structures are updated only during the garbage
- * collecting phase, if any.
- */
-
-struct subzone;
-
-typedef struct zone {			/* Zone descriptor */
-	gchar **zn_free;			/**< Pointer to first free block */
-	struct subzone *zn_next;	/**< Next allocated zone chunk, null if none */
-	gpointer zn_arena;			/**< Base address of zone arena */
-	gint zn_refcnt;				/**< How many references to that zone? */
-	gint zn_size;				/**< Size of blocks in zone */
-	gint zn_hint;				/**< Hint size, for next zone extension */
-	gint zn_cnt;				/**< Amount of used blocks in zone */
-} zone_t;
-
-#define zcount(z)	((z)->zn_cnt)	/**< Amount of allocated blocks */
+struct zone;
+typedef struct zone zone_t;
 
 /*
  * Memory allocation routines.
@@ -89,22 +67,13 @@ void zdestroy(zone_t *zone);
 #define REMAP_ZALLOC
 #endif
 
-#ifdef REMAP_ZALLOC
-
-#ifdef TRACK_ZALLOC
+#if defined(REMAP_ZALLOC) && defined(TRACK_ZALLOC)
 #error "TRACK_ZALLOC and REMAP_ZALLOC are mutually exclusive"
-#endif
-
-#define zalloc(z)	g_malloc((z)->zn_size)
-#define zfree(z,o)	g_free(o)
-
-#else	/* !REMAP_ZALLOC */
+#endif	/* REMAP_ZALLOC && TRACK_ZALLOC */
 
 gpointer zalloc(zone_t *);
 void zfree(zone_t *, gpointer);
 
-#endif	/* REMAP_ZALLOC */
-
 #ifdef TRACK_ZALLOC
 
 #define zalloc(z)	zalloc_track(z, __FILE__, __LINE__)
