@@ -1,5 +1,5 @@
 /*
- * $Id: zlib_util.h 12242 2006-11-06 20:41:55Z cbiere $
+ * $Id: zlib_util.h 12500 2006-11-26 10:53:21Z cbiere $
  *
  * Copyright (c) 2002-2003, Raphael Manfredi
  *
@@ -36,7 +36,7 @@
 #ifndef _zlib_util_h_
 #define _zlib_util_h_
 
-#include <glib.h>
+#include "common.h"
 
 /**
  * Incremental deflater stream.
