@@ -1,5 +1,5 @@
 /*
- * $Id: main.c 12271 2006-11-09 06:11:22Z cbiere $
+ * $Id: main.c 14052 2007-07-06 21:28:36Z cbiere $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi
  *
@@ -27,7 +27,7 @@
  * @ingroup core
  * @file
  *
- * Main functions for Gtk-Gnutella.
+ * Main functions for gtk-gnutella.
  *
  * @author Raphael Manfredi
  * @date 2001-2003
@@ -38,7 +38,6 @@
 
 #define CORE_SOURCES
 
-#include "core/gdb.h"
 #include "core/ban.h"
 #include "core/bitzi.h"
 #include "core/bogons.h"
@@ -57,7 +56,6 @@
 #include "core/gnet_stats.h"
 #include "core/gnutella.h"
 #include "core/guid.h"
-#include "core/gwcache.h"
 #include "core/hcache.h"
 #include "core/hostiles.h"
 #include "core/hosts.h"
@@ -86,9 +84,11 @@
 #include "core/tls_common.h"
 #include "core/tsync.h"
 #include "core/tx.h"
+#include "core/udp.h"
 #include "core/uhc.h"
 #include "core/upload_stats.h"
-#include "core/verify.h"
+#include "core/verify_sha1.h"
+#include "core/verify_tth.h"
 #include "core/version.h"
 #include "core/vmsg.h"
 #include "core/whitelist.h"
@@ -99,10 +99,10 @@
 #include "lib/atoms.h"
 #include "lib/bg.h"
 #include "lib/cq.h"
+#include "lib/crash.h"
 #include "lib/crc.h"
 #include "lib/dbus_util.h"
 #include "lib/eval.h"
-#include "lib/fragcheck.h"
 #include "lib/glib-missing.h"
 #include "lib/halloc.h"
 #include "lib/iso3166.h"
@@ -122,7 +122,6 @@
 #endif
 
 #if defined(USE_GTK1) || defined(USE_GTK2)
-#include "ui/gtk/drop.h"
 #include "ui/gtk/gui.h"
 #include "ui/gtk/icon.h"
 #include "ui/gtk/main.h"
@@ -137,7 +136,7 @@
 
 #include "lib/override.h"		/* Must be the last header included */
 
-RCSID("$Id: main.c 12271 2006-11-09 06:11:22Z cbiere $")
+RCSID("$Id: main.c 14052 2007-07-06 21:28:36Z cbiere $")
 
 #define SLOW_UPDATE_PERIOD		20	/**< Update period for `main_slow_update' */
 #define EXIT_GRACE				30	/**< Seconds to wait before exiting */
@@ -149,7 +148,7 @@ RCSID("$Id: main.c 12271 2006-11-09 06:11:22Z cbiere $")
 static guint main_slow_update = 0;
 static gboolean exiting = FALSE;
 static volatile sig_atomic_t from_atexit = FALSE;
-static volatile int signal_received = 0;
+static volatile sig_atomic_t signal_received = 0;
 static volatile sig_atomic_t shutdown_requested = 0;
 static volatile sig_atomic_t sig_hup_received = 0;
 static jmp_buf atexit_env;
@@ -179,6 +178,16 @@ sig_hup(int n)
 	sig_hup_received = 1;
 }
 
+static void
+sig_chld(int n)
+{
+	int saved_errno = errno;
+	(void) n;
+	while (waitpid(-1, NULL, WNOHANG) > 0)
+		continue;
+	errno = saved_errno;
+}
+
 #if defined(FRAGCHECK) || defined(MALLOC_STATS)
 static volatile sig_atomic_t signal_malloc = 0;
 
@@ -222,38 +231,37 @@ gboolean
 debugging(guint t)
 {
 	return
-		ban_debug > t ||
-		bitzi_debug > t ||
-		bootstrap_debug > t ||
-		dbg > t ||
-		dh_debug > t ||
-		dht_debug > t ||
-		dmesh_debug > t ||
-		download_debug > t ||
-		dq_debug > t ||
-		fileinfo_debug > t ||
-		ggep_debug > t ||
-		gmsg_debug > t ||
-		gwc_debug > t ||
-		hsep_debug > t ||
-		http_debug > t ||
-		lib_debug > t ||
-		node_debug > t ||
-		oob_proxy_debug > t ||
-		parq_debug > t ||
-		pcache_debug > t ||
-		qrp_debug > t ||
-		query_debug > t ||
-		routing_debug > t ||
-		rudp_debug > t ||
-		search_debug > t ||
-		share_debug > t ||
-		socket_debug > t ||
-		tls_debug > t ||
-		udp_debug > t ||
-		upload_debug > t ||
-		url_debug > t ||
-		vmsg_debug > t ||
+		GNET_PROPERTY(ban_debug) > t ||
+		GNET_PROPERTY(bitzi_debug) > t ||
+		GNET_PROPERTY(bootstrap_debug) > t ||
+		GNET_PROPERTY(dbg) > t ||
+		GNET_PROPERTY(dh_debug) > t ||
+		GNET_PROPERTY(dht_debug) > t ||
+		GNET_PROPERTY(dmesh_debug) > t ||
+		GNET_PROPERTY(download_debug) > t ||
+		GNET_PROPERTY(dq_debug) > t ||
+		GNET_PROPERTY(fileinfo_debug) > t ||
+		GNET_PROPERTY(ggep_debug) > t ||
+		GNET_PROPERTY(gmsg_debug) > t ||
+		GNET_PROPERTY(hsep_debug) > t ||
+		GNET_PROPERTY(http_debug) > t ||
+		GNET_PROPERTY(lib_debug) > t ||
+		GNET_PROPERTY(node_debug) > t ||
+		GNET_PROPERTY(oob_proxy_debug) > t ||
+		GNET_PROPERTY(parq_debug) > t ||
+		GNET_PROPERTY(pcache_debug) > t ||
+		GNET_PROPERTY(qrp_debug) > t ||
+		GNET_PROPERTY(query_debug) > t ||
+		GNET_PROPERTY(routing_debug) > t ||
+		GNET_PROPERTY(rudp_debug) > t ||
+		GNET_PROPERTY(search_debug) > t ||
+		GNET_PROPERTY(share_debug) > t ||
+		GNET_PROPERTY(socket_debug) > t ||
+		GNET_PROPERTY(tls_debug) > t ||
+		GNET_PROPERTY(udp_debug) > t ||
+		GNET_PROPERTY(upload_debug) > t ||
+		GNET_PROPERTY(url_debug) > t ||
+		GNET_PROPERTY(vmsg_debug) > t ||
 
 		/* Above line left blank for easy "!}sort" under vi */
 		0;
@@ -363,7 +371,7 @@ gtk_gnutella_exit(gint n)
 
 	exiting = TRUE;
 
-#define DO(fn) 	do { exit_step = STRINGIFY(fn); fn(); } while (0)
+#define DO(fn) 	do { exit_step = #fn; fn(); } while (0)
 
 	DO(shell_close);
 	DO(file_info_close_pre);
@@ -371,12 +379,12 @@ gtk_gnutella_exit(gint n)
 	DO(upload_close);	/* Done before upload_stats_close() for stats update */
 	DO(upload_stats_close);
 	DO(parq_close);
+	DO(verify_sha1_close);
+	DO(verify_tth_close);
 	DO(download_close);
 	DO(pproxy_close);
 	DO(http_close);
-	DO(gwc_close);
 	DO(uhc_close);
-	DO(verify_close);
 	DO(move_close);
 
 	/*
@@ -423,15 +431,19 @@ gtk_gnutella_exit(gint n)
 	 * have more connections to flush.
 	 */
 
-	if (current_peermode == NODE_P_ULTRA)
+	if (GNET_PROPERTY(current_peermode) == NODE_P_ULTRA)
 		exit_grace *= 2;
 
 	while (node_bye_pending()) {
 		time_t now = time(NULL);
 		time_delta_t d;
 
+		if (signal_received)
+			break;
+
 		if ((d = delta_time(now, exit_time)) >= exit_grace)
 			break;
+
 		main_gui_shutdown_tick(exit_grace - d);
 		sleep(1);
 	}
@@ -444,17 +456,19 @@ gtk_gnutella_exit(gint n)
 	dh_close();
 	dq_close();
 	hsep_close();
-	drop_close();
 	file_info_close();
 	ext_close();
 	share_close();
 	node_close();
+	udp_close();
 	routing_close();	/* After node_close() */
 	bsched_close();
 	dmesh_close();
 	host_close();
 	hcache_close();		/* After host_close() */
 	bogons_close();		/* Idem, since host_close() can touch the cache */
+	tx_collect();		/* Prevent spurious leak notifications */
+	rx_collect();		/* Idem */
 	hostiles_close();
 	spam_close();
 	gip_close();
@@ -463,6 +477,7 @@ gtk_gnutella_exit(gint n)
 	whitelist_close();
 	features_close();
 	clock_close();
+	vmsg_close();
 	watcher_close();
 	tsync_close();
 	cq_close();
@@ -478,7 +493,6 @@ gtk_gnutella_exit(gint n)
 	adns_close();
 	dbus_util_close();  /* After adns_close() to avoid strange crashes */
 	tls_cache_close();
-	gdb_close();
 	file_object_close();
 	settings_close();	/* Must come after hcache_close() */
 	atoms_close();
@@ -518,9 +532,8 @@ static void
 slow_main_timer(time_t now)
 {
 	static guint i = 0;
-	static time_t last_warn = 0;
 
-	if (cpu_debug) {
+	if (GNET_PROPERTY(cpu_debug)) {
 		static tm_t since = { 0, 0 };
 		static gdouble user = 0.0;
 		static gdouble sys = 0.0;
@@ -547,7 +560,6 @@ slow_main_timer(time_t now)
 		file_info_store_if_dirty();
 		break;
 	case 4:
-		gwc_store_if_dirty();
 		hcache_store_if_dirty(HOST_ULTRA);
 		break;
 	default:
@@ -564,11 +576,6 @@ slow_main_timer(time_t now)
 
 	node_slow_timer(now);
 	ignore_timer(now);
-
-	if (delta_time(now, last_warn) > 600) {
-		version_ancient_warn();
-		last_warn = now;
-	}
 }
 
 #if !defined(USE_TOPLESS)
@@ -635,7 +642,7 @@ check_cpu_usage(void)
 	load_avg += (load >> 3) - (load_avg >> 3);
 	avg = load_avg / 100;
 
-	if (dbg > 1 && last_cpu)
+	if (GNET_PROPERTY(cpu_debug) > 1 && last_cpu)
 		g_message("CPU: %.3f secs in %.3f secs (~%.3f%% @ cover=%.2f) avg=%d%%",
 			cpu - last_cpu, elapsed, cpu_percent, coverage, avg);
 
@@ -652,12 +659,12 @@ check_cpu_usage(void)
 	 * the average load enough to disable the "overloaded" condition.
 	 */
 
-	if (avg >= LOAD_HIGH_WATERMARK && !overloaded_cpu) {
+	if (avg >= LOAD_HIGH_WATERMARK && !GNET_PROPERTY(overloaded_cpu)) {
 		if (debugging(0))
 			g_message("high average CPU load (%d%%), entering overloaded state",
 				avg);
 		gnet_prop_set_boolean_val(PROP_OVERLOADED_CPU, TRUE);
-	} else if (overloaded_cpu && avg < LOAD_LOW_WATERMARK) {
+	} else if (GNET_PROPERTY(overloaded_cpu) && avg < LOAD_LOW_WATERMARK) {
 		if (debugging(0))
 			g_message("average CPU load (%d%%) low, leaving overloaded state",
 				avg);
@@ -735,11 +742,11 @@ main_timer(gpointer p)
 	 * detected a high CPU load.
 	 */
 
-	if (!overloaded_cpu) {
+	if (!GNET_PROPERTY(overloaded_cpu)) {
 		icon_timer();
 	}
 
-	bg_sched_timer(overloaded_cpu);			/* Background tasks */
+	bg_sched_timer(GNET_PROPERTY(overloaded_cpu));	/* Background tasks */
 
 	return TRUE;
 }
@@ -778,7 +785,7 @@ log_handler(const gchar *domain, GLogLevelFlags level,
 	ct = localtime(&now);
 
 	switch (level) {
-#define CASE(x) case CAT2(G_LOG_LEVEL_,x): prefix = STRINGIFY(x); break;
+#define CASE(x) case CAT2(G_LOG_LEVEL_,x): prefix = #x; break;
 
 	CASE(CRITICAL)
 	CASE(ERROR)
@@ -832,76 +839,131 @@ log_init(void)
 	}
 }
 
-/**
- * Closes all file descriptors greater or equal to ``fd''.
- */
-static void
-close_fds(gint fd)
-{
-	g_assert(fd >= 0);
-
-#ifdef F_CLOSEM
-	if (-1 == fcntl(fd, F_CLOSEM))
-#endif
-	{
-		gdouble start;
-		gint num_fds = compat_max_fd();
-
-		start = tm_cputime(NULL, NULL);
-		for (/* NOTHING */; fd < num_fds; fd++) {
-			close(fd);
-
-			/* Just in case we're trying to close a bazillion fds on a vax */
-			if (0 == (fd & 0xff) && tm_cputime(NULL, NULL) - start > 5) {
-				g_warning("Aborted closing file descriptors after "
-					"exceeding 5s CPU time (fd=%d)", fd);
-				break;
-			}
-		}
-	}
-}
-
 extern char **environ;
 
 enum main_arg {
 	main_arg_daemonize,
+	main_arg_exec_on_crash,
 	main_arg_geometry,
 	main_arg_help,
 	main_arg_log_stderr,
 	main_arg_log_stdout,
+	main_arg_no_halloc,
+	main_arg_no_xshm,
+	main_arg_pause_on_crash,
 	main_arg_ping,
 	main_arg_shell,
 	main_arg_version,
 
+	/* Passed through for Gtk+/GDK/GLib */
+	main_arg_class,
+	main_arg_g_fatal_warnings,
+	main_arg_gdk_debug,
+	main_arg_gdk_no_debug,
+	main_arg_gtk_debug,
+	main_arg_gtk_no_debug,
+	main_arg_gtk_module,
+	main_arg_name,
+
 	num_main_args
 };
 
+enum arg_type {
+	ARG_TYPE_NONE,
+	ARG_TYPE_TEXT,
+	ARG_TYPE_PATH
+};
+
 static struct {
 	const enum main_arg id;
 	const gchar * const name;
 	const gchar * const summary;
-	const gboolean has_arg;
+	const enum arg_type type;
 	const gchar *arg;
 	gboolean used;
 } options[] = {
-#define OPTION(name, summary, has_arg) \
-	{ main_arg_ ## name , STRINGIFY(name), summary, has_arg, NULL, FALSE }
+#define OPTION(name, type, summary) \
+	{ main_arg_ ## name , #name, summary, ARG_TYPE_ ## type, NULL, FALSE }
 
-#define OPTION_FLAG(name, summary)	OPTION(name, summary, FALSE)
-#define OPTION_WARG(name, summary)	OPTION(name, summary, TRUE)
-	OPTION_FLAG(daemonize,	"Daemonize the process."),
-	OPTION_WARG(geometry,	"Placement of the main GUI window."),
-	OPTION_FLAG(help, 		"Print this message."),
-	OPTION_WARG(log_stderr,	"Log standard output to a file."),
-	OPTION_WARG(log_stdout,	"Log standard error output to a file."),
-	OPTION_FLAG(ping,		"Check whether gtk-gnutella is running."),
-	OPTION_FLAG(shell,		"Access the local shell interface."),
-	OPTION_FLAG(version,	"Show version information."),
-#undef OPTION_WARG
-#undef OPTION_FLAG
+	OPTION(daemonize, 		NONE, "Daemonize the process."),
+	OPTION(exec_on_crash, 	PATH, "Execute a command on crash."),
+	OPTION(geometry,		TEXT, "Placement of the main GUI window."),
+	OPTION(help, 			NONE, "Print this message."),
+	OPTION(log_stderr,		PATH, "Log standard output to a file."),
+	OPTION(log_stdout,		PATH, "Log standard error output to a file."),
+#ifdef USE_HALLOC
+	OPTION(no_halloc,		NONE, "Disable malloc() replacement."),
+#else
+	OPTION(no_halloc,		NONE, NULL),	/* ignore silently */
+#endif	/* USE_HALLOC */
+	OPTION(no_xshm,			NONE, "Disable MIT shared memory extension."),
+	OPTION(pause_on_crash, 	NONE, "Pause the process on crash."),
+	OPTION(ping,			NONE, "Check whether gtk-gnutella is running."),
+	OPTION(shell,			NONE, "Access the local shell interface."),
+	OPTION(version,			NONE, "Show version information."),
+
+	/* These are handled by Gtk+/GDK/GLib */
+	OPTION(class,				TEXT, NULL),
+	OPTION(g_fatal_warnings,	NONE, NULL),
+	OPTION(gdk_debug,			TEXT, NULL),
+	OPTION(gdk_no_debug,		TEXT, NULL),
+	OPTION(gtk_debug,			TEXT, NULL),
+	OPTION(gtk_no_debug,		TEXT, NULL),
+	OPTION(gtk_module,			TEXT, NULL),
+	OPTION(name,				TEXT, NULL),
 #undef OPTION
 };
 
+static inline gchar
+underscore_to_hyphen(gchar c)
+{
+	return '_' == c ? '-' : c;
+}
+
+/**
+ * Checks whether two strings qualify as equivalent, the ASCII underscore
+ * character and the ASCII hyphen character are considered equivalent.
+ *
+ * @return whether the two strings qualify as equivalent or not.
+ */
+static gboolean
+option_match(const gchar *a, const gchar *b)
+{
+	g_assert(a);
+	g_assert(b);
+
+	do {
+		if (underscore_to_hyphen(*a) != underscore_to_hyphen(*b)) {
+			return FALSE;
+		}
+		b++;
+	} while ('\0' != *a++);
+
+	return TRUE;
+}
+
+/**
+ * Copies the given option name to a static buffer replacing underscores
+ * with hyphens.
+ *
+ * @return a pointer to a static buffer holding the pretty version of the
+ *         option name. 
+ */
+static const gchar *
+option_pretty_name(const gchar *name)
+{
+	static gchar buf[128];
+	size_t i;
+
+	for (i = 0; i < G_N_ELEMENTS(buf) - 1; i++) {
+		if ('\0' == name[i])
+			break;
+		buf[i] = underscore_to_hyphen(name[i]);
+	}
+	buf[i] = '\0';
+	return buf;
+}
+
 static gint
 reopen_log_files(void)
 {
@@ -939,67 +1001,146 @@ usage(int exit_code)
 	
 	STATIC_ASSERT(G_N_ELEMENTS(options) == num_main_args);
 	for (i = 0; i < G_N_ELEMENTS(options); i++) {
-		const gchar *arg;
-		size_t pad;
-
 		g_assert(options[i].id == i);
 
-		arg = options[i].has_arg ? " <argument>" : "";
-		pad = strlen(options[i].name) + strlen(arg);
-		if (pad < 24) {
-			pad = 24 - pad;
-		} else {
-			pad = 0;
-		}
+		if (options[i].summary) {
+			const gchar *arg, *name;
+			size_t pad;
 
-		fprintf(f, "  --%s%s%-*s%s\n",
-			options[i].name,
-			arg,
-			(gint) pad, "",
-			options[i].summary);
+			arg = "";
+			name = option_pretty_name(options[i].name);
+			switch (options[i].type) {
+			case ARG_TYPE_NONE:
+				break;
+			case ARG_TYPE_TEXT:
+				arg = " <argument>";
+				break;
+			case ARG_TYPE_PATH:
+				arg = " <path>";
+				break;
+			}
+			
+			pad = strlen(name) + strlen(arg);
+			if (pad < 24) {
+				pad = 24 - pad;
+			} else {
+				pad = 0;
+			}
+
+			fprintf(f, "  --%s%s%-*s%s\n",
+				name, arg, (gint) pad, "", options[i].summary);
+		}
 	}
 	
 	exit(exit_code);
 }
 
+/* NOTE: This function must not allocate any memory. */
+static void
+prehandle_arguments(char **argv)
+{
+	argv++;
+
+	while (argv[0]) {
+		const gchar *s;
+		guint i;
+
+		s = is_strprefix(argv[0], "--");
+		if (NULL == s || '\0' == s[0])
+			break;
+
+		argv++;
+
+		for (i = 0; i < G_N_ELEMENTS(options); i++) {
+			if (option_match(options[i].name, s))
+				break;
+		}
+		if (G_N_ELEMENTS(options) == i)
+			return;
+
+		if (main_arg_no_halloc == i) {
+			options[i].used = TRUE;
+		}
+
+		switch (options[i].type) {
+		case ARG_TYPE_NONE:
+			break;
+		case ARG_TYPE_TEXT:
+		case ARG_TYPE_PATH:
+			if (NULL == argv[0] || '-' == argv[0][0])
+				return;
+
+			argv++;
+			break;
+		}
+	}
+}
+
+
 static void
 handle_arguments(int argc, char **argv)
 {
+	const char *argv0;
 	guint i;
 
 	STATIC_ASSERT(G_N_ELEMENTS(options) == num_main_args);
-
 	for (i = 0; i < G_N_ELEMENTS(options); i++) {
 		g_assert(options[i].id == i);
-		options[i].used = FALSE;
-		options[i].arg = NULL;
 	}
 
+	argv0 = argv[0];
+	argv++;
+	argc--;
+
 	while (argc > 0) {
-		const gchar *s = argv[0];
+		const gchar *s;
+
+		s = is_strprefix(argv[0], "--");
+		if (!s)
+			usage(EXIT_FAILURE);
+		if ('\0' == s[0])
+			break;
 
 		argv++;
 		argc--;
-		
-		s = is_strprefix(s, "--");
-		if (!s || '\0' == s[0]) {
-			continue;
-		}
 
 		for (i = 0; i < G_N_ELEMENTS(options); i++) {
-			if (0 != strcmp(options[i].name, s)) {
-				continue;
+			if (option_match(options[i].name, s))
+				break;
+		}
+		if (G_N_ELEMENTS(options) == i) {
+			fprintf(stderr, "Unknown option \"--%s\"\n", s);
+			usage(EXIT_FAILURE);
+		}
+
+		options[i].used = TRUE;
+		switch (options[i].type) {
+		case ARG_TYPE_NONE:
+			break;
+		case ARG_TYPE_TEXT:
+		case ARG_TYPE_PATH:
+			if (argc < 0 || NULL == argv[0] || '-' == argv[0][0]) {
+				fprintf(stderr, "Missing argument for \"--%s\"\n", s);
+				usage(EXIT_FAILURE);
 			}
-			options[i].used = TRUE;
-			if (options[i].has_arg) {
-				if (argc < 1 || NULL == argv[0] || '-' == argv[0][0]) {
-					fprintf(stderr, "Missing argument for %s\n", s);
+			switch (options[i].type) {
+			case ARG_TYPE_TEXT:
+				options[i].arg = g_strdup(argv[0]);
+				break;
+			case ARG_TYPE_PATH:
+				options[i].arg = absolute_pathname(argv[0]);
+				if (NULL == options[i].arg) {
+					fprintf(stderr,
+						"Could not determine absolute path for \"--%s\"\n", s);
 					usage(EXIT_FAILURE);
 				}
-				options[i].arg = g_strdup(argv[0]);
-				argv++;
-				argc--;
+				break;
+			case ARG_TYPE_NONE:
+				g_assert_not_reached();
 			}
+			argv++;
+			argc--;
+			break;
 		}
 	}
 
@@ -1007,11 +1148,28 @@ handle_arguments(int argc, char **argv)
 		exit(EXIT_FAILURE);
 	}
 
+	crash_init(options[main_arg_exec_on_crash].arg, argv0,
+		options[main_arg_pause_on_crash].used);
+
 	if (options[main_arg_help].used) {
 		usage(EXIT_SUCCESS);
 	}
 	if (options[main_arg_version].used) {
 		printf("%s\n", version_build_string());
+
+		printf("GLib %u.%u.%u (compiled against %u.%u.%u)\n",
+			glib_major_version, glib_minor_version, glib_micro_version,
+			GLIB_MAJOR_VERSION, GLIB_MINOR_VERSION, GLIB_MICRO_VERSION);
+
+#ifndef USE_TOPLESS
+		printf("Gtk+ %u.%u.%u (compiled against %u.%u.%u)\n",
+			gtk_major_version, gtk_minor_version, gtk_micro_version,
+			GTK_MAJOR_VERSION, GTK_MINOR_VERSION, GTK_MICRO_VERSION);
+#endif	/* USE_TOPLESS */
+
+		if (tls_version_string())
+			printf("%s\n", tls_version_string());
+
 		exit(EXIT_SUCCESS);
 	}
 	if (options[main_arg_ping].used) {
@@ -1036,30 +1194,38 @@ handle_arguments(int argc, char **argv)
 			exit(EXIT_FAILURE);
 		}
 	}
+
 }
 
 int
 main(int argc, char **argv)
 {
-
-#ifdef FRAGCHECK
-	fragcheck_init();
-#endif
-	halloc_init();
-
-	misc_init();
-
-	tm_now_exact(&start_time);
-
 	if (compat_is_superuser()) {
 		fprintf(stderr, "Never ever run this as root!\n");
 		exit(EXIT_FAILURE);
 	}
 
-	close_fds(3); /* Just in case */
+	/*
+	 * This must be run before we allocate memory because we might
+	 * use mmap() with /dev/zero and then accidently close this
+	 * file descriptor.
+	 */
+	close_file_descriptors(3); /* Just in case */
+
+	prehandle_arguments(argv);
+
+	if (!options[main_arg_no_halloc].used) {
+		halloc_init();
+	}
+	
+	misc_init();
+	tm_now_exact(&start_time);
 
 	set_signal(SIGINT, SIG_IGN);	/* ignore SIGINT in adns (e.g. for gdb) */
 	set_signal(SIGHUP, sig_hup);
+#ifdef SIGCHLD
+	set_signal(SIGCHLD, sig_chld);
+#endif
 #ifdef SIGPIPE
 	set_signal(SIGPIPE, SIG_IGN);
 #endif
@@ -1108,27 +1274,26 @@ main(int argc, char **argv)
 	iso3166_init();
 	dbus_util_init();
 	vendor_init();
-	vmsg_init();
 
-	main_gui_early_init(argc, argv);
+	main_gui_early_init(argc, argv, options[main_arg_no_xshm].used);
 
 	cq_init();
+	vmsg_init();
 	tsync_init();
 	watcher_init();
 	hcache_init();			/* before settings_init() */
+	bsched_early_init();	/* before settings_init() */
 	settings_init();
-	gdb_init();
 	tls_global_init();
 	tls_cache_init();
-    hcache_retrieve_all();	/* after settings_init() */
 	hostiles_init();
 	spam_init();
 	bogons_init();
 	gip_init();
 	guid_init();
-	gwc_init();
 	uhc_init();
-	verify_init();
+	verify_sha1_init();
+	verify_tth_init();
 	move_init();
 	ignore_init();
 	file_info_init();
@@ -1139,6 +1304,7 @@ main(int argc, char **argv)
 	gmsg_init();
 	bsched_init();
 	node_init();
+    hcache_retrieve_all();	/* after settings_init() and node_init() */
 	routing_init();
 	search_init();
 	share_init();
@@ -1167,8 +1333,6 @@ main(int argc, char **argv)
 	main_gui_init();
 	node_post_init();
 
-	drop_init();
-
 	download_restore_state();
 	ntp_init();
 
