@@ -1,5 +1,5 @@
 /*
- * $Id: columns.h 11566 2006-08-17 02:50:23Z cbiere $
+ * $Id: columns.h 13676 2007-05-16 18:44:58Z cbiere $
  *
  * Copyright (c) 2001-2004, Raphael Manfredi & Richard Eckart
  *
@@ -136,6 +136,7 @@ enum c_fi {
 	c_fi_filename = 0,
 	c_fi_size,
 	c_fi_done,
+	c_fi_uploaded,
 	c_fi_sources,
 	c_fi_status,
 #define FILEINFO_VISIBLE_COLUMNS ((guint) c_fi_status + 1)
@@ -147,13 +148,14 @@ enum c_fi {
  */
 
 enum c_sr_columns {
-    c_sr_filename = 0,
+    c_sr_filename,
 	c_sr_ext,
     c_sr_charset,
     c_sr_size,
 	c_sr_count,
     c_sr_loc,
     c_sr_meta,	/**< meta-data info column */
+    c_sr_vendor,
     c_sr_info,
     c_sr_route,
     c_sr_protocol,
@@ -163,8 +165,10 @@ enum c_sr_columns {
     c_sr_hostile,
     c_sr_owned,
     c_sr_sha1,
-	c_sr_num
+	c_sr_ctime,
+
 #define SEARCH_RESULTS_VISIBLE_COLUMNS ((guint) c_sr_num)
+	c_sr_num
 };
 
 /**
