@@ -1,5 +1,5 @@
 /*
- * $Id: filter_core.c,v 1.34 2005/12/15 10:20:29 cbiere Exp $
+ * $Id: filter_core.c 13777 2007-05-29 00:23:11Z cbiere $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi, Richard Eckart
  *
@@ -36,11 +36,12 @@
 
 #include "common.h"
 
-RCSID("$Id: filter_core.c,v 1.34 2005/12/15 10:20:29 cbiere Exp $");
+RCSID("$Id: filter_core.c 13777 2007-05-29 00:23:11Z cbiere $")
 
 #include "filter.h"
 #include "filter_core.h"
 #include "search.h"
+#include "settings.h"
 #include "gtk-missing.h"
 
 #ifdef USE_GTK1
@@ -76,6 +77,8 @@ typedef struct shadow {
     guint32 fail_count;
 } shadow_t;
 
+typedef GList *filter_add_rule_func_t(GList *, gpointer);
+
 /**
  * Structure holding "global" variables during filtering.
  */
@@ -87,9 +90,9 @@ struct filter_context {
 	 * Those variables are initialized as needed.
 	 */
 
-	gchar *l_name;				/**< Lower-cased file name */
+	const gchar *l_name;		/**< Lower-cased file name; atom */
 	size_t l_len;				/**< Length of lower-cased representation */
-	gchar *utf8_name;			/**< Normalized UTF-8 version of name */
+	const gchar *utf8_name;		/**< Normalized UTF-8 version of name; atom */
 	size_t utf8_len;			/**< Length of UTF-8 name representation */
 };
 
@@ -130,6 +133,31 @@ GList *filters_current = NULL;
 /***
  *** Implementation
  ***/
+
+#define WIDGET(name) \
+static GtkWidget * name ## _protected_ ; \
+ \
+GtkWidget *gui_ ## name (void) \
+{ \
+	return name ## _protected_ ; \
+} \
+ \
+void \
+gui_ ## name ## _set (GtkWidget *w) \
+{ \
+	name ## _protected_ = w; \
+} \
+ \
+GtkWidget * \
+gui_ ## name ## _lookup(const gchar *id) \
+{ \
+	return lookup_widget(gui_ ## name (), id); \
+}
+
+WIDGET(filter_dialog)
+WIDGET(popup_filter_rule)
+#undef WIDGET
+
 void
 dump_ruleset(const GList *ruleset)
 {
@@ -204,11 +232,11 @@ shadow_find(filter_t *f)
 
     l = g_list_find_custom(shadow_filters, f, shadow_filter_eq);
     if (l != NULL) {
-        if (gui_debug >= 6)
+        if (GUI_PROPERTY(gui_debug) >= 6)
             g_message("shadow found for: %s", f->name);
         return l->data;
     } else {
-        if (gui_debug >= 6)
+        if (GUI_PROPERTY(gui_debug) >= 6)
             g_message("no shadow found for: %s", f->name);
         return NULL;
     }
@@ -228,7 +256,7 @@ shadow_new(filter_t *f)
     g_assert(f != NULL);
     g_assert(f->name != NULL);
 
-    if (gui_debug >= 6)
+    if (GUI_PROPERTY(gui_debug) >= 6)
         g_message("creating shadow for: %s", f->name);
 
     shadow = g_new0(shadow_t, 1);
@@ -264,7 +292,7 @@ shadow_cancel(shadow_t *shadow)
     g_assert(shadow != NULL);
     g_assert(shadow->filter != NULL);
 
-    if (gui_debug >= 6)
+    if (GUI_PROPERTY(gui_debug) >= 6)
         g_message("cancel shadow for filter: %s", shadow->filter->name);
 
     for (r = shadow->added; r != NULL; r = r->next)
@@ -301,7 +329,7 @@ shadow_commit(shadow_t *shadow)
 
     realf = shadow->filter;
 
-    if (gui_debug >= 6) {
+    if (GUI_PROPERTY(gui_debug) >= 6) {
         g_message("committing shadow for filter:");
         dump_shadow(shadow);
     }
@@ -354,7 +382,7 @@ shadow_commit(shadow_t *shadow)
     shadow_filters = g_list_remove(shadow_filters, shadow);
     G_FREE_NULL(shadow);
 
-    if (gui_debug >= 6) {
+    if (GUI_PROPERTY(gui_debug) >= 6) {
         g_message("after commit filter looks like this");
         dump_filter(realf);
     }
@@ -402,9 +430,9 @@ filter_open_dialog(void)
 
     current_search = search_gui_get_current_search();
 
-    if (filter_dialog == NULL) {
-        filter_dialog = filter_gui_create_dlg_filters();
-        g_assert(filter_dialog != NULL);
+    if (gui_filter_dialog() == NULL) {
+        gui_filter_dialog_set(filter_gui_create_dlg_filters());
+        g_assert(gui_filter_dialog() != NULL);
 
         filter_gui_init();
         filter_refresh_display(filters_current);
@@ -428,23 +456,23 @@ filter_close_dialog(gboolean commit)
     } else
         filter_revert_changes();
 
-    if (filter_dialog != NULL) {
+    if (gui_filter_dialog() != NULL) {
         gint32 coord[4] = {0, 0, 0, 0};
 
-        gdk_window_get_root_origin(filter_dialog->window, &coord[0], &coord[1]);
-        gdk_drawable_get_size(filter_dialog->window, &coord[2], &coord[3]);
+        gdk_window_get_root_origin(gui_filter_dialog()->window, &coord[0], &coord[1]);
+        gdk_drawable_get_size(gui_filter_dialog()->window, &coord[2], &coord[3]);
 
         gui_prop_set_guint32(PROP_FILTER_DLG_COORDS, (guint32 *) coord, 0, 4);
 
-        *(guint32 *) &filter_main_divider_pos =
+        *(guint32 *) &GUI_PROPERTY(filter_main_divider_pos) =
             gtk_paned_get_position
-                (GTK_PANED(lookup_widget(filter_dialog, "hpaned_filter_main")));
+                (GTK_PANED(gui_filter_dialog_lookup("hpaned_filter_main")));
 
 #ifdef FILTER_HIDE_ON_CLOSE
-        gtk_widget_hide(filter_dialog);
+        gtk_widget_hide(gui_filter_dialog());
 #else
-        gtk_object_destroy(GTK_OBJECT(filter_dialog));
-        filter_dialog = NULL;
+        gtk_object_destroy(GTK_OBJECT(gui_filter_dialog()));
+        gui_filter_dialog() = NULL;
 #endif /* FILTER_HIDE_ON_CLOSE */
     }
 }
@@ -636,14 +664,13 @@ filter_new_jump_rule(filter_t *target, guint16 flags)
 
 
 rule_t *
-filter_new_sha1_rule(const gchar *sha1, const gchar *filename,
+filter_new_sha1_rule(const struct sha1 *sha1, const gchar *filename,
 	filter_t *target, guint16 flags)
 {
    	rule_t *f;
 
     g_assert(target != NULL);
-    g_assert(filename != NULL);
-    g_assert(utf8_is_valid_string(filename));
+    g_assert(NULL == filename || utf8_is_valid_string(filename));
 
     f = g_new0(rule_t, 1);
 
@@ -651,8 +678,8 @@ filter_new_sha1_rule(const gchar *sha1, const gchar *filename,
 
   	f->target = target;
     f->u.sha1.hash = sha1 != NULL ? g_memdup(sha1, SHA1_RAW_SIZE) : NULL;
-    f->u.sha1.filename = g_strdup(filename);
-    f->flags  = flags;
+    f->u.sha1.filename = g_strdup(filename ? filename : "");
+    f->flags = flags;
     set_flags(f->flags, RULE_FLAG_VALID);
 
     return f;
@@ -762,8 +789,8 @@ filter_close_search(search_t *s)
     g_assert(s != NULL);
     g_assert(s->filter != NULL);
 
-    if (gui_debug >= 6)
-        g_message("closing search (freeing filter): %s", s->query);
+    if (GUI_PROPERTY(gui_debug) >= 6)
+        g_message("closing search (freeing filter): %s", search_gui_query(s));
 
     shadow = shadow_find(s->filter);
     if (shadow != NULL) {
@@ -854,7 +881,7 @@ filter_revert_changes(void)
 {
     GList *iter;
 
-    if (gui_debug >= 5)
+    if (GUI_PROPERTY(gui_debug) >= 5)
         g_message("Canceling all changes to filters/rules");
 
     filter_gui_freeze_filters();
@@ -993,20 +1020,26 @@ filter_rule_condition_to_string(const rule_t *r)
 
 			uint64_to_string_buf(r->u.size.upper, smax_64, sizeof smax_64);
 			gm_snprintf(tmp, sizeof tmp , _("If filesize is exactly %s (%s)"),
-				smax_64, short_size(r->u.size.upper));
+				smax_64,
+				short_size(r->u.size.upper, show_metric_units()));
 		} else if (r->u.size.lower == 0) {
             gchar smax_64[UINT64_DEC_BUFLEN];
 
 			uint64_to_string_buf(r->u.size.upper + 1, smax_64, sizeof smax_64);
 			gm_snprintf(tmp, sizeof tmp,
 				_("If filesize is smaller than %s (%s)"),
-				smax_64, short_size(r->u.size.upper + 1));
+				smax_64,
+				short_size(r->u.size.upper + 1, show_metric_units()));
 		} else {
             gchar smin[256], smax[256];
             gchar smin_64[UINT64_DEC_BUFLEN], smax_64[UINT64_DEC_BUFLEN];
 
-            g_strlcpy(smin, short_size(r->u.size.lower), sizeof smin);
-            g_strlcpy(smax, short_size(r->u.size.upper), sizeof smax);
+            g_strlcpy(smin,
+				short_size(r->u.size.lower, show_metric_units()),
+				sizeof smin);
+            g_strlcpy(smax,
+				short_size(r->u.size.upper, show_metric_units()),
+				sizeof smax);
 			uint64_to_string_buf(r->u.size.lower, smin_64, sizeof smin_64);
 			uint64_to_string_buf(r->u.size.upper, smax_64, sizeof smax_64);
 
@@ -1195,7 +1228,7 @@ filter_new(const gchar *name)
     g_assert(utf8_is_valid_string(name));
 
     f = g_malloc0(sizeof *f);
-    f->name = g_strdup(name);
+    f->name = atom_str_get(name);
     f->ruleset = NULL;
     f->search = NULL;
     f->visited = FALSE;
@@ -1245,14 +1278,17 @@ filter_add_to_session(filter_t *f)
 void
 filter_new_for_search(search_t *s)
 {
+	const gchar *query;
     filter_t *f;
 
     g_assert(s != NULL);
-    g_assert(s->query != NULL);
-    g_assert(utf8_is_valid_string(s->query));
+
+	query = search_gui_query(s);
+    g_assert(query);
+    g_assert(utf8_is_valid_string(query));
 
     f = g_new0(filter_t, 1);
-    f->name = g_strdup(s->query);
+    f->name = atom_str_get(query);
     f->ruleset = NULL;
     f->search = NULL;
     f->visited = FALSE;
@@ -1345,7 +1381,7 @@ filter_free(filter_t *f)
 	G_LIST_FOREACH_SWAPPED(copy, filter_remove_rule, f);
     g_list_free(copy);
 
-    G_FREE_NULL(f->name);
+    atom_str_free_null(&f->name);
     G_FREE_NULL(f);
 }
 
@@ -1359,7 +1395,7 @@ filter_free_rule(rule_t *r)
 {
     g_assert(r != NULL);
 
-    if (gui_debug >= 6)
+    if (GUI_PROPERTY(gui_debug) >= 6)
         g_message("freeing rule: %s", filter_rule_to_string(r));
 
     switch (r->type) {
@@ -1408,18 +1444,19 @@ filter_free_rule(rule_t *r)
 
 
 /**
- * Append a new rule to a filter. If necessary also update the shadow.
+ * Add a new rule to a filter. If necessary also update the shadow.
  * The addition of the rule cannot be cancelled by canceling the
  * shadow. If no shadow for the filters exists, none is created.
  */
 void
-filter_append_rule(filter_t *f, rule_t * const r)
+filter_add_rule(filter_t *f, rule_t * const r, filter_add_rule_func_t func)
 {
     shadow_t *shadow;
     shadow_t *target_shadow;
 
     g_assert(f != NULL);
     g_assert(r != NULL);
+    g_assert(func);
     g_assert(r->target != NULL);
 
     shadow = shadow_find(f);
@@ -1435,9 +1472,9 @@ filter_append_rule(filter_t *f, rule_t * const r)
     /*
      * We add the rule to the filter increase the refcount on the target.
      */
-    f->ruleset = g_list_append(f->ruleset, r);
+    f->ruleset = (*func)(f->ruleset, r);
     r->target->refcount ++;
-    if (gui_debug >= 6)
+    if (GUI_PROPERTY(gui_debug) >= 6)
         g_message("increased refcount on \"%s\" to %d",
             r->target->name, r->target->refcount);
 
@@ -1445,7 +1482,7 @@ filter_append_rule(filter_t *f, rule_t * const r)
      * If a shadow for our filter exists, we add it there also.
      */
     if (shadow != NULL)
-        shadow->current = g_list_append(shadow->current, r);
+        shadow->current = (*func)(shadow->current, r);
 
     /*
      * If a shadow for the target exists, we increase refcount there too.
@@ -1453,7 +1490,7 @@ filter_append_rule(filter_t *f, rule_t * const r)
     if (target_shadow != NULL) {
         target_shadow->refcount ++;
 
-        if (gui_debug >= 6)
+        if (GUI_PROPERTY(gui_debug) >= 6)
             g_message("increased refcount on shadow of \"%s\" to %d",
                 target_shadow->filter->name, target_shadow->refcount);
     }
@@ -1472,7 +1509,23 @@ filter_append_rule(filter_t *f, rule_t * const r)
     }
 }
 
+/**
+ * Append a new rule to a filter. If necessary also update the shadow.
+ */
+void
+filter_append_rule(filter_t *f, rule_t * const r)
+{
+  filter_add_rule(f, r, g_list_append);
+}
 
+/**
+ * Prepend a new rule to a filter. If necessary also update the shadow.
+ */
+void
+filter_prepend_rule(filter_t *f, rule_t * const r)
+{
+  filter_add_rule(f, r, g_list_prepend);
+}
 
 /**
  * Append a new rule to the filter shadow. This call will fail
@@ -1489,7 +1542,7 @@ filter_append_rule_to_session(filter_t *f, rule_t * const r)
     g_assert(f != NULL);
     g_assert(r->target != NULL);
 
-    if (gui_debug >= 4)
+    if (GUI_PROPERTY(gui_debug) >= 4)
         g_message("appending rule to filter: %s <- %s (%p)",
             f->name, filter_rule_to_string(r),
 			cast_to_gconstpointer(r->target));
@@ -1528,7 +1581,7 @@ filter_append_rule_to_session(filter_t *f, rule_t * const r)
         target_shadow = shadow_new(r->target);
 
     target_shadow->refcount ++;
-    if (gui_debug >= 6)
+    if (GUI_PROPERTY(gui_debug) >= 6)
         g_message("increased refcount on shadow of \"%s\" to %d",
             target_shadow->filter->name, target_shadow->refcount);
 
@@ -1651,7 +1704,7 @@ filter_remove_rule(filter_t *f, rule_t *r)
     if (in_filter) {
         r->target->refcount --;
 
-        if (gui_debug >= 6)
+        if (GUI_PROPERTY(gui_debug) >= 6)
             g_message("decreased refcount on \"%s\" to %d",
                 r->target->name, r->target->refcount);
     }
@@ -1660,7 +1713,7 @@ filter_remove_rule(filter_t *f, rule_t *r)
         if (target_shadow != NULL) {
             target_shadow->refcount --;
 
-            if (gui_debug >= 6)
+            if (GUI_PROPERTY(gui_debug) >= 6)
                 g_message("decreased refcount on shadow of \"%s\" to %d",
                     target_shadow->filter->name, target_shadow->refcount);
         }
@@ -1704,7 +1757,7 @@ filter_remove_rule_from_session(filter_t *f, rule_t * const r)
     g_assert(r != NULL);
     g_assert(f != NULL);
 
-    if (gui_debug >= 4)
+    if (GUI_PROPERTY(gui_debug) >= 4)
         g_message("removing rule in filter: %s -> %s",
             f->name, filter_rule_to_string(r));
 
@@ -1729,7 +1782,7 @@ filter_remove_rule_from_session(filter_t *f, rule_t * const r)
         target_shadow = shadow_new(r->target);
 
     target_shadow->refcount --;
-    if (gui_debug >= 6)
+    if (GUI_PROPERTY(gui_debug) >= 6)
         g_message("decreased refcount on shadow of \"%s\" to %d",
             target_shadow->filter->name, target_shadow->refcount);
 
@@ -1740,7 +1793,7 @@ filter_remove_rule_from_session(filter_t *f, rule_t * const r)
          * not committed. We removed it from the added list
          * and free the ressources.
          */
-        if (gui_debug >= 4)
+        if (GUI_PROPERTY(gui_debug) >= 4)
             g_message("while removing from %s: removing from added: %s",
                 f->name, filter_rule_to_string(r));
         shadow->added = g_list_remove(shadow->added, r);
@@ -1752,7 +1805,7 @@ filter_remove_rule_from_session(filter_t *f, rule_t * const r)
          */
         g_assert(g_list_find(shadow->removed, r) == NULL);
 
-        if (gui_debug >= 4)
+        if (GUI_PROPERTY(gui_debug) >= 4)
             g_message("while removing from %s: adding to removed: %s",
                 f->name, filter_rule_to_string(r));
 
@@ -1804,7 +1857,7 @@ filter_replace_rule_in_session(filter_t *f,
     filter = g_list_find(shadow->current, old_rule);
     g_assert(filter != NULL);
 
-    if (gui_debug >= 4) {
+    if (GUI_PROPERTY(gui_debug) >= 4) {
         gchar f1[4096];
 		const gchar *f2;
 
@@ -1823,7 +1876,7 @@ filter_replace_rule_in_session(filter_t *f,
         target_shadow = shadow_new(old_rule->target);
 
     target_shadow->refcount --;
-    if (gui_debug >= 6)
+    if (GUI_PROPERTY(gui_debug) >= 6)
         g_message("decreased refcount on shadow of \"%s\" to %d",
             target_shadow->filter->name, target_shadow->refcount);
 
@@ -1863,7 +1916,7 @@ filter_replace_rule_in_session(filter_t *f,
         target_shadow = shadow_new(new_rule->target);
 
     target_shadow->refcount ++;
-    if (gui_debug >= 6)
+    if (GUI_PROPERTY(gui_debug) >= 6)
         g_message("increased refcount on shadow of \"%s\" to %d",
             target_shadow->filter->name, target_shadow->refcount);
 
@@ -1916,10 +1969,10 @@ filter_adapt_order(void)
     shadow_t *shadow;
     GtkCList *clist;
 
-    if (!work_filter || filter_dialog == NULL)
+    if (!work_filter || gui_filter_dialog() == NULL)
         return;
 
-    clist = GTK_CLIST(lookup_widget(filter_dialog, "clist_filter_rules"));
+    clist = GTK_CLIST(gui_filter_dialog_lookup("clist_filter_rules"));
 
     /*
      * Create a new shadow if necessary.
@@ -1955,10 +2008,10 @@ filter_adapt_order(void)
     GtkTreeView *tv;
 	GtkTreeModel *model;
 
-    if (!work_filter || filter_dialog == NULL)
+    if (!work_filter || gui_filter_dialog() == NULL)
         return;
 
-    tv = GTK_TREE_VIEW(lookup_widget(filter_dialog, "treeview_filter_rules"));
+    tv = GTK_TREE_VIEW(gui_filter_dialog_lookup("treeview_filter_rules"));
 	model = gtk_tree_view_get_model(tv);
 
     /*
@@ -1988,7 +2041,7 @@ do {																\
     (r)->match_count++;												\
     (prop_count)++;													\
     (r)->target->match_count++;										\
-    if (gui_debug >= 10)											\
+    if (GUI_PROPERTY(gui_debug) >= 10)											\
         g_message("matched rule: %s", filter_rule_to_string((r)));	\
 } while (0)
 
@@ -2031,7 +2084,7 @@ filter_apply(filter_t *filter, struct filter_context *ctx, filter_result_t *res)
 		gint i;
 
         r = list->data;
-        if (gui_debug >= 10)
+        if (GUI_PROPERTY(gui_debug) >= 10)
             g_message("trying to match against: %s", filter_rule_to_string(r));
 
         if (RULE_IS_ACTIVE(r)) {
@@ -2040,8 +2093,8 @@ filter_apply(filter_t *filter, struct filter_context *ctx, filter_result_t *res)
                 match = TRUE;
                 break;
             case RULE_TEXT: {
-				gchar *l_name = ctx->l_name;
-				gchar *utf8_name = ctx->utf8_name;
+				const gchar *l_name = ctx->l_name;
+				const gchar *utf8_name = ctx->utf8_name;
 
 				if (utf8_name == NULL) {
 					ctx->utf8_name = utf8_name = atom_str_get(rec->utf8_name);
@@ -2049,7 +2102,7 @@ filter_apply(filter_t *filter, struct filter_context *ctx, filter_result_t *res)
 				}
 
 				if (l_name == NULL) {
-					l_name = utf8_strlower_copy(utf8_name);
+					gchar *s = utf8_strlower_copy(utf8_name);
 
 					/*
 					 * Cache for further rules, to avoid costly utf8
@@ -2057,11 +2110,11 @@ filter_apply(filter_t *filter, struct filter_context *ctx, filter_result_t *res)
 					 * rule they have configured.
 					 */
 
-					ctx->l_name = atom_str_get(l_name);
+					ctx->l_name = atom_str_get(s);
 					ctx->l_len = strlen(ctx->l_name);
-
-					G_FREE_NULL(l_name);
 					l_name = ctx->l_name;
+
+					G_FREE_NULL(s);
 				}
 
                 switch (r->u.text.type) {
@@ -2360,10 +2413,8 @@ filter_record(search_t *sch, const struct record *rec)
 	 * Cleanup cached variables.
 	 */
 
-	if (ctx.utf8_name != NULL)
-		atom_str_free(ctx.utf8_name);
-	if (ctx.l_name != NULL)
-		atom_str_free(ctx.l_name);
+	atom_str_free_null(&ctx.utf8_name);
+	atom_str_free_null(&ctx.l_name);
 
 	return result;
 }
@@ -2378,7 +2429,7 @@ filter_shutdown(void)
 {
     GList *f;
 
-    if (gui_debug >= 5)
+    if (GUI_PROPERTY(gui_debug) >= 5)
         g_message("shutting down filters");
 
     /*
@@ -2445,7 +2496,7 @@ filter_init(void)
 
     filters_current = g_list_copy(filters);
 
-    popup_filter_rule = create_popup_filter_rule();
+    gui_popup_filter_rule_set(create_popup_filter_rule());
 }
 
 
@@ -2666,9 +2717,10 @@ filter_add_drop_sha1_rule(const struct record *rec, filter_t *filter)
     rule = filter_new_sha1_rule(rec->sha1, s,
         filter_get_drop_target(), RULE_FLAG_ACTIVE);
 
-    filter_append_rule(filter, rule);
-	if (s != rec->name)
+    filter_prepend_rule(filter, rule);
+	if (s != rec->name) {
 		G_FREE_NULL(s);
+	}
 }
 
 /**
@@ -2689,9 +2741,10 @@ filter_add_drop_name_rule(const struct record *rec, filter_t *filter)
     rule = filter_new_text_rule(s, RULE_TEXT_EXACT, TRUE,
         filter_get_drop_target(), RULE_FLAG_ACTIVE);
 
-    filter_append_rule(filter, rule);
-	if (s != rec->name)
+    filter_prepend_rule(filter, rule);
+	if (s != rec->name) {
 		G_FREE_NULL(s);
+	}
 }
 
 /**
@@ -2710,7 +2763,7 @@ filter_add_drop_host_rule(const struct record *rec, filter_t *filter)
     rule = filter_new_ip_rule(rec->results_set->addr, -1,
         		filter_get_drop_target(), RULE_FLAG_ACTIVE);
 
-    filter_append_rule(filter, rule);
+    filter_prepend_rule(filter, rule);
 }
 
 /**
@@ -2733,8 +2786,9 @@ filter_add_download_sha1_rule(const struct record *rec, filter_t *filter)
             filter_get_download_target(), RULE_FLAG_ACTIVE);
 
         filter_append_rule(filter, rule);
-		if (s != rec->name)
+		if (s != rec->name) {
 			G_FREE_NULL(s);
+		}
     }
 }
 
@@ -2757,8 +2811,9 @@ filter_add_download_name_rule(const struct record *rec, filter_t *filter)
         filter_get_download_target(), RULE_FLAG_ACTIVE);
 
     filter_append_rule(filter, rule);
-	if (s != rec->name)
+	if (s != rec->name) {
 		G_FREE_NULL(s);
+	}
 }
 
 /* vi: set ts=4 sw=4 cindent: */
