@@ -1,5 +1,5 @@
 /*
- * $Id: downloads.c 12242 2006-11-06 20:41:55Z cbiere $
+ * $Id: downloads.c 13930 2007-06-22 23:03:25Z cbiere $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi, Richard Eckart
  *
@@ -25,7 +25,7 @@
 
 #include "gtk/gui.h"
 
-RCSID("$Id: downloads.c 12242 2006-11-06 20:41:55Z cbiere $")
+RCSID("$Id: downloads.c 13930 2007-06-22 23:03:25Z cbiere $")
 
 #include "gtk/downloads.h"
 #include "gtk/downloads_common.h"
@@ -71,9 +71,9 @@ static GHashTable *parents_gui_time;	/**< Time at which parent was updated */
 static GHashTable *parents_children;
 static GHashTable *parents_queue_children;
 
-static GtkCTree *ctree_downloads = NULL;
-static GtkCTree *ctree_downloads_queue = NULL;
-static GtkNotebook *notebook = NULL;
+static GtkCTree *ctree_downloads;
+static GtkCTree *ctree_downloads_queue;
+static GtkNotebook *notebook;
 
 static gboolean ctree_downloads_frozen = FALSE;
 static gboolean ctree_downloads_queue_frozen = FALSE;
@@ -467,7 +467,8 @@ void gui_update_download_hostcount(struct download *d)
 /**
  * Initialize local data structures.
  */
-void downloads_gui_init(void)
+void
+downloads_gui_init(void)
 {
     GtkCList *clist_queue;
     GtkCList *clist;
@@ -986,6 +987,7 @@ gui_update_download(struct download *d, gboolean force)
 		switch (d->status) {
 		case GTA_DL_ACTIVE_QUEUED:
 		case GTA_DL_RECEIVING:
+		case GTA_DL_IGNORING:
 		case GTA_DL_HEADERS:
 		case GTA_DL_PUSH_SENT:
 		case GTA_DL_CONNECTING:
@@ -1011,8 +1013,12 @@ gui_update_download(struct download *d, gboolean force)
 	switch (d->status) {
 	case GTA_DL_ACTIVE_QUEUED:	/* JA, 31 jan 2003 Active queueing */
 		{
-			gint elapsed = delta_time(now, d->last_update);
-
+			time_delta_t elapsed;
+			
+			elapsed = delta_time(now, d->last_update);
+			elapsed = MAX(0, elapsed);
+			elapsed = MIN(elapsed, INT_MAX);
+			
 			rw = gm_snprintf(tmpstr, sizeof(tmpstr), _("Queued"));
 
 			if (guc_get_parq_dl_position(d) > 0) {
@@ -1042,7 +1048,10 @@ gui_update_download(struct download *d, gboolean force)
 
 			if (
 				parent_gui_needs_update(d, now) &&
-				!downloads_gui_any_status(d, GTA_DL_RECEIVING)
+				(
+					!downloads_gui_any_status(d, GTA_DL_RECEIVING) &&
+					!downloads_gui_any_status(d, GTA_DL_IGNORING)
+				)
 			)
 				downloads_gui_update_parent_status(d, now, _("Queued"));
 		}
@@ -1165,7 +1174,7 @@ gui_update_download(struct download *d, gboolean force)
 
 	case GTA_DL_COMPLETED:
 		if (d->last_update != d->start_date) {
-			guint32 t = delta_time(d->last_update, d->start_date);
+			time_delta_t t = delta_time(d->last_update, d->start_date);
 			
 			gm_snprintf(tmpstr, sizeof(tmpstr), "%s (%s) %s",
 				FILE_INFO_COMPLETE(fi) ? _("Completed") : _("Chunk done"),
@@ -1246,14 +1255,13 @@ gui_update_download(struct download *d, gboolean force)
 		break;
 
 	case GTA_DL_RECEIVING:
+	case GTA_DL_IGNORING:
 		if (d->pos + download_buffered(d) > d->skip) {
-			gdouble p = 0;
 			gint bps;
 			guint32 avg_bps;
 			filesize_t downloaded;
 
 			downloaded = d->pos - d->skip + download_buffered(d);
-			p = guc_download_source_progress(d);
 
 			if (d->bio) {
 				bps = bio_bps(d->bio);
@@ -1277,7 +1285,7 @@ gui_update_download(struct download *d, gboolean force)
 
                 s = remain / avg_bps;
 
-				if (now - d->last_update > IO_STALLED)
+				if (delta_time(now, d->last_update) > IO_STALLED)
 					rw += gm_snprintf(&tmpstr[rw], sizeof(tmpstr)-rw, " %s",
 						_("(stalled)"));
 				else
@@ -1301,8 +1309,10 @@ gui_update_download(struct download *d, gboolean force)
 					}
 				}
 			} else {
-				rw = gm_snprintf(tmpstr, sizeof(tmpstr), "%4.02f%% %s", p,
-					(now - d->last_update > IO_STALLED) ? _("(stalled)") : "");
+				rw = gm_snprintf(tmpstr, sizeof(tmpstr), "%4.02f%% %s",
+					  guc_download_source_progress(d),
+					  delta_time(now, d->last_update) > IO_STALLED
+					  	? _("(stalled)") : "");
 			}
 
 			/*
@@ -1310,7 +1320,7 @@ gui_update_download(struct download *d, gboolean force)
 			 */
 
 			if (d->ranges != NULL)
-				gm_snprintf(&tmpstr[rw], sizeof(tmpstr)-rw,
+				rw += gm_snprintf(&tmpstr[rw], sizeof(tmpstr)-rw,
 					" <PFS %4.02f%%>", d->ranges_size * 100.0 / fi->size);
 
 			/*
@@ -1319,9 +1329,13 @@ gui_update_download(struct download *d, gboolean force)
 			 */
 
 			if (d->served_reqs)
-				gm_snprintf(&tmpstr[rw], sizeof(tmpstr)-rw,
+				rw += gm_snprintf(&tmpstr[rw], sizeof(tmpstr)-rw,
 					" #%u", d->served_reqs + 1);
 
+			if (GTA_DL_IGNORING == d->status)
+				rw += gm_snprintf(&tmpstr[rw], sizeof(tmpstr)-rw,
+					" (%s)", _("ignoring"));
+
 			a = tmpstr;
 		} else
 			a = _("Connected");
@@ -1333,9 +1347,13 @@ gui_update_download(struct download *d, gboolean force)
 
 	case GTA_DL_TIMEOUT_WAIT:
 		{
-			gint when = d->timeout_delay - (now - d->last_update);
-			gm_snprintf(tmpstr, sizeof(tmpstr),
-				_("Retry in %ds"), MAX(0, when));
+			time_delta_t when;
+			
+			when = d->timeout_delay - delta_time(now, d->last_update);
+			when = MAX(0, when);
+			when = MIN(when, INT_MAX);
+			gm_snprintf(tmpstr, sizeof(tmpstr), _("Retry in %us"),
+				(unsigned) when);
 		}
 		a = tmpstr;
 		break;
@@ -1432,7 +1450,10 @@ gui_update_download(struct download *d, gboolean force)
 							c_dl_status, a);
 						record_parent_gui_update(key, now);
 
-					} else if (GTA_DL_RECEIVING == d->status) {
+					} else if (
+						GTA_DL_RECEIVING == d->status ||
+						GTA_DL_IGNORING == d->status
+					) {
 						guint32 s;
 
 						active_src = fi->recvcount;
@@ -1482,19 +1503,25 @@ gui_update_download_abort_resume(void)
     gboolean do_queue  = FALSE;
     gboolean abort_sha1 = FALSE;
 
+	/*
+	 * settings_gui_init() triggers this function before downloads_gui_init()
+	 * has run.
+	 */
+	if (NULL == ctree_downloads)
+		return;
+
     node_list = g_list_copy(GTK_CLIST(ctree_downloads)->selection);
 	data_list = downloads_gui_collect_ctree_data(ctree_downloads,
 		node_list, FALSE, TRUE);
 
     for (l = data_list; NULL != l; l = g_list_next(l)) {
-		d = (struct download *) l->data;
-
-        if (!d) {
-			g_warning
-				("gui_update_download_abort_resume(): row has NULL data\n");
+		d = l->data;
+        if (NULL == d) {
+			g_warning("gui_update_download_abort_resume(): row has NULL data");
 			continue;
 		}
 
+		download_check(d);
 		g_assert(d->status != GTA_DL_REMOVED);
 
 		switch (d->status) {
@@ -1523,6 +1550,7 @@ gui_update_download_abort_resume(void)
 		case GTA_DL_REQ_SENT:
 		case GTA_DL_HEADERS:
 		case GTA_DL_RECEIVING:
+		case GTA_DL_IGNORING:
 		case GTA_DL_ACTIVE_QUEUED:
 		case GTA_DL_SINKING:
 			do_abort = TRUE;
