@@ -1,5 +1,5 @@
 /*
- * $Id: downloads_cb.c 12195 2006-11-03 02:20:22Z cbiere $
+ * $Id: downloads_cb.c 13855 2007-06-17 00:21:31Z cbiere $
  *
  * Copyright (c) 2001-2003, Richard Eckart
  *
@@ -23,10 +23,11 @@
  *----------------------------------------------------------------------
  */
 
+#include "common.h"
+
+RCSID("$Id: downloads_cb.c 13855 2007-06-17 00:21:31Z cbiere $")
+
 #include "gtk/gui.h"
-
-RCSID("$Id: downloads_cb.c 12195 2006-11-03 02:20:22Z cbiere $")
-
 #include "gtk/downloads.h"
 #include "gtk/downloads_common.h"
 #include "gtk/statusbar.h"
@@ -116,7 +117,7 @@ on_ctree_downloads_resize_column(GtkCList *unused_clist,
 {
 	(void) unused_clist;
 	(void) unused_udata;
-	*(gint *) &dl_active_col_widths[column] = width;
+	*(gint *) &GUI_PROPERTY(dl_active_col_widths)[column] = width;
 }
 
 gboolean
@@ -550,13 +551,10 @@ on_popup_downloads_copy_url_activate(GtkMenuItem *unused_menuitem,
         /*
          * if "copy url" is done repeatedly, we have to make sure the
          * memory of the previous selection is freed, because we may not
-         * recieve a "selection_clear" signal.
+         * receive a "selection_clear" signal.
          *      --BLUE, 24/04/2002
          */
-        if (selected_url != NULL) {
-            G_FREE_NULL(selected_url);
-        }
-
+		G_FREE_NULL(selected_url);
         selected_url = guc_download_build_url(d);
     }
 }
@@ -820,13 +818,10 @@ on_popup_queue_copy_url_activate(GtkMenuItem *unused_menuitem,
         /*
          * if "copy url" is done repeatedly, we have to make sure the
          * memory of the previous selection is freed, because we may not
-         * recieve a "selection_clear" signal.
+         * receive a "selection_clear" signal.
          *      --BLUE, 24/04/2002
          */
-        if (selected_url != NULL) {
-            G_FREE_NULL(selected_url);
-        }
-
+		G_FREE_NULL(selected_url);
         selected_url = guc_download_build_url(d);
     }
 }
@@ -1003,21 +998,17 @@ on_entry_queue_regex_activate(GtkEditable *editable, gpointer unused_udata)
 	GtkCTreeRow *row;
 	struct download *d, *dtemp;
 	gboolean child_selected, node_expanded;
-    gint i;
-  	gint n;
-    gint m = 0;
-    gint  err;
-    gchar * regex;
+    gint i, n, m = 0, err, flags;
+    gchar *regex;
 	regex_t re;
 
 	(void) unused_udata;
     regex = STRTRACK(gtk_editable_get_chars(GTK_EDITABLE(editable), 0, -1));
 	g_return_if_fail(regex != NULL);
 
-    err = regcomp(&re,
-                  regex,
-                  REG_EXTENDED|REG_NOSUB|(queue_regex_case ? 0 : REG_ICASE));
-
+	flags = REG_EXTENDED | REG_NOSUB;
+	flags |= GUI_PROPERTY(queue_regex_case) ? 0 : REG_ICASE;
+    err = regcomp(&re, regex, flags);
    	if (err) {
         char buf[1024];
 		regerror(err, &re, buf, sizeof buf);
@@ -1076,7 +1067,7 @@ on_entry_queue_regex_activate(GtkEditable *editable, gpointer unused_udata)
 
 					n = regexec(&re, dtemp->file_name, 0, NULL, 0);
 					if (0 != n) {
-						n = regexec(&re, download_outname(dtemp), 0, NULL, 0);
+						n = regexec(&re, download_basename(dtemp), 0, NULL, 0);
 					}
 					if (0 == n) {
 						gtk_ctree_select(ctree_downloads_queue(), child);
@@ -1098,7 +1089,7 @@ on_entry_queue_regex_activate(GtkEditable *editable, gpointer unused_udata)
 			/* Not a header entry */
             if (
 				(n = regexec(&re, d->file_name, 0, NULL, 0)) == 0 ||
-				(n = regexec(&re, download_outname(d), 0, NULL, 0)) == 0
+				(n = regexec(&re, download_basename(d), 0, NULL, 0)) == 0
 			) {
                 gtk_ctree_select(ctree_downloads_queue(), node);
                 m ++;
@@ -1181,7 +1172,7 @@ on_ctree_downloads_queue_resize_column(GtkCList *unused_clist,
 {
 	(void) unused_clist;
 	(void) unused_udata;
-	*(gint *) &dl_queued_col_widths[column] = width;
+	*(gint *) &GUI_PROPERTY(dl_queued_col_widths)[column] = width;
 }
 
 void
@@ -1282,9 +1273,7 @@ on_popup_downloads_selection_clear_event(GtkWidget *unused_widget,
 {
 	(void) unused_widget;
 	(void) unused_event;
-    if (selected_url != NULL) {
-        G_FREE_NULL(selected_url);
-    }
+	G_FREE_NULL(selected_url);
     return TRUE;
 }
 
