@@ -1,5 +1,5 @@
 /*
- * $Id: fileinfo.c,v 1.23 2006/02/06 14:35:54 cbiere Exp $
+ * $Id: fileinfo.c 13971 2007-06-24 22:25:27Z cbiere $
  *
  * Copyright (c) 2003, Richard Eckart
  *
@@ -35,13 +35,15 @@
 
 #include "gtk/gui.h"
 
-RCSID("$Id: fileinfo.c,v 1.23 2006/02/06 14:35:54 cbiere Exp $");
+RCSID("$Id: fileinfo.c 13971 2007-06-24 22:25:27Z cbiere $")
 
+#include "gtk/columns.h"
+#include "gtk/drag.h"
 #include "gtk/filter.h"
+#include "gtk/gtk-missing.h"
+#include "gtk/settings.h"
 #include "gtk/statusbar.h"
 #include "gtk/visual_progress.h"
-#include "gtk/columns.h"
-#include "gtk/gtk-missing.h"
 
 #include "if/gui_property_priv.h"
 #include "if/bridge/ui2c.h"
@@ -52,109 +54,24 @@ RCSID("$Id: fileinfo.c,v 1.23 2006/02/06 14:35:54 cbiere Exp $");
 
 static gnet_fi_t last_shown = 0;
 static gboolean  last_shown_valid = FALSE;
-static GHashTable *fi_updates = NULL;
+static GHashTable *fi_updates;
 
 /*
  * Together visible_fi and hidden_fi are a list of all fileinfo handles
  * the the gui knows about.
  */
-static GSList *visible_fi = NULL;
-static GSList *hidden_fi  = NULL;
+static GSList *visible_fi;
+static GSList *hidden_fi;
 
 static regex_t filter_re;
 
-static GtkCList *clist_fileinfo = NULL;		/* Cached lookup_widget() */
+static GtkCList *clist_fileinfo;		/* Cached lookup_widget() */
 
-static void
-drag_begin(GtkWidget *unused_widget, GdkDragContext *unused_drag_ctx,
-	gpointer udata)
+static gchar *
+fi_gui_get_file_url(GtkWidget *unused_widget)
 {
-	gchar **url_ptr = udata;
-
 	(void) unused_widget;
-	(void) unused_drag_ctx;
-
-	g_assert(url_ptr != NULL);
-	G_FREE_NULL(*url_ptr);
-	
-	if (last_shown_valid) {
-		gnet_fi_info_t *fi = NULL;
-    	gnet_fi_status_t fis;
-
-    	guc_fi_get_status(last_shown, &fis);
-
-		/* Allow partials but not unstarted files */
-		if (fis.done > 0) {
-			const gchar *path;
-			gchar *save_path = NULL;
-
-			fi = guc_fi_get_info(last_shown);
-			g_assert(fi != NULL);
-			if (fis.done < fis.size) {
-				path = fi->path;
-			} else {
-				/* XXX: This is a hack since the final destination might
-				 *		might be different e.g., due to a filename clash
-				 *		or because the PROP_MOVE_FILE_PATH changed in the
-				 *		meantime. */
-				save_path = gnet_prop_get_string(PROP_MOVE_FILE_PATH, NULL, 0);
-				path = save_path;
-			}
-
-			if (path && fi->file_name) {
-				gchar *escaped;
-				gchar *pathname;
-
-				pathname = make_pathname(path, fi->file_name);
-				escaped = url_escape(pathname);
-				if (escaped != pathname)
-					G_FREE_NULL(pathname);
-
-				*url_ptr = g_strconcat("file://", escaped, (void *) 0);
-
-				G_FREE_NULL(escaped);
-			}
-			G_FREE_NULL(save_path);
-
-    		guc_fi_free_info(fi);
-		}
-	}
-}
-
-static void
-drag_data_get(GtkWidget *unused_widget, GdkDragContext *unused_drag_ctx,
-	GtkSelectionData *data, guint unused_info, guint unused_stamp,
-	gpointer udata)
-{
-	gchar **url_ptr = udata;
-
-	(void) unused_widget;
-	(void) unused_drag_ctx;
-	(void) unused_info;
-	(void) unused_stamp;
-
-	g_assert(url_ptr != NULL);
-	if (*url_ptr) {
-		const gchar *drag_data = *url_ptr;
-		
-    	gtk_selection_data_set(data, GDK_SELECTION_TYPE_STRING,
-			8 /* CHAR_BIT */, cast_to_gconstpointer(drag_data),
-			strlen(drag_data));
-		G_FREE_NULL(*url_ptr);
-	}
-}
-
-static void
-drag_end(GtkWidget *unused_widget, GdkDragContext *unused_drag_ctx,
-	gpointer udata)
-{
-	gchar **url_ptr = udata;
-
-	(void) unused_widget;
-	(void) unused_drag_ctx;
-
-	g_assert(url_ptr != NULL);
-	G_FREE_NULL(*url_ptr);
+	return last_shown_valid ? guc_file_info_get_file_url(last_shown) : NULL;
 }
 
 void
@@ -163,7 +80,7 @@ on_clist_fileinfo_resize_column(GtkCList *unused_clist,
 {
 	(void) unused_clist;
 	(void) unused_udata;
-    *(gint *) &file_info_col_widths[column] = width;
+    *(gint *) &GUI_PROPERTY(file_info_col_widths)[column] = width;
 }
 
 /* Cache for fi_gui_fill_info. This is global so it can be freed
@@ -179,7 +96,7 @@ static gnet_fi_info_t *last_fi = NULL;
  * filled from the given `fih'.
  */
 static gnet_fi_info_t *
-fi_gui_fill_info(gnet_fi_t fih, gchar *titles[c_fi_num])
+fi_gui_fill_info(gnet_fi_t fih, const gchar *titles[c_fi_num])
 {
     /* Clear info from last call. We keep this around so we don't
      * have to strdup entries from it when passing them to the
@@ -199,31 +116,43 @@ fi_gui_fill_info(gnet_fi_t fih, gchar *titles[c_fi_num])
 
 /* XXX -- factorize this code with GTK2's one */
 static void
-fi_gui_fill_status(gnet_fi_t fih, gchar *titles[c_fi_num])
+fi_gui_fill_status(gnet_fi_t fih, const gchar *titles[c_fi_num])
 {
     static gchar fi_sources[32];
     static gchar fi_status[256];
     static gchar fi_done[SIZE_FIELD_MAX+10];
     static gchar fi_size[SIZE_FIELD_MAX];
+    static gchar fi_uploaded[SIZE_FIELD_MAX];
     gnet_fi_status_t s;
 
     guc_fi_get_status(fih, &s);
 
     gm_snprintf(fi_sources, sizeof(fi_sources), "%d/%d/%d",
-        s.recvcount, s.aqueued_count+s.pqueued_count, s.lifecount);
+        s.recvcount, s.aqueued_count + s.pqueued_count, s.lifecount);
     titles[c_fi_sources] = fi_sources;
 
     if (s.done) {
+		gdouble d;
+
+		d = s.size > 0 ? ((gdouble) s.done / s.size) * 100.0 : 100.0;
         gm_snprintf(fi_done, sizeof(fi_done), "%s (%.1f%%)",
-            short_size(s.done), ((float) s.done / s.size)*100.0);
+            short_size(s.done, show_metric_units()), d);
         titles[c_fi_done] = fi_done;
     } else {
         titles[c_fi_done] = "-";
     }
 
-    g_strlcpy(fi_size, short_size(s.size), sizeof(fi_size));
-    titles[c_fi_size]    = fi_size;
+    g_strlcpy(fi_size, short_size(s.size, show_metric_units()), sizeof fi_size);
+    titles[c_fi_size] = fi_size;
 
+    if (s.uploaded) {
+    	g_strlcpy(fi_uploaded, short_size(s.uploaded, show_metric_units()),
+			sizeof fi_uploaded);
+        titles[c_fi_uploaded] = fi_uploaded;
+    } else {
+        titles[c_fi_uploaded] = "-";
+    }
+	
     if (s.recvcount) {
 		guint32 secs = 0;
 
@@ -232,7 +161,7 @@ fi_gui_fill_status(gnet_fi_t fih, gchar *titles[c_fi_num])
 
         gm_snprintf(fi_status, sizeof(fi_status),
             _("Downloading (%s)  TR: %s"),
-			short_rate(s.recv_last_rate),
+			short_rate(s.recv_last_rate, show_metric_units()),
 			secs ? short_time(secs) : "-");
 
         titles[c_fi_status] = fi_status;
@@ -240,7 +169,7 @@ fi_gui_fill_status(gnet_fi_t fih, gchar *titles[c_fi_num])
 		gint rw;
 
 		rw = gm_snprintf(fi_status, sizeof(fi_status),
-				"%s", _("Finished"));
+				"%s", s.seeding ? _("Seeding") : _("Finished"));
 
 		if (s.has_sha1) {
 			if (s.sha1_hashed == s.size)
@@ -252,15 +181,15 @@ fi_gui_fill_status(gnet_fi_t fih, gchar *titles[c_fi_num])
 			else
 				rw += gm_snprintf(&fi_status[rw], sizeof(fi_status)-rw,
 						"; %s %s (%.1f%%)", _("Computing SHA1"),
-						short_size(s.sha1_hashed),
-						((float) s.sha1_hashed / s.size) * 100.0);
+						short_size(s.sha1_hashed, show_metric_units()),
+						((gdouble) s.sha1_hashed / s.size) * 100.0);
 		}
 
 		if (s.copied > 0 && s.copied < s.size) 
 			rw += gm_snprintf(&fi_status[rw], sizeof(fi_status)-rw,
 					"; %s %s (%.1f%%)", _("Moving"),
-					short_size(s.copied),
-					((float) s.copied / s.size) * 100.0);
+					short_size(s.copied, show_metric_units()),
+					((gdouble) s.copied / s.size) * 100.0);
 
         titles[c_fi_status] = fi_status;
     } else if (s.lifecount == 0) {
@@ -283,7 +212,7 @@ fi_gui_fill_status(gnet_fi_t fih, gchar *titles[c_fi_num])
 static void
 fi_gui_set_details(gnet_fi_t fih)
 {
-    gnet_fi_info_t *fi = NULL;
+    gnet_fi_info_t *fi;
     gnet_fi_status_t fis;
     gchar **aliases;
     guint n;
@@ -296,15 +225,20 @@ fi_gui_set_details(gnet_fi_t fih)
     guc_fi_get_status(fih, &fis);
     aliases = guc_fi_get_aliases(fih);
 
-    cl_aliases = GTK_CLIST(lookup_widget(main_window, "clist_fi_aliases"));
+    cl_aliases = GTK_CLIST(gui_main_window_lookup("clist_fi_aliases"));
 
-    gtk_label_set_text(
-        GTK_LABEL(lookup_widget(main_window, "label_fi_filename")),
+    gtk_entry_printf(
+		GTK_ENTRY(gui_main_window_lookup("entry_fi_sha1")),
+		"%s%s",
+		fi->sha1 ? "urn:sha1:" : _("<none>"),
+		fi->sha1 ? sha1_base32(fi->sha1) : "");
+    gtk_entry_set_text(
+        GTK_ENTRY(gui_main_window_lookup("entry_fi_filename")),
         fi->file_name);
 	uint64_to_string_buf(fis.size, bytes, sizeof bytes);
-    gtk_label_printf(
-        GTK_LABEL(lookup_widget(main_window, "label_fi_size")),
-        _("%s (%s bytes)"), short_size(fis.size), bytes);
+    gtk_entry_printf(
+        GTK_ENTRY(gui_main_window_lookup("entry_fi_size")),
+        _("%s (%s bytes)"), short_size(fis.size, show_metric_units()), bytes);
 
     gtk_clist_freeze(cl_aliases);
     gtk_clist_clear(cl_aliases);
@@ -320,7 +254,7 @@ fi_gui_set_details(gnet_fi_t fih)
 
 	vp_draw_fi_progress(last_shown_valid, last_shown);
 
-	gtk_widget_set_sensitive(lookup_widget(main_window, "button_fi_purge"),
+	gtk_widget_set_sensitive(gui_main_window_lookup("button_fi_purge"),
 							 TRUE);
 }
 
@@ -330,17 +264,23 @@ fi_gui_set_details(gnet_fi_t fih)
 static void
 fi_gui_clear_details(void)
 {
+	static const gchar *widgets[] = {
+		"entry_fi_filename",
+		"entry_fi_sha1",
+		"entry_fi_size",
+	};
+	guint i;
+		
     last_shown_valid = FALSE;
 
-    gtk_label_set_text(
-        GTK_LABEL(lookup_widget(main_window, "label_fi_filename")),
-        "");
-    gtk_label_set_text(
-        GTK_LABEL(lookup_widget(main_window, "label_fi_size")),
-        "");
+	for (i = 0; i < G_N_ELEMENTS(widgets); i++) {
+		GtkEntry *entry = GTK_ENTRY(gui_main_window_lookup(widgets[i]));
+		gtk_entry_set_text(entry, "");
+	}
+
     gtk_clist_clear(
-        GTK_CLIST(lookup_widget(main_window, "clist_fi_aliases")));
-    gtk_widget_set_sensitive(lookup_widget(main_window, "button_fi_purge"),
+        GTK_CLIST(gui_main_window_lookup("clist_fi_aliases")));
+    gtk_widget_set_sensitive(gui_main_window_lookup("button_fi_purge"),
         FALSE);
 
     vp_draw_fi_progress(last_shown_valid, last_shown);
@@ -374,9 +314,9 @@ static void
 fi_gui_add_row(gnet_fi_t fih)
 {
     GtkCList *clist = clist_fileinfo;
+    const gchar *titles[c_fi_num];
     gint row;
     guint n;
-    gchar *titles[c_fi_num];
 	gnet_fi_info_t *info;
 	gboolean filter_match;
 	GSList *l;
@@ -414,7 +354,7 @@ fi_gui_add_row(gnet_fi_t fih)
             titles[n] = "";
     }
 
-    row = gtk_clist_append(clist, titles);
+    row = gtk_clist_append(clist, deconstify_gpointer(titles));
     gtk_clist_set_row_data(clist, row, GUINT_TO_POINTER(fih));
 }
 
@@ -448,21 +388,19 @@ fi_gui_remove_row(gnet_fi_t fih, gboolean hide)
 static void
 fi_gui_set_filter_regex(gchar *s)
 {
-    gint err;
-    GSList *sl;
-    gint row;
-    GSList *old_hidden = g_slist_copy(hidden_fi);
+    GSList *sl, *old_hidden = g_slist_copy(hidden_fi);
     GtkCList *clist_fi;
     char *fallback_re = ".";
+    gint err, flags, row;
 
     if (s == NULL) {
         s = fallback_re;
     }
 
     /* Recompile the row filter*/
-    err = regcomp(&filter_re, s,
-                  REG_EXTENDED|REG_NOSUB|(fi_regex_case ? 0 : REG_ICASE));
-
+	flags = REG_EXTENDED | REG_NOSUB;
+	flags |= GUI_PROPERTY(fi_regex_case) ? 0 : REG_ICASE;
+    err = regcomp(&filter_re, s, flags);
    	if (err) {
         gchar buf[1024];
 		regerror(err, &filter_re, buf, sizeof buf);
@@ -513,7 +451,7 @@ static void
 fi_gui_update(gnet_fi_t fih, gboolean full)
 {
     GtkCList *clist = clist_fileinfo;
-	gchar    *titles[c_fi_num];
+	const gchar    *titles[c_fi_num];
     gint      row;
     guint     n;
 
@@ -623,20 +561,22 @@ on_button_fi_purge_clicked(GtkButton *unused_button, gpointer unused_udata)
 
 	(void) unused_button;
 	(void) unused_udata;
+	
+    gtk_clist_freeze(clist);
     sl_handles = clist_collect_data(clist, TRUE, NULL);
     if (sl_handles) {
 		GSList *sl;
 
-		for (sl = sl_handles; sl != NULL; sl = g_slist_next(sl))
+		for (sl = sl_handles; sl != NULL; sl = g_slist_next(sl)) {
 			if (GPOINTER_TO_UINT(sl->data) == last_shown) {
 				last_shown_valid = FALSE;
 				break;
 			}
-
+		}
         guc_fi_purge_by_handle_list(sl_handles);
     }
-
     g_slist_free(sl_handles);
+    gtk_clist_thaw(clist);
 }
 
 void
@@ -655,12 +595,6 @@ on_entry_fi_regex_activate(GtkEditable *editable, gpointer unused_udata)
 void
 fi_gui_init(void)
 {
-    static const GtkTargetEntry targets[] = {
-        { "STRING", 0, 23 },
-        { "text/plain", 0, 23 },
-    };
-	static gchar *dnd_url; /* Holds the URL to set the drag data */
-
 	fi_updates = g_hash_table_new(NULL, NULL);
 
     guc_fi_add_listener(fi_gui_fi_added, EV_FI_ADDED, FREQ_SECS, 0);
@@ -670,27 +604,19 @@ fi_gui_init(void)
     guc_fi_add_listener(fi_gui_fi_status_changed_transient,
 		EV_FI_STATUS_CHANGED_TRANSIENT, FREQ_SECS, 0);
 
-    clist_fileinfo = GTK_CLIST(lookup_widget(main_window, "clist_fileinfo"));
+    clist_fileinfo = GTK_CLIST(gui_main_window_lookup("clist_fileinfo"));
 
     gtk_clist_set_column_justification(clist_fileinfo,
         c_fi_size, GTK_JUSTIFY_RIGHT);
+    gtk_clist_set_column_justification(clist_fileinfo,
+        c_fi_uploaded, GTK_JUSTIFY_RIGHT);
 
-    gtk_clist_column_titles_passive(clist_fileinfo);
+    gtk_clist_column_titles_active(clist_fileinfo);
 
     /* Initialize the row filter */
     fi_gui_set_filter_regex(NULL);
-	
-	/* Initialize drag support */
-	gtk_drag_source_set(GTK_WIDGET(clist_fileinfo),
-		GDK_BUTTON1_MASK | GDK_BUTTON2_MASK, targets, G_N_ELEMENTS(targets),
-		GDK_ACTION_DEFAULT | GDK_ACTION_COPY | GDK_ACTION_ASK);
 
-    gtk_signal_connect(GTK_OBJECT(clist_fileinfo), "drag-data-get",
-        drag_data_get, &dnd_url);
-    gtk_signal_connect(GTK_OBJECT(clist_fileinfo), "drag-begin",
-        drag_begin, &dnd_url);
-    gtk_signal_connect(GTK_OBJECT(clist_fileinfo), "drag-end",
-        drag_end, &dnd_url);
+	drag_attach(GTK_WIDGET(clist_fileinfo), fi_gui_get_file_url);
 }
 
 void
@@ -757,4 +683,177 @@ fi_gui_update_display(time_t now)
 #endif
 }
 
+static inline guint
+fi_gui_relative_done(const gnet_fi_status_t *s, gboolean percent)
+{
+	if (percent) {
+		return filesize_per_100(s->size, s->done);
+	} else {
+		return filesize_per_1000(s->size, s->done);
+	}
+}
+
+static inline guint
+fi_gui_numeric_status(const gnet_fi_status_t *s)
+{
+	guint v;
+
+	v = fi_gui_relative_done(s, TRUE);
+	v |= (s->lifecount > 0)						? (1 <<  7) : 0;
+	v |= (s->aqueued_count || s->pqueued_count)	? (1 <<  8) : 0;
+	v |= (s->recvcount > 0)						? (1 <<  9) : 0;
+	v |= (s->size > 0 && s->size == s->done)	? (1 << 10) : 0;
+
+	return v;
+}
+
+static gint 
+fi_gui_cmp_filename(GtkCList *unused_clist,
+	gconstpointer ptr1, gconstpointer ptr2)
+{
+    gnet_fi_info_t *a_fi, *b_fi;
+    gnet_fi_t a, b;
+	gint r;
+   
+	(void) unused_clist;
+	a = GPOINTER_TO_UINT(((const GtkCListRow *) ptr1)->data);
+    b = GPOINTER_TO_UINT(((const GtkCListRow *) ptr2)->data);
+
+	a_fi = guc_fi_get_info(a);
+	b_fi = guc_fi_get_info(b);
+	r = strcmp(a_fi->file_name, b_fi->file_name);
+	guc_fi_free_info(b_fi);
+	guc_fi_free_info(a_fi);
+
+	return r;
+}
+
+static gint 
+fi_gui_cmp_size(GtkCList *unused_clist,
+	gconstpointer ptr1, gconstpointer ptr2)
+{
+    gnet_fi_status_t a, b;
+    gnet_fi_t fi_a, fi_b;
+
+	(void) unused_clist;
+	fi_a = GPOINTER_TO_UINT(((const GtkCListRow *) ptr1)->data);
+    fi_b = GPOINTER_TO_UINT(((const GtkCListRow *) ptr2)->data);
+
+    guc_fi_get_status(fi_a, &a);
+    guc_fi_get_status(fi_b, &b);
+
+	return CMP(a.size, b.size);
+}
+
+static gint 
+fi_gui_cmp_done(GtkCList *unused_clist,
+	gconstpointer ptr1, gconstpointer ptr2)
+{
+    gnet_fi_status_t a, b;
+    gnet_fi_t fi_a, fi_b;
+	gint ret;
+
+	(void) unused_clist;
+	fi_a = GPOINTER_TO_UINT(((const GtkCListRow *) ptr1)->data);
+    fi_b = GPOINTER_TO_UINT(((const GtkCListRow *) ptr2)->data);
+
+    guc_fi_get_status(fi_a, &a);
+    guc_fi_get_status(fi_b, &b);
+
+	ret = CMP(fi_gui_relative_done(&a, FALSE), fi_gui_relative_done(&b, FALSE));
+	return 0 == ret ? CMP(a.done, b.done) : ret;
+}
+
+static gint 
+fi_gui_cmp_uploaded(GtkCList *unused_clist,
+	gconstpointer ptr1, gconstpointer ptr2)
+{
+    gnet_fi_status_t a, b;
+    gnet_fi_t fi_a, fi_b;
+
+	(void) unused_clist;
+	fi_a = GPOINTER_TO_UINT(((const GtkCListRow *) ptr1)->data);
+    fi_b = GPOINTER_TO_UINT(((const GtkCListRow *) ptr2)->data);
+
+    guc_fi_get_status(fi_a, &a);
+    guc_fi_get_status(fi_b, &b);
+
+	return CMP(a.uploaded, b.uploaded);
+}
+
+static gint 
+fi_gui_cmp_sources(GtkCList *unused_clist,
+	gconstpointer ptr1, gconstpointer ptr2)
+{
+    gnet_fi_status_t a, b;
+    gnet_fi_t fi_a, fi_b;
+	gint r;
+
+	(void) unused_clist;
+	fi_a = GPOINTER_TO_UINT(((const GtkCListRow *) ptr1)->data);
+    fi_b = GPOINTER_TO_UINT(((const GtkCListRow *) ptr2)->data);
+
+    guc_fi_get_status(fi_a, &a);
+    guc_fi_get_status(fi_b, &b);
+
+	r = CMP(a.recvcount, b.recvcount);
+	if (0 == r) {
+		r = CMP(a.aqueued_count + a.pqueued_count,
+				b.aqueued_count + b.pqueued_count);
+		if (0 == r) {
+			r = CMP(a.lifecount, b.lifecount);
+		}
+	}
+	return r;
+}
+
+static gint 
+fi_gui_cmp_status(GtkCList *unused_clist,
+	gconstpointer ptr1, gconstpointer ptr2)
+{
+    gnet_fi_status_t a, b;
+    gnet_fi_t fi_a, fi_b;
+
+	(void) unused_clist;
+	fi_a = GPOINTER_TO_UINT(((const GtkCListRow *) ptr1)->data);
+    fi_b = GPOINTER_TO_UINT(((const GtkCListRow *) ptr2)->data);
+
+    guc_fi_get_status(fi_a, &a);
+    guc_fi_get_status(fi_b, &b);
+
+	return CMP(fi_gui_numeric_status(&a), fi_gui_numeric_status(&b));
+}
+
+void
+on_clist_fileinfo_click_column(GtkCList *clist, gint column,
+	gpointer unused_udata)
+{
+	static gint sort_col = c_fi_num;
+	static gboolean sort_invert;
+
+	(void) unused_udata;
+	
+	g_assert(column >= 0 && column < c_fi_num);
+
+	switch ((enum c_fi) column) {
+#define CASE(x) case c_fi_ ## x : \
+		gtk_clist_set_compare_func(clist, fi_gui_cmp_ ## x ); break;
+	CASE(filename)
+	CASE(size)
+	CASE(done)
+	CASE(uploaded)
+	CASE(sources)
+	CASE(status)
+#undef CASE
+	case c_fi_num:
+		g_assert_not_reached();
+	}
+
+	sort_invert = sort_col == column && !sort_invert;
+	sort_col = column;
+	gtk_clist_set_sort_type(clist,
+		sort_invert ? GTK_SORT_DESCENDING : GTK_SORT_ASCENDING);
+	gtk_clist_sort(clist);
+}
+
 /* vi: set ts=4 sw=4 cindent: */
