@@ -1,5 +1,5 @@
 /*
- * $Id: search_cb.c 12268 2006-11-09 00:55:57Z cbiere $
+ * $Id: search_cb.c 13990 2007-06-30 00:59:50Z cbiere $
  *
  * Copyright (c) 2001-2005, Raphael Manfredi, Richard Eckart
  *
@@ -37,12 +37,13 @@
  * @date 2005
  */
 
-#include "gtk/gui.h"
+#include "common.h"
 
-RCSID("$Id: search_cb.c 12268 2006-11-09 00:55:57Z cbiere $")
+RCSID("$Id: search_cb.c 13990 2007-06-30 00:59:50Z cbiere $")
 
 #include <gdk/gdkkeysyms.h>
 
+#include "gtk/gui.h"
 #include "gtk/gtkcolumnchooser.h"
 #include "gtk/search.h"
 #include "gtk/statusbar.h"
@@ -62,16 +63,82 @@ RCSID("$Id: search_cb.c 12268 2006-11-09 00:55:57Z cbiere $")
 #include "if/core/sockets.h"
 
 #include "lib/atoms.h"
+#include "lib/cq.h"
 #include "lib/glib-missing.h"
 #include "lib/iso3166.h"
 #include "lib/vendors.h"
 #include "lib/utf8.h"
 #include "lib/override.h"		/* Must be the last header included */
 
+static gint search_details_selected_row = -1;
+static gchar *selected_text;
+
+static record_t *selected_record; 
+
+gchar * 
+search_details_get_text(GtkWidget *widget)
+{
+	gchar *text = NULL;
+
+	if (
+		search_details_selected_row >= 0 &&
+		gtk_clist_get_text(GTK_CLIST(widget), search_details_selected_row, 1,
+			&text)
+	) {
+		return g_strdup(text);
+	} else {
+		return NULL;
+	}
+}
+
 /***
  *** Private functions
  ***/
 
+/* Display XML data from the result if any */
+static void
+search_set_xml_metadata(const record_t *rc)
+{
+	GtkText *xml;
+
+	xml = GTK_TEXT(gui_main_window_lookup("text_result_info_xml"));
+	gtk_text_freeze(xml);
+	gtk_text_set_point(xml, 0);
+	gtk_text_forward_delete(xml, gtk_text_get_length(xml));
+	if (rc) {
+		gchar *text;
+	
+		text = rc->xml ? search_xml_indent(rc->xml) : NULL;
+		gtk_text_set_point(xml, 0);
+		gtk_text_insert(xml, NULL, NULL, NULL,
+			text ? lazy_utf8_to_ui_string(text) : "", (-1));
+		G_FREE_NULL(text);
+	}
+	gtk_text_thaw(xml);
+}
+
+static GtkCList *clist_search_details;
+
+void
+search_gui_clear_details(void)
+{
+	if (clist_search_details) {
+		gtk_clist_clear(clist_search_details);
+	}
+}
+
+void
+search_gui_append_detail(const gchar *name, const gchar *value)
+{
+ 	const gchar *titles[2];
+
+	g_return_if_fail(clist_search_details);
+
+	titles[0] = name;
+	titles[1] = EMPTY_STRING(value);
+    gtk_clist_append(clist_search_details, (gchar **) titles);
+}
+	
 /**
  *	Activates/deactivates buttons and popups based on what is selected
  */
@@ -79,180 +146,24 @@ RCSID("$Id: search_cb.c 12268 2006-11-09 00:55:57Z cbiere $")
  * Set or clear (when rc == NULL) the information about the record.
  */
 static void
-search_gui_set_details(const record_t *rc)
+search_gui_refresh_details(const record_t *rc)
 {
-	enum info_idx {
-		info_filename = 0,
-		info_sha1,
-		info_source,
-		info_size,
-		info_guid,
-		info_timestamp,
-		info_vendor,
-		info_index,
-		info_tag,
-		info_country,
-		info_speed,
-
-		num_infos
-	};
-	static GtkEntry *entry[num_infos];
-	static GtkText *xml;
-	static gboolean initialized;
-	guint j;
-
-	if (!initialized) {
-		static const struct {
-			const gchar *name;
-		} widgets[] = {
-#define D(x) STRINGIFY(CAT2(entry_result_,x))
-			{ D(info_filename) },
-			{ D(info_sha1) },
-			{ D(info_source) },
-			{ D(info_size) },
-			{ D(info_guid) },
-			{ D(info_timestamp) },
-			{ D(info_vendor) },
-			{ D(info_index) },
-			{ D(info_tag) },
-			{ D(info_country) },
-			{ D(info_speed) },
-#undef D
-		};
-		guint i;
-		
-		STATIC_ASSERT(G_N_ELEMENTS(widgets) == G_N_ELEMENTS(entry));
-		STATIC_ASSERT(G_N_ELEMENTS(widgets) == num_infos);
-		
-		initialized = TRUE;
-
-		for (i = 0; i < G_N_ELEMENTS(entry); i++)
-			entry[i] = GTK_ENTRY(gui_main_window_lookup(widgets[i].name));
-
-		xml = GTK_TEXT(gui_main_window_lookup(
-							STRINGIFY(CAT2(text_result_,info_xml))));
+	if (NULL == clist_search_details) {
+		static const gchar name[] = "clist_search_details";
+		clist_search_details = GTK_CLIST(gui_main_window_lookup(name));
 	}
+	g_return_if_fail(clist_search_details);
 
-	if (NULL == rc) {
-		guint i;
-
-		for (i = 0; i < G_N_ELEMENTS(entry); i++)
-			gtk_entry_set_text(entry[i], "");
-
-		gtk_text_freeze(xml);
-		gtk_text_set_point(xml, 0);
-		gtk_text_forward_delete(xml, gtk_text_get_length(xml));
-		gtk_text_thaw(xml);
-
-		return;
-	}
-
-	for (j = 0; j < num_infos; j++) {
-		GtkEntry *e = entry[j];
-
-		switch ((enum info_idx) j) {
-		case info_filename:
-			gtk_entry_set_text(e, lazy_utf8_to_ui_string(rc->utf8_name));
-			break;
-			
-		case info_sha1:
-			gtk_entry_printf(e, "%s%s",
-				rc->sha1 ? "urn:sha1:" : _("<no SHA1 known>"),
-				rc->sha1 ? sha1_base32(rc->sha1) : "");
-			break;
-			
-		case info_source:
-			gtk_entry_set_text(e,
-				rc->results_set->hostname
-					? hostname_port_to_string(rc->results_set->hostname,
-						rc->results_set->port)
-					: host_addr_port_to_string(rc->results_set->addr,
-						rc->results_set->port));
-			break;
-
-		case info_country:
-			gtk_entry_printf(e, "%s (%s)",
-				iso3166_country_name(rc->results_set->country),
-				iso3166_country_cc(rc->results_set->country));
-			break;
-
-		case info_size:
-			{
-				gchar bytes[32];
-				
-				uint64_to_string_buf(rc->size, bytes, sizeof bytes);
-				gtk_entry_printf(e, _("%s (%s bytes)"),
-					short_size(rc->size, show_metric_units()), bytes);
-			}
-			break;
-
-		case info_guid:
-			gtk_entry_set_text(e, guid_hex_str(rc->results_set->guid));
-			break;
-
-		case info_timestamp:
-			/* The ".24" is there to discard the trailing '\n' (see ctime(3) */
-			gtk_entry_printf(entry[info_timestamp], "%24.24s",
-				ctime(&rc->results_set->stamp));
-			break;
-
-		case info_vendor:
-			{	
-				const gchar *vendor, *ver;
-
-				vendor = lookup_vendor_name(rc->results_set->vcode);
-				ver = vendor ? rc->results_set->version : NULL;
-				gtk_entry_printf(e, "%s%s%s",
-					EMPTY_STRING(vendor),
-					ver ? "/" : "",
-					EMPTY_STRING(ver));
-			}
-			break;
-
-		case info_index:
-			gtk_entry_printf(e, "%lu", (gulong) rc->file_index);
-			break;
-
-		case info_tag:
-			gtk_entry_set_text(e, EMPTY_STRING(rc->results_set->query));
-			break;
-			
-		case info_speed:
-			gtk_entry_printf(e, "%u", rc->results_set->speed);
-			break;
-
-		case num_infos:
-			g_assert_not_reached();
-		}
-	}
-
-	if (rc->xml) {
-		gchar *xml_text = search_xml_indent(rc->xml);
-		if (xml_text) {
-			gtk_text_freeze(xml);
-			gtk_text_set_point(xml, 0);
-			gtk_text_insert(xml, NULL, NULL, NULL,
-					lazy_utf8_to_ui_string(xml_text), -1);
-			gtk_text_thaw(xml);
-			G_FREE_NULL(xml_text);
-		}
-	}
+    gtk_clist_freeze(clist_search_details);
+	search_gui_set_details(rc);
+    gtk_clist_thaw(clist_search_details);
+	search_set_xml_metadata(rc);
 }
 
-/**
- * Autoselects all searches matching given node in given tree, if the
- * unexpanded root of the tree is selected.  Otherwise, select only the
- * node on which they clicked.
- *
- * @return the amount of entries selected.
- */
-gint
-search_cb_autoselect(GtkCTree *ctree, GtkCTreeNode *node)
+record_t *
+search_gui_get_record(GtkCTree *ctree, GtkCTreeNode *node)
 {
-	GtkCTreeNode *child;
 	gui_record_t *grc;
-	record_t *rc;
-    guint32 sel_sources = 0;
 
 	/*
      * Rows with NULL data can appear when inserting new rows
@@ -264,46 +175,58 @@ search_cb_autoselect(GtkCTree *ctree, GtkCTreeNode *node)
      * Can this really happen???
      *      -- Richard, 18/04/2004
      */
-	grc = gtk_ctree_node_get_row_data(ctree, node);
-    if (grc == NULL) {
-        g_warning("search_cb_autoselect: row with NULL data detected");
-        return 0;
-    }
+	grc = node ? gtk_ctree_node_get_row_data(ctree, node) : NULL;
+	return grc ? grc->shared_record : NULL;
+}
 
-	rc = grc->shared_record;
+/**
+ * Autoselects all searches matching given node in given tree, if the
+ * unexpanded root of the tree is selected.  Otherwise, select only the
+ * node on which they clicked.
+ *
+ * @return the amount of entries selected.
+ */
+static gint
+search_cb_autoselect(GtkCTree *ctree, GtkCTreeNode *node)
+{
+    guint32 sel_sources = 0;
 
-	/*
-	 * Update details about the selected search.
-	 */
-	search_gui_set_details(rc);
+	g_return_val_if_fail(ctree, 0);
+	g_return_val_if_fail(node, 0);
+
+	gtk_signal_handler_block_by_func(GTK_OBJECT(ctree),
+		GTK_SIGNAL_FUNC(on_ctree_search_results_select_row), NULL);
 
 	/*
 	 * If the selected node is expanded, select it only.
 	 */
 
 	gtk_ctree_select(ctree, node);
-
-	if (GTK_CTREE_ROW(node)->expanded)
-		return 1;
-
-	/*
-	 * Node is not expanded.  Select all its children.
-	 */
-
 	sel_sources++;		/* We already selected the parent (folded) node */
 
-	for (
-		child = GTK_CTREE_ROW(node)->children;
-        child != NULL;
-		child = GTK_CTREE_NODE_SIBLING(child)
-	) {
-		gtk_ctree_select(ctree, child);
-		sel_sources++;
+	if (!GTK_CTREE_ROW(node)->expanded) {
+		GtkCTreeNode *child;
+
+		/*
+		 * Node is not expanded.  Select all its children.
+		 */
+
+		for (
+			child = GTK_CTREE_ROW(node)->children;
+			child != NULL;
+			child = GTK_CTREE_NODE_SIBLING(child)
+		) {
+			gtk_ctree_select(ctree, child);
+			sel_sources++;
+		}
+
+		if (sel_sources > 1)
+			statusbar_gui_message(15,
+				"auto selected %d sources by urn:sha1", sel_sources);
 	}
 
-    if (sel_sources > 1)
-        statusbar_gui_message(15,
-            "auto selected %d sources by urn:sha1", sel_sources);
+	gtk_signal_handler_unblock_by_func(GTK_OBJECT(ctree),
+		GTK_SIGNAL_FUNC(on_ctree_search_results_select_row), NULL);
 
 	return sel_sources;
 }
@@ -322,17 +245,16 @@ void
 on_search_notebook_switch(GtkNotebook *notebook, GtkNotebookPage *unused_page,
 	gint page_num, gpointer unused_udata)
 {
-	search_t *sch;
+	search_t *search;
 
 	(void) unused_page;
 	(void) unused_udata;
 
-	sch = (search_t *) gtk_object_get_user_data(
-		GTK_OBJECT(gtk_notebook_get_nth_page(notebook, page_num)));
+	search = gtk_object_get_user_data(
+				GTK_OBJECT(gtk_notebook_get_nth_page(notebook, page_num)));
 
-	g_return_if_fail(sch);
-
-    search_gui_set_current_search(sch);
+	g_return_if_fail(search);
+    search_gui_set_current_search(search);
 }
 
 /**
@@ -394,9 +316,6 @@ on_button_search_clear_clicked(GtkButton *unused_button, gpointer unused_udata)
         (gui_main_window_lookup("button_search_clear"), FALSE);
 }
 
-
-/**
- */
 void
 on_button_search_close_clicked(GtkButton *unused_button, gpointer unused_udata)
 {
@@ -410,9 +329,6 @@ on_button_search_close_clicked(GtkButton *unused_button, gpointer unused_udata)
         search_gui_close_search(search);
 }
 
-
-/**
- */
 void
 on_button_search_download_clicked(GtkButton *unused_button,
 	gpointer unused_udata)
@@ -423,9 +339,6 @@ on_button_search_download_clicked(GtkButton *unused_button,
     search_gui_download_files();
 }
 
-
-/**
- */
 void
 on_button_search_collapse_all_clicked(GtkButton *unused_button,
 	gpointer unused_udata)
@@ -436,9 +349,6 @@ on_button_search_collapse_all_clicked(GtkButton *unused_button,
     search_gui_collapse_all();
 }
 
-
-/**
- */
 void
 on_button_search_expand_all_clicked(GtkButton *unused_button,
 	gpointer unused_udata)
@@ -449,8 +359,6 @@ on_button_search_expand_all_clicked(GtkButton *unused_button,
     search_gui_expand_all();
 }
 
-/**
- */
 void
 on_button_search_filter_clicked(GtkButton *unused_button, gpointer unused_udata)
 {
@@ -460,8 +368,6 @@ on_button_search_filter_clicked(GtkButton *unused_button, gpointer unused_udata)
 	filter_open_dialog();
 }
 
-/**
- */
 gboolean
 on_clist_search_results_key_press_event(GtkWidget *unused_widget,
 	GdkEventKey *event, gpointer unused_udata)
@@ -498,8 +404,7 @@ on_clist_search_results_button_press_event(GtkWidget *widget,
 	case 1:
         /* left click section */
 		if (event->type == GDK_2BUTTON_PRESS) {
-			gtk_signal_emit_stop_by_name(GTK_OBJECT(widget),
-				"button_press_event");
+			gui_signal_stop_emit_by_name(widget, "button_press_event");
 			return FALSE;
 		}
 		if (event->type == GDK_BUTTON_PRESS) {
@@ -515,8 +420,7 @@ on_clist_search_results_button_press_event(GtkWidget *widget,
 				 * 2 clicks within 250 msec == doubleclick.
 				 * Surpress further events
 				 */
-				gtk_signal_emit_stop_by_name(GTK_OBJECT(widget),
-					"button_press_event");
+				gui_signal_stop_emit_by_name(widget, "button_press_event");
 
 				if (gtk_clist_get_selection_info(GTK_CLIST(widget), event->x,
 					event->y, &row, &column)) {
@@ -541,7 +445,7 @@ on_clist_search_results_button_press_event(GtkWidget *widget,
 			item = GTK_MENU_ITEM(
 					gui_popup_search_lookup("popup_search_toggle_tabs"));
         	gtk_label_set(GTK_LABEL(item->item.bin.child),
-				search_results_show_tabs
+				GUI_PROPERTY(search_results_show_tabs)
 					? _("Show search list")
 					: _("Show tabs"));
 			gtk_menu_popup(GTK_MENU(gui_popup_search()), NULL, NULL, NULL, NULL,
@@ -590,6 +494,141 @@ on_clist_search_results_click_column(GtkCList *clist, gint column,
 	search_gui_sort_column(search, column); /* Sort column, draw arrow */
 }
 
+void
+on_clist_search_details_selection_get(GtkWidget *unused_widget,
+	GtkSelectionData *data, guint unused_info,
+	guint unused_eventtime, gpointer unused_udata)
+{
+	(void) unused_widget;
+	(void) unused_info;
+	(void) unused_udata;
+	(void) unused_eventtime;
+
+    gtk_selection_data_set(data, GDK_SELECTION_TYPE_STRING,
+		8 /* CHAR_BIT */,
+		(guchar *) selected_text,
+		selected_text ? strlen(selected_text) : 0);
+}
+
+gboolean
+on_clist_search_details_key_press_event(GtkWidget *widget,
+	GdkEventKey *event, gpointer unused_udata)
+{
+    g_assert(event != NULL);
+
+	(void) unused_udata;
+
+	switch (event->keyval) {
+	guint modifier;
+	case GDK_c:
+		modifier = gtk_accelerator_get_default_mod_mask() & event->state;
+		if (GDK_CONTROL_MASK == modifier) {
+			if (gtk_selection_owner_set(widget,
+					GDK_SELECTION_PRIMARY, GDK_CURRENT_TIME)
+			) {
+				G_FREE_NULL(selected_text);
+				selected_text = search_details_get_text(widget);
+			}
+			return TRUE;
+		}
+		break;
+	}
+	return FALSE;
+}
+
+gint
+on_clist_search_details_selection_clear_event(GtkWidget *unused_widget,
+	GdkEventSelection *unused_event)
+{
+	(void) unused_widget;
+	(void) unused_event;
+	G_FREE_NULL(selected_text);
+    return TRUE;
+}
+
+void
+on_clist_search_details_select_row(GtkCList *unused_clist,
+	gint row, gint unused_column, GdkEventButton *unused_event,
+	gpointer unused_udata)
+{
+	(void) unused_clist;
+	(void) unused_column;
+	(void) unused_event;
+	(void) unused_udata;
+	search_details_selected_row = row;
+}
+
+void
+on_clist_search_details_unselect_row(GtkCList *unused_clist,
+	gint unused_row, gint unused_column, GdkEventButton *unused_event,
+	gpointer unused_udata)
+{
+	(void) unused_clist;
+	(void) unused_row;
+	(void) unused_column;
+	(void) unused_event;
+	(void) unused_udata;
+	search_details_selected_row = -1;
+}
+
+static cevent_t *row_selected_ev;
+
+#define ROW_SELECT_TIMEOUT	100 /* milliseconds */
+
+static gint
+gui_record_cmp(gconstpointer p, gconstpointer q)
+{
+	const gui_record_t *a = p, *b = q;
+	return a->shared_record != b->shared_record;
+}
+
+static void
+row_selected_expire(cqueue_t *unused_cq, gpointer unused_udata)
+{
+	search_t *search;
+
+	(void) unused_cq;
+	(void) unused_udata;
+
+	row_selected_ev = NULL;
+
+    search = search_gui_get_current_search();
+	if (search) {
+    	search_gui_refresh_popup();
+		search_gui_refresh_details(selected_record);
+		if (selected_record) {
+			GtkCTreeNode *node;
+			gui_record_t grc;
+			
+			grc.shared_record = selected_record;
+        	node = gtk_ctree_find_by_row_data_custom(GTK_CTREE(search->tree),
+					gtk_ctree_node_nth(GTK_CTREE(search->tree), 0),
+					&grc, gui_record_cmp);
+			search_cb_autoselect(GTK_CTREE(search->tree), GTK_CTREE_NODE(node));
+		}
+	} else {
+		search_gui_clear_details();
+	}
+}
+
+static void
+selected_row_changed(GtkCTree *ctree, GtkCTreeNode *node)
+{
+	if (selected_record) {
+		search_gui_unref_record(selected_record);
+	}
+	selected_record = search_gui_get_record(ctree, GTK_CTREE_NODE(node));
+	if (selected_record) {
+		search_gui_ref_record(selected_record);
+	}
+
+	if (row_selected_ev) {
+		cq_resched(callout_queue, row_selected_ev, ROW_SELECT_TIMEOUT);
+	} else {
+		row_selected_ev = cq_insert(callout_queue, ROW_SELECT_TIMEOUT,
+							row_selected_expire, NULL);
+	}
+}
 
 /**
  *	This function is called when the user selects a row in the
@@ -599,61 +638,23 @@ void
 on_ctree_search_results_select_row(GtkCTree *ctree,
 	GList *node, gint unused_column, gpointer unused_udata)
 {
-    static gboolean active = FALSE;
-
 	(void) unused_column;
 	(void) unused_udata;
 
-    /*
-     * We need to avoid recursion to prevent memory corruption.
-     */
-	if (active)
-		return;
-
-	if (NULL == node)
-		return;
-
-    active = TRUE;
-
-    /* actually using the "active" guard should be enough, but
-     * I'll try to block the signal too, just to be on the save side.
-     */
-    /* FIXME: Use either guard or signal blocking. Signal blocking
-     *        should be preferred to get better performance.
-	 */
-    gtk_signal_handler_block_by_func(GTK_OBJECT(ctree),
-		GTK_SIGNAL_FUNC(on_ctree_search_results_select_row),
-		NULL);
-
-    search_gui_refresh_popup();
-	search_cb_autoselect(ctree, GTK_CTREE_NODE(node));
-
-    gtk_signal_handler_unblock_by_func(GTK_OBJECT(ctree),
-        GTK_SIGNAL_FUNC(on_ctree_search_results_select_row),
-        NULL);
-
-    active = FALSE;
+	selected_row_changed(ctree, GTK_CTREE_NODE(node));
 }
 
-
-/**
- */
 void
-on_ctree_search_results_unselect_row(GtkCTree *unused_ctree, GList *unused_node,
+on_ctree_search_results_unselect_row(GtkCTree *ctree, GList *unused_node,
 	gint unused_column, gpointer unused_udata)
 {
-	(void) unused_ctree;
 	(void) unused_node;
 	(void) unused_column;
 	(void) unused_udata;
 
-	search_gui_set_details(NULL);	/* Clear details about results */
-    search_gui_refresh_popup();
+	selected_row_changed(ctree, NULL);
 }
 
-
-/**
- */
 void
 on_ctree_search_results_resize_column(GtkCList *unused_clist, gint column,
 	gint width, gpointer unused_udata)
@@ -729,7 +730,7 @@ search_gui_browse_selected(void)
     search = search_gui_get_current_search();
     g_assert(search != NULL);
 
-    ctree = search->ctree;
+    ctree = search->tree;
 	selected = GTK_CLIST(ctree)->selection;
 
 	if (selected == NULL) {
@@ -799,14 +800,14 @@ add_filter(filter_t *filter, GFunc filter_add_func, GCompareFunc cfn)
     search = search_gui_get_current_search();
     g_assert(search != NULL);
 
-    gtk_clist_freeze(GTK_CLIST(search->ctree));
+    gtk_clist_freeze(GTK_CLIST(search->tree));
 
-	node_list = g_list_copy(GTK_CLIST(search->ctree)->selection);
-	data_list = search_cb_collect_ctree_data(search->ctree, node_list, cfn);
+	node_list = g_list_copy(GTK_CLIST(search->tree)->selection);
+	data_list = search_cb_collect_ctree_data(search->tree, node_list, cfn);
 
     g_slist_foreach(data_list, filter_add_func, filter);
 
-    gtk_clist_thaw(GTK_CLIST(search->ctree));
+    gtk_clist_thaw(GTK_CLIST(search->tree));
 	g_slist_free(data_list);
 	g_list_free(node_list);
 }
@@ -923,9 +924,9 @@ on_popup_search_config_cols_activate(GtkMenuItem *unused_menuitem,
 
     search = search_gui_get_current_search();
     g_return_if_fail(search != NULL);
-    g_assert(search->ctree != NULL);
+    g_assert(search->tree != NULL);
 
-    cc = gtk_column_chooser_new(GTK_WIDGET(search->ctree));
+    cc = gtk_column_chooser_new(GTK_WIDGET(search->tree));
     gtk_menu_popup(GTK_MENU(cc), NULL, NULL, NULL, NULL, 1, 0);
 
     /* GtkColumnChooser takes care of cleaning up itself */
@@ -949,16 +950,16 @@ on_popup_search_metadata_activate(GtkMenuItem *unused_menuitem,
     search = search_gui_get_current_search();
     g_assert(search != NULL);
 
-    gtk_clist_freeze(GTK_CLIST(search->ctree));
+    gtk_clist_freeze(GTK_CLIST(search->tree));
 
-	node_list = g_list_copy(GTK_CLIST(search->ctree)->selection);
-	data_list = search_cb_collect_ctree_data(search->ctree,
+	node_list = g_list_copy(GTK_CLIST(search->tree)->selection);
+	data_list = search_cb_collect_ctree_data(search->tree,
 					node_list, gui_record_sha1_eq);
 
 	/* Make sure the column is actually visible. */
 	{
 		static const gint min_width = 80;
-		GtkCList *clist = GTK_CLIST(search->ctree);
+		GtkCList *clist = GTK_CLIST(search->tree);
 
     	gtk_clist_set_column_visibility(clist, c_sr_meta, TRUE);
 		if (clist->column[c_sr_meta].width < min_width)
@@ -974,10 +975,24 @@ on_popup_search_metadata_activate(GtkMenuItem *unused_menuitem,
 
 	G_SLIST_FOREACH(data_list, search_gui_queue_bitzi_by_sha1);
 
-	gtk_clist_thaw(GTK_CLIST(search->ctree));
+	gtk_clist_thaw(GTK_CLIST(search->tree));
 	g_slist_free(data_list);
 	g_list_free(node_list);
 }
 
+void
+search_gui_callbacks_shutdown(void)
+{
+	/*
+ 	 *	Remove delayed callbacks
+ 	 */
+	cq_cancel(callout_queue, &row_selected_ev);
+	search_gui_clear_details();
+	if (selected_record) {
+		search_gui_unref_record(selected_record);
+		selected_record = NULL;
+	}
+}
+
 /* -*- mode: cc-mode; tab-width:4; -*- */
 /* vi: set ts=4 sw=4 cindent: */
