@@ -1,5 +1,5 @@
 /*
- * $Id: search_cb.h 12268 2006-11-09 00:55:57Z cbiere $
+ * $Id: search_cb.h 13868 2007-06-18 01:30:56Z cbiere $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi, Richard Eckart
  *
@@ -26,6 +26,7 @@
 #ifndef _gtk1_search_cb_h_
 #define _gtk1_search_cb_h_
 
+#include "common.h"
 #include <gtk/gtk.h>
 
 void search_gui_set_cursor_position(gint x, gint y);
@@ -53,5 +54,21 @@ void on_ctree_search_results_select_row(GtkCTree *ctree, GList *node, gint colum
 void on_ctree_search_results_unselect_row(GtkCTree *ctree, GList *node, gint column, gpointer user_data);
 void on_ctree_search_results_resize_column(GtkCList * clist, gint column, gint width, gpointer user_data);
 
+gchar *search_details_get_text(GtkWidget *widget);
+void on_clist_search_details_select_row(GtkCList *unused_clist,
+	gint row, gint unused_column, GdkEventButton *unused_event,
+	gpointer unused_udata);
+void on_clist_search_details_unselect_row(GtkCList *unused_clist,
+	gint unused_row, gint unused_column, GdkEventButton *unused_event,
+	gpointer unused_udata);
+gboolean on_clist_search_details_key_press_event(GtkWidget *unused_widget,
+	GdkEventKey *event, gpointer unused_udata);
+void on_clist_search_details_selection_get(GtkWidget *unused_widget,
+	GtkSelectionData *data, guint unused_info,
+	guint unused_eventtime, gpointer unused_udata);
+gint on_clist_search_details_selection_clear_event(GtkWidget *unused_widget,
+	GdkEventSelection *unused_event);
+
+void search_gui_callbacks_shutdown(void);
 
 #endif /* _gtk1_search_cb_h_ */
