@@ -1,5 +1,5 @@
 /*
- * $Id: search_stats.c 12195 2006-11-03 02:20:22Z cbiere $
+ * $Id: search_stats.c 13777 2007-05-29 00:23:11Z cbiere $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi
  * Copyright (c) 2002, Michael Tesch
@@ -46,10 +46,11 @@
  * Released with gtk-gnutella & its license.
  */
 
+#include "common.h"
+
+RCSID("$Id: search_stats.c 13777 2007-05-29 00:23:11Z cbiere $")
+
 #include "gtk/gui.h"
-
-RCSID("$Id: search_stats.c 12195 2006-11-03 02:20:22Z cbiere $")
-
 #include "gtk/gtk-missing.h"
 #include "gtk/search_stats.h"
 #include "gtk/columns.h"
@@ -211,8 +212,8 @@ stats_hash_to_clist(gpointer key, gpointer value, gpointer unused_udata)
 
 	/* try to keep the number of infrequent terms down */
 	if (
-		((gfloat) val->total_cnt / (val->periods + 2.0)) * 100 <
-			search_stats_delcoef
+		(1.0 * val->total_cnt / (val->periods + 2.0)) * 100 <
+			GUI_PROPERTY(search_stats_delcoef)
 	) {
 		G_FREE_NULL(key);
 		G_FREE_NULL(val);
@@ -375,7 +376,7 @@ search_stats_gui_update(time_t now)
 
     if (
 		delta_time(now, last_update) <
-			(time_delta_t) search_stats_update_interval
+			(time_delta_t) GUI_PROPERTY(search_stats_update_interval)
 	)
         return;
 
