@@ -1,5 +1,5 @@
 /*
- * $Id: upload_stats.c 12195 2006-11-03 02:20:22Z cbiere $
+ * $Id: upload_stats.c 13057 2007-03-11 23:04:51Z cbiere $
  *
  * Copyright (c) 2001-2003, Richard Eckart
  * Copyright (c) 2002, Michael Tesch
@@ -55,10 +55,11 @@
  * Released with gtk-gnutella & its license
  */
 
+#include "common.h"
+
+RCSID("$Id: upload_stats.c 13057 2007-03-11 23:04:51Z cbiere $")
+
 #include "gtk/gui.h"
-
-RCSID("$Id: upload_stats.c 12195 2006-11-03 02:20:22Z cbiere $")
-
 #include "gtk/upload_stats.h"
 #include "gtk/columns.h"
 #include "gtk/settings.h"
@@ -121,32 +122,32 @@ upload_stats_gui_shutdown(void)
 }
 
 void
-upload_stats_gui_add(struct ul_stats *s)
+upload_stats_gui_add(const struct ul_stats *us)
 {
 	GtkCList *clist = clist_ul_stats();
-	gchar *rowdata[5];
+	const gchar *rowdata[5];
 	gint row;
 	gchar size_tmp[16];
 	gchar attempts_tmp[16];
 	gchar complete_tmp[16];
 	gchar norm_tmp[16];
 
-	g_strlcpy(size_tmp, short_size(s->size, show_metric_units()),
+	g_strlcpy(size_tmp, short_size(us->size, show_metric_units()),
 		sizeof size_tmp);
-	gm_snprintf(attempts_tmp, sizeof attempts_tmp, "%u", s->attempts);
-	gm_snprintf(complete_tmp, sizeof complete_tmp, "%u", s->complete);
-	gm_snprintf(norm_tmp, sizeof norm_tmp, "%.3f", s->norm);
+	gm_snprintf(attempts_tmp, sizeof attempts_tmp, "%u", us->attempts);
+	gm_snprintf(complete_tmp, sizeof complete_tmp, "%u", us->complete);
+	gm_snprintf(norm_tmp, sizeof norm_tmp, "%.3f", us->norm);
 
-	rowdata[c_us_filename] = s->filename;
+	rowdata[c_us_filename] = us->filename;
 	rowdata[c_us_size] = size_tmp;
 	rowdata[c_us_attempts] = attempts_tmp;
 	rowdata[c_us_complete] = complete_tmp;
 	rowdata[c_us_norm] = norm_tmp;
 
-    row = gtk_clist_insert(clist, 0, rowdata);
+    row = gtk_clist_insert(clist, 0, deconstify_gpointer(rowdata));
 	ul_rows++;
 
-	gtk_clist_set_row_data_full(clist, row, s, NULL);
+	gtk_clist_set_row_data_full(clist, row, deconstify_gpointer(us), NULL);
 
     /* FIXME: should use auto_sort? */
 	gtk_clist_sort(clist);
@@ -157,7 +158,7 @@ upload_stats_gui_add(struct ul_stats *s)
  * Called when a row of the upload stats should be updated
  */
 void
-upload_stats_gui_update(const gchar *name, guint64 size)
+upload_stats_gui_update(const struct ul_stats *us)
 {
 	GtkCList *clist = clist_ul_stats();
 	gint row;
@@ -165,7 +166,7 @@ upload_stats_gui_update(const gchar *name, guint64 size)
 	static gchar tmpstr[16];
 
 	/* find this file in the clist_ul_stats */
-	row = ul_find_row_by_upload(name, size, &s);
+	row = ul_find_row_by_upload(us->filename, us->size, &s);
 	if (-1 == row) {
 		g_assert_not_reached();
 		return;
