@@ -1,5 +1,5 @@
 /*
- * $Id: uploads.c 12195 2006-11-03 02:20:22Z cbiere $
+ * $Id: uploads.c 13954 2007-06-24 02:47:23Z cbiere $
  *
  * Copyright (c) 2001-2003, Richard Eckart
  *
@@ -23,10 +23,11 @@
  *----------------------------------------------------------------------
  */
 
+#include "common.h"
+
+RCSID("$Id: uploads.c 13954 2007-06-24 02:47:23Z cbiere $")
+
 #include "gtk/gui.h"
-
-RCSID("$Id: uploads.c 12195 2006-11-03 02:20:22Z cbiere $")
-
 #include "interface-glade.h"
 #include "gtk/uploads.h"
 #include "gtk/uploads_common.h"
@@ -182,11 +183,8 @@ upload_row_data_t *
 uploads_gui_get_row_data(gnet_upload_t uhandle)
 {
     upload_row_data_t *rd;
-	gint row;
 
-    row =  find_row(uhandle, &rd);
-
-	return row == -1 ? NULL : rd;
+	return find_row(uhandle, &rd) < 0 ? NULL : rd;
 }
 
 static void
@@ -348,21 +346,16 @@ uploads_gui_early_init(void)
 void
 uploads_gui_init(void)
 {
-    gtk_clist_set_column_justification(
-        GTK_CLIST(gui_main_window_lookup("clist_uploads")),
-        c_ul_size, GTK_JUSTIFY_RIGHT);
+	GtkCList *clist;
 
-    gtk_clist_set_column_justification(
-        GTK_CLIST(gui_main_window_lookup("clist_uploads")),
-        c_ul_progress, GTK_JUSTIFY_RIGHT);
-
-	gtk_clist_column_titles_passive(
-        GTK_CLIST(gui_main_window_lookup("clist_uploads")));
+	clist = GTK_CLIST(gui_main_window_lookup("clist_uploads"));
+    gtk_clist_set_column_justification(clist, c_ul_size, GTK_JUSTIFY_RIGHT);
+    gtk_clist_set_column_justification(clist, c_ul_progress, GTK_JUSTIFY_RIGHT);
+	gtk_clist_column_titles_passive(clist);
 
     guc_upload_add_upload_added_listener(upload_added);
     guc_upload_add_upload_removed_listener(upload_removed);
-    guc_upload_add_upload_info_changed_listener
-		(upload_info_changed);
+    guc_upload_add_upload_info_changed_listener(upload_info_changed);
 }
 
 /**
@@ -373,8 +366,7 @@ uploads_gui_shutdown(void)
 {
     guc_upload_remove_upload_added_listener(upload_added);
     guc_upload_remove_upload_removed_listener(upload_removed);
-    guc_upload_remove_upload_info_changed_listener
-		(upload_info_changed);
+    guc_upload_remove_upload_info_changed_listener(upload_info_changed);
 }
 
 /**
