@@ -1,5 +1,5 @@
 /*
- * $Id: monitor.c,v 1.16 2005/11/09 20:11:57 cbiere Exp $
+ * $Id: monitor.c 13971 2007-06-24 22:25:27Z cbiere $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi, Richard Eckart
  *
@@ -36,19 +36,21 @@
 
 #include "gtk/gui.h"
 
-RCSID("$Id: monitor.c,v 1.16 2005/11/09 20:11:57 cbiere Exp $");
+RCSID("$Id: monitor.c 13971 2007-06-24 22:25:27Z cbiere $")
 
+#include "gtk/drag.h"
 #include "gtk/monitor.h"
 #include "gtk/monitor_cb.h"
 
 #include "if/gui_property_priv.h"
 #include "if/bridge/ui2c.h"
 
-#include "lib/utf8.h"
 #include "lib/glib-missing.h"
+#include "lib/utf8.h"
+
 #include "lib/override.h"		/* Must be the last header included */
 
-static GtkListStore *monitor_model = NULL;
+static GtkListStore *monitor_model;
 
 enum {
    QUERY_COLUMN = 0,
@@ -56,7 +58,6 @@ enum {
 };
 
 
-
 /***
  *** Callbacks
  ***/
@@ -74,13 +75,13 @@ monitor_gui_add(query_type_t type, const gchar *item,
 	 * show, that's why we don't just the remove the first item. */
 	n = gtk_tree_model_iter_n_children(GTK_TREE_MODEL(monitor_model), NULL);
 
-	if (n > 0 && (guint) n >= monitor_max_items) {
+	if (n > 0 && (guint) n >= GUI_PROPERTY(monitor_max_items)) {
 		GtkTreeIter iter;
 		gboolean ok;
 
 		/* Children are enumerated from 0 to number-1 */
 		ok = gtk_tree_model_iter_nth_child(GTK_TREE_MODEL(monitor_model),
-					&iter, NULL, MAX(1, monitor_max_items) - 1);
+					&iter, NULL, MAX(1, GUI_PROPERTY(monitor_max_items)) - 1);
 		while (ok) {
 			GtkTreeIter next = iter;
 			ok = gtk_tree_model_iter_next(GTK_TREE_MODEL(monitor_model), &next);
@@ -89,7 +90,7 @@ monitor_gui_add(query_type_t type, const gchar *item,
 		}
 	}
 
-	if (monitor_max_items > 0) {
+	if (GUI_PROPERTY(monitor_max_items) > 0) {
 		const gchar *charset_ptr, *s;
 		gchar *dbuf;
 		GtkTreeIter iter;
@@ -115,6 +116,27 @@ monitor_gui_add(query_type_t type, const gchar *item,
 	}
 }
 
+static gchar *
+monitor_gui_get_text(GtkWidget *widget)
+{
+	GtkTreeModel *model;
+	GtkTreeIter iter;
+
+	g_return_val_if_fail(widget, NULL);
+
+	if (drag_get_iter(GTK_TREE_VIEW(widget), &model, &iter)) {
+		static const GValue zero_value;
+		GValue value;
+
+		value = zero_value;
+		gtk_tree_model_get_value(model, &iter, QUERY_COLUMN, &value);
+		return g_strdup(g_value_get_string(&value));
+	} else {
+		return NULL;
+	}
+}
+
+
 /***
  *** Public functions
  ***/
@@ -131,10 +153,9 @@ monitor_gui_init(void)
     monitor_model = gtk_list_store_new(MONITOR_COLUMNS, G_TYPE_STRING);
 
     /* Get the monitor widget */
-    tree = lookup_widget(main_window, "treeview_monitor");
+    tree = gui_main_window_lookup("treeview_monitor");
 
-    gtk_tree_view_set_model
-        (GTK_TREE_VIEW(tree), GTK_TREE_MODEL(monitor_model));
+    gtk_tree_view_set_model(GTK_TREE_VIEW(tree), GTK_TREE_MODEL(monitor_model));
 
     /* The view now holds a reference.  We can get rid of our own
      * reference */
@@ -153,6 +174,8 @@ monitor_gui_init(void)
 
 	g_signal_connect(G_OBJECT(tree), "button_press_event",
 		G_CALLBACK(on_treeview_monitor_button_press_event), NULL);
+
+	drag_attach(GTK_WIDGET(tree), monitor_gui_get_text);
 }
 
 void
