@@ -1,5 +1,5 @@
 /* 
- * $Id: search_cb.c 12268 2006-11-09 00:55:57Z cbiere $
+ * $Id: search_cb.c 14003 2007-07-01 15:14:59Z cbiere $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi, Richard Eckart
  *
@@ -36,20 +36,21 @@
 
 #include "gtk/gui.h"
 
-RCSID("$Id: search_cb.c 12268 2006-11-09 00:55:57Z cbiere $")
+RCSID("$Id: search_cb.c 14003 2007-07-01 15:14:59Z cbiere $")
 
 #include <gdk/gdkkeysyms.h>
 
 #include "search_cb.h"
 
-#include "gtk/gtkcolumnchooser.h"
-#include "gtk/search.h"
-#include "gtk/statusbar.h"
-#include "gtk/misc.h"
 #include "gtk/columns.h"
-#include "gtk/notebooks.h"
+#include "gtk/drag.h"
 #include "gtk/gtk-missing.h"
+#include "gtk/gtkcolumnchooser.h"
+#include "gtk/misc.h"
+#include "gtk/notebooks.h"
+#include "gtk/search.h"
 #include "gtk/settings.h"
+#include "gtk/statusbar.h"
 
 #include "if/gui_property.h"
 #include "if/gui_property_priv.h"
@@ -57,6 +58,7 @@ RCSID("$Id: search_cb.c 12268 2006-11-09 00:55:57Z cbiere $")
 #include "if/bridge/ui2c.h"
 #include "if/core/sockets.h"
 
+#include "lib/cq.h"
 #include "lib/glib-missing.h"
 #include "lib/iso3166.h"
 #include "lib/utf8.h"
@@ -65,7 +67,7 @@ RCSID("$Id: search_cb.c 12268 2006-11-09 00:55:57Z cbiere $")
 
 /* Privat variables */
 
-static search_t *search_selected = NULL;
+static search_t *search_selected;
 
 /***
  *** Private functions
@@ -271,8 +273,7 @@ on_tree_view_search_results_button_press_event(GtkWidget *widget,
 	case 1:
         /* left click section */
 		if (event->type == GDK_2BUTTON_PRESS) {
-			gtk_signal_emit_stop_by_name(GTK_OBJECT(widget),
-				"button_press_event");
+			gui_signal_stop_emit_by_name(widget, "button_press_event");
 			return FALSE;
 		}
 		if (event->type == GDK_BUTTON_PRESS) {
@@ -281,10 +282,8 @@ on_tree_view_search_results_button_press_event(GtkWidget *widget,
 				 * 2 clicks within 250 msec == doubleclick.
 				 * Surpress further events
 				 */
-				gtk_signal_emit_stop_by_name(GTK_OBJECT(widget),
-					"button_press_event");
+				gui_signal_stop_emit_by_name(widget, "button_press_event");
 				search_gui_download_files();
-
 				return TRUE;
 			} else {
 				click_time = event->time;
@@ -300,7 +299,7 @@ on_tree_view_search_results_button_press_event(GtkWidget *widget,
         	gtk_label_set(GTK_LABEL((GTK_MENU_ITEM(
 				gui_popup_search_lookup("popup_search_toggle_tabs"))
                 	->item.bin.child)),
-				search_results_show_tabs ?
+				GUI_PROPERTY(search_results_show_tabs) ?
 					_("Show search list") : _("Show tabs"));
 			gtk_menu_popup(GTK_MENU(gui_popup_search()), NULL, NULL, NULL, NULL,
 				event->button, event->time);
@@ -328,20 +327,21 @@ on_button_search_download_clicked(GtkButton *unused_button,
     search_gui_download_files();
 }
 
-gboolean
+void
 on_tree_view_search_results_click_column(GtkTreeViewColumn *column,
 	gpointer udata)
 {
 	GtkTreeSortable *model;
-	search_t *sch = udata;
+	search_t *search = udata;
 	GtkSortType order;
 	gint sort_col;
 
 	/* The default treeview is empty */
-	if (!sch)
-		return FALSE;
+	if (!search)
+		return;
 
-	model = GTK_TREE_SORTABLE(sch->model);
+	model = GTK_TREE_SORTABLE(
+				gtk_tree_view_get_model(GTK_TREE_VIEW(column->tree_view)));
 
 	/*
 	 * Here we enforce a tri-state sorting. Normally, Gtk+ would only
@@ -362,10 +362,10 @@ on_tree_view_search_results_click_column(GtkTreeViewColumn *column,
 	gtk_tree_sortable_get_sort_column_id(model, &sort_col, NULL);
 
 	/* If the user switched to another sort column, reset the sort order. */
-	if (sch->sort_col != sort_col) {
+	if (search->sort_col != sort_col) {
 		guint32 rank = 0;
 
-		sch->sort_order = SORT_NONE;
+		search->sort_order = SORT_NONE;
 		/*
 		 * Iterate over all rows and record their current rank/position so
 	 	 * that re-sorting is stable.
@@ -374,70 +374,46 @@ on_tree_view_search_results_click_column(GtkTreeViewColumn *column,
 			search_gui_update_rank, &rank);
 	}
 
-	sch->sort_col = sort_col;
+	search->sort_col = sort_col;
 
 	/* The search has to keep state about the sort order itself because
 	 * Gtk+ knows only ASCENDING/DESCENDING but not NONE (unsorted). */
-	switch (sch->sort_order) {
+	switch (search->sort_order) {
 	case SORT_NONE:
 	case SORT_NO_COL:
-		sch->sort_order = SORT_ASC;
+		search->sort_order = SORT_ASC;
 		break;
 	case SORT_ASC:
-		sch->sort_order = SORT_DESC;
+		search->sort_order = SORT_DESC;
 		break;
 	case SORT_DESC:
-		sch->sort_order = SORT_NONE;
+		search->sort_order = SORT_NONE;
 		break;
 	}
 
-	if (SORT_NONE == sch->sort_order) {
+	if (SORT_NONE == search->sort_order) {
 		/*
 		 * Reset the sorting and let the arrow disappear from the
 		 * header. Gtk+ actually seems to change the order of the
 		 * rows back to the original order (i.e., chronological).
 		 */
-		gtk_tree_view_column_set_sort_indicator(column, FALSE);
 #if GTK_CHECK_VERSION(2,6,0)
 		gtk_tree_sortable_set_sort_column_id(model,
 			GTK_TREE_SORTABLE_UNSORTED_SORT_COLUMN_ID, order);
-		gtk_tree_view_column_set_sort_indicator(column, FALSE);
 #endif /* Gtk+ >= 2.6.0 */
+		gtk_tree_view_column_set_sort_indicator(column, FALSE);
 	} else {
 		/*
 		 * Enforce the order as decided from the search state. Gtk+
 		 * might disagree but it'll do as told.
 		 */
 		gtk_tree_sortable_set_sort_column_id(model, sort_col,
-			SORT_ASC == sch->sort_order
+			SORT_ASC == search->sort_order
 				? GTK_SORT_ASCENDING : GTK_SORT_DESCENDING);
 		gtk_tree_view_column_set_sort_indicator(column, TRUE);
 	}
 	/* Make the column stays clickable. */
 	gtk_tree_view_column_set_clickable(column, TRUE);
-
-	return FALSE;
-}
-
-static const gchar *
-search_get_vendor_from_record(const record_t *rc)
-{
-	const gchar *s;
-
-	g_assert(rc != NULL);
-
-	s = lookup_vendor_name(rc->results_set->vcode);
-	if (s == NULL)
-		return _("Unknown");
-
-	if (rc->results_set->version) {
-		static gchar buf[128];
-
-		gm_snprintf(buf, sizeof buf, "%s/%s", s, rc->results_set->version);
-		return buf;
-	}
-
-	return s;
 }
 
 void
@@ -447,10 +423,9 @@ search_update_tooltip(GtkTreeView *tv, GtkTreePath *path)
 	GtkTreeIter iter;
 	const record_t *rc;
 
-	g_assert(tv != NULL);
-	if (!path) {
-		rc = NULL;
-	} else {
+	g_return_if_fail(tv);
+
+	if (path) {
 		model = gtk_tree_view_get_model(tv);
 		if (!gtk_tree_model_get_iter(model, &iter, path)) {
 			g_warning("gtk_tree_model_get_iter() failed");
@@ -458,6 +433,8 @@ search_update_tooltip(GtkTreeView *tv, GtkTreePath *path)
 		}
 
 		rc = search_gui_get_record_at_path(tv, path);
+	} else {
+		rc = NULL;
 	}
 
 	if (!rc) {
@@ -466,27 +443,14 @@ search_update_tooltip(GtkTreeView *tv, GtkTreePath *path)
 		gtk_tooltips_set_tip(settings_gui_tooltips(), GTK_WIDGET(tv),
 			_("Move the cursor over a row to see details."), NULL);
 		w = settings_gui_tooltips()->tip_window;
-#if 0
-		if (w)
-			gtk_widget_hide(w);
-#endif
 	} else {
-		gchar text[4096], extra[1024];
-		gboolean has_extra;
+		gchar text[4096];
 
-		if (rc->utf8_name && utf8_can_dejap(rc->utf8_name)) {
-			utf8_dejap(extra, sizeof extra, rc->utf8_name);
-		} else {
-			0[extra] = '\0';
-		}
-
-		has_extra = '\0' != 0[extra];
 		gm_snprintf(text, sizeof text,
 			"%s %s\n"
 			"%s %s (%s)\n"
 			"%s %.64s\n"
-			"%s %s"
-			"%s%s",
+			"%s %s",
 			_("Peer:"),
 			host_addr_port_to_string(rc->results_set->addr,
 				rc->results_set->port),
@@ -494,156 +458,195 @@ search_update_tooltip(GtkTreeView *tv, GtkTreePath *path)
 			iso3166_country_name(rc->results_set->country),
 			iso3166_country_cc(rc->results_set->country),
 			_("Vendor:"),
-			search_get_vendor_from_record(rc),
+			search_gui_get_vendor(rc->results_set),
 			_("Size:"),
-			short_size(rc->size, show_metric_units()),
-			has_extra ? "\nExtra: " : "",
-			has_extra ? extra : "");
+			short_size(rc->size, show_metric_units()));
 
 		gtk_tooltips_set_tip(settings_gui_tooltips(), GTK_WIDGET(tv),
 			text, NULL);
 	}
 }
 
+gchar *
+search_details_get_text(GtkWidget *widget)
+{
+	GtkTreeModel *model;
+	GtkTreeIter iter;
+
+	g_return_val_if_fail(widget, NULL);
+
+	if (drag_get_iter(GTK_TREE_VIEW(widget), &model, &iter)) {
+		static const GValue zero_value;
+		GValue value;
+
+		value = zero_value;
+		gtk_tree_model_get_value(model, &iter, 1, &value);
+		return g_strdup(g_value_get_string(&value));
+	} else {
+		return NULL;
+	}
+}
+
+/* Display XML data from the result if any */
+static void
+search_set_xml_metadata(const record_t *rc)
+{
+	GtkTextBuffer *txt;
+	gchar *xml_txt;
+		
+	txt = gtk_text_view_get_buffer(GTK_TEXT_VIEW(
+					gui_main_window_lookup("textview_result_info_xml")));
+	
+	/*
+	 * Character set detection usually fails here because XML
+	 * is mostly ASCII so that the thresholds are not reached.
+	 */
+	if (rc->xml) {
+		gchar *s = unknown_to_utf8_normalized(rc->xml, UNI_NORM_GUI, NULL);
+		xml_txt = search_xml_indent(s);
+		if (rc->xml != s) {
+			G_FREE_NULL(s);
+		}
+	} else {
+		xml_txt = NULL;
+	}
+	gtk_text_buffer_set_text(txt, EMPTY_STRING(xml_txt), -1);
+	G_FREE_NULL(xml_txt);
+}
+
+static GtkTreeView *treeview_search_details;
+
+void
+search_gui_clear_details(void)
+{
+	GtkTreeModel *model;
+
+	g_return_if_fail(treeview_search_details);
+
+	model = gtk_tree_view_get_model(treeview_search_details);
+	gtk_list_store_clear(GTK_LIST_STORE(model));
+}
+
+void
+search_gui_append_detail(const gchar *title, const gchar *value)
+{
+	GtkTreeModel *model;
+	GtkTreeIter iter;
+
+	g_return_if_fail(treeview_search_details);
+
+	model = gtk_tree_view_get_model(treeview_search_details);
+	gtk_list_store_append(GTK_LIST_STORE(model), &iter);
+	gtk_list_store_set(GTK_LIST_STORE(model), &iter, 0, title, 1, value, (-1));
+}
+
+static void
+search_gui_refresh_details(const record_t *rc)
+{
+	if (NULL == treeview_search_details) {
+		static const gchar name[] = "treeview_search_details";
+		treeview_search_details = GTK_TREE_VIEW(gui_main_window_lookup(name));
+	}
+	g_return_if_fail(treeview_search_details);
+
+	g_object_freeze_notify(G_OBJECT(treeview_search_details));
+	search_gui_set_details(rc);
+	g_object_thaw_notify(G_OBJECT(treeview_search_details));
+	search_set_xml_metadata(rc);
+}
+
 static void
 search_update_details(GtkTreeView *tv, GtkTreePath *path)
 {
-	const record_t *rc = NULL;
-	gchar bytes[UINT64_DEC_BUFLEN];
-	gchar *xml_txt;
-
-	g_assert(tv != NULL);
-	g_assert(path != NULL);
-
-	rc = search_gui_get_record_at_path(tv, path);
-	g_return_if_fail(rc != NULL);
+	g_assert(tv);
+	g_assert(path);
 	
-	gtk_entry_set_text(
-		GTK_ENTRY(gui_main_window_lookup("entry_result_info_filename")),
-		rc->name
-			? lazy_unknown_to_utf8_normalized(rc->name, UNI_NORM_GUI, NULL)
-			: "");
-	
-	gtk_entry_printf(
-		GTK_ENTRY(gui_main_window_lookup("entry_result_info_sha1")),
-		"%s%s",
-		rc->sha1 ? "urn:sha1:" : _("<no SHA1 known>"),
-		rc->sha1 ? sha1_base32(rc->sha1) : "");
-
-	if (rc->results_set->hostname)
-		gtk_entry_set_text(GTK_ENTRY(
-					gui_main_window_lookup("entry_result_info_source")),
-				hostname_port_to_string(
-					rc->results_set->hostname, rc->results_set->port));
-	else
-		gtk_entry_set_text(GTK_ENTRY(
-					gui_main_window_lookup("entry_result_info_source")),
-				host_addr_port_to_string(rc->results_set->addr,
-					rc->results_set->port));
-
-	uint64_to_string_buf(rc->size, bytes, sizeof bytes);
-	gtk_entry_printf(
-			GTK_ENTRY(gui_main_window_lookup("entry_result_info_size")),
-			_("%s (%s bytes)"), short_size(rc->size, show_metric_units()),
-			bytes);
-
-	gtk_entry_set_text(
-			GTK_ENTRY(gui_main_window_lookup("entry_result_info_guid")),
-			guid_hex_str(rc->results_set->guid));
-
-	gtk_entry_printf(GTK_ENTRY(
-				gui_main_window_lookup("entry_result_info_timestamp")),
-			"%24.24s", ctime(&rc->results_set->stamp));
-			/* discard trailing '\n' (see ctime(3) */
-
-	gtk_entry_set_text(
-			GTK_ENTRY(gui_main_window_lookup("entry_result_info_vendor")),
-			search_get_vendor_from_record(rc));
-
-	gtk_entry_printf(
-			GTK_ENTRY(gui_main_window_lookup("entry_result_info_index")),
-			"%lu", (gulong) rc->file_index);
-
-	{
-		const gchar *query = rc->results_set->query;
-		GtkEntry *entry;
-	   
-		entry = GTK_ENTRY(gui_main_window_lookup("entry_result_info_tag"));
-		query = rc->results_set->query;
-		if (query) {
-			gchar *s;
-
-			s = unknown_to_utf8_normalized(query, UNI_NORM_GUI, NULL);
-			gtk_entry_set_text(entry, s ? s : "");
-			if (query != s) {
-				G_FREE_NULL(s);
-			}
-		} else {
-			gtk_entry_set_text(entry, "");
-		}
-	}
-
-	/* Display XML data from the result if any */
-	{
-		GtkTextBuffer *txt;
-		
-		txt = gtk_text_view_get_buffer(GTK_TEXT_VIEW(
-					gui_main_window_lookup("textview_result_info_xml")));
-	
-		/*
-		 * Character set detection usually fails here because XML
-		 * is mostly ASCII so that the thresholds are not reached.
-		 */
-		if (rc->xml) {
-			gchar *s = unknown_to_utf8_normalized(rc->xml, UNI_NORM_GUI, NULL);
-			xml_txt = search_xml_indent(s);
-			if (rc->xml != s) {
-				G_FREE_NULL(s);
-			}
-		} else {
-			xml_txt = NULL;
-		}
-		gtk_text_buffer_set_text(txt, EMPTY_STRING(xml_txt), -1);
-		G_FREE_NULL(xml_txt);
-	}
-
-#if 0
-	/**
-	 * Dump the raw filename to enhance the character set detection.
-	 * The following helps to figure out what the actual encoding is:
-	 *
-	 *	for cs in $(iconv -l); do
-	 *	  printf "${cs}: ${filename}" | iconv -f "${cs}" -t UTF-8 2>/dev/null
-	 *	done
-	 */
-	g_message("filename[] = \"%s\"",
-		rc->name ? lazy_string_to_printf_escape(rc->name) : "(null)");
-	(void) lazy_string_to_printf_escape("");	/* release buffer if any */
-#endif
+	search_gui_refresh_details(search_gui_get_record_at_path(tv, path));
 }
 
+static cevent_t *row_selected_ev;
+
+#define ROW_SELECT_TIMEOUT	100 /* milliseconds */
+
+static void
+row_selected_expire(cqueue_t *unused_cq, gpointer unused_udata)
+{
+	GtkTreePath *path = NULL;
+	GtkTreeView *tv;
+	search_t *search;
+
+	(void) unused_cq;
+	(void) unused_udata;
+
+	row_selected_ev = NULL;
+    search = search_gui_get_current_search();
+
+	if (search) {
+		tv = GTK_TREE_VIEW(search->tree);
+		gtk_tree_view_get_cursor(tv, &path, NULL);
+	} else {
+		tv = NULL;
+		path = NULL;
+	}
+	if (path) {
+		search_update_tooltip(tv, path);
+		search_update_details(tv, path);
+		gtk_tree_path_free(path);
+       	search_gui_refresh_popup();
+	} else {
+		search_gui_clear_details();
+	}
+}
 
 /**
  * This function is called when the user selectes a row in the
  * search results pane. Autoselection takes place here.
  */
 void
-on_tree_view_search_results_select_row(GtkTreeView *tv, gpointer unused_udata)
+on_tree_view_search_results_select_row(GtkTreeView *unused_tv,
+	gpointer unused_udata)
 {
-	GtkTreePath *path;
+	(void) unused_tv;
+	(void) unused_udata;
+
+	if (row_selected_ev) {
+		cq_resched(callout_queue, row_selected_ev, ROW_SELECT_TIMEOUT);
+	} else {
+		row_selected_ev = cq_insert(callout_queue, ROW_SELECT_TIMEOUT,
+							row_selected_expire, NULL);
+	}
+}
+
+gboolean
+on_treeview_search_details_key_press_event(GtkWidget *widget,
+	GdkEventKey *event, gpointer unused_udata)
+{
+    g_assert(event != NULL);
 
 	(void) unused_udata;
 
-	gtk_tree_view_get_cursor(tv, &path, NULL);
-	if (path != NULL) {
-		search_update_tooltip(tv, path);
-		search_update_details(tv, path);
-		gtk_tree_path_free(path);
-		path = NULL;
-       	search_gui_refresh_popup();
+	switch (event->keyval) {
+	guint modifier;
+	case GDK_c:
+		modifier = gtk_accelerator_get_default_mod_mask() & event->state;
+		if (GDK_CONTROL_MASK == modifier) {
+			gchar *text;
+			size_t length;
+
+			text = search_details_get_text(widget);
+			length = text ? strlen(text) : 0;
+			length = length < INT_MAX ? length : 0;
+			gtk_clipboard_set_text(gtk_clipboard_get(GDK_SELECTION_PRIMARY),
+				text, length);
+			G_FREE_NULL(text);
+			return TRUE;
+		}
+		break;
 	}
+	return FALSE;
 }
 
+
 void
 on_button_search_passive_clicked(GtkButton *unused_button,
 	gpointer unused_udata)
@@ -699,8 +702,9 @@ on_popup_search_drop_name_activate(GtkMenuItem *unused_menuitem,
 	(void) unused_udata;
 
     search = search_gui_get_current_search();
-    g_assert(search != NULL);
-	selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(search->tree_view));
+	g_return_if_fail(search);
+
+	selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(search->tree));
     sl = tree_selection_collect_data(selection,
 			search_gui_get_record, gui_record_name_eq);
     g_slist_foreach(sl, (GFunc) filter_add_drop_name_rule, search->filter);
@@ -719,8 +723,9 @@ on_popup_search_drop_sha1_activate(GtkMenuItem *unused_menuitem,
 	(void) unused_udata;
 
     search = search_gui_get_current_search();
-    g_assert(search != NULL);
-	selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(search->tree_view));
+	g_return_if_fail(search);
+
+	selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(search->tree));
     sl = tree_selection_collect_data(selection,
 			search_gui_get_record, gui_record_sha1_eq);
     g_slist_foreach(sl, (GFunc) filter_add_drop_sha1_rule, search->filter);
@@ -739,8 +744,9 @@ on_popup_search_drop_host_activate(GtkMenuItem *unused_menuitem,
 	(void) unused_udata;
 
     search = search_gui_get_current_search();
-    g_assert(search != NULL);
-	selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(search->tree_view));
+	g_return_if_fail(search);
+
+	selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(search->tree));
     sl = tree_selection_collect_data(selection,
 			search_gui_get_record, gui_record_host_eq);
     g_slist_foreach(sl, (GFunc) filter_add_drop_host_rule, search->filter);
@@ -759,8 +765,9 @@ on_popup_search_drop_name_global_activate(GtkMenuItem *unused_menuitem,
 	(void) unused_udata;
 
     search = search_gui_get_current_search();
-    g_assert(search != NULL);
-	selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(search->tree_view));
+	g_return_if_fail(search);
+
+	selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(search->tree));
     sl = tree_selection_collect_data(selection,
 			search_gui_get_record, gui_record_name_eq);
     g_slist_foreach(sl, (GFunc) filter_add_drop_name_rule,
@@ -780,8 +787,9 @@ on_popup_search_drop_sha1_global_activate(GtkMenuItem *unused_menuitem,
 	(void) unused_udata;
 
     search = search_gui_get_current_search();
-    g_assert(search != NULL);
-	selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(search->tree_view));
+	g_return_if_fail(search);
+
+	selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(search->tree));
     sl = tree_selection_collect_data(selection,
 			search_gui_get_record, gui_record_sha1_eq);
     g_slist_foreach(sl, (GFunc) filter_add_drop_sha1_rule,
@@ -801,8 +809,9 @@ on_popup_search_drop_host_global_activate(GtkMenuItem *unused_menuitem,
 	(void) unused_udata;
 
     search = search_gui_get_current_search();
-    g_assert(search != NULL);
-	selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(search->tree_view));
+	g_return_if_fail(search);
+
+	selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(search->tree));
     sl = tree_selection_collect_data(selection,
 			search_gui_get_record, gui_record_host_eq);
     g_slist_foreach(sl, (GFunc) filter_add_drop_host_rule,
@@ -821,9 +830,9 @@ on_popup_search_config_cols_activate(GtkMenuItem *unused_menuitem,
 	(void) unused_udata;
 
     search = search_gui_get_current_search();
-	g_return_if_fail(NULL != search);
+	g_return_if_fail(search);
 
-	cc = gtk_column_chooser_new(GTK_WIDGET(search->tree_view));
+	cc = gtk_column_chooser_new(GTK_WIDGET(search->tree));
    	gtk_menu_popup(GTK_MENU(cc), NULL, NULL, NULL, NULL, 1, 0);
 
 	/* GtkColumnChooser takes care of cleaning up itself */
@@ -875,19 +884,18 @@ void
 search_gui_browse_selected(void)
 {
 	search_t *search;
-    GSList *sl;
-   
+
 	search = search_gui_get_current_search();
-	if (!search)
-		return;
-
-    sl = tree_selection_collect_data(
-			gtk_tree_view_get_selection(GTK_TREE_VIEW(search->tree_view)),
-			search_gui_get_record,
-			gui_record_host_eq);
-    g_slist_foreach(sl, search_gui_browse_selected_helper, NULL);
-    g_slist_free(sl);
+	if (search) {
+    	GSList *sl;
 
+		sl = tree_selection_collect_data(
+				gtk_tree_view_get_selection(GTK_TREE_VIEW(search->tree)),
+				search_gui_get_record,
+				gui_record_host_eq);
+		g_slist_foreach(sl, search_gui_browse_selected_helper, NULL);
+		g_slist_free(sl);
+	}
 }
 
 void
@@ -897,22 +905,19 @@ on_popup_search_copy_magnet_activate(GtkMenuItem *unused_item,
 	search_t *search;
 	GtkTreeView *tv;
 	GtkTreeIter iter;
-	GtkTreePath *path;
+	GtkTreePath *path = NULL;
 	GtkTreeModel *model;
 
 	(void) unused_item;
 	(void) unused_udata;
 
 	search = search_gui_get_current_search();
-	if (!search)
-		return;
+	g_return_if_fail(search);
 
-	tv = GTK_TREE_VIEW(search->tree_view);
+	tv = GTK_TREE_VIEW(search->tree);
 	gtk_tree_view_get_cursor(tv, &path, NULL);
-	if (!path) {
-		return;
-	}
-	
+	g_return_if_fail(path);
+
 	model = gtk_tree_view_get_model(tv);
 	if (gtk_tree_model_get_iter(model, &iter, path)) {
 		gchar *url;
@@ -933,11 +938,12 @@ on_popup_search_copy_magnet_activate(GtkMenuItem *unused_item,
 
 
 void
-search_callbacks_shutdown(void)
+search_gui_callbacks_shutdown(void)
 {
 	/*
  	 *	Remove delayed callbacks
  	 */
+	cq_cancel(callout_queue, &row_selected_ev);
 }
 
 /* -*- mode: cc-mode; tab-width:4; -*- */
