@@ -1,5 +1,5 @@
 /*
- * $Id: uploads.c,v 1.5 2004/10/25 03:47:50 cbiere Exp $
+ * $Id: uploads.c 13623 2007-05-13 00:38:34Z cbiere $
  *
  * Copyright (c) 2001-2003, Richard Eckart
  *
@@ -23,30 +23,46 @@
  *----------------------------------------------------------------------
  */
 
+/**
+ * @ingroup gtk
+ * @file
+ *
+ * Needs short description here.
+ *
+ * @author Richard Eckart
+ * @date 2001-2003
+ */
+
 #include "gtk/gui.h"
 
 #include "interface-glade.h"
+#if !GTK_CHECK_VERSION(2,5,0)
 #include "pbarcellrenderer.h"
+#endif
 
 #include "gtk/uploads.h"
 #include "gtk/uploads_common.h"
 #include "gtk/columns.h"
 #include "gtk/gtk-missing.h"
+#include "gtk/misc.h"
 #include "gtk/notebooks.h"
+#include "gtk/settings.h"
 
 #include "if/gui_property.h"
 #include "if/bridge/ui2c.h"
 
 #include "lib/atoms.h"
+#include "lib/host_addr.h"
 #include "lib/glib-missing.h"
-#include "lib/utf8.h"
 #include "lib/iso3166.h"
+#include "lib/misc.h"
+#include "lib/tm.h"
 #include "lib/walloc.h"
 #include "lib/override.h"		/* Must be the last header included */
 
-RCSID("$Id: uploads.c,v 1.5 2004/10/25 03:47:50 cbiere Exp $");
+RCSID("$Id: uploads.c 13623 2007-05-13 00:38:34Z cbiere $")
 
-#define UPDATE_MIN	300		/* Update screen every 5 minutes at least */
+#define UPDATE_MIN	300		/**< Update screen every 5 minutes at least */
 
 static gboolean uploads_remove_lock = FALSE;
 static gboolean uploads_shutting_down = FALSE;
@@ -55,13 +71,11 @@ static GtkTreeView *treeview_uploads = NULL;
 static GtkListStore *store_uploads = NULL;
 static GtkWidget *button_uploads_clear_completed = NULL;
 
-/* hash table for fast handle -> GtkTreeIter mapping */
+/** hash table for fast handle -> GtkTreeIter mapping */
 static GHashTable *upload_handles = NULL;
-/* list of all *removed* uploads; contains the handles */
+/** list of all *removed* uploads; contains the handles */
 static GSList *sl_removed_uploads = NULL;
 
-static inline upload_row_data_t *find_upload(gnet_upload_t u);
-
 static void uploads_gui_update_upload_info(const gnet_upload_info_t *u);
 static void uploads_gui_add_upload(gnet_upload_info_t *u);
 
@@ -77,14 +91,38 @@ static const char * const column_titles[UPLOADS_GUI_VISIBLE_COLUMNS] = {
 };
 
 typedef struct remove_row_ctx {
-	gboolean force;			/* If false, rows will only be removed, if 
-							 * their `entry_removal_timeout' has expired. */
-	time_t now; 			/* Current time, used to decide whether row
-							 * should be finally removed. */
-	GSList *sl_remaining;	/* Contains row data for not yet removed rows. */	
+	gboolean force;			/**< If false, rows will only be removed, if
+							 **  their `entry_removal_timeout' has expired. */
+	time_t now; 			/**< Current time, used to decide whether row
+							 **  should be finally removed. */
+	GSList *sl_remaining;	/**< Contains row data for not yet removed rows. */
 } remove_row_ctx_t;
 
 
+/**
+ * Tries to fetch upload_row_data associated with the given upload handle.
+ *
+ * @return a pointer the upload_row_data.
+ */
+static inline upload_row_data_t *
+find_upload(gnet_upload_t u)
+{
+	upload_row_data_t *rd;
+	gpointer key;
+	gboolean found;
+
+	found = g_hash_table_lookup_extended(upload_handles, GUINT_TO_POINTER(u),
+				NULL, &key);
+	g_assert(found);
+	rd = key;
+
+	g_assert(NULL != rd);
+	g_assert(rd->valid);
+	g_assert(rd->handle == u);
+
+	return rd;
+}
+
 /***
  *** Callbacks
  ***/
@@ -96,9 +134,9 @@ on_button_press_event(GtkWidget *unused_widget, GdkEventButton *event,
 	(void) unused_widget;
 	(void) unused_udata;
 
-	if (3 == event->button) { 
+	if (3 == event->button) {
         /* Right click section (popup menu) */
-		gtk_menu_popup(GTK_MENU(popup_uploads), NULL, NULL, NULL, NULL,
+		gtk_menu_popup(GTK_MENU(gui_popup_uploads()), NULL, NULL, NULL, NULL,
 			event->button, event->time);
 		return TRUE;
     }
@@ -121,7 +159,7 @@ upload_removed(gnet_upload_t uh, const gchar *reason,
 
 	(void) running;
 	(void) registered;
-	
+
     /* Invalidate row and remove it from the GUI if autoclear is on */
 	rd = find_upload(uh);
 	g_assert(NULL != rd);
@@ -154,12 +192,21 @@ upload_added(gnet_upload_t n, guint32 running, guint32 registered)
     guc_upload_free_info(info);
 }
 
+/**
+ * Fetch the GUI row data associated with upload handle.
+ */
+upload_row_data_t *
+uploads_gui_get_row_data(gnet_upload_t uhandle)
+{
+	return find_upload(uhandle);
+}
+
 /**
  * Callback: called when upload information was changed by the backend.
- * This updates the upload information in the gui. 
+ * This updates the upload information in the gui.
  */
 static void
-upload_info_changed(gnet_upload_t u, 
+upload_info_changed(gnet_upload_t u,
     guint32 running, guint32 registered)
 {
     gnet_upload_info_t *info;
@@ -172,7 +219,7 @@ upload_info_changed(gnet_upload_t u,
     guc_upload_free_info(info);
 }
 
-#define COMPARE_FUNC(field, code) \
+#define COMPARE_FUNC(field) \
 static gint CAT2(compare_,field)( \
 	GtkTreeModel *model, GtkTreeIter *a, GtkTreeIter *b, gpointer user_data) \
 { \
@@ -181,54 +228,29 @@ static gint CAT2(compare_,field)( \
 	(void) user_data; \
 	gtk_tree_model_get(model, a, c_ul_data, &rd_a, (-1)); \
 	gtk_tree_model_get(model, b, c_ul_data, &rd_b, (-1)); \
-	code \
-}
+	{
 
-COMPARE_FUNC(hosts, {
-	guint32 ip_a = rd_a->ip;
-	guint32 ip_b = rd_b->ip;
-	return CMP(ip_a, ip_b);
-});
+#define COMPARE_FUNC_END } }
 
-COMPARE_FUNC(sizes, {
+COMPARE_FUNC(hosts)
+	return host_addr_cmp(rd_a->addr, rd_b->addr);
+COMPARE_FUNC_END
+
+COMPARE_FUNC(sizes)
 	return CMP(rd_b->size, rd_a->size);
-});
+COMPARE_FUNC_END
 
-COMPARE_FUNC(ranges, {
+COMPARE_FUNC(ranges)
 	filesize_t u = rd_a->range_end - rd_a->range_start;
 	filesize_t v = rd_b->range_end - rd_b->range_start;
-	gint s = CMP(v, u); 
+	gint s = CMP(v, u);
 	return 0 != s ? s : CMP(rd_a->range_start, rd_b->range_start);
-});
+COMPARE_FUNC_END
 
 /***
  *** Private functions
  ***/
 
-/**
- * Tries to fetch upload_row_data associated with the given upload handle.
- *
- * @return a pointer the upload_row_data.
- */
-static inline upload_row_data_t *
-find_upload(gnet_upload_t u)
-{
-	upload_row_data_t *rd;
-	gpointer key;
-	gboolean found;
-
-	found = g_hash_table_lookup_extended(upload_handles, GUINT_TO_POINTER(u),
-				NULL, &key);
-	g_assert(found);
-	rd = key;
-
-	g_assert(NULL != rd);
-	g_assert(rd->valid);
-	g_assert(rd->handle == u);
-
-	return rd;
-}
-
 static void
 uploads_gui_update_upload_info(const gnet_upload_info_t *u)
 {
@@ -236,16 +258,24 @@ uploads_gui_update_upload_info(const gnet_upload_info_t *u)
     upload_row_data_t *rd = NULL;
 	gnet_upload_status_t status;
 	size_t range_len;
+	gint progress;
 
 	rd = find_upload(u->upload_handle);
 	g_assert(NULL != rd);
- 
-	rd->last_update  = time(NULL);	
 
-	if (u->ip != rd->ip) {
-		rd->ip = u->ip;
+	rd->last_update  = tm_time();
+
+	if (
+		!host_addr_equal(u->addr, rd->addr) ||
+		!host_addr_equal(u->gnet_addr, rd->gnet_addr) ||
+		u->gnet_port != rd->gnet_port
+	) {
+		rd->addr = u->addr;
+		rd->gnet_addr = u->gnet_addr;
+		rd->gnet_port = u->gnet_port;
+
 		gtk_list_store_set(store_uploads, &rd->iter,
-			c_ul_host, ip_to_gchar(rd->ip), (-1));
+			c_ul_host, uploads_gui_host_string(u), (-1));
 	}
 
 	if (u->range_start != rd->range_start || u->range_end != rd->range_end) {
@@ -255,18 +285,20 @@ uploads_gui_update_upload_info(const gnet_upload_info_t *u)
 		rd->range_end  = u->range_end;
 
 		if (u->range_start == 0 && u->range_end == 0)
-			g_strlcpy(str, "...", sizeof(str));
+			g_strlcpy(str, "...", sizeof str);
 		else {
-			range_len = gm_snprintf(str, sizeof(str), "%s%s",
-				u->partial ? "*" : "",
-				compact_size(u->range_end - u->range_start + 1));
+			range_len = gm_snprintf(str, sizeof str, "%s%s",
+				short_size(u->range_end - u->range_start + 1,
+					show_metric_units()),
+				u->partial ? _(" (partial)") : "");
 
-			if ((guint) range_len < sizeof(str)) {
+			if ((guint) range_len < sizeof str) {
 				if (u->range_start)
 					range_len += gm_snprintf(&str[range_len],
-									sizeof(str)-range_len,
-									" @ %s", compact_size(u->range_start));
-				g_assert((guint) range_len < sizeof(str));
+									sizeof str - range_len,
+									" @ %s", short_size(u->range_start,
+												show_metric_units()));
+				g_assert((guint) range_len < sizeof str);
 			}
 		}
 
@@ -276,29 +308,25 @@ uploads_gui_update_upload_info(const gnet_upload_info_t *u)
 	if (u->file_size != rd->size) {
 		rd->size = u->file_size;
 		gtk_list_store_set(store_uploads, &rd->iter,
-			c_ul_size, short_size(rd->size),
+			c_ul_size, short_size(rd->size, show_metric_units()),
 			(-1));
 	}
 
-	/* Exploit that u->name is an atom! */ 
+	/* Exploit that u->name is an atom! */
 	if (u->name != rd->name) {
-		g_assert(NULL != u->name);
-		if (NULL != rd->name)
-			atom_str_free(rd->name);
-		rd->name = atom_str_get(u->name);
+		atom_str_free_null(&rd->name);
+		rd->name = u->name ? atom_str_get(u->name) : NULL;
 		gtk_list_store_set(store_uploads, &rd->iter,
-			c_ul_filename, lazy_locale_to_utf8(rd->name, 0),
+			c_ul_filename, rd->name,
 			(-1));
 	}
 
-	/* Exploit that u->user_agent is an atom! */ 
+	/* Exploit that u->user_agent is an atom! */
 	if (u->user_agent != rd->user_agent) {
-		g_assert(NULL != u->user_agent);
-		if (NULL != rd->user_agent)
-			atom_str_free(rd->user_agent);
-		rd->user_agent = atom_str_get(u->user_agent);
+		atom_str_free_null(&rd->user_agent);
+		rd->user_agent = u->user_agent ? atom_str_get(u->user_agent) : NULL;
 		gtk_list_store_set(store_uploads, &rd->iter,
-			c_ul_agent, lazy_locale_to_utf8(rd->user_agent, 0),
+			c_ul_agent, rd->user_agent,
 			(-1));
 	}
 
@@ -312,8 +340,9 @@ uploads_gui_update_upload_info(const gnet_upload_info_t *u)
 	guc_upload_get_status(u->upload_handle, &status);
 	rd->status = status.status;
 
+	progress = 100.0 * uploads_gui_progress(&status, rd);
 	gtk_list_store_set(store_uploads, &rd->iter,
-		c_ul_progress, force_range(uploads_gui_progress(&status, rd), 0.0, 1.0),
+		c_ul_progress, CLAMP(progress, 0, 100),
 		c_ul_status, uploads_gui_status_str(&status, rd),
 		(-1));
 
@@ -332,26 +361,28 @@ uploads_gui_update_upload_info(const gnet_upload_info_t *u)
 void
 uploads_gui_add_upload(gnet_upload_info_t *u)
 {
-	gint range_len;
+	gint range_len, progress;
 	const gchar *titles[UPLOADS_GUI_VISIBLE_COLUMNS];
-    upload_row_data_t *rd = walloc(sizeof(*rd));
+    upload_row_data_t *rd = walloc(sizeof *rd);
 	gnet_upload_status_t status;
 	static gchar size_tmp[256];
 
-	memset(titles, 0, sizeof(titles));
+	memset(titles, 0, sizeof titles);
 
     rd->handle      = u->upload_handle;
     rd->range_start = u->range_start;
     rd->range_end   = u->range_end;
 	rd->size		= u->file_size;
     rd->start_date  = u->start_date;
-	rd->ip			= u->ip;
+	rd->addr		= u->addr;
 	rd->name		= NULL != u->name ? atom_str_get(u->name) : NULL;
 	rd->country	    = u->country;
 	rd->user_agent	= NULL != u->user_agent
 						? atom_str_get(u->user_agent) : NULL;
 	rd->push		= u->push;
 	rd->valid		= TRUE;
+    rd->gnet_addr   = u->gnet_addr;
+    rd->gnet_port   = u->gnet_port;
 
 	guc_upload_get_status(u->upload_handle, &status);
     rd->status = status.status;
@@ -361,43 +392,32 @@ uploads_gui_add_upload(gnet_upload_info_t *u)
     else {
 		static gchar range_tmp[256];	/* MUST be static! */
 
-        range_len = gm_snprintf(range_tmp, sizeof(range_tmp), "%s%s",
-			u->partial ? "*" : "",
-            compact_size(u->range_end - u->range_start + 1));
+        range_len = gm_snprintf(range_tmp, sizeof range_tmp, "%s%s",
+            short_size(u->range_end - u->range_start + 1,
+				show_metric_units()),
+			u->partial ? _(" (partial)") : "");
 
         if (u->range_start)
             range_len += gm_snprintf(
-                &range_tmp[range_len], sizeof(range_tmp)-range_len,
-                " @ %s", compact_size(u->range_start));
-    
-        g_assert((guint) range_len < sizeof(range_tmp));
+                &range_tmp[range_len], sizeof range_tmp - range_len,
+                " @ %s", short_size(u->range_start, show_metric_units()));
+
+        g_assert((guint) range_len < sizeof range_tmp);
 
         titles[c_ul_range] = range_tmp;
     }
 
-	g_strlcpy(size_tmp, short_size(u->file_size), sizeof(size_tmp)); 
+	g_strlcpy(size_tmp, short_size(u->file_size, show_metric_units()),
+		sizeof size_tmp);
     titles[c_ul_size] = size_tmp;
 
-	if (NULL != u->user_agent) {
-		static gchar str[256];	/* MUST be static! */
-		gchar *agent;
-
-		agent = lazy_locale_to_utf8(u->user_agent, 0);
-		if (u->user_agent != agent) {
-			g_strlcpy(str, agent, sizeof(str));
-			agent = str;
-		}
-    	titles[c_ul_agent] = agent;
-	} else
-		titles[c_ul_agent] = "...";
-
+   	titles[c_ul_agent] = u->user_agent ? u->user_agent : "...";
 	titles[c_ul_loc] = iso3166_country_cc(u->country);
-
-	titles[c_ul_filename] = NULL != u->name
-								? lazy_locale_to_utf8(u->name, 0) : "...";
-	titles[c_ul_host]     = ip_to_gchar(u->ip);
+	titles[c_ul_filename] = u->name ? u->name : "...";
+	titles[c_ul_host] = uploads_gui_host_string(u);
 	titles[c_ul_status] = uploads_gui_status_str(&status, rd);
 
+	progress = 100.0 * uploads_gui_progress(&status, rd);
     gtk_list_store_append(store_uploads, &rd->iter);
     gtk_list_store_set(store_uploads, &rd->iter,
 		c_ul_size, titles[c_ul_size],
@@ -406,9 +426,7 @@ uploads_gui_add_upload(gnet_upload_info_t *u)
 		c_ul_host, titles[c_ul_host],
 		c_ul_loc, titles[c_ul_loc],
 		c_ul_agent, titles[c_ul_agent],
-		c_ul_progress, 
-			force_range(
-				uploads_gui_progress(&status, rd), 0.0, 1.0),
+		c_ul_progress, CLAMP(progress, 0, 100),
 		c_ul_status, titles[c_ul_status],
 		c_ul_fg, NULL,
 		c_ul_data, rd,
@@ -421,49 +439,47 @@ add_column(gint column_id, GtkTreeIterCompareFunc sortfunc, GtkType column_type)
 {
 	GtkTreeViewColumn *column;
 	GtkCellRenderer *renderer;
-	guint32 width;
-	gboolean visible;
+	gint xpad;
 
 	g_assert(column_id >= 0 && (guint) column_id < UPLOADS_GUI_VISIBLE_COLUMNS);
 	g_assert(NULL != treeview_uploads);
 	g_assert(NULL != store_uploads);
 
-	gui_prop_get_guint32(PROP_UPLOADS_COL_WIDTHS, &width, column_id, 1);
-	gui_prop_get_boolean(PROP_UPLOADS_COL_VISIBLE, &visible, column_id, 1);
 	if (column_type == GTK_TYPE_CELL_RENDERER_PROGRESS) {
+		xpad = 0;
 		renderer = gtk_cell_renderer_progress_new();
 		column = gtk_tree_view_column_new_with_attributes(
 					_(column_titles[column_id]), renderer,
 					"value", column_id,
 					NULL);
 	} else { /* if (column_type == GTK_TYPE_CELL_RENDERER_TEXT) { */
+		xpad = GUI_CELL_RENDERER_XPAD;
 		renderer = gtk_cell_renderer_text_new();
 		gtk_cell_renderer_text_set_fixed_height_from_font(
 			GTK_CELL_RENDERER_TEXT(renderer), 1);
 		g_object_set(renderer,
 			"foreground-set", TRUE,
-			NULL);
+			(void *) 0);
 		column = gtk_tree_view_column_new_with_attributes(
 					_(column_titles[column_id]), renderer,
 					"foreground-gdk", c_ul_fg,
-					"text", column_id, 
-					NULL);
+					"text", column_id,
+					(void *) 0);
 	}
 
 	g_object_set(renderer,
 		"xalign", (gfloat) 0.0,
-		"xpad", GUI_CELL_RENDERER_XPAD,
+		"xpad", xpad,
 		"ypad", GUI_CELL_RENDERER_YPAD,
-		NULL);
+		(void *) 0);
 
 	g_object_set(G_OBJECT(column),
+		"fixed-width", 1,
 		"min-width", 1,
-		"fixed-width", MAX(1, width),
 		"reorderable", TRUE,
 		"resizable", TRUE,
 		"sizing", GTK_TREE_VIEW_COLUMN_FIXED,
-		"visible", visible,
-		NULL);
+		(void *) 0);
 
 	gtk_tree_view_column_set_sort_column_id(column, column_id);
 	gtk_tree_view_append_column(treeview_uploads, column);
@@ -473,6 +489,38 @@ add_column(gint column_id, GtkTreeIterCompareFunc sortfunc, GtkType column_type)
 			column_id, sortfunc, GINT_TO_POINTER(column_id), NULL);
 }
 
+static GtkListStore *
+create_uploads_model(void)
+{
+	static GType columns[c_ul_num];
+	GtkListStore *store;
+	guint i;
+
+	STATIC_ASSERT(c_ul_num == G_N_ELEMENTS(columns));
+#define SET(c, x) case (c): columns[i] = (x); break
+	for (i = 0; i < G_N_ELEMENTS(columns); i++) {
+		switch (i) {
+		SET(c_ul_filename, G_TYPE_STRING);
+		SET(c_ul_host, G_TYPE_STRING);
+		SET(c_ul_loc, G_TYPE_STRING);
+		SET(c_ul_size, G_TYPE_STRING);
+		SET(c_ul_range, G_TYPE_STRING);
+		SET(c_ul_agent, G_TYPE_STRING);
+		SET(c_ul_progress, G_TYPE_INT);
+		SET(c_ul_status, G_TYPE_STRING);
+		SET(c_ul_fg, GDK_TYPE_COLOR);
+		SET(c_ul_data, G_TYPE_POINTER);
+		default:
+			g_assert_not_reached();
+		}
+	}
+#undef SET
+
+	store = gtk_list_store_newv(G_N_ELEMENTS(columns), columns);
+	return GTK_LIST_STORE(store);
+}
+
+
 /***
  *** Public functions
  ***/
@@ -480,24 +528,11 @@ add_column(gint column_id, GtkTreeIterCompareFunc sortfunc, GtkType column_type)
 void
 uploads_gui_early_init(void)
 {
-    popup_uploads = create_popup_uploads();
 }
 
 void
 uploads_gui_init(void)
 {
-	GType types[] = {
-		G_TYPE_STRING,
-		G_TYPE_STRING,
-		G_TYPE_STRING,
-		G_TYPE_STRING,
-		G_TYPE_STRING,
-		G_TYPE_STRING,
-		G_TYPE_FLOAT,
-		G_TYPE_STRING,
-		GDK_TYPE_COLOR,
-		G_TYPE_POINTER
-	};
 	static const struct {
 		gint id;
 		GtkTreeIterCompareFunc sortfunc;
@@ -508,53 +543,45 @@ uploads_gui_init(void)
 		{ c_ul_size, 		compare_sizes },
 		{ c_ul_range, 		compare_ranges },
 		{ c_ul_agent, 		NULL },
-		{ c_ul_progress, 	NULL }, 
+		{ c_ul_progress, 	NULL },
 		{ c_ul_status, 		NULL },
 	};
-	GtkTreeView *treeview;
 	size_t i;
 
-	STATIC_ASSERT(G_N_ELEMENTS(types) == c_ul_num);
 	STATIC_ASSERT(G_N_ELEMENTS(cols) == UPLOADS_GUI_VISIBLE_COLUMNS);
-	store_uploads = gtk_list_store_newv(G_N_ELEMENTS(types), types);
+	store_uploads = create_uploads_model();
 
-	button_uploads_clear_completed = lookup_widget(main_window,
-		"button_uploads_clear_completed");
-	treeview_uploads = treeview =
-		GTK_TREE_VIEW(lookup_widget(main_window, "treeview_uploads"));
-	gtk_tree_view_set_model(treeview, GTK_TREE_MODEL(store_uploads));
+	button_uploads_clear_completed =
+		gui_main_window_lookup("button_uploads_clear_completed");
+	treeview_uploads =
+		GTK_TREE_VIEW(gui_main_window_lookup("treeview_uploads"));
+	gtk_tree_view_set_model(treeview_uploads, GTK_TREE_MODEL(store_uploads));
 
 	for (i = 0; i < G_N_ELEMENTS(cols); i++)
 		add_column(cols[i].id, cols[i].sortfunc,
 			c_ul_progress == cols[i].id
 				? GTK_TYPE_CELL_RENDERER_PROGRESS
 				: GTK_TYPE_CELL_RENDERER_TEXT);
-	
+
+	tree_view_restore_widths(treeview_uploads, PROP_UPLOADS_COL_WIDTHS);
+	tree_view_restore_visibility(treeview_uploads, PROP_UPLOADS_COL_VISIBLE);
+
 	upload_handles = g_hash_table_new(NULL, NULL);
 
     guc_upload_add_upload_added_listener(upload_added);
     guc_upload_add_upload_removed_listener(upload_removed);
-    guc_upload_add_upload_info_changed_listener
-		(upload_info_changed);
-	
-	g_signal_connect(GTK_OBJECT(treeview), "button_press_event",
+    guc_upload_add_upload_info_changed_listener(upload_info_changed);
+
+	g_signal_connect(GTK_OBJECT(treeview_uploads), "button_press_event",
 		G_CALLBACK(on_button_press_event), NULL);
 }
 
 static inline void
-free_row_data(upload_row_data_t *rd, gpointer user_data)
+free_row_data(upload_row_data_t *rd)
 {
-	(void) user_data;
-
-	if (NULL != rd->user_agent) {
-		atom_str_free(rd->user_agent);
-		rd->user_agent = NULL;
-	}
-	if (NULL != rd->name) {
-		atom_str_free(rd->name);
-		rd->user_agent = NULL;
-	}
-	wfree(rd, sizeof(*rd));
+	atom_str_free_null(&rd->user_agent);
+	atom_str_free_null(&rd->name);
+	wfree(rd, sizeof *rd);
 }
 
 static inline void
@@ -564,7 +591,7 @@ free_handle(gpointer key, gpointer value,
 	(void) key;
 	(void) user_data;
 
-	free_row_data(value, NULL);
+	free_row_data(value);
 }
 
 static inline void
@@ -573,27 +600,33 @@ remove_row(upload_row_data_t *rd, remove_row_ctx_t *ctx)
 	g_assert(NULL != rd);
 	if (ctx->force || upload_should_remove(ctx->now, rd)) {
     	gtk_list_store_remove(store_uploads, &rd->iter);
-		free_row_data(rd, NULL);
+		free_row_data(rd);
 	} else
 		ctx->sl_remaining = g_slist_prepend(ctx->sl_remaining, rd);
 }
 
 static inline void
-update_row(gpointer key, gpointer data, gpointer user_data)
+update_row(gpointer key, gpointer data, gpointer unused_udata)
 {
-	time_t now = *(time_t *) user_data;
+	time_t now;
 	upload_row_data_t *rd = data;
 	gnet_upload_status_t status;
+	guint progress;
 
+	(void) unused_udata;
 	g_assert(NULL != rd);
 	g_assert(GPOINTER_TO_UINT(key) == rd->handle);
+
+	now = tm_time();
 	if (delta_time(now, rd->last_update) < 2)
 		return;
 
 	rd->last_update = now;
 	guc_upload_get_status(rd->handle, &status);
+	progress = 100.0 * uploads_gui_progress(&status, rd);
+	progress = MIN(progress, 100);
 	gtk_list_store_set(store_uploads, &rd->iter,
-		c_ul_progress, force_range(uploads_gui_progress(&status, rd), 0.0, 1.0),
+		c_ul_progress, progress,
 		c_ul_status, uploads_gui_status_str(&status, rd),
 		(-1));
 }
@@ -604,9 +637,7 @@ update_row(gpointer key, gpointer data, gpointer user_data)
 void
 uploads_gui_update_display(time_t now)
 {
-    static time_t last_update = 0;
-    static gboolean locked = FALSE;
-	remove_row_ctx_t ctx = { FALSE, now, NULL };
+    static time_t last_update;
 	gint current_page;
 	static GtkNotebook *notebook = NULL;
 
@@ -619,7 +650,7 @@ uploads_gui_update_display(time_t now)
 	 */
 
 	if (notebook == NULL)
-		notebook = GTK_NOTEBOOK(lookup_widget(main_window, "notebook_main"));
+		notebook = GTK_NOTEBOOK(gui_main_window_lookup("notebook_main"));
 
 	current_page = gtk_notebook_get_current_page(notebook);
 	if (
@@ -629,26 +660,31 @@ uploads_gui_update_display(time_t now)
 		return;
 	}
 
-    if (last_update == now)
-        return;
+    if (last_update != now) {
+    	static gboolean locked = FALSE;
+		remove_row_ctx_t ctx;
 
-    last_update = now;
+    	last_update = now;
+		ctx.force = FALSE;
+		ctx.now = now;
+		ctx.sl_remaining = NULL;
 
-    g_return_if_fail(!locked);
-    locked = TRUE;
+		g_return_if_fail(!locked);
+		locked = TRUE;
 
-	/* Remove all rows with `removed' uploads. */
-	G_SLIST_FOREACH(sl_removed_uploads, remove_row, &ctx);
-	g_slist_free(sl_removed_uploads);
-	sl_removed_uploads = ctx.sl_remaining;
+		/* Remove all rows with `removed' uploads. */
+		G_SLIST_FOREACH_WITH_DATA(sl_removed_uploads, remove_row, &ctx);
+		g_slist_free(sl_removed_uploads);
+		sl_removed_uploads = ctx.sl_remaining;
 
-	/* Update the status column for all active uploads. */ 
-	g_hash_table_foreach(upload_handles, update_row, &now);
-       
-	if (NULL == sl_removed_uploads)
-		gtk_widget_set_sensitive(button_uploads_clear_completed, FALSE);
+		/* Update the status column for all active uploads. */
+		g_hash_table_foreach(upload_handles, update_row, NULL);
 
-    locked = FALSE;
+		if (NULL == sl_removed_uploads)
+			gtk_widget_set_sensitive(button_uploads_clear_completed, FALSE);
+
+		locked = FALSE;
+	}
 }
 
 static gboolean
@@ -665,14 +701,14 @@ uploads_clear_helper(gpointer user_data)
 
 	/* Remove all rows with `removed' uploads. */
 
-	G_SLIST_FOREACH(sl_removed_uploads, remove_row, &ctx);
+	G_SLIST_FOREACH_WITH_DATA(sl_removed_uploads, remove_row, &ctx);
 	g_slist_free(sl_removed_uploads);
 	sl_removed_uploads = ctx.sl_remaining;
 
     for (sl = sl_removed_uploads; sl; sl = g_slist_next(sl_removed_uploads)) {
 		remove_row((upload_row_data_t *) sl->data, &ctx);
 		/* Interrupt and come back later to prevent GUI stalling. */
-		if (0 == (++counter & 0x7f)) { 
+		if (0 == (++counter & 0x7f)) {
 			/* Remember which elements haven't been traversed yet. */
 			sl_remaining = g_slist_remove_link(sl, sl);
 			break;
@@ -682,13 +718,13 @@ uploads_clear_helper(gpointer user_data)
 	 * by ctx->remaining. */
 	g_slist_free(sl_removed_uploads);
 	sl_removed_uploads = g_slist_concat(ctx.sl_remaining, sl_remaining);
-		
+
     if (NULL == sl_removed_uploads) {
 		gtk_widget_set_sensitive(button_uploads_clear_completed, FALSE);
     	uploads_remove_lock = FALSE;
     	return FALSE; /* Finished. */
     }
-    
+
     return TRUE; /* More rows to remove. */
 }
 
@@ -697,7 +733,7 @@ uploads_gui_clear_completed(void)
 {
 	if (!uploads_remove_lock) {
 		uploads_remove_lock = TRUE;
-		gtk_timeout_add(100, uploads_clear_helper, store_uploads); 
+		gtk_timeout_add(100, uploads_clear_helper, store_uploads);
 	}
 }
 
@@ -705,24 +741,23 @@ uploads_gui_clear_completed(void)
  * Unregister callbacks in the backend and clean up.
  */
 void
-uploads_gui_shutdown(void) 
+uploads_gui_shutdown(void)
 {
 	uploads_shutting_down = TRUE;
-	
+
 	tree_view_save_widths(treeview_uploads, PROP_UPLOADS_COL_WIDTHS);
 	tree_view_save_visibility(treeview_uploads, PROP_UPLOADS_COL_VISIBLE);
 
     guc_upload_remove_upload_added_listener(upload_added);
     guc_upload_remove_upload_removed_listener(upload_removed);
-    guc_upload_remove_upload_info_changed_listener
-		(upload_info_changed);
+    guc_upload_remove_upload_info_changed_listener(upload_info_changed);
 
 	gtk_list_store_clear(store_uploads);
 
 	g_hash_table_foreach(upload_handles, free_handle, NULL);
 	g_hash_table_destroy(upload_handles);
 	upload_handles = NULL;
-	G_SLIST_FOREACH(sl_removed_uploads, free_row_data, NULL);
+	G_SLIST_FOREACH(sl_removed_uploads, free_row_data);
 	g_slist_free(sl_removed_uploads);
 	sl_removed_uploads = NULL;
 }
