@@ -1,5 +1,5 @@
 /*
- * $Id: main.c 12268 2006-11-09 00:55:57Z cbiere $
+ * $Id: main.c 13990 2007-06-30 00:59:50Z cbiere $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi & Richard Eckart
  *
@@ -36,7 +36,7 @@
 
 #include "gui.h"
 
-RCSID("$Id: main.c 12268 2006-11-09 00:55:57Z cbiere $")
+RCSID("$Id: main.c 13990 2007-06-30 00:59:50Z cbiere $")
 
 #ifdef I_PWD
 #include <pwd.h>
@@ -77,7 +77,6 @@ RCSID("$Id: main.c 12268 2006-11-09 00:55:57Z cbiere $")
 
 #include "if/bridge/ui2c.h"
 
-#include "lib/file.h"
 #include "lib/glib-missing.h"
 #include "lib/tm.h"
 #include "lib/utf8.h"
@@ -109,6 +108,7 @@ gui_ ## name ## _lookup(const gchar *id) \
 
 WIDGET(dlg_about)
 WIDGET(dlg_faq)
+WIDGET(dlg_ancient)
 WIDGET(dlg_prefs)
 WIDGET(dlg_quit)
 WIDGET(main_window)
@@ -250,12 +250,12 @@ gui_init_menu(void)
 
 	g_object_unref(store);
 
-	g_signal_connect(G_OBJECT(treeview), "cursor-changed",
-		G_CALLBACK(on_main_gui_treeview_menu_cursor_changed), NULL);
-	g_signal_connect(G_OBJECT(treeview), "row-collapsed",
-		G_CALLBACK(on_main_gui_treeview_menu_row_collapsed), NULL);
-	g_signal_connect(G_OBJECT(treeview), "row-expanded",
-		G_CALLBACK(on_main_gui_treeview_menu_row_expanded), NULL);
+	gui_signal_connect(treeview, "cursor-changed",
+		on_main_gui_treeview_menu_cursor_changed, NULL);
+	gui_signal_connect(treeview, "row-collapsed",
+		on_main_gui_treeview_menu_row_collapsed, NULL);
+	gui_signal_connect(treeview, "row-expanded",
+		on_main_gui_treeview_menu_row_expanded, NULL);
 }
 
 static void
@@ -484,6 +484,7 @@ gui_init_dlg_about(void)
 		"FlashCode <flashcode@flashtux.org>",
 		"U\304\237ur \303\207etin <ugur.jnmbk@gmail.com>",
 		"Lloyd Bryant <lloydbaz@msn.com>",
+		"Martijn van Oosterhout <kleptog@svana.org>",
     };
 	GtkWidget *text;
     guint i;
@@ -507,93 +508,6 @@ gui_init_dlg_about(void)
     	Q_("translation_credit|"));
 }
 
-static void
-gui_init_dlg_faq(void)
-{
-	static const gchar faq_file[] = "FAQ";
-	static file_path_t fp[4];
-	GtkWidget *text;
-	const gchar *lang;
-	guint i = 0;
-	FILE *f;
-
-	text = gui_dlg_faq_lookup("textview_faq");
-	lang = locale_get_language();
-
-	file_path_set(&fp[i++], make_pathname(PRIVLIB_EXP, lang), faq_file);
-	file_path_set(&fp[i++], PRIVLIB_EXP G_DIR_SEPARATOR_S "en", faq_file);
-	
-#ifndef OFFICIAL_BUILD
-	file_path_set(&fp[i++],
-		make_pathname(PACKAGE_EXTRA_SOURCE_DIR, lang), faq_file);
-
-	file_path_set(&fp[i++],
-		PACKAGE_EXTRA_SOURCE_DIR G_DIR_SEPARATOR_S "en", faq_file);
-#endif /* !OFFICIAL_BUILD */
-
-	g_assert(i <= G_N_ELEMENTS(fp));
-
-	f = file_config_open_read_norename("FAQ", fp, i);
-	if (f) {
-		gchar buf[4096];
-		gboolean tag = FALSE;
-
-		while (fgets(buf, sizeof buf, f)) {
-			const gchar *s;
-			gchar *p;
-
-			if (!utf8_is_valid_string(buf)) {
-				text_widget_append(GTK_WIDGET(text),
-					_("\nThe FAQ document is damaged.\n"));
-				break;
-			}
-
-			/* Strip HTML tags */
-			for (s = buf, p = buf; *s != '\0'; s++) {
-				if (tag) {
-					if (*s == '>')
-						tag = FALSE;
-				} else if (*s != '<') {
-					*p++ = *s;
-				} else {
-					tag = TRUE;
-				}
-			}
-			*p = '\0';
-			
-			/* Convert entities */
-			p = buf;
-			for (s = buf; '\0' != *s; s++) {
-				if ('&' == *s) {
-					const gchar *endptr;
-					guint32 uc;
-
-					uc = html_decode_entity(s, &endptr);
-					if ((guint32) -1 != uc) {
-						guint n;
-
-						n = utf8_encode_char(uc, p, (endptr - s) + 1);
-						p += n;
-						s = endptr;
-						continue;
-					}
-				}
-				*p++ = *s;
-			}
-			*p = '\0';
-			
-			text_widget_append(GTK_WIDGET(text), lazy_utf8_to_ui_string(buf));
-		}
-
-		fclose(f);
-	} else {
-		static const gchar msg[] =
-		N_(	"The FAQ document could not be loaded. Please read the online FAQ "
-			"at http://gtk-gnutella.sourceforge.net/?page=faq instead.");
-		text_widget_append(GTK_WIDGET(text), _(msg));
-	}
-}
-
 /**
  * Searches for the gktrc file to use. Order in which they are scanned:
  *
@@ -601,7 +515,6 @@ gui_init_dlg_faq(void)
  *	- $HOME/.gtk/gtkrc
  *	- $HOME/.gtk1/gtkrc ($HOME/.gtk2/gtkrc if GTK2 interface is used)
  *	- $GTK_GNUTELLA_DIR/gtkrc
- *	- ./gtkrc
  *
  * Where the last one can overrule settings from earlier resource files.
  */
@@ -640,8 +553,6 @@ main_gui_gtkrc_init(void)
 	userrc = make_pathname(guc_settings_config_dir(), rcfn);
 	gtk_rc_parse(userrc);
 	G_FREE_NULL(userrc);
-
-	gtk_rc_parse("." G_DIR_SEPARATOR_S "gtkrc");
 }
 
 /***
@@ -655,13 +566,16 @@ main_gui_gtkrc_init(void)
  *      -- Richard, 6/9/2002
  */
 void
-main_gui_early_init(gint argc, gchar **argv)
+main_gui_early_init(gint argc, gchar **argv, gboolean disable_xshm)
 {
 	/* Glade inits */
 
 	gtk_set_locale();
 	gtk_init(&argc, &argv);
 
+	if (disable_xshm)
+		gdk_set_use_xshm(FALSE);
+
 	add_pixmap_directory(PRIVLIB_EXP G_DIR_SEPARATOR_S "pixmaps");
 #ifndef OFFICIAL_BUILD
 	add_pixmap_directory(PACKAGE_SOURCE_DIR G_DIR_SEPARATOR_S "pixmaps");
@@ -677,7 +591,7 @@ main_gui_early_init(gint argc, gchar **argv)
 	gui_init_dlg_about();
 
     gui_dlg_faq_set(create_dlg_faq());
-    gui_init_dlg_faq();
+    gui_dlg_ancient_set(create_dlg_ancient());
 
     gui_dlg_prefs_set(create_dlg_prefs());
 	gui_init_dlg_prefs();
@@ -807,13 +721,8 @@ main_gui_run(const gchar *geometry_spec)
         gui_main_window_lookup("label_statusbar_uptime"),
         8, 8);
 
-#ifdef USE_GTK2
-	g_signal_connect(GTK_OBJECT(gui_main_window_lookup("notebook_main")),
-		"switch-page", G_CALLBACK(on_notebook_main_switch_page), NULL);
-#else
-	gtk_signal_connect(GTK_OBJECT(gui_main_window_lookup("notebook_main")),
-		"switch-page", on_notebook_main_switch_page, NULL);
-#endif /* USE_GTK2 */
+	gui_signal_connect(gui_main_window_lookup("notebook_main"), "switch-page",
+		on_notebook_main_switch_page, NULL);
 
 	/*
 	 * Make sure the application starts in the Gnet pane.
