@@ -1,5 +1,5 @@
 /*
- * $Id: main_cb.c 12195 2006-11-03 02:20:22Z cbiere $
+ * $Id: main_cb.c 13180 2007-03-21 17:46:22Z cbiere $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi, Richard Eckart
  *
@@ -25,22 +25,83 @@
 
 #include "gui.h"
 
-RCSID("$Id: main_cb.c 12195 2006-11-03 02:20:22Z cbiere $")
+RCSID("$Id: main_cb.c 13180 2007-03-21 17:46:22Z cbiere $")
 
-#include "main_cb.h"
+#include "html_view.h"
 #include "main.h"
+#include "main_cb.h"
 #include "misc.h"
 #include "notebooks.h"
+#include "settings.h"
 
 #include "if/gui_property.h"
 #include "if/bridge/ui2c.h"
 
+#include "lib/file.h"
+#include "lib/utf8.h"
+
 #include "lib/override.h"	/* Must be the last header included */
 
 /***
  *** Private functions
  ***/
 
+static struct html_view *faq_html_view;
+
+static void
+load_faq(void)
+{
+	static const gchar faq_file[] = "FAQ";
+	static file_path_t fp[4];
+	GtkWidget *textview;
+	const gchar *lang;
+	guint i = 0;
+	FILE *f;
+
+	html_view_free(&faq_html_view);
+
+	textview = gui_dlg_faq_lookup("textview_faq");
+	lang = locale_get_language();
+
+	file_path_set(&fp[i++], make_pathname(PRIVLIB_EXP, lang), faq_file);
+	file_path_set(&fp[i++], PRIVLIB_EXP G_DIR_SEPARATOR_S "en", faq_file);
+	
+#ifndef OFFICIAL_BUILD
+	file_path_set(&fp[i++],
+		make_pathname(PACKAGE_EXTRA_SOURCE_DIR, lang), faq_file);
+
+	file_path_set(&fp[i++],
+		PACKAGE_EXTRA_SOURCE_DIR G_DIR_SEPARATOR_S "en", faq_file);
+#endif /* !OFFICIAL_BUILD */
+
+	g_assert(i <= G_N_ELEMENTS(fp));
+
+	f = file_config_open_read_norename("FAQ", fp, i);
+	if (f) {
+		faq_html_view = html_view_load_file(textview, fileno(f));
+		fclose(f);
+	} else {
+		static const gchar msg[] =
+		N_(
+			"<html>"
+			"<head>"
+			"<title>Frequently Asked Questions</title>"
+			"</head>"
+			"<body>"
+			"<p>"
+			"The FAQ document could not be loaded. Please read the "
+			"<a href=\"http://gtk-gnutella.sourceforge.net/?page=faq\">"
+			"FAQ online</a> instead."
+			"</p>"
+			"</body>"
+			"</html>"
+		);
+
+		faq_html_view = html_view_load_memory(textview, array_from_string(msg));
+	}
+}
+
+
 static void
 quit(gboolean force)
 {
@@ -105,6 +166,7 @@ on_menu_faq_activate(GtkMenuItem *unused_menuitem, gpointer unused_udata)
 	(void) unused_udata;
 
 	g_return_if_fail(gui_dlg_faq());
+	load_faq();
     gtk_widget_show(gui_dlg_faq());
 	g_return_if_fail(gui_dlg_faq()->window);
 	gdk_window_raise(gui_dlg_faq()->window);
@@ -164,6 +226,17 @@ on_dlg_about_delete_event(GtkWidget *unused_widget, GdkEvent *unused_event,
 	return TRUE;
 }
 
+gboolean
+on_dlg_ancient_delete_event(GtkWidget *unused_widget, GdkEvent *unused_event,
+	gpointer unused_udata)
+{
+	(void) unused_widget;
+	(void) unused_event;
+	(void) unused_udata;
+
+	ancient_version_dialog_hide();
+	return TRUE;
+}
 /***
  *** FAQ dialog
  ***/
@@ -177,6 +250,7 @@ on_dlg_faq_delete_event(GtkWidget *unused_widget, GdkEvent *unused_event,
 
 	g_return_val_if_fail(gui_dlg_faq(), TRUE);
 
+	html_view_free(&faq_html_view);
 	gtk_widget_hide(gui_dlg_faq());
 	return TRUE;
 }
