@@ -1,5 +1,5 @@
 /*
- * $Id: main_cb.h 10844 2006-05-14 01:52:03Z cbiere $
+ * $Id: main_cb.h 13180 2007-03-21 17:46:22Z cbiere $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi, Richard Eckart
  *
@@ -58,12 +58,15 @@ void
 on_menu_keyboard_shortcuts_activate(GtkMenuItem *unused_menuitem,
 	gpointer unused_udata);
 
+void on_button_ancient_close_clicked(GtkButton *button, gpointer user_data);
+gboolean on_dlg_ancient_delete_event(
+    GtkWidget *widget, GdkEvent *event, gpointer user_data);
+
 /***
  *** FAQ dialog
  ***/
-void on_button_faq_close_clicked(GtkButton *button, gpointer user_data);
-gboolean on_dlg_faq_delete_event(
-    GtkWidget *widget, GdkEvent *event, gpointer user_data);
+gboolean on_dlg_faq_delete_event(GtkWidget *widget, GdkEvent *event,
+		gpointer user_data);
 
 /***
  *** Prefs dialog
