@@ -1,5 +1,5 @@
 /*
- * $Id: nodes_common.c,v 1.21 2005/09/11 03:40:17 daichik Exp $
+ * $Id: nodes_common.c 13777 2007-05-29 00:23:11Z cbiere $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi, Richard Eckart
  *
@@ -36,9 +36,10 @@
 
 #include "gui.h"
 
-RCSID("$Id: nodes_common.c,v 1.21 2005/09/11 03:40:17 daichik Exp $");
+RCSID("$Id: nodes_common.c 13777 2007-05-29 00:23:11Z cbiere $")
 
 #include "nodes_common.h"
+#include "settings.h"
 
 #include "gtk/statusbar.h"
 
@@ -81,7 +82,7 @@ nodes_gui_common_status_str(const gnet_node_status_t *n)
 		if (n->sent || n->received) {
 			size_t slen = 0;
 
-			if (!node_show_detailed_info) {
+			if (!GUI_PROPERTY(node_show_detailed_info)) {
 				gm_snprintf(gui_tmp, sizeof(gui_tmp),
 					"TX=%d RX=%d Q=%d,%d%% %s",
 					n->sent, n->received,
@@ -92,34 +93,38 @@ nodes_gui_common_status_str(const gnet_node_status_t *n)
 				break;
 			}
 
-			if (n->tx_compressed && show_gnet_info_txc)
+			if (n->tx_compressed && GUI_PROPERTY(show_gnet_info_txc))
 				slen += gm_snprintf(gui_tmp, sizeof(gui_tmp), "TXc=%d,%d%%",
 					(gint) n->sent, (gint) (n->tx_compression_ratio * 100));
 			else
 				slen += gm_snprintf(gui_tmp, sizeof(gui_tmp), "TX=%d",
 							(gint) n->sent);
 
-			if (show_gnet_info_tx_speed || show_gnet_info_tx_wire) {
+			if (
+				GUI_PROPERTY(show_gnet_info_tx_speed) ||
+				GUI_PROPERTY(show_gnet_info_tx_wire)
+			) {
 				gboolean is_first = TRUE;
 
 				slen += gm_snprintf(&gui_tmp[slen], sizeof(gui_tmp)-slen,
 					" (" /* ')' */);
 
-				if (show_gnet_info_tx_wire) {
+				if (GUI_PROPERTY(show_gnet_info_tx_wire)) {
 					slen += gm_snprintf(&gui_tmp[slen], sizeof(gui_tmp)-slen,
-						"%s", compact_size(n->tx_written));
+					"%s", compact_size(n->tx_written, show_metric_units()));
 					is_first = FALSE;
 				}
 
-				if (show_gnet_info_tx_speed)
+				if (GUI_PROPERTY(show_gnet_info_tx_speed))
 					slen += gm_snprintf(&gui_tmp[slen], sizeof(gui_tmp)-slen,
-						"%s%s", is_first ? "" : ", ", compact_rate(n->tx_bps));
+						"%s%s", is_first ? "" : ", ",
+						compact_rate(n->tx_bps, show_metric_units()));
 
 				slen += gm_snprintf(&gui_tmp[slen], sizeof(gui_tmp)-slen,
 					/* '(' */ ")");
 			}
 
-			if (n->rx_compressed && show_gnet_info_rxc)
+			if (n->rx_compressed && GUI_PROPERTY(show_gnet_info_rxc))
 				slen += gm_snprintf(&gui_tmp[slen], sizeof(gui_tmp)-slen,
 					" RXc=%d,%d%%",
 					(gint) n->received, (gint) (n->rx_compression_ratio * 100));
@@ -127,51 +132,57 @@ nodes_gui_common_status_str(const gnet_node_status_t *n)
 				slen += gm_snprintf(&gui_tmp[slen], sizeof(gui_tmp)-slen,
 					" RX=%d", (gint) n->received);
 
-			if (show_gnet_info_rx_speed || show_gnet_info_rx_wire) {
+			if (
+				GUI_PROPERTY(show_gnet_info_rx_speed) ||
+				GUI_PROPERTY(show_gnet_info_rx_wire)
+			) {
 				gboolean is_first = TRUE;
 
 				slen += gm_snprintf(&gui_tmp[slen], sizeof(gui_tmp)-slen,
 					" (" /* ')' */);
 
-				if (show_gnet_info_rx_wire) {
+				if (GUI_PROPERTY(show_gnet_info_rx_wire)) {
 					slen += gm_snprintf(&gui_tmp[slen], sizeof(gui_tmp)-slen,
-						"%s", compact_size(n->rx_given));
+						"%s", compact_size(n->rx_given, show_metric_units()));
 					is_first = FALSE;
 				}
 
-				if (show_gnet_info_rx_speed)
+				if (GUI_PROPERTY(show_gnet_info_rx_speed))
 					slen += gm_snprintf(&gui_tmp[slen], sizeof(gui_tmp)-slen,
-						"%s%s", is_first ? "" : ", ", compact_rate(n->rx_bps));
+						"%s%s", is_first ? "" : ", ",
+						compact_rate(n->rx_bps, show_metric_units()));
 
 				slen += gm_snprintf(&gui_tmp[slen], sizeof(gui_tmp)-slen,
 					/* '(' */ ")");
 			}
 
 			if (
-				show_gnet_info_tx_queries || show_gnet_info_rx_queries ||
-				show_gnet_info_gen_queries || show_gnet_info_sq_queries
+				GUI_PROPERTY(show_gnet_info_tx_queries) ||
+				GUI_PROPERTY(show_gnet_info_rx_queries) ||
+				GUI_PROPERTY(show_gnet_info_gen_queries) ||
+				GUI_PROPERTY(show_gnet_info_sq_queries)
 			) {
 				gboolean is_first = TRUE;
 
 				slen += gm_snprintf(&gui_tmp[slen], sizeof(gui_tmp)-slen,
 					" Query(" /* ')' */);
 
-				if (show_gnet_info_gen_queries) {
+				if (GUI_PROPERTY(show_gnet_info_gen_queries)) {
 					slen += gm_snprintf(&gui_tmp[slen], sizeof(gui_tmp)-slen,
 						"Gen=%d", n->squeue_sent);
 					is_first = FALSE;
 				}
-				if (show_gnet_info_sq_queries) {
+				if (GUI_PROPERTY(show_gnet_info_sq_queries)) {
 					slen += gm_snprintf(&gui_tmp[slen], sizeof(gui_tmp)-slen,
 						"%sQ=%d", is_first ? "" : ", ", n->squeue_count);
 					is_first = FALSE;
 				}
-				if (show_gnet_info_tx_queries) {
+				if (GUI_PROPERTY(show_gnet_info_tx_queries)) {
 					slen += gm_snprintf(&gui_tmp[slen], sizeof(gui_tmp)-slen,
 						"%sTX=%u", is_first ? "" : ", ", n->tx_queries);
 					is_first = FALSE;
 				}
-				if (show_gnet_info_rx_queries)
+				if (GUI_PROPERTY(show_gnet_info_rx_queries))
 					slen += gm_snprintf(&gui_tmp[slen], sizeof(gui_tmp)-slen,
 						"%sRX=%u", is_first ? "" : ", ", n->rx_queries);
 
@@ -179,18 +190,21 @@ nodes_gui_common_status_str(const gnet_node_status_t *n)
 					/* '(' */ ")");
 			}
 
-			if (show_gnet_info_tx_hits || show_gnet_info_rx_hits) {
+			if (
+				GUI_PROPERTY(show_gnet_info_tx_hits) ||
+				GUI_PROPERTY(show_gnet_info_rx_hits)
+			) {
 				gboolean is_first = TRUE;
 
 				slen += gm_snprintf(&gui_tmp[slen], sizeof(gui_tmp)-slen,
 					" QHit(" /* ')' */);
 
-				if (show_gnet_info_tx_hits) {
+				if (GUI_PROPERTY(show_gnet_info_tx_hits)) {
 					slen += gm_snprintf(&gui_tmp[slen], sizeof(gui_tmp)-slen,
 						"TX=%u", n->tx_qhits);
 					is_first = FALSE;
 				}
-				if (show_gnet_info_rx_hits)
+				if (GUI_PROPERTY(show_gnet_info_rx_hits))
 					slen += gm_snprintf(&gui_tmp[slen], sizeof(gui_tmp)-slen,
 						"%sRX=%u", is_first ? "" : ", ", n->rx_qhits);
 
@@ -198,18 +212,21 @@ nodes_gui_common_status_str(const gnet_node_status_t *n)
 					/* '(' */ ")");
 			}
 
-			if (show_gnet_info_tx_dropped || show_gnet_info_rx_dropped) {
+			if (
+				GUI_PROPERTY(show_gnet_info_tx_dropped) ||
+				GUI_PROPERTY(show_gnet_info_rx_dropped)
+			) {
 				gboolean is_first = TRUE;
 
 				slen += gm_snprintf(&gui_tmp[slen], sizeof(gui_tmp)-slen,
 					" Drop(" /* ')' */);
 
-				if (show_gnet_info_tx_dropped) {
+				if (GUI_PROPERTY(show_gnet_info_tx_dropped)) {
 					slen += gm_snprintf(&gui_tmp[slen], sizeof(gui_tmp)-slen,
 						"TX=%u", n->tx_dropped);
 					is_first = FALSE;
 				}
-				if (show_gnet_info_rx_dropped)
+				if (GUI_PROPERTY(show_gnet_info_rx_dropped))
 					slen += gm_snprintf(&gui_tmp[slen], sizeof(gui_tmp)-slen,
 						"%sRX=%u", is_first ? "" : ", ", n->rx_dropped);
 
@@ -217,27 +234,37 @@ nodes_gui_common_status_str(const gnet_node_status_t *n)
 					/* '(' */ ")");
 			}
 
-			if (show_gnet_info_shared_size || show_gnet_info_shared_files) {
+			if (
+				GUI_PROPERTY(show_gnet_info_shared_size) ||
+				GUI_PROPERTY(show_gnet_info_shared_files)
+			) {
 				gboolean is_first = TRUE;
 
 				slen += gm_snprintf(&gui_tmp[slen], sizeof(gui_tmp)-slen,
 					" Lib(" /* ')' */);
 
-				if (show_gnet_info_shared_size && n->gnet_info_known) {
+				if (
+					GUI_PROPERTY(show_gnet_info_shared_size) &&
+					n->gnet_info_known
+				) {
 					slen += gm_snprintf(&gui_tmp[slen], sizeof(gui_tmp)-slen,
-						"%s", compact_kb_size(
-							n->gnet_files_count ? n->gnet_kbytes_count : 0));
+						"%s",
+						compact_kb_size(n->gnet_files_count
+							? n->gnet_kbytes_count : 0, show_metric_units()));
 					is_first = FALSE;
 				}
-				if (show_gnet_info_shared_files && n->gnet_info_known)
+				if (
+					GUI_PROPERTY(show_gnet_info_shared_files) &&
+					n->gnet_info_known
+				) {
 					slen += gm_snprintf(&gui_tmp[slen], sizeof(gui_tmp)-slen,
 						"%s#=%u", is_first ? "" : ", ", n->gnet_files_count);
-
+				}
 				slen += gm_snprintf(&gui_tmp[slen], sizeof(gui_tmp)-slen,
 					/* '(' */ "%s)", n->gnet_info_known ? "" : "?");
 			}
 
-			if (show_gnet_info_qrp_stats) {
+			if (GUI_PROPERTY(show_gnet_info_qrp_stats)) {
 				if (n->has_qrp)
 					slen += gm_snprintf(&gui_tmp[slen], sizeof(gui_tmp)-slen,
 						" QRP=%u%%",
@@ -246,17 +273,19 @@ nodes_gui_common_status_str(const gnet_node_status_t *n)
 				if (n->qrt_slots != 0)
 					slen += gm_snprintf(&gui_tmp[slen], sizeof(gui_tmp)-slen,
 						" QRT(%s, g=%d, f=%d%%, t=%d%%, e=%d%%)",
-						compact_size(n->qrt_slots), n->qrt_generation,
+						compact_size(n->qrt_slots, show_metric_units()),
+						n->qrt_generation,
 						n->qrt_fill_ratio, n->qrt_pass_throw,
 						(guint) (n->qrp_efficiency * 100.0));
 			}
 
-			if (show_gnet_info_dbw)
+			if (GUI_PROPERTY(show_gnet_info_dbw))
 				slen += gm_snprintf(&gui_tmp[slen], sizeof(gui_tmp)-slen,
-				" Dup=%u Bad=%u W=%u H=%u",
-				n->n_dups, n->n_bad, n->n_weird, n->n_hostile);
+				" Dup=%u Bad=%u W=%u H=%u S=%u E=%u",
+				n->n_dups, n->n_bad, n->n_weird,
+				n->n_hostile, n->n_spam, n->n_evil);
 
-			if (show_gnet_info_rt) {
+			if (GUI_PROPERTY(show_gnet_info_rt)) {
 				slen += gm_snprintf(&gui_tmp[slen], sizeof(gui_tmp)-slen,
 				" RT(avg=%d, last=%d", n->rt_avg, n->rt_last);	/* ) */
 				if (n->tcp_rtt)
@@ -304,109 +333,41 @@ nodes_gui_common_status_str(const gnet_node_status_t *n)
 	return a;
 }
 
-/**
- * Display a summary of the node flags.
- *
- * The stuff in the Flags column means:
- *
- *  - 012345678AB (offset)
- *  - NIrwqxZPFhE
- *  - ^^^^^^^^^^^
- *  - ||||||||||+ E indicates a TLS encrypted connection
- *  - |||||||||+  hops flow triggerd (h), or total query flow control (f)
- *  - ||||||||+   flow control (F), or pending data in queue (d)
- *  - |||||||+    indicates whether we're a push proxy (P) / node is proxy (p)
- *  - ||||||+     indicates whether RX, TX or both (Z) are compressed
- *  - |||||+      indicates whether we sent our last-hop QRT to remote UP
- *  - ||||+       indicates whether we sent/received a QRT, or send/receive one
- *  - |||+        indicates whether node is writable
- *  - ||+         indicates whether node is readable
- *  - |+          indicates connection type (Incoming, Outgoing, Ponging)
- *  - +           indicates peer mode (Normal, Ultra, Leaf)
- */
-const gchar *
-nodes_gui_common_flags_str(const gnet_node_flags_t *flags)
-{
-	static gchar status[] = "NIrwqTRPFhS";
-
-	switch (flags->peermode) {
-		case NODE_P_UNKNOWN:	status[0] = '-'; break;
-		case NODE_P_ULTRA:		status[0] = 'U'; break;
-		case NODE_P_NORMAL:		status[0] = 'N'; break;
-		case NODE_P_LEAF:		status[0] = 'L'; break;
-		case NODE_P_CRAWLER:	status[0] = 'C'; break;
-		case NODE_P_UDP:		status[0] = 'P'; break;
-		default:				g_assert(0); break;
-	}
-
-	status[1] = flags->incoming ? 'I' : 'O';
-	status[2] = flags->readable ? 'r' : '-';
-	status[3] = flags->writable ? 'w' : '-';
-
-	switch (flags->qrt_state) {
-		case QRT_S_SENT: case QRT_S_RECEIVED:		status[4] = 'Q'; break;
-		case QRT_S_SENDING: case QRT_S_RECEIVING:	status[4] = 'q'; break;
-		case QRT_S_PATCHING:						status[4] = 'p'; break;
-		default:									status[4] = '-';
-	}
-
-	switch (flags->uqrt_state) {
-		case QRT_S_SENT:		status[5] = 'X'; break;
-		case QRT_S_SENDING:		status[5] = 'x'; break;
-		case QRT_S_PATCHING:	status[5] = 'p'; break;
-		default:				status[5] = '-';
-	}
-
-	status[6] =
-		flags->tx_compressed && flags->rx_compressed ? 'Z' :
-		flags->tx_compressed ? 'T' :
-		flags->rx_compressed ? 'R' : '-';
-
-	if (flags->is_push_proxied)  status[7] = 'P';
-	else if (flags->is_proxying) status[7] = 'p';
-	else status[7] = '-';
-
-	if (flags->in_tx_swift_control) status[8]     = 'S';
-	else if (flags->in_tx_flow_control) status[8] = 'F';
-	else if (flags->mqueue_above_lowat) status[8] = 'D';
-	else if (!flags->mqueue_empty) status[8]      = 'd';
-	else status[8]                                = '-';
-
-	if (flags->hops_flow == 0)
-		status[9] = 'f';
-	else if (flags->hops_flow < GTA_NORMAL_TTL)
-		status[9] = 'h';
-	else
-		status[9] = '-';
-
-	status[10] = flags->tls ? 'E' : '-';
-
-	status[sizeof(status) - 1] = '\0';
-	return status;
-}
-
 struct add_node_context {
 	guint32 flags;
 	guint16 port;
 };
 
 static void
-add_node_helper(const host_addr_t *addr, gpointer data)
+add_node_helper(const host_addr_t *addrs, size_t n, gpointer data)
 {
 	struct add_node_context *ctx = data;
 
+	g_assert(addrs);
 	g_assert(ctx);
 	g_assert(0 != ctx->port);
 
-	if (addr)
-		guc_node_add(*addr, ctx->port, ctx->flags);
+	if (n > 0) {
+		guc_node_add(addrs[random_raw() % n], ctx->port, ctx->flags);
+	}
 
 	wfree(ctx, sizeof *ctx);
 }
 
 /**
- * Try to connect to the node given by the addr string in the form
- * [ip]:[port]. Port may be omitted.
+ * Try to connect to the list of nodes given by in following form:
+ *
+ * list = <node> | <node>, 1*<node>
+ * port = 1..65535
+ * hostname = 1*[a-zA-Z0-9.-]
+ * node = hostname [":" <port>]
+ *       | <IPv4 address>[":" <port>]
+ *		 | <IPv6 address>
+ *		 | "[" <IPv6 address> "]:" <port>
+ * peer = ["tls:"]<node>
+ *
+ * If the port is omitted, the default port (GTA_PORT: 6346) is used.
+ * The case-insensitive prefix "tls:" requests a TLS (encrypted) connection.
  */
 void
 nodes_gui_common_connect_by_name(const gchar *line)
@@ -431,14 +392,14 @@ nodes_gui_common_connect_by_name(const gchar *line)
 
 		addr = zero_host_addr;
 		port = GTA_PORT;
-		flags = CONNECT_F_FORCE;
+		flags = SOCK_F_FORCE;
 		endptr = NULL;
 		hostname = NULL;
 		hostname_len = 0;
 
 		endptr = is_strcaseprefix(q, "tls:");
 		if (endptr) {
-			flags |= CONNECT_F_TLS;
+			flags |= SOCK_F_TLS;
 			q = endptr;
 		}
 
@@ -455,16 +416,14 @@ nodes_gui_common_connect_by_name(const gchar *line)
 		q = endptr;
 
 		if (':' == *q) {
-			guint32 v;
 			gint error;
 
-			v = parse_uint32(&q[1], &endptr, 10, &error);
-			if (error || 0 == v || v > 0xffff) {
+			port = parse_uint16(&q[1], &endptr, 10, &error);
+			if (error || 0 == port) {
 				g_message("Cannot parse port");
 				break;
 			}
 
-			port = v;
 			q = skip_ascii_spaces(endptr);
 		} else {
 			q = skip_ascii_spaces(endptr);
