@@ -1,5 +1,5 @@
 /*
- * $Id: search.h 11712 2006-08-23 22:06:25Z cbiere $
+ * $Id: search.h 13868 2007-06-18 01:30:56Z cbiere $
  *
  * Copyright (c) 2001-2003, Richard Eckart
  *
@@ -32,6 +32,7 @@
 #include "if/core/search.h"
 #include "if/core/bitzi.h"
 #include "lib/hashlist.h"
+#include "lib/slist.h"
 #include "lib/prop.h"
 
 /**
@@ -40,15 +41,10 @@
 struct search {
     gnet_search_t search_handle;	/**< Search handle */
 
-	gchar      *query;				/**< The query string; always UTF-8 */
-	gboolean    enabled;
-
 #ifdef USE_GTK2
-	GtkWidget  *tree_view;			/**< GtkTreeView for this search */
-	GtkTreeModel	*model;			/**< GtkTreeModel for the GtkTreeView
-										 so it can be detached from it */
+	GtkTreeView  *tree;				/**< GtkTreeView for this search */
 #else
-	GtkCTree   *ctree;			   	/**< GtkCTree for this search */
+	GtkCTree   *tree;			   	/**< GtkCTree for this search */
 #endif /* USE_GTK2 */
 
 	GHashTable *parents;			/**< table of mount iterators for
@@ -66,17 +62,12 @@ struct search {
 	gint        tab_updating;		/**< ID for timeout func. to be cancelled */
 	guint32     unseen_items;		/**< How many items haven't been seen yet */
 
-	gboolean    passive;			/**< Is this a passive search? */
-	gboolean    massive_update;		/**< massive update in process */
-	gboolean    browse;				/**< Is this a browse-host search? */
-	gboolean    local;				/**< Is this a local search? */
-
-	hash_list_t *r_sets;			/**< The results sets of this search */
-
 	GHashTable *dups;				/**< keep a record of dups. */
 
     filter_t   *filter;				/**< filter ruleset bound to this search */
 
+	slist_t	*queue;					/**< records to be inserted */
+
 	/*
 	 * Search stats.
 	 */
@@ -129,8 +120,7 @@ void search_gui_discard_files(void);
 
 void search_gui_sort_column(search_t *search, gint column);
 
-void search_gui_add_record(
-	search_t *sch, record_t *rc, GString *vinfo, GdkColor *fg, GdkColor *bg);
+void search_gui_add_record(search_t *sch, record_t *rc, enum gui_color color);
 
 gboolean gui_search_update_tab_label(struct search *);
 void gui_search_clear_results(void);
@@ -142,7 +132,6 @@ const record_t *search_gui_get_record_at_path(GtkTreeView *tv,
 					GtkTreePath *path);
 #else
 void gui_search_force_update_tab_label(struct search *);
-GtkCTreeNode *find_parent_with_sha1(GHashTable *ht, gpointer key);
 #endif /* USE_GTK2 */
 
 void search_gui_expand_all(void);
