@@ -1,5 +1,5 @@
 /*
- * $Id: search_common.h 12268 2006-11-09 00:55:57Z cbiere $
+ * $Id: search_common.h 14002 2007-07-01 14:26:27Z cbiere $
  *
  * Copyright (c) 2003, Raphael Manfredi
  *
@@ -34,6 +34,10 @@ void search_remove_got_results_listener(search_got_results_listener_t l);
 
 #define TAB_UPDATE_TIME	5		/**< Update search tabs after 5 seconds */
 
+typedef enum {
+	RESULTS_SET_MAGIC = 0xa44eb853U
+} results_set_magic_t;
+
 /**
  * A results_set structure factorizes the common information from a Query Hit
  * packet, and then has a list of individual records, one for each hit.
@@ -49,36 +53,30 @@ void search_remove_got_results_listener(search_got_results_listener_t l);
  *     is purely descriptive anyway.
  */
 typedef struct results_set {
-	gint refcount;			/**< Number of "struct search" this belongs to */
+	results_set_magic_t magic;
+	gint num_recs;
 
-	gchar *guid;			/**< Servent's GUID (atom) */
-	host_addr_t addr;
-	guint8 hops;
-	guint8 ttl;
-	guint16 port;
-	guint16 speed;
-	guint32 status;				/**< Parsed status bits from trailer */
-	time_t  stamp;				/**< Reception time of the hit */
-	union vendor_code vcode;	/**< Vendor code */
-	gchar *version;				/**< Version information (atom) */
-	gint country;				/**< Country code -- encoded ISO3166 */
-	gnet_host_vec_t *proxies;	/**< Optional: known push proxies */
-	gchar *hostname;			/**< Optional: server's hostname (atom) */
-	gchar *query;				/**< Optional: original query (atom) */
-	host_addr_t last_hop;		/**< IP of delivering node */
+	const gchar *guid;			/**< Servent's GUID (atom) */
+	const gchar *version;		/**< Version information (atom) */
+	const gchar *hostname;		/**< Optional: server's hostname (atom) */
+	const gchar *query;			/**< Optional: original query (atom) */
 
-	guint32 num_recs;
 	GSList *records;
     GSList *schl;
-} results_set_t;
+	gnet_host_vec_t *proxies;	/**< Optional: known push proxies */
 
-/**
- * Host vector held in query hits.
- */
-typedef struct host_vec {
-	gnet_host_t *hvec;			/**< Vector of alternate locations */
-	gint hvcnt;					/**< Amount of hosts in vector */
-} host_vec_t;
+	host_addr_t addr;
+	host_addr_t last_hop;		/**< IP of delivering node */
+	time_t  stamp;				/**< Reception time of the hit */
+
+	vendor_code_t vcode;		/**< Vendor code */
+	guint32 status;				/**< Parsed status bits from trailer */
+	guint16 country;			/**< Country code -- encoded ISO3166 */
+	guint16 port;
+	guint16 speed;
+	guint8 hops;
+	guint8 ttl;
+} results_set_t;
 
 typedef enum {
 	RECORD_MAGIC = 0x3fb9c04e
@@ -94,25 +92,44 @@ typedef enum {
  * many different hash tables (one per search).
  */
 typedef struct record {
-	results_set_t *results_set;	/**< Parent, containing record */
-	gint refcount;				/**< Number of hash tables it has been put to */
 	record_magic_t magic;		/**< Magic ID */
+	gint refcount;				/**< Number of hash tables it has been put to */
 
-	gchar  *name;				/**< Filename (atom) */
-	gchar  *ext;				/**< File extension (atom) */
-	gchar  *utf8_name;			/**< Filename converted to UTF-8 (atom) */
+	results_set_t *results_set;	/**< Parent, containing record */
+	const gchar *name;			/**< Filename (atom) */
+	const gchar *ext;			/**< File extension (atom) */
+	const gchar *utf8_name;		/**< Path/Filename converted to UTF-8 (atom) */
 	const gchar *charset;		/**< Detected charset of name (static const) */
-	gchar  *sha1;				/**< SHA1 URN (binary form, atom) */
-	gchar  *xml;				/**< Optional XML data string (atom) */
-	gchar  *tag;				/**< Optional tag data string (atom) */
-	gchar  *info;				/**< Short version of tag (atom) */
-	gchar  *path;				/**< Optional path (atom) */
+	const struct sha1 *sha1;	/**< SHA1 URN (binary form, atom) */
+	const gchar *xml;			/**< Optional XML data string (atom) */
+	const gchar *tag;			/**< Optional tag data string (atom) */
+	const gchar *info;			/**< Short version of tag (atom) */
+	const gchar *path;			/**< Optional path (atom) */
 	gnet_host_vec_t *alt_locs;	/**< Optional alternate locations for record */
 	filesize_t size;			/**< Size of file, in bytes */
+	time_t  create_time;		/**< Create Time of file; zero if unknown */
 	guint32 file_index;			/**< Index for GET command */
     flag_t  flags;              /**< same flags as in gnet_record_t */
 } record_t;
 
+static inline void
+record_check(const record_t * const rc)
+{
+	g_assert(rc);
+	g_assert(rc->magic == RECORD_MAGIC);
+	g_assert(rc->refcount >= 0);
+	g_assert(rc->refcount < INT_MAX);
+}
+
+static inline void
+results_set_check(const results_set_t * const rs)
+{
+	g_assert(rs);
+	g_assert(rs->magic == RESULTS_SET_MAGIC);
+	g_assert(rs->num_recs >= 0);
+	g_assert(rs->num_recs < INT_MAX);
+}
+
 struct query {
 	gchar *text;
 	GList *rules;
@@ -120,15 +137,22 @@ struct query {
 };
 
 enum gui_color {
-	GUI_COLOR_SPAM,
-	GUI_COLOR_HOSTILE,
-	GUI_COLOR_UNREQUESTED,
+	GUI_COLOR_DEFAULT,
 	GUI_COLOR_DOWNLOADING,
+	GUI_COLOR_HOSTILE,
+	GUI_COLOR_IGNORED,
+	GUI_COLOR_MARKED,
+	GUI_COLOR_MAYBE_SPAM,
+	GUI_COLOR_SPAM,
+	GUI_COLOR_UNREQUESTED,
+
 	NUM_GUI_COLORS
 };
 
-void gui_color_init(void);
+
+void gui_color_init(GtkWidget *widget);
 GdkColor *gui_color_get(enum gui_color id);
+void gui_color_set(const enum gui_color id, GdkColor *color);
 void gui_search_get_colors(search_t *sch,
 		GdkColor **mark_color, GdkColor **ignore_color);
 
@@ -146,25 +170,13 @@ void search_gui_set_current_search(search_t *sch);
 void search_gui_forget_current_search(void);
 void search_gui_current_search(search_t *sch);
 
-void search_gui_free_alt_locs(record_t *rc);
-void search_gui_host_vec_free(gnet_host_vec_t *v);
-gnet_host_vec_t *search_gui_proxies_clone(gnet_host_vec_t *v);
-void search_gui_free_proxies(results_set_t *rs);
-void search_gui_clean_r_set(results_set_t *rs);
-void search_gui_free_r_set(results_set_t *rs);
-void search_gui_dispose_results(results_set_t *rs);
 void search_gui_ref_record(record_t *rc);
 void search_gui_unref_record(record_t *rc);
-void search_gui_free_r_sets(search_t *sch);
 guint search_gui_hash_func(gconstpointer key);
 gint search_gui_hash_key_compare(gconstpointer a, gconstpointer b);
-void search_gui_remove_r_set(search_t *sch, results_set_t *rs);
-gboolean search_gui_result_is_dup(search_t *sch, record_t *rc);
-const gchar * search_gui_get_route(const record_t *rc);
+const gchar *search_gui_get_route(const struct results_set *rs);
 search_t *search_gui_find(gnet_search_t sh);
-gchar *search_gui_get_filename_extension(const gchar *filename_utf8);
-record_t *search_gui_create_record(results_set_t *rs, gnet_record_t *r) ;
-void search_gui_check_alt_locs(results_set_t *rs, record_t *rc);
+const gchar *search_gui_get_filename_extension(const gchar *filename_utf8);
 void search_gui_set_sort_defaults(void);
 void search_gui_store_searches(void);
 void search_gui_retrieve_searches(void);
@@ -190,15 +202,21 @@ void search_gui_option_menu_searches_update(void);
 void search_gui_option_menu_searches_select(const search_t *sch);
 
 void search_gui_browse_selected(void);
-gboolean search_gui_handle_magnet(const gchar *url, const gchar **error_str);
-gboolean search_gui_handle_http(const gchar *url, const gchar **error_str);
-gboolean search_gui_handle_urn(const gchar *url, const gchar **error_str);
+gboolean search_gui_insert_query(const gchar *text);
 
 gchar *search_xml_indent(const gchar *s);
 
-void on_option_menu_search_changed(GtkOptionMenu *option_menu, gpointer unused_udata);
+void on_option_menu_search_changed(GtkOptionMenu *option_menu,
+			gpointer unused_udata);
 
-gint search_gui_cmp_sha1s(const gchar *a, const gchar *b);
+void on_spinbutton_search_reissue_timeout_activate(GtkEditable *editable,
+			gpointer user_data);
+gboolean on_spinbutton_search_reissue_timeout_focus_out_event(GtkWidget *widget,
+			GdkEventFocus *event, gpointer user_data);
+void on_spinbutton_search_reissue_timeout_changed(GtkEditable *editable,
+			gpointer unused_udata);
+
+gint search_gui_cmp_sha1s(const struct sha1 *a, const struct sha1 *b);
 
 void search_gui_duplicate_search(search_t *search);
 void search_gui_restart_search(search_t *search);
@@ -272,6 +290,24 @@ GSList *search_gui_get_selected_searches(void);
 gboolean search_gui_has_selected_item(search_t *search);
 void search_gui_refresh_popup(void);
 void search_gui_search_list_clicked(GtkWidget *widget, GdkEventButton *event);
+void search_gui_flush_queues(void);
+
+const gchar *search_gui_query(const search_t *search);
+gboolean search_gui_is_browse(const search_t *search);
+gboolean search_gui_is_enabled(const search_t *search);
+gboolean search_gui_is_local(const search_t *search);
+gboolean search_gui_is_passive(const search_t *search);
+
+void search_gui_download(record_t *rc);
+const gchar *search_gui_nice_size(const record_t *rc);
+const gchar *search_gui_get_vendor(const struct results_set *rs);
+void search_gui_set_details(const record_t *rc);
+
+/* FIXME: This does not belong here. */
+gchar *gnet_host_vec_to_string(const gnet_host_vec_t *hvec);
+
+void search_gui_clear_details(void);
+void search_gui_append_detail(const gchar *title, const gchar *value);
 
 #endif /* _gtk_search_common_h_ */
 
