@@ -1,5 +1,5 @@
 /*
- * $Id: search_xml.c 12141 2006-10-27 01:17:39Z cbiere $
+ * $Id: search_xml.c 13777 2007-05-29 00:23:11Z cbiere $
  *
  * Copyright (c) 2002-2003, Richard Eckart
  *
@@ -35,7 +35,7 @@
 
 #include "gui.h"
 
-RCSID("$Id: search_xml.c 12141 2006-10-27 01:17:39Z cbiere $")
+RCSID("$Id: search_xml.c 13777 2007-05-29 00:23:11Z cbiere $")
 
 #include <libxml/tree.h>
 #include <libxml/parser.h>
@@ -340,7 +340,7 @@ target_to_string(filter_t *target)
 
 	if (!target_map) {
 		target_new_id(TRUE); /* Reset */
-		target_map = g_hash_table_new(NULL, NULL);
+		target_map = g_hash_table_new(pointer_hash_func, NULL);
 	}
 
 	if (!g_hash_table_lookup_extended(target_map, target, NULL, &value)) {
@@ -455,7 +455,7 @@ search_store_xml(void)
     } else {
 		gchar *filename;
 
-        if (gui_debug >= 3)
+        if (GUI_PROPERTY(gui_debug) >= 3)
             g_message("saved searches file: %s", filename_new);
 
 		filename = make_pathname(settings_gui_config_dir(), search_file_xml);
@@ -560,7 +560,7 @@ search_retrieve_xml(void)
 	}
 	G_FREE_NULL(path_orig);
 
-    id_map = g_hash_table_new(NULL, NULL);
+    id_map = g_hash_table_new(pointer_hash_func, NULL);
 
     /*
      * find nodes and add them to the list, this just
@@ -577,7 +577,7 @@ search_retrieve_xml(void)
      * shadows.
      */
 
-    if (gui_debug >= 6)
+    if (GUI_PROPERTY(gui_debug) >= 6)
         g_message("resolving UIDs");
 
     for (f = filters; f != NULL; f = f_next) {
@@ -588,7 +588,7 @@ search_retrieve_xml(void)
 
 		f_next = g_list_next(f);
 
-        if (gui_debug >= 6) {
+        if (GUI_PROPERTY(gui_debug) >= 6) {
             g_message("\n\nresolving on filter:");
             dump_filter(filter);
         }
@@ -618,7 +618,7 @@ search_retrieve_xml(void)
                  * We circumvent the shadows, so we must do refcounting
                  * manually here.
                  */
-                if (gui_debug >= 7) {
+                if (GUI_PROPERTY(gui_debug) >= 7) {
                     g_message("increasing refcount on \"%s\" to %d",
                         rule->target->name, rule->target->refcount + 1);
 				}
@@ -627,7 +627,7 @@ search_retrieve_xml(void)
             }
         }
 
-        if (gui_debug >= 6) {
+        if (GUI_PROPERTY(gui_debug) >= 6) {
 			g_message("resolved filter:");
             dump_filter(filter);
         }
@@ -648,17 +648,17 @@ search_retrieve_xml(void)
         gboolean borked = FALSE;
         const GList *s;
 
-        if (gui_debug >= 6)
+        if (GUI_PROPERTY(gui_debug) >= 6)
             g_message("verifying bindings...");
 
         for (s = search_gui_get_searches(); s != NULL; s = g_list_next(s)) {
             search_t *search = s->data;
 
             if (search->filter->search == search) {
-                if (gui_debug >= 6)
-                    g_message("binding ok for: %s", search->query);
+                if (GUI_PROPERTY(gui_debug) >= 6)
+                    g_message("binding ok for: %s", search_gui_query(search));
             } else {
-                g_warning("binding broken for: %s", search->query);
+                g_warning("binding broken for: %s", search_gui_query(search));
                 borked = TRUE;
             }
         }
@@ -715,27 +715,28 @@ search_to_xml(xmlNodePtr parent, search_t *s)
     GList *iter;
 
     g_assert(s != NULL);
-    g_assert(s->query != NULL);
+    g_assert(search_gui_query(s) != NULL);
     g_assert(parent != NULL);
 
-	if (s->browse || s->local)
+	if (search_gui_is_browse(s) || search_gui_is_local(s))
 		return;			/* Don't persist "browse host" searches. */
 
-    if (gui_debug >= 6) {
+    if (GUI_PROPERTY(gui_debug) >= 6) {
         g_message(
 			"saving search: %s (%p enabled=%d)\n"
 			"  -- filter is bound to: %p\n"
 			"  -- search is         : %p",
-			s->query, cast_to_gconstpointer(s), s->enabled,
+			search_gui_query(s), cast_to_gconstpointer(s),
+			search_gui_is_enabled(s),
 			cast_to_gconstpointer(s->filter->search),
 			cast_to_gconstpointer(s));
     }
 
     newxml = xml_new_empty_child(parent, NODE_SEARCH);
-    xml_prop_set(newxml, TAG_SEARCH_QUERY, s->query);
+    xml_prop_set(newxml, TAG_SEARCH_QUERY, search_gui_query(s));
 
-	xml_prop_printf(newxml, TAG_SEARCH_ENABLED, "%u", s->enabled);
-    xml_prop_printf(newxml, TAG_SEARCH_PASSIVE, "%u", TO_BOOL(s->passive));
+	xml_prop_printf(newxml, TAG_SEARCH_ENABLED, "%u", search_gui_is_enabled(s));
+    xml_prop_printf(newxml, TAG_SEARCH_PASSIVE, "%u", search_gui_is_passive(s));
     xml_prop_printf(newxml, TAG_SEARCH_REISSUE_TIMEOUT, "%u",
 		guc_search_get_reissue_timeout(s->search_handle));
     xml_prop_printf(newxml, TAG_SEARCH_CREATE_TIME, "%s",
@@ -764,12 +765,12 @@ filter_to_xml(xmlNodePtr parent, filter_t *f)
      * Don't store the builtin targets or bound rulesets
      */
     if (filter_is_builtin(f) || filter_is_bound(f)) {
-        if (gui_debug >= 7)
+        if (GUI_PROPERTY(gui_debug) >= 7)
             g_message("not saving bound/builtin: %s", f->name);
         return;
     }
 
-    if (gui_debug >= 6) {
+    if (GUI_PROPERTY(gui_debug) >= 6) {
 		g_message(
 			"saving filter: %s\n"
 			"  -- bound   : %p",
@@ -1096,7 +1097,7 @@ xml_to_search(xmlNodePtr xmlnode, gpointer unused_udata)
 	if (0 == lifetime && 0 == (flags & SEARCH_F_PASSIVE))
 		flags &= ~SEARCH_F_ENABLED;
 
-    if (gui_debug >= 4) {
+    if (GUI_PROPERTY(gui_debug) >= 4) {
         g_message("adding new %s %s search: %s",
 			(flags & SEARCH_F_ENABLED) ? "enabled" : "disabled",
 			(flags & SEARCH_F_PASSIVE) ? "passive" : "active",
@@ -1109,11 +1110,14 @@ xml_to_search(xmlNodePtr xmlnode, gpointer unused_udata)
 
 	xml_free_null(&query);
 
-    /*
-     * Also parse all children.
-     */
-	for (node = xmlnode->children; node != NULL; node = node->next)
-        parse_xml(node, search->filter);
+	if (search) {
+		/*
+		 * Also parse all children.
+		 */
+		for (node = xmlnode->children; node != NULL; node = node->next) {
+			parse_xml(node, search->filter);
+		}
+	}
 }
 
 static void
@@ -1163,7 +1167,7 @@ xml_to_filter(xmlNodePtr xmlnode, gpointer unused_udata)
 			goto failure;
         }
     } else {
-        if (gui_debug >= 4)
+        if (GUI_PROPERTY(gui_debug) >= 4)
             g_message("adding new filter: %s", name);
         filter = filter_new(name);
         filters = g_list_append(filters, filter);
@@ -1256,7 +1260,7 @@ xml_to_text_rule(xmlNodePtr xmlnode, gpointer data)
     rule = filter_new_text_rule(match, type, case_sensitive, target, flags);
     clear_flags(rule->flags, RULE_FLAG_VALID);
 
-    if (gui_debug >= 4) {
+    if (GUI_PROPERTY(gui_debug) >= 4) {
         g_message("added to filter \"%s\" rule with target %p",
             filter->name, cast_to_gconstpointer(rule->target));
 	}
@@ -1325,7 +1329,7 @@ xml_to_ip_rule(xmlNodePtr xmlnode, gpointer data)
     rule = filter_new_ip_rule(addr, mask, target, flags);
     clear_flags(rule->flags, RULE_FLAG_VALID);
 
-    if (gui_debug >= 4) {
+    if (GUI_PROPERTY(gui_debug) >= 4) {
         g_message("added to filter \"%s\" rule with target %p",
             filter->name, cast_to_gconstpointer(rule->target));
 	}
@@ -1389,7 +1393,7 @@ xml_to_size_rule(xmlNodePtr xmlnode, gpointer data)
     rule = filter_new_size_rule(lower, upper, target, flags);
     clear_flags(rule->flags, RULE_FLAG_VALID);
 
-    if (gui_debug >= 4) {
+    if (GUI_PROPERTY(gui_debug) >= 4) {
         g_message("added to filter \"%s\" rule with target %p",
             filter->name, cast_to_gconstpointer(rule->target));
 	}
@@ -1428,7 +1432,7 @@ xml_to_jump_rule(xmlNodePtr xmlnode, gpointer data)
     rule = filter_new_jump_rule(target,flags);
     clear_flags(rule->flags, RULE_FLAG_VALID);
 
-    if (gui_debug >= 4) {
+    if (GUI_PROPERTY(gui_debug) >= 4) {
         g_message("added to filter \"%s\" rule with target %p",
             filter->name, cast_to_gconstpointer(rule->target));
 	}
@@ -1442,7 +1446,7 @@ failure:
 static void
 xml_to_sha1_rule(xmlNodePtr xmlnode, gpointer data)
 {
-    const gchar *hash = NULL;
+    const struct sha1 *sha1 = NULL;
     gchar *filename = NULL;
     gchar *buf;
     rule_t *rule;
@@ -1462,14 +1466,14 @@ xml_to_sha1_rule(xmlNodePtr xmlnode, gpointer data)
 
     buf = STRTRACK(xml_get_string(xmlnode, TAG_RULE_SHA1_HASH));
     if (buf != NULL) {
-		hash = strlen(buf) == SHA1_BASE32_SIZE ? base32_sha1(buf) : NULL;
+		sha1 = strlen(buf) == SHA1_BASE32_SIZE ? base32_sha1(buf) : NULL;
 		xml_free_null(&buf);
-		if (!hash) {
+		if (!sha1) {
         	g_warning("xml_to_sha1_rule: Invalidly encoded SHA1");
 			return;
 		}
 	} else {
-		hash = NULL;
+		sha1 = NULL;
 	}
 
     buf = STRTRACK(xml_get_string(xmlnode, TAG_RULE_TARGET));
@@ -1482,10 +1486,10 @@ xml_to_sha1_rule(xmlNodePtr xmlnode, gpointer data)
 	xml_free_null(&buf);
 
     flags = get_rule_flags_from_xml(xmlnode);
-    rule = filter_new_sha1_rule(hash, filename, target, flags);
+    rule = filter_new_sha1_rule(sha1, filename, target, flags);
     clear_flags(rule->flags, RULE_FLAG_VALID);
 
-    if (gui_debug >= 4) {
+    if (GUI_PROPERTY(gui_debug) >= 4) {
         g_message("added to filter \"%s\" rule with target %p",
             filter->name, cast_to_gconstpointer(rule->target));
 	}
@@ -1565,7 +1569,7 @@ xml_to_flag_rule(xmlNodePtr xmlnode, gpointer data)
     rule = filter_new_flag_rule(stable, busy, push, target, flags);
     clear_flags(rule->flags, RULE_FLAG_VALID);
 
-    if (gui_debug >= 4) {
+    if (GUI_PROPERTY(gui_debug) >= 4) {
         g_message("added to filter \"%s\" rule with target %p",
             filter->name, cast_to_gconstpointer(rule->target));
 	}
@@ -1632,7 +1636,7 @@ xml_to_state_rule(xmlNodePtr xmlnode, gpointer data)
     rule = filter_new_state_rule(display, download, target, flags);
     clear_flags(rule->flags, RULE_FLAG_VALID);
 
-    if (gui_debug >= 4) {
+    if (GUI_PROPERTY(gui_debug) >= 4) {
         g_message("added to filter \"%s\" rule with target %p",
             filter->name, cast_to_gconstpointer(rule->target));
 	}
