@@ -1,5 +1,5 @@
 /*
- * $Id: settings_cb.c 12195 2006-11-03 02:20:22Z cbiere $
+ * $Id: settings_cb.c 13990 2007-06-30 00:59:50Z cbiere $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi, Richard Eckart
  *
@@ -36,7 +36,7 @@
 
 #include "gui.h"
 
-RCSID("$Id: settings_cb.c 12195 2006-11-03 02:20:22Z cbiere $")
+RCSID("$Id: settings_cb.c 13990 2007-06-30 00:59:50Z cbiere $")
 
 #include "gtk-missing.h"
 #include "misc.h"
@@ -69,44 +69,6 @@ RCSID("$Id: settings_cb.c 12195 2006-11-03 02:20:22Z cbiere $")
         CAT2(pref,_prop_set_boolean)(p, &val, 0, 1);                    \
     } while (0)
 
-void
-on_spinbutton_search_reissue_timeout_changed(GtkEditable *editable,
-		gpointer unused_udata)
-{
-    static gboolean lock = FALSE;
-    search_t *current_search;
-    guint32 timeout_real;
-    guint32 timeout;
-
-	(void) unused_udata;
-
-    if (lock)		/* Prevents recursion */
-        return;
-
-    lock = TRUE;
-
-    current_search = search_gui_get_current_search();
-
-    if (
-		!current_search ||
-		!guc_search_is_active(current_search->search_handle)
-	) {
-        lock = FALSE;
-        return;
-    }
-
-    timeout = gtk_spin_button_get_value(GTK_SPIN_BUTTON(editable));
-
-    guc_search_set_reissue_timeout(current_search->search_handle, timeout);
-    timeout_real = guc_search_get_reissue_timeout(
-						current_search->search_handle);
-
-    if (timeout != timeout_real)
-        gtk_spin_button_set_value(GTK_SPIN_BUTTON(editable), timeout_real);
-
-    lock = FALSE;
-}
-
 static void
 on_entry_config_proxy_hostname_activate_helper(const host_addr_t *addrs,
 		size_t n, gpointer unused_udata)
@@ -972,24 +934,9 @@ void
 on_menu_searchbar_visible_activate(GtkMenuItem *menuitem,
 	gpointer unused_udata)
 {
-	GtkWidget *viewport, *entry;
-	
 	(void) unused_udata;
-	
-	viewport = gui_main_window_lookup("viewport_searchbar");
-	entry = gui_main_window_lookup("entry_search");
 
-	if (GTK_WIDGET_VISIBLE(viewport)) {
-		if (GTK_WIDGET_HAS_FOCUS(entry))
-			gtk_widget_hide(viewport);
-	} else {
-		gtk_widget_show(viewport);
-	}
-
-	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(menuitem),
-		GTK_WIDGET_VISIBLE(viewport));
-	if (GTK_WIDGET_VISIBLE(viewport) && !GTK_WIDGET_HAS_FOCUS(entry))
-		gtk_widget_grab_focus(entry);
+	checkmenu_changed(gui, PROP_SEARCHBAR_VISIBLE, menuitem);
 }
 
 void
