@@ -1,5 +1,5 @@
 /*
- * $Id: upload_stats_cb.c,v 1.6 2005/09/11 03:40:17 daichik Exp $
+ * $Id: upload_stats_cb.c 13777 2007-05-29 00:23:11Z cbiere $
  *
  * Copyright (c) 2003, Raphael Manfredi
  * Copyright (c) 2001-2003, Richard Eckart
@@ -38,7 +38,7 @@
 
 #include "gui.h"
 
-RCSID("$Id: upload_stats_cb.c,v 1.6 2005/09/11 03:40:17 daichik Exp $");
+RCSID("$Id: upload_stats_cb.c 13777 2007-05-29 00:23:11Z cbiere $")
 
 #include "upload_stats_cb.h"
 #include "upload_stats.h"
@@ -192,7 +192,7 @@ on_clist_ul_stats_resize_column(GtkCList *unused_clist,
 	(void) unused_clist;
 	(void) unused_udata;
     /* FIXME: use properties */
-	*(gint *) &ul_stats_col_widths[column] = width;
+	*(gint *) &GUI_PROPERTY(ul_stats_col_widths)[column] = width;
 }
 
 #endif /* USE_GTK1 */
@@ -206,8 +206,8 @@ on_popup_upload_stats_config_cols_activate(GtkMenuItem *unused_menuitem,
 
 	(void) unused_menuitem;
 	(void) unused_udata;
-    cc = gtk_column_chooser_new(
-			lookup_widget(main_window, "treeview_ul_stats"));
+
+    cc = gtk_column_chooser_new(gui_main_window_lookup("treeview_ul_stats"));
     gtk_menu_popup(GTK_MENU(cc), NULL, NULL, NULL, NULL, 0, GDK_CURRENT_TIME);
 }
 #endif /* USE_GTK2 */
