@@ -1,5 +1,5 @@
 /*
- * $Id: uploads.h 10360 2005-12-11 13:52:52Z rmanfredi $
+ * $Id: uploads.h 13870 2007-06-18 03:27:25Z cbiere $
  *
  * Copyright (c) 2001-2003, Richard Eckart
  *
@@ -43,10 +43,10 @@ typedef struct upload_row_data {
     GtkTreeIter   iter;
 	host_addr_t   addr;
 	filesize_t    size;
+	const gchar   *user_agent;	/**< atom */
+	const gchar   *name;		/**< atom */
 	gboolean      push;
-	gchar         *user_agent;	/**< atom */
-	gchar         *name;		/**< atom */
-	gint          country;
+	guint16       country;
 #endif /* USE_GTK2 */
 } upload_row_data_t;
 
