@@ -1,5 +1,5 @@
 /*
- * $Id: uploads_cb.c 12195 2006-11-03 02:20:22Z cbiere $
+ * $Id: uploads_cb.c 13777 2007-05-29 00:23:11Z cbiere $
  *
  * Copyright (c) 2001-2003, Richard Eckart
  *
@@ -25,7 +25,7 @@
 
 #include "gui.h"
 
-RCSID("$Id: uploads_cb.c 12195 2006-11-03 02:20:22Z cbiere $")
+RCSID("$Id: uploads_cb.c 13777 2007-05-29 00:23:11Z cbiere $")
 
 #include "uploads_cb.h"
 #include "uploads.h"
@@ -126,7 +126,7 @@ on_clist_uploads_resize_column(GtkCList *unused_clist,
 	(void) unused_udata;
 
     /* FIXME: use properties */
-	*(gint *) &uploads_col_widths[column] = width;
+	*(gint *) &GUI_PROPERTY(uploads_col_widths)[column] = width;
 }
 
 void
