@@ -1,5 +1,5 @@
 /*
- * $Id: uploads_common.c,v 1.25 2005/12/11 13:52:51 rmanfredi Exp $
+ * $Id: uploads_common.c 13500 2007-05-05 17:54:08Z rmanfredi $
  *
  * Copyright (c) 2001-2003, Richard Eckart
  *
@@ -25,11 +25,12 @@
 
 #include "gui.h"
 
-RCSID("$Id: uploads_common.c,v 1.25 2005/12/11 13:52:51 rmanfredi Exp $");
+RCSID("$Id: uploads_common.c 13500 2007-05-05 17:54:08Z rmanfredi $")
 
 #include "uploads.h"			/* For upload_row_data_t */
 #include "uploads_common.h"
 #include "search_common.h"
+#include "settings.h"
 
 #include "if/gui_property.h"
 #include "if/gnet_property.h"
@@ -42,40 +43,25 @@ RCSID("$Id: uploads_common.c,v 1.25 2005/12/11 13:52:51 rmanfredi Exp $");
 #include "lib/tm.h"
 #include "lib/override.h"		/* Must be the last header included */
 
-#define IO_STALLED		60		/**< If nothing exchanged after that many secs */
-
-/**
- * Invoked from the core when we discover the Gnutella address and port
- * of the uploading party.
- */
-void
-uploads_gui_set_gnet_addr(gnet_upload_t u, host_addr_t addr, guint16 port)
-{
-	upload_row_data_t *rd = uploads_gui_get_row_data(u);
-
-	if (rd != NULL) {
-		rd->gnet_addr = addr;
-		rd->gnet_port = port;
-	}
-}
+#define IO_STALLED		60	/**< If nothing exchanged after that many secs */
 
 /**
  *
  * @returns a floating point value from [0:1] which indicates
  * the total progress of the upload.
  */
-gfloat
+gdouble
 uploads_gui_progress(const gnet_upload_status_t *u,
 	const upload_row_data_t *data)
 {
-	gfloat progress = 0.0;
-	filesize_t requested;
+	gdouble progress = 0.0;
 
 	if (u->pos < data->range_start) /* No progress yet */
 		return 0.0;
 
 	switch (u->status) {
     case GTA_UL_HEADERS:
+    case GTA_UL_EXPECTING:
     case GTA_UL_WAITING:
     case GTA_UL_PFSP_WAITING:
     case GTA_UL_ABORTED:
@@ -90,15 +76,12 @@ uploads_gui_progress(const gnet_upload_status_t *u,
 		progress = 1.0;
 		break;
 	case GTA_UL_SENDING:
-		requested = data->range_end - data->range_start + 1;
-		if (requested != 0) {
-			/*
-			 * position divided by 1 percentage point, found by dividing
-			 * the total size by 100
-			 */
-			progress = (gfloat) (u->pos - data->range_start) / requested;
-		} else {
-			progress = 0.0;
+		{
+			filesize_t requested, done;
+
+			requested = data->range_end - data->range_start + 1;
+			done = requested > 0 ? u->pos - data->range_start : 0;
+			progress = filesize_per_100(requested, done) / 100.0;
 		}
 		break;
 	}
@@ -124,14 +107,16 @@ uploads_gui_status_str(const gnet_upload_status_t *u,
 
     case GTA_UL_COMPLETE:
 		{
-			gint t = delta_time(u->last_update, data->start_date);
+			time_delta_t d = delta_time(data->last_update, data->start_date);
 	        filesize_t requested = data->range_end - data->range_start + 1;
 
 			gm_snprintf(tmpstr, sizeof(tmpstr),
-				_("%sCompleted (%s) %s"),
-				u->parq_quick ? "* " : "",
-				t > 0 ? short_rate(requested / t) : _("< 1s"),
-				t > 0 ? short_time(t) : "");
+				"%s (%s) %s%s #%u", _("Completed"),
+				d > 0 ? short_rate(requested / d, show_metric_units())
+						: _("< 1s"),
+				d > 0 ? short_time(d) : "",
+				u->parq_quick ? " (quick)" : "",
+				u->reqnum);
 		}
         break;
 
@@ -141,24 +126,29 @@ uploads_gui_status_str(const gnet_upload_status_t *u,
 			filesize_t tr = (data->range_end + 1 - u->pos) / MAX(1, u->avg_bps);
 			gdouble p = uploads_gui_progress(u, data);
 			time_t now = tm_time();
-			gboolean stalled = delta_time(now, u->last_update) > IO_STALLED;
+			gboolean stalled = delta_time(now, data->last_update) > IO_STALLED;
 			gchar pbuf[32];
 
 			gm_snprintf(pbuf, sizeof pbuf, "%5.02f%% ", p * 100.0);
-			gm_snprintf(tmpstr, sizeof tmpstr, _("%s%s(%s) TR: %s"),
-				u->parq_quick ? "* " : "",
+			gm_snprintf(tmpstr, sizeof tmpstr, _("%s(%s) TR: %s %s #%u"),
 				p > 1.0 ? pbuf : "",
-				stalled ? _("stalled") : short_rate(u->bps),
-				short_time(tr));
+				stalled ? _("stalled")
+					: short_rate(u->bps, show_metric_units()),
+				short_time(tr),
+				u->parq_quick ? " (quick)" : "",
+				u->reqnum);
 		}
 		break;
 
     case GTA_UL_HEADERS:
         return _("Waiting for headers...");
 
-    case GTA_UL_WAITING:
+    case GTA_UL_EXPECTING:
         return _("Waiting for further request...");
 
+    case GTA_UL_WAITING:
+        return _("Reading follow-up request...");
+
     case GTA_UL_PFSP_WAITING:
         return _("Unavailable range, waiting retry...");
 
@@ -250,6 +240,7 @@ upload_should_remove(time_t now, const upload_row_data_t *ul)
 	case GTA_UL_SENDING:
 	case GTA_UL_HEADERS:
 	case GTA_UL_WAITING:
+	case GTA_UL_EXPECTING:
 	case GTA_UL_QUEUED:
 	case GTA_UL_QUEUE:
 	case GTA_UL_QUEUE_WAITING:
@@ -258,14 +249,17 @@ upload_should_remove(time_t now, const upload_row_data_t *ul)
 	}
 
 	if (0 != prop) {
-		guint32 grace;
+		guint32 val;
+		time_delta_t grace;
+
+		gnet_prop_get_guint32_val(PROP_ENTRY_REMOVAL_TIMEOUT, &val);
+		grace = val;
 
-		gnet_prop_get_guint32_val(PROP_ENTRY_REMOVAL_TIMEOUT, &grace);
 		if (delta_time(now, ul->last_update) > grace) {
-			gboolean val;
+			gboolean auto_remove;
 
-			gui_prop_get_boolean_val(prop, &val);
-			return val;
+			gui_prop_get_boolean_val(prop, &auto_remove);
+			return auto_remove;
 		}
 	}
 
@@ -278,10 +272,23 @@ upload_should_remove(time_t now, const upload_row_data_t *ul)
 const gchar * 
 uploads_gui_host_string(const gnet_upload_info_t *u)
 {
-	static gchar buf[MAX_HOSTLEN + sizeof " (E)"];
+	static gchar buf[1024];
+	const gchar *peer;
+
+	if (u->gnet_port && is_host_addr(u->gnet_addr)) {
+		peer = host_addr_port_to_string(u->gnet_addr, u->gnet_port);
+	} else {
+		peer = NULL;
+	}
 
 	concat_strings(buf, sizeof buf,
-		host_addr_to_string(u->addr), u->encrypted ? " (E)" : "", (void *) 0);
+		host_addr_to_string(u->addr),
+		u->encrypted ? " (E) " : " ",
+		peer ? " <" : "",
+		peer ? peer : "",
+		peer ? ">"  : "",
+		(void *) 0);
+
 	return buf;
 }
 
@@ -292,7 +299,7 @@ void
 uploads_gui_browse_host(host_addr_t addr, guint16 port)
 {
 	if (host_addr_is_routable(addr) && port != 0)
-		search_gui_new_browse_host(NULL, addr, port, NULL, FALSE, NULL);
+		search_gui_new_browse_host(NULL, addr, port, NULL, NULL, 0);
 }
 
 /* vi: set ts=4 sw=4 cindent: */
